@@ -1,6 +1,6 @@
-(* $Id: colorize.ml 7372 2015-11-18 18:07:59Z jr_reuter $
+(* colorize.ml --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -8,7 +8,7 @@
        with contributions from
        Christian Speckner <cnspeckn@googlemail.com>
        Marco Sekulla <marco.sekulla@kit.edu>
-       Soyoung Shim <soyoung.shim@desy.de>
+       So Young Shim <soyoung.shim@desy.de>
 
    WHIZARD is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
@@ -24,21 +24,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
-let rcs_file = RCS.parse "Colorize" ["Colorizing Monochrome Models"]
-    { RCS.revision = "$Revision: 7372 $";
-      RCS.date = "$Date: 2015-11-18 19:07:59 +0100 (Wed, 18 Nov 2015) $";
-      RCS.author = "$Author: jr_reuter $";
-      RCS.source
-        = "$URL: svn+ssh://jr_reuter@login.hepforge.org/hepforge/svn/whizard/trunk/omega/src/colorize.ml $" }
-
 (* \thocwmodulesection{Colorizing a Monochrome Model} *)
 
 module It (M : Model.T) = 
   struct
 
-    let rcs = RCS.rename rcs_file "Colorize.It()"
-        [ "Colorizing Generic Monochrome Models"]
-
     open Coupling
 
     module C = Color
@@ -1542,9 +1532,6 @@ module It (M : Model.T) =
 module Gauge (M : Model.Gauge) = 
   struct
 
-    let rcs = RCS.rename rcs_file "Colorize.Gauge()"
-        [ "Colorizing Monochrome Gauge Models"]
-
     module CM = It(M)
 
     type flavor = CM.flavor
