@@ -1,6 +1,6 @@
-(* $Id: config.mli 6465 2015-01-10 15:22:31Z jr_reuter $
+(* config.mli --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -22,6 +22,12 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
+val version : string
+val date : string
+val status : string
+
+val default_UFO_dir : string
+
 (* Cache writing is attempted in the order [[system_cache_dir]], [[user_cache_dir]], [["."]]
    and cache reading in the opposite order. *)
 
