@@ -1,6 +1,6 @@
-(* $Id: modellib_MSSM.mli 6465 2015-01-10 15:22:31Z jr_reuter $
+(* modellib_MSSM.mli --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -40,11 +40,3 @@ module MSSM_no_4_ckm : MSSM_flags
 module MSSM_Grav : MSSM_flags
 module MSSM_Hgg : MSSM_flags
 module MSSM : functor (F: MSSM_flags) -> Model.T with module Ch = Charges.QQ
-
-(*i
- *  Local Variables:
- *  mode:caml
- *  indent-tabs-mode:nil
- *  page-delimiter:"^(\\* .*\n"
- *  End:
-i*)
