@@ -1,6 +1,6 @@
-(* $Id: modellib_NMSSM.ml 6465 2015-01-10 15:22:31Z jr_reuter $
+(* modellib_NMSSM.ml --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -23,13 +23,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
-let rcs_file = RCS.parse "Modellib_NMSSM" ["NMSSM"]
-    { RCS.revision = "$Revision: 6465 $";
-      RCS.date = "$Date: 2015-01-10 16:22:31 +0100 (Sat, 10 Jan 2015) $";
-      RCS.author = "$Author: jr_reuter $";
-      RCS.source
-        = "$URL: svn+ssh://jr_reuter@login.hepforge.org/hepforge/svn/whizard/trunk/omega/src/modellib_NMSSM.ml $" }
-
 (* \thocwmodulesection{Next-to-Minimal Supersymmetric Standard Model} *)
 
 (* This is based on the NMSSM implementation by Felix Braam. Note that for the 
@@ -62,9 +55,6 @@ module NMSSM_Hgg : NMSSM_flags =
 
 module NMSSM_func (Flags : NMSSM_flags) = 
   struct
-    let rcs = RCS.rename rcs_file "Modellib_NMSSM.NMSSM"
-        [ "NMSSM" ]
-
     open Coupling
 
     let default_width = ref Timelike
@@ -1597,11 +1587,3 @@ generalization to complex parameters is obvious. *)
           ^ "," ^ string_of_sfm m2 ^ ")"
       
   end
-
-(*i
- *  Local Variables:
- *  mode:caml
- *  indent-tabs-mode:nil
- *  page-delimiter:"^(\\* .*\n"
- *  End:
-i*)
