@@ -1,6 +1,6 @@
-(* $Id: modellib_NMSSM.mli 6465 2015-01-10 15:22:31Z jr_reuter $
+(* modellib_NMSSM.mli --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -39,12 +39,3 @@ module NMSSM : NMSSM_flags
 module NMSSM_CKM : NMSSM_flags
 module NMSSM_Hgg : NMSSM_flags
 module NMSSM_func : functor (F: NMSSM_flags) -> Model.T with module Ch = Charges.QQ
-
-
-(*i
- *  Local Variables:
- *  mode:caml
- *  indent-tabs-mode:nil
- *  page-delimiter:"^(\\* .*\n"
- *  End:
-i*)
