@@ -1,6 +1,6 @@
-(* $Id: modellib_NoH.mli 5045 2014-03-21 17:09:34Z msekulla $
+(* modellib_NoH.mli --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -39,11 +39,3 @@ module NoH_k_matrix : NoH_flags
 
 module NoH : functor (F : NoH_flags) -> Model.Gauge with module Ch = Charges.QQ
 module AltH: functor (F : NoH_flags) -> Model.Gauge with module Ch = Charges.QQ
-
-(*i
- *  Local Variables:
- *  mode:caml
- *  indent-tabs-mode:nil
- *  page-delimiter:"^(\\* .*\n"
- *  End:
-i*)
