@@ -1,6 +1,6 @@
-(* $Id: modellib_PSSSM.ml 6465 2015-01-10 15:22:31Z jr_reuter $
+(* modellib_PSSSM.ml --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -21,13 +21,6 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
 
-let rcs_file = RCS.parse "Modellib_PSSSM" ["Extended SUSY models"]
-    { RCS.revision = "$Revision: 6465 $";
-      RCS.date = "$Date: 2015-01-10 16:22:31 +0100 (Sat, 10 Jan 2015) $";
-      RCS.author = "$Author: jr_reuter $";
-      RCS.source
-        = "$URL: svn+ssh://jr_reuter@login.hepforge.org/hepforge/svn/whizard/trunk/omega/src/modellib_PSSSM.ml $" }
-
 (* \thocwmodulesection{Extended Supersymmetric Standard Model(s)} *)
 
 (* This is based on the NMSSM implementation by Felix Braam, and extended to
@@ -52,9 +45,6 @@ module PSSSM_QCD : extMSSM_flags =
 
 module ExtMSSM (Flags : extMSSM_flags) = 
   struct
-    let rcs = RCS.rename rcs_file "Modellib_PSSSM.NMSSM"
-        [ "Extended SUSY models" ]
-
     open Coupling
 
     let default_width = ref Timelike
@@ -68,7 +58,9 @@ module ExtMSSM (Flags : extMSSM_flags) =
         "custom_width", Arg.String (fun f -> default_width := Custom f),
         "use custom width";
         "cancel_widths", Arg.Unit (fun () -> default_width := Vanishing),
-        "use vanishing width"]
+        "use vanishing width";
+        "cms_width", Arg.Unit (fun () -> default_width := Complex_Mass),
+        "use complex mass scheme"]
 
 
 (*additional combinatorics *)
@@ -1976,11 +1968,3 @@ generalization to complex parameters is obvious. *)
       | G_NLQC -> "g_nlqc"
       
   end
-
-(*i
- *  Local Variables:
- *  mode:caml
- *  indent-tabs-mode:nil
- *  page-delimiter:"^(\\* .*\n"
- *  End:
-i*)
