@@ -1,10 +1,11 @@
-(* $Id: modellib_SM.mli 6264 2014-11-14 15:40:49Z fbach $
+(* modellib_Zprime.mli --
 
-   Copyright (C) 1999-2014 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
        Juergen Reuter <juergen.reuter@desy.de>
+       with contributions from
        Christian Speckner <cnspeckn@googlemail.com>
 
    WHIZARD is free software; you can redistribute it and/or modify it
@@ -32,11 +33,3 @@ module type SM_flags =
 module SM_no_anomalous : SM_flags
 
 module Zprime : functor (F : SM_flags) -> Model.Gauge with module Ch = Charges.QQ
-
-(*i
- *  Local Variables:
- *  mode:caml
- *  indent-tabs-mode:nil
- *  page-delimiter:"^(\\* .*\n"
- *  End:
-i*)
