@@ -1,6 +1,6 @@
-(* $Id: modeltools.ml 6465 2015-01-10 15:22:31Z jr_reuter $
+(* modeltools.ml --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -22,13 +22,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
-let rcs_file = RCS.parse "Modeltools" ["Lagragians"]
-    { RCS.revision = "$Revision: 6465 $";
-      RCS.date = "$Date: 2015-01-10 16:22:31 +0100 (Sat, 10 Jan 2015) $";
-      RCS.author = "$Author: jr_reuter $";
-      RCS.source
-        = "$URL: svn+ssh://jr_reuter@login.hepforge.org/hepforge/svn/whizard/trunk/omega/src/modeltools.ml $" }
-
 (* \thocwmodulesection{Compilation} *)
 
 (* Flavors and coupling constants:  flavors can be tested for equality
@@ -242,6 +235,58 @@ module Fusions (F : Flavor) : Fusions with type f = F.f and type c = F.c =
 
   end
 
+module type Constant =
+  sig
+    type t
+    val of_string : string -> t
+  end
+
+module Constant (M : Model.T) : Constant with type t = M.constant =
+  struct
+
+    type t = M.constant
+
+    module String_Key =
+      struct
+        type t = string
+        let hash = Hashtbl.hash
+        let equal = (=)
+      end
+    module String_Hash = Hashtbl.Make (String_Key)
+
+    let table = String_Hash.create 37
+
+    let fill_table table vs =
+      List.iter
+        (fun (_, _, c) ->
+          String_Hash.add table (M.constant_symbol c) c)
+        vs
+
+    (* Delay loading of the tables until the first use, so that
+       [M.vertices] can be initialized from a file.  *)
+
+    let tables_filled = ref false
+
+    let fill_tables () =
+      if not !tables_filled then begin
+	let (v3, v4, vn) = M.vertices () in
+	fill_table table v3;
+	fill_table table v4;
+	fill_table table vn;
+	tables_filled := true
+      end
+
+    let of_string name =
+      try
+	fill_tables ();
+        String_Hash.find table name
+      with
+      | Not_found ->
+          invalid_arg
+            ("Constant(Model).of_string: unknown coupling constant: " ^ name)
+
+  end
+
 (* \thocwmodulesection{Mutable Models} *)
 
 module Mutable (FGC : sig type f and g and c end) =
@@ -250,13 +295,15 @@ module Mutable (FGC : sig type f and g and c end) =
     type gauge = FGC.g
     type constant = FGC.c
 
+    let init () = ()
+
     let options = Options.empty
 
     module Ch = Charges.Null
     let charges _ = ()
 
     exception Uninitialized of string
-    let unitialized name =
+    let uninitialized name =
       raise (Uninitialized name)
       
 (* Note that [lookup] works, by the magic of currying, for any arity.  But
@@ -271,57 +318,63 @@ module Mutable (FGC : sig type f and g and c end) =
       (update, lookup)
 
     let set_color, color =
-      declare (fun f -> unitialized "color")
+      declare (fun f -> uninitialized "color")
     let set_pdg, pdg =
-      declare (fun f -> unitialized "pdg")
+      declare (fun f -> uninitialized "pdg")
     let set_lorentz, lorentz =
-      declare (fun f -> unitialized "lorentz")
+      declare (fun f -> uninitialized "lorentz")
     let set_propagator, propagator =
-      declare (fun f -> unitialized "propagator")
+      declare (fun f -> uninitialized "propagator")
     let set_width, width =
-      declare (fun f -> unitialized "width")
+      declare (fun f -> uninitialized "width")
     let set_goldstone, goldstone =
-      declare (fun f -> unitialized "goldstone")
+      declare (fun f -> uninitialized "goldstone")
     let set_conjugate, conjugate =
-      declare (fun f -> unitialized "conjugate")
+      declare (fun f -> uninitialized "conjugate")
     let set_fermion, fermion =
-      declare (fun f -> unitialized "fermion")
+      declare (fun f -> uninitialized "fermion")
     let set_max_degree, max_degree =
-      declare (fun () -> unitialized "max_degree")
+      declare (fun () -> uninitialized "max_degree")
     let set_vertices, vertices =
-      declare (fun () -> unitialized "vertices")
+      declare (fun () -> (* ([], [], []) *) uninitialized "vertices" )
     let set_fuse2, fuse2 =
-      declare (fun f1 f2 -> unitialized "fuse2")
+      declare (fun f1 f2 -> uninitialized "fuse2")
     let set_fuse3, fuse3 =
-      declare (fun f1 f2 f3 -> unitialized "fuse3")
+      declare (fun f1 f2 f3 -> uninitialized "fuse3")
     let set_fuse, fuse =
-      declare (fun f -> unitialized "fuse")
+      declare (fun f -> uninitialized "fuse")
     let set_flavors, flavors =
       declare (fun () -> [])
     let set_external_flavors, external_flavors =
-      declare (fun () -> [("unitialized", [])])
+      declare (fun () -> [("uninitialized", [])])
     let set_parameters, parameters =
-      declare (fun f -> unitialized "parameters")
+      declare (fun f -> uninitialized "parameters")
     let set_flavor_of_string, flavor_of_string =
-      declare (fun f -> unitialized "flavor_of_string")
+      declare (fun f -> uninitialized "flavor_of_string")
     let set_flavor_to_string, flavor_to_string =
-      declare (fun f -> unitialized "flavor_to_string")
+      declare (fun f -> uninitialized "flavor_to_string")
     let set_flavor_to_TeX, flavor_to_TeX =
-      declare (fun f -> unitialized "flavor_to_TeX")
+      declare (fun f -> uninitialized "flavor_to_TeX")
     let set_flavor_symbol, flavor_symbol =
-      declare (fun f -> unitialized "flavor_symbol")
+      declare (fun f -> uninitialized "flavor_symbol")
     let set_gauge_symbol, gauge_symbol =
-      declare (fun f -> unitialized "gauge_symbol")
+      declare (fun f -> uninitialized "gauge_symbol")
     let set_mass_symbol, mass_symbol =
-      declare (fun f -> unitialized "mass_symbol")
+      declare (fun f -> uninitialized "mass_symbol")
     let set_width_symbol, width_symbol =
-      declare (fun f -> unitialized "width_symbol")
+      declare (fun f -> uninitialized "width_symbol")
     let set_constant_symbol, constant_symbol =
-      declare (fun f -> unitialized "constant_symbol")
+      declare (fun f -> uninitialized "constant_symbol")
+
+    module F = Fusions (struct
+      type f = flavor
+      type c = constant
+      let compare = compare
+      let conjugate = conjugate
+    end)
 
     let setup ~color ~pdg ~lorentz ~propagator ~width ~goldstone
         ~conjugate ~fermion ~max_degree ~vertices 
-        ~fuse:(fuse2, fuse3, fusen)
         ~flavors ~parameters ~flavor_of_string ~flavor_to_string
         ~flavor_to_TeX ~flavor_symbol
         ~gauge_symbol ~mass_symbol ~width_symbol ~constant_symbol =
@@ -334,13 +387,14 @@ module Mutable (FGC : sig type f and g and c end) =
       set_conjugate conjugate;
       set_fermion fermion;
       set_max_degree (fun () -> max_degree);
-      set_vertices vertices;
-      set_fuse2 fuse2;
-      set_fuse3 fuse3;
-      set_fuse fusen;
-      set_external_flavors (fun f -> flavors);
+      set_vertices (fun () -> vertices);
+      let table = F.of_vertices vertices in
+      set_fuse2 (F.fuse2 table);
+      set_fuse3 (F.fuse3 table);
+      set_fuse (F.fuse table);
+      set_external_flavors (fun () -> flavors);
       let flavors = ThoList.flatmap snd flavors in
-      set_flavors (fun f -> flavors);
+      set_flavors (fun () -> flavors);
       set_parameters parameters;
       set_flavor_of_string flavor_of_string;
       set_flavor_to_string flavor_to_string;
@@ -351,14 +405,45 @@ module Mutable (FGC : sig type f and g and c end) =
       set_width_symbol width_symbol;
       set_constant_symbol constant_symbol
 
-    let rcs = RCS.rename rcs_file "Models.Mutable" ["Mutable Model"]
   end
 
-(*i
- *  Local Variables:
- *  mode:caml
- *  indent-tabs-mode:nil
- *  page-delimiter:"^(\\* .*\n"
- *  End:
-i*)
-
+module Static (M : Model.T) =
+  struct
+    type flavor = M.flavor
+    type gauge = M.gauge
+    type constant = M.constant
+    module Ch = M.Ch
+    let color = M.color
+    let charges = M.charges
+    let pdg = M.pdg
+    let lorentz = M.lorentz
+    let propagator = M.propagator
+    let width = M.width
+    let conjugate = M.conjugate
+    let fermion = M.fermion
+    let max_degree = M.max_degree
+    let vertices = M.vertices
+    let fuse2 = M.fuse2
+    let fuse3 = M.fuse3
+    let fuse = M.fuse
+    let flavors = M.flavors
+    let external_flavors = M.external_flavors
+    let goldstone = M.goldstone
+    let parameters = M.parameters
+    let flavor_of_string = M.flavor_of_string
+    let flavor_to_string = M.flavor_to_string
+    let flavor_to_TeX = M.flavor_to_TeX
+    let flavor_symbol = M.flavor_symbol
+    let gauge_symbol = M.gauge_symbol
+    let mass_symbol = M.mass_symbol
+    let width_symbol = M.width_symbol
+    let constant_symbol = M.constant_symbol
+    let options = M.options
+    let init () = ()
+    let setup ~color ~pdg ~lorentz ~propagator ~width ~goldstone
+        ~conjugate ~fermion ~max_degree ~vertices 
+        ~flavors ~parameters ~flavor_of_string ~flavor_to_string
+        ~flavor_to_TeX ~flavor_symbol
+        ~gauge_symbol ~mass_symbol ~width_symbol ~constant_symbol =
+      ()
+  end
