@@ -1,6 +1,6 @@
-(* $Id: momentum.mli 6465 2015-01-10 15:22:31Z jr_reuter $
+(* momentum.mli --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -204,7 +204,6 @@ module type T =
 
         end
 
-    val rcs : RCS.t
   end
 
 module Lists : T
