@@ -1,6 +1,6 @@
-(* $Id: omega_SYM.ml 6465 2015-01-10 15:22:31Z jr_reuter $
+(* omega_SYM.ml --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -23,19 +23,9 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
 
-let rcs_file = RCS.parse "omega_SYM"
-    ["Super Yang-Mills (incomplete, just for stress-testing Colorize.It())"]
-    { RCS.revision = "$Revision: 6465 $";
-      RCS.date = "$Date: 2015-01-10 16:22:31 +0100 (Sat, 10 Jan 2015) $";
-      RCS.author = "$Author: jr_reuter $";
-      RCS.source
-        = "$URL: svn+ssh://jr_reuter@login.hepforge.org/hepforge/svn/whizard/trunk/omega/src/omega_SYM.ml $" }
-
 module SYM = 
   struct
 
-    let rcs = rcs_file
-
     open Coupling
 
     let options = Options.empty
