@@ -1,6 +1,6 @@
-(* $Id: partition.mli 6465 2015-01-10 15:22:31Z jr_reuter $
+(* partition.mli --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -35,8 +35,6 @@ val triples : int -> int -> int -> (int * int * int) list
    \end{equation} *)
 val tuples : int -> int -> int -> int -> int list list
 
-val rcs : RCS.t
-
 (*i
  *  Local Variables:
  *  mode:caml
