@@ -1,6 +1,6 @@
-(* $Id: whizard.ml 6465 2015-01-10 15:22:31Z jr_reuter $
+(* whizard.ml --
 
-   Copyright (C) 1999-2015 by
+   Copyright (C) 1999-2017 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -22,13 +22,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)
 
-let rcs = RCS.parse "Whizard" ["Whizard Interface"]
-    { RCS.revision = "$Revision: 6465 $";
-      RCS.date = "$Date: 2015-01-10 16:22:31 +0100 (Sat, 10 Jan 2015) $";
-      RCS.author = "$Author: jr_reuter $";
-      RCS.source
-        = "$URL: svn+ssh://jr_reuter@login.hepforge.org/hepforge/svn/whizard/trunk/omega/src/whizard.ml $" }
-
 open Printf
 
 module type T =
@@ -260,9 +253,6 @@ i*)
           fprintf ch "\n    grove\n";
           fprintf ch "    %s\n" (whizard_tree d);
           analyze_tree ch d) ds) t.trees;
-      fprintf ch "\n! O'Mega revision control information:\n";
-      List.iter (fun s -> fprintf ch "!    %s\n" s)
-        (ThoList.flatmap RCS.summary (rcs :: M.rcs :: F.rcs_list));
       fprintf ch "\n"
 i*)
 
