@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -39,7 +39,7 @@ module beams
   use iso_varying_string, string_t => varying_string
   use io_units
   use format_defs, only: FMT_19
-  use unit_tests, only: nearly_equal
+  use numeric_utils
   use diagnostics
   use md5
   use lorentz
@@ -86,8 +86,9 @@ module beams
      procedure :: get_energy => beam_data_get_energy
      procedure :: get_sqrts => beam_data_get_sqrts
      procedure :: cm_frame => beam_data_cm_frame
+     procedure :: get_polarization => beam_data_get_polarization
      procedure :: get_helicity_state_matrix => beam_data_get_helicity_state_matrix
-     procedure :: is_initialized => beam_data_is_initialized 
+     procedure :: is_initialized => beam_data_is_initialized
      procedure :: get_md5sum => beam_data_get_md5sum
      procedure :: init_structure => beam_data_init_structure
      procedure :: init_sqrts => beam_data_init_sqrts
@@ -150,7 +151,7 @@ contains
        write (u, "(1x,A)") "Beam data (decay):"
        if (verb) then
           call write_prt (1)
-          call beam_data%pmatrix(1)%write (u)          
+          call beam_data%pmatrix(1)%write (u)
           write (u, *) "R.f. momentum:"
           call vector4_write (beam_data%p_cm(1), u)
           write (u, *) "Lab momentum:"
@@ -162,7 +163,7 @@ contains
        write (u, "(1x,A)") "Beam data (collision):"
        if (verb) then
           call write_prt (1)
-          call beam_data%pmatrix(1)%write (u)          
+          call beam_data%pmatrix(1)%write (u)
           call write_prt (2)
           call beam_data%pmatrix(2)%write (u)
           call write_sqrts
@@ -251,10 +252,9 @@ contains
     integer :: i
     allocate (e (beam_data%n))
     if (beam_data%initialized) then
-       do i = 1, beam_data%n          
+       do i = 1, beam_data%n
           e(i) = energy (beam_data%p(i))
        end do
-       !!! e = energy (beam_data%p)       
     else
        e = 0
     end if
@@ -265,13 +265,21 @@ contains
     real(default) :: sqrts
     sqrts = beam_data%sqrts
   end function beam_data_get_sqrts
-  
+
   function beam_data_cm_frame (beam_data) result (flag)
     class(beam_data_t), intent(in) :: beam_data
     logical :: flag
     flag = beam_data%lab_is_cm_frame
   end function beam_data_cm_frame
-  
+
+  function beam_data_get_polarization (beam_data) result (pol)
+    class(beam_data_t), intent(in) :: beam_data
+    real(default), dimension(2) :: pol
+    if (beam_data%n /= 2) &
+         call msg_fatal ("Beam data: can only treat scattering processes.")
+    pol = beam_data%pmatrix%get_simple_pol ()
+  end function beam_data_get_polarization
+
   function beam_data_get_helicity_state_matrix (beam_data) result (state_hel)
     type(state_matrix_t) :: state_hel
     class(beam_data_t), intent(in) :: beam_data
@@ -279,12 +287,9 @@ contains
     integer :: i
     allocate (pol (beam_data%n))
     do i = 1, beam_data%n
-       call polarization_init_pmatrix (pol(i), beam_data%pmatrix(i))
+       call pol(i)%init_pmatrix (beam_data%pmatrix(i))
     end do
     call combine_polarization_states (pol, state_hel)
-    do i = 1, beam_data%n
-       call polarization_final (pol(i))
-    end do
   end function beam_data_get_helicity_state_matrix
 
   function beam_data_is_initialized (beam_data) result (initialized)
@@ -348,7 +353,7 @@ contains
        end select
     end if
   end subroutine beam_data_init_structure
-    
+
   subroutine beam_data_init_sqrts (beam_data, sqrts, flv, smatrix, pol_f)
     class(beam_data_t), intent(out) :: beam_data
     real(default), intent(in) :: sqrts
@@ -391,7 +396,7 @@ contains
     !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug
     !!! p = vector4_moving (e, p3)
     do i = 1, beam_data%n
-       p(i) = vector4_moving (e(i), p3(i))    
+       p(i) = vector4_moving (e(i), p3(i))
     end do
     p0 = sum (p)
     beam_data%p = p
@@ -413,7 +418,7 @@ contains
     end select
     call beam_data_finish_initialization (beam_data, flv, smatrix, pol_f)
   end subroutine beam_data_init_momenta
-    
+
   subroutine beam_data_finish_initialization (beam_data, flv, smatrix, pol_f)
     type(beam_data_t), intent(inout) :: beam_data
     type(flavor_t), dimension(:), intent(in) :: flv
@@ -481,16 +486,20 @@ contains
   subroutine beam_init (beam, beam_data)
     type(beam_t), intent(out) :: beam
     type(beam_data_t), intent(in), target :: beam_data
-    type(quantum_numbers_mask_t), dimension(beam_data%n) :: mask
+    logical, dimension(beam_data%n) :: polarized, diagonal
+    type(quantum_numbers_mask_t), dimension(beam_data%n) :: mask, mask_d
     type(state_matrix_t), target :: state_hel, state_fc, state_tmp
     type(state_iterator_t) :: it_hel, it_tmp
     type(quantum_numbers_t), dimension(:), allocatable :: qn
-    !type(polarization_t), dimension(:), allocatable :: pol
-    integer :: i
-    real(default), dimension(:,:), allocatable :: pol_matrix
+    complex(default) :: value
+    real(default), parameter :: tolerance = 100 * epsilon (1._default)
+    polarized = beam_data%pmatrix%is_polarized ()
+    diagonal = beam_data%pmatrix%is_diagonal ()
     mask = quantum_numbers_mask (.false., .false., &
-         .not. beam_data%pmatrix%is_polarized (), &
-         mask_hd = beam_data%pmatrix%is_diagonal ())
+         mask_h = .not. polarized, &
+         mask_hd = diagonal)
+    mask_d = quantum_numbers_mask (.false., .false., .false., &
+         mask_hd = polarized .and. diagonal)
     call beam%int%basic_init &
          (0, 0, beam_data%n, mask=mask, store_values=.true.)
     state_hel = beam_data%get_helicity_state_matrix ()
@@ -502,8 +511,15 @@ contains
     call it_hel%init (state_hel)
     call it_tmp%init (state_tmp)
     do while (it_hel%is_valid ())
-       call beam%int%add_state (it_tmp%get_quantum_numbers (), &
-            value=it_hel%get_matrix_element ())
+       qn = it_tmp%get_quantum_numbers ()
+       value = it_hel%get_matrix_element ()
+       if (any (qn%are_redundant (mask_d))) then
+          ! skip off-diagonal elements for diagonal polarization
+       else if (abs (value) <= tolerance) then
+          ! skip zero entries
+       else
+          call beam%int%add_state (qn, value = value)
+       end if
        call it_hel%advance ()
        call it_tmp%advance ()
     end do
@@ -519,10 +535,11 @@ contains
     call beam%int%final ()
   end subroutine beam_final
 
-  subroutine beam_write (beam, unit, verbose, show_momentum_sum, show_mass)
+  subroutine beam_write (beam, unit, verbose, show_momentum_sum, show_mass, col_verbose)
     type(beam_t), intent(in) :: beam
     integer, intent(in), optional :: unit
     logical, intent(in), optional :: verbose, show_momentum_sum, show_mass
+    logical, intent(in), optional :: col_verbose
     integer :: u
     u = given_output_unit (unit);  if (u < 0)  return
     select case (beam%int%get_n_out ())
@@ -531,7 +548,8 @@ contains
     end select
     call beam%int%basic_write &
          (unit, verbose = verbose, show_momentum_sum = &
-            show_momentum_sum, show_mass = show_mass)
+          show_momentum_sum, show_mass = show_mass, &
+          col_verbose = col_verbose)
   end subroutine beam_write
 
   subroutine beam_assign (beam_out, beam_in)
