@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -38,7 +38,7 @@ module sf_aux
   use kinds, only: default
   use io_units
   use constants, only: twopi
-  use unit_tests, only: vanishes
+  use numeric_utils
 
   use lorentz
 
@@ -132,10 +132,14 @@ contains
     real(default) :: tp, tm
     if (present (x))  d%x = x
     if (present (xb)) d%xb = xb
-    if (.not. vanishes (d%xb)) then
-       d%pb = sqrt (max (d%E**2 - d%u / d%xb**2, 0._default))
+    if (vanishes (d%u)) then
+       d%pb = d%E
     else
-       d%pb = 0
+       if (.not. vanishes (d%xb)) then
+          d%pb = sqrt (max (d%E**2 - d%u / d%xb**2, 0._default))
+       else
+          d%pb = 0
+       end if
     end if
     tp = -2 * d%xb * d%E**2 + d%s + d%u
     tm = -2 * d%xb * d%p * d%pb
@@ -184,7 +188,7 @@ contains
        end if
     end if
   end subroutine splitting_inverse_t
-    
+
   subroutine splitting_sample_phi (d, r)
     class(splitting_data_t), intent(inout) :: d
     real(default), intent(in) :: r
@@ -224,21 +228,24 @@ contains
           q(1) = vector4_moving (d%xb * d%E, q1)
           q(2) = vector4_moving (d%x * d%E, q2)
        end if
-    else       
-       den = 2 * d%xb * d%p * d%pb       
+    else
+       den = 2 * d%xb * d%p * d%pb
        tt0 = max (d%t - d%t0, 0._default)
        tt1 = min (d%t - d%t1, 0._default)
        if (den**2 <= epsilon(den)) then
-          st2 = 1
+          st2 = 0
        else
           st2 = - (tt0 * tt1) / den ** 2
        end if
-       if (st2 > 1) then 
+       if (st2 > 1) then
           st2 = 1
        end if
        ct2 = 1 - st2
        st = sqrt (max (st2, 0._default))
        ct = sqrt (max (ct2, 0._default))
+       if ((d%t - d%t0 + d%t - d%t1) < 0) then
+          ct = - ct
+       end if
        sp = sin (d%phi)
        cp = cos (d%phi)
        rot = rotation_to_2nd (3, space_part (k))
@@ -248,7 +255,7 @@ contains
        q(2) = rot * vector4_moving (d%x * d%E, q2)
     end if
   end function splitting_split_momentum
-    
+
   elemental subroutine on_shell (p, m2, keep)
     type(vector4_t), intent(inout) :: p
     real(default), intent(in) :: m2
@@ -334,7 +341,7 @@ contains
        end if
     end if
   end subroutine splitting_recover
-       
+
   function splitting_get_x (sd) result (x)
     class(splitting_data_t), intent(in) :: sd
     real(default) :: x
