@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -40,7 +40,7 @@ module sf_epa
   use io_units
   use constants, only: pi
   use format_defs, only: FMT_17, FMT_19
-  use unit_tests
+  use numeric_utils
   use diagnostics
   use physics_defs, only: PHOTON
   use lorentz
@@ -52,8 +52,9 @@ module sf_epa
   use state_matrices
   use polarizations
   use interactions
+  use sf_aux
   use sf_base
-  
+
   implicit none
   private
 
@@ -84,20 +85,21 @@ module sf_epa
      real(default) :: dc
      integer :: error = NONE
      logical :: recoil = .false.
+     logical :: keep_energy = .true.
    contains
      procedure :: init => epa_data_init
      procedure :: check => epa_data_check
      procedure :: write => epa_data_write
      procedure :: get_n_par => epa_data_get_n_par
      procedure :: get_pdg_out => epa_data_get_pdg_out
-     procedure :: allocate_sf_int => epa_data_allocate_sf_int  
+     procedure :: allocate_sf_int => epa_data_allocate_sf_int
   end type epa_data_t
 
   type, extends (sf_int_t) :: epa_t
      type(epa_data_t), pointer :: data => null ()
      real(default) :: x  = 0
      real(default) :: xb = 0
-     real(default) :: E  = 0     
+     real(default) :: E  = 0
      real(default), dimension(:), allocatable :: charge2
    contains
      procedure :: type_string => epa_type_string
@@ -108,19 +110,20 @@ module sf_epa
      procedure :: inverse_kinematics => epa_inverse_kinematics
      procedure :: recover_x => sf_epa_recover_x
      procedure :: apply => epa_apply
-  end type epa_t 
-  
+  end type epa_t
+
 
 contains
 
-  subroutine epa_data_init &
-       (data, model, pdg_in, alpha, x_min, q_min, E_max, mass, recoil)
+  subroutine epa_data_init (data, model, pdg_in, alpha, &
+       x_min, q_min, E_max, mass, recoil, keep_energy)
     class(epa_data_t), intent(inout) :: data
     class(model_data_t), intent(in), target :: model
     type(pdg_array_t), intent(in) :: pdg_in
     real(default), intent(in) :: alpha, x_min, q_min, E_max
     real(default), intent(in), optional :: mass
     logical, intent(in), optional :: recoil
+    logical, intent(in), optional :: keep_energy
     integer :: n_flv, i
     data%model => model
     n_flv = pdg_array_get_length (pdg_in)
@@ -140,17 +143,20 @@ contains
     select case (char (data%model%get_name ()))
     case ("QCD","Test")
        data%error = NO_EPA;  return
-    end select     
+    end select
     if (present (recoil)) then
        data%recoil = recoil
     end if
+    if (present (keep_energy)) then
+       data%keep_energy = keep_energy
+    end if
     if (present (mass)) then
        data%mass = mass
     else
        data%mass = data%flv_in(1)%get_mass ()
        if (any (data%flv_in%get_mass () /= data%mass)) then
           data%error = MASS_MIX;  return
-       end if 
+       end if
     end if
     if (max (data%mass, data%q_min) == 0) then
        data%error = ZERO_QMIN;  return
@@ -181,7 +187,7 @@ contains
     end select
   end subroutine epa_data_check
 
-  subroutine epa_data_write (data, unit, verbose) 
+  subroutine epa_data_write (data, unit, verbose)
     class(epa_data_t), intent(in) :: data
     integer, intent(in), optional :: unit
     logical, intent(in), optional :: verbose
@@ -195,18 +201,19 @@ contains
           call data%flv_in(i)%write (u)
        end do
        write (u, *)
-       write (u, "(3x,A," // FMT_19 // ")") "  alpha  = ", data%alpha
-       write (u, "(3x,A," // FMT_19 // ")") "  x_min  = ", data%x_min
-       write (u, "(3x,A," // FMT_19 // ")") "  x_max  = ", data%x_max
-       write (u, "(3x,A," // FMT_19 // ")") "  q_min  = ", data%q_min
-       write (u, "(3x,A," // FMT_19 // ")") "  q_max  = ", data%q_max
-       write (u, "(3x,A," // FMT_19 // ")") "  E_max  = ", data%e_max
-       write (u, "(3x,A," // FMT_19 // ")") "  mass   = ", data%mass
-       write (u, "(3x,A," // FMT_19 // ")") "  a      = ", data%a
-       write (u, "(3x,A," // FMT_19 // ")") "  c0     = ", data%c0
-       write (u, "(3x,A," // FMT_19 // ")") "  c1     = ", data%c1
-       write (u, "(3x,A," // FMT_19 // ")") "  log    = ", data%log
-       write (u, "(3x,A,L2)")      "  recoil = ", data%recoil
+       write (u, "(3x,A," // FMT_19 // ")") "  alpha    = ", data%alpha
+       write (u, "(3x,A," // FMT_19 // ")") "  x_min    = ", data%x_min
+       write (u, "(3x,A," // FMT_19 // ")") "  x_max    = ", data%x_max
+       write (u, "(3x,A," // FMT_19 // ")") "  q_min    = ", data%q_min
+       write (u, "(3x,A," // FMT_19 // ")") "  q_max    = ", data%q_max
+       write (u, "(3x,A," // FMT_19 // ")") "  E_max    = ", data%e_max
+       write (u, "(3x,A," // FMT_19 // ")") "  mass     = ", data%mass
+       write (u, "(3x,A," // FMT_19 // ")") "  a        = ", data%a
+       write (u, "(3x,A," // FMT_19 // ")") "  c0       = ", data%c0
+       write (u, "(3x,A," // FMT_19 // ")") "  c1       = ", data%c1
+       write (u, "(3x,A," // FMT_19 // ")") "  log      = ", data%log
+       write (u, "(3x,A,L2)")      "  recoil   = ", data%recoil
+       write (u, "(3x,A,L2)")      "  keep en. = ", data%keep_energy
     else
        write (u, "(3x,A)") "[undefined]"
     end if
@@ -221,29 +228,29 @@ contains
        n = 1
     end if
   end function epa_data_get_n_par
-  
+
   subroutine epa_data_get_pdg_out (data, pdg_out)
     class(epa_data_t), intent(in) :: data
     type(pdg_array_t), dimension(:), intent(inout) :: pdg_out
     pdg_out(1) = PHOTON
   end subroutine epa_data_get_pdg_out
-  
+
   subroutine epa_data_allocate_sf_int (data, sf_int)
     class(epa_data_t), intent(in) :: data
     class(sf_int_t), intent(inout), allocatable :: sf_int
     allocate (epa_t :: sf_int)
   end subroutine epa_data_allocate_sf_int
-  
+
   function epa_type_string (object) result (string)
     class(epa_t), intent(in) :: object
     type(string_t) :: string
     if (associated (object%data)) then
-       string = "EPA: equivalent photon approx." 
+       string = "EPA: equivalent photon approx."
     else
        string = "EPA: [undefined]"
     end if
   end function epa_type_string
-  
+
   subroutine epa_write (object, unit, testflag)
     class(epa_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -257,25 +264,25 @@ contains
           write (u, "(3x,A," // FMT_17 // ")")  "x =", object%x
           if (object%status >= SF_FAILED_EVALUATION) then
              write (u, "(3x,A," // FMT_17 // ")")  "E =", object%E
-          end if          
+          end if
        end if
        call object%base_write (u, testflag)
     else
        write (u, "(1x,A)")  "EPA data: [undefined]"
     end if
   end subroutine epa_write
-    
+
   subroutine epa_init (sf_int, data)
     class(epa_t), intent(out) :: sf_int
     class(sf_data_t), intent(in), target :: data
     type(quantum_numbers_mask_t), dimension(3) :: mask
     integer, dimension(3) :: hel_lock
-    type(polarization_t) :: pol
-    type(quantum_numbers_t), dimension(1) :: qn_fc, qn_hel
+    type(polarization_t), target :: pol
+    type(quantum_numbers_t), dimension(1) :: qn_fc
     type(flavor_t) :: flv_photon
     type(color_t) :: col_photon
-    type(quantum_numbers_t) :: qn_photon, qn, qn_rad
-    type(state_iterator_t) :: it_hel
+    type(quantum_numbers_t) :: qn_hel, qn_photon, qn, qn_rad
+    type(polarization_iterator_t) :: it_hel
     integer :: i
     mask = quantum_numbers_mask (.false., .false., &
          mask_h = [.false., .false., .true.])
@@ -283,36 +290,41 @@ contains
     select type (data)
     type is (epa_data_t)
        call sf_int%base_init (mask, [data%mass**2], &
-            [data%mass**2], [0._default], hel_lock = hel_lock)       
+            [data%mass**2], [0._default], hel_lock = hel_lock)
        sf_int%data => data
        call flv_photon%init (PHOTON, data%model)
        call col_photon%init ()
        call qn_photon%init (flv_photon, col_photon)
        do i = 1, size (data%flv_in)
-          call polarization_init_generic (pol, data%flv_in(i))
+          call pol%init_generic (data%flv_in(i))
           call qn_fc(1)%init ( &
                flv = data%flv_in(i), &
                col = color_from_flavor (data%flv_in(i), 1))
-          call it_hel%init (pol%state)
+          call it_hel%init (pol)
           do while (it_hel%is_valid ())
              qn_hel = it_hel%get_quantum_numbers ()
-             qn = qn_hel(1) .merge. qn_fc(1)
+             qn = qn_hel .merge. qn_fc(1)
              qn_rad = qn
              call qn_rad%tag_radiated ()
              call sf_int%add_state ([qn, qn_rad, qn_photon])
              call it_hel%advance ()
           end do
-          call polarization_final (pol)
+          !  call pol%final ()
        end do
        call sf_int%freeze ()
+       if (data%keep_energy) then
+          sf_int%on_shell_mode = KEEP_ENERGY
+       else
+          sf_int%on_shell_mode = KEEP_MOMENTUM
+       end if
        call sf_int%set_incoming ([1])
        call sf_int%set_radiated ([2])
        call sf_int%set_outgoing ([3])
     end select
   end subroutine epa_init
-    
+
   subroutine epa_setup_constants (sf_int)
-    class(epa_t), intent(inout) :: sf_int
+    class(epa_t), intent(inout), target :: sf_int
     type(state_iterator_t) :: it
     type(flavor_t) :: flv
     integer :: i, n_me
@@ -327,7 +339,7 @@ contains
     end do
     sf_int%status = SF_INITIAL
   end subroutine epa_setup_constants
-  
+
   subroutine epa_complete_kinematics (sf_int, x, f, r, rb, map)
     class(epa_t), intent(inout) :: sf_int
     real(default), dimension(:), intent(out) :: x
@@ -337,14 +349,14 @@ contains
     logical, intent(in) :: map
     real(default) :: xb1
     real(default) :: delta, sqrt_delta, lx
-    if (map) then                                   
+    if (map) then
        associate (data => sf_int%data)
          delta = data%log ** 2 -  4 * (r(1) * data%c1 + rb(1) * data%c0)
          if (delta > 0) then
             sqrt_delta = sqrt (delta)
             lx = (data%log - sqrt_delta) / 2
          else
-            sf_int%status = SF_FAILED_KINEMATICS          
+            sf_int%status = SF_FAILED_KINEMATICS
             f = 0
             return
          end if
@@ -359,11 +371,11 @@ contains
           sf_int%status = SF_FAILED_KINEMATICS
           f = 0
           return
-       end if       
-    end if       
+       end if
+    end if
     xb1 = 1 - x(1)
     if (size(x) == 3)  x(2:3) = r(2:3)
-    call sf_int%split_momentum (x, xb1) 
+    call sf_int%split_momentum (x, xb1)
     select case (sf_int%status)
     case (SF_DONE_KINEMATICS)
        sf_int%x = x(1)
@@ -432,7 +444,7 @@ contains
     sf_int%x  = x(1)
     sf_int%xb = 1 - x(1)
   end subroutine sf_epa_recover_x
-  
+
   subroutine epa_apply (sf_int, scale)
     class(epa_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
