@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -65,12 +65,12 @@ contains
     type(pdg_array_t), dimension(1) :: pdg_out
     integer, dimension(:), allocatable :: pdg1
     class(sf_data_t), allocatable :: data
-    
+
     write (u, "(A)")  "* Test output: sf_ewa_1"
     write (u, "(A)")  "*   Purpose: initialize and display &
          &test structure function data"
     write (u, "(A)")
-    
+
     write (u, "(A)")  "* Create empty data object"
     write (u, "(A)")
 
@@ -99,11 +99,11 @@ contains
     call data%get_pdg_out (pdg_out)
     pdg1 = pdg_out(1)
     write (u, "(2x,99(1x,I0))")  pdg1
-        
+
     write (u, "(A)")
     write (u, "(A)")  "* Initialize for W boson"
     write (u, "(A)")
-    
+
     deallocate (data)
     allocate (ewa_data_t :: data)
     select type (data)
@@ -121,9 +121,9 @@ contains
     call data%get_pdg_out (pdg_out)
     pdg1 = pdg_out(1)
     write (u, "(2x,99(1x,I0))")  pdg1
-            
+
     call model%final ()
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_ewa_1"
 
@@ -141,12 +141,12 @@ contains
     real(default) :: E
     real(default), dimension(:), allocatable :: r, rb, x
     real(default) :: f
-    
+
     write (u, "(A)")  "* Test output: sf_ewa_2"
     write (u, "(A)")  "*   Purpose: initialize and fill &
          &test structure function object"
     write (u, "(A)")
-    
+
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
@@ -155,23 +155,23 @@ contains
     pdg_in = 2
 
     call reset_interaction_counter ()
-    
+
     allocate (ewa_data_t :: data)
     select type (data)
     type is (ewa_data_t)
        call data%init (model, pdg_in, 0.01_default, &
-            500._default, 3000._default, .false., .false.)
+            500._default, 3000._default, .false., .true.)
        call data%set_id (24)
     end select
-       
+
     write (u, "(A)")  "* Initialize structure-function object"
     write (u, "(A)")
-    
+
     call data%allocate_sf_int (sf_int)
     call sf_int%init (data)
     call sf_int%set_beam_index ([1])
     call sf_int%setup_constants ()
-    
+
     call sf_int%write (u)
 
     write (u, "(A)")
@@ -182,66 +182,66 @@ contains
     call pacify (k, 1e-10_default)
     call vector4_write (k, u)
     call sf_int%seed_kinematics ([k])
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Set kinematics for r=0.4, no EWA mapping, collinear"
     write (u, "(A)")
-    
+
     allocate (r (data%get_n_par ()))
     allocate (rb(size (r)))
     allocate (x (size (r)))
-    
+
     r = 0.4_default
     rb = 1 - r
     call sf_int%complete_kinematics (x, f, r, rb, map=.false.)
-    
+
     write (u, "(A,9(1x,F10.7))")  "r =", r
     write (u, "(A,9(1x,F10.7))")  "rb=", rb
     write (u, "(A,9(1x,F10.7))")  "x =", x
     write (u, "(A,9(1x,F10.7))")  "f =", f
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Recover x from momenta"
     write (u, "(A)")
-    
+
     q = sf_int%get_momenta (outgoing=.true.)
     call sf_int%final ()
     deallocate (sf_int)
-    
+
     call data%allocate_sf_int (sf_int)
     call sf_int%init (data)
     call sf_int%set_beam_index ([1])
     call sf_int%setup_constants ()
-    
+
     call sf_int%seed_kinematics ([k])
     call sf_int%set_momenta (q, outgoing=.true.)
     call sf_int%recover_x (x)
     call sf_int%inverse_kinematics (x, f, r, rb, map=.false., &
          set_momenta=.true.)
-    
+
     write (u, "(A,9(1x,F10.7))")  "r =", r
     write (u, "(A,9(1x,F10.7))")  "rb=", rb
     write (u, "(A,9(1x,F10.7))")  "x =", x
-    write (u, "(A,9(1x,F10.7))")  "f =", f    
-    
+    write (u, "(A,9(1x,F10.7))")  "f =", f
+
     write (u, "(A)")
     write (u, "(A)")  "* Evaluate EWA structure function"
     write (u, "(A)")
-    
+
     call sf_int%apply (scale = 100._default)
     call sf_int%write (u)
-        
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
-    
+
     call sf_int%final ()
     call model%final ()
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_ewa_2"
-    
+
   end subroutine sf_ewa_2
-  
+
   subroutine sf_ewa_3 (u)
     integer, intent(in) :: u
     type(model_data_t), target :: model
@@ -254,12 +254,12 @@ contains
     real(default) :: E
     real(default), dimension(:), allocatable :: r, rb, x
     real(default) :: f
-    
+
     write (u, "(A)")  "* Test output: sf_ewa_3"
     write (u, "(A)")  "*   Purpose: initialize and fill &
          &test structure function object"
     write (u, "(A)")
-    
+
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
@@ -268,23 +268,23 @@ contains
     pdg_in = 2
 
     call reset_interaction_counter ()
-    
+
     allocate (ewa_data_t :: data)
     select type (data)
     type is (ewa_data_t)
        call data%init (model, pdg_in, 0.01_default, &
-            500._default, 3000._default, .false., .false.)
+            500._default, 3000._default, .false., .true.)
        call data%set_id (24)
     end select
-    
+
     write (u, "(A)")  "* Initialize structure-function object"
-    write (u, "(A)")    
-    
+    write (u, "(A)")
+
     call data%allocate_sf_int (sf_int)
     call sf_int%init (data)
     call sf_int%set_beam_index ([1])
     call sf_int%setup_constants ()
-    
+
     call sf_int%write (u)
 
     write (u, "(A)")
@@ -299,39 +299,39 @@ contains
     write (u, "(A)")
     write (u, "(A)")  "* Set kinematics for r=0.4, with EWA mapping, collinear"
     write (u, "(A)")
-    
+
     allocate (r (data%get_n_par ()))
     allocate (rb(size (r)))
     allocate (x (size (r)))
-    
+
     r = 0.4_default
     rb = 1 - r
     call sf_int%complete_kinematics (x, f, r, rb, map=.true.)
-    
+
     write (u, "(A,9(1x,F10.7))")  "r =", r
     write (u, "(A,9(1x,F10.7))")  "rb=", rb
     write (u, "(A,9(1x,F10.7))")  "x =", x
     write (u, "(A,9(1x,F10.7))")  "f =", f
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Recover x from momenta"
     write (u, "(A)")
-    
+
     q = sf_int%get_momenta (outgoing=.true.)
     call sf_int%final ()
     deallocate (sf_int)
-    
+
     call data%allocate_sf_int (sf_int)
     call sf_int%init (data)
     call sf_int%set_beam_index ([1])
     call sf_int%setup_constants ()
-    
+
     call sf_int%seed_kinematics ([k])
     call sf_int%set_momenta (q, outgoing=.true.)
     call sf_int%recover_x (x)
     call sf_int%inverse_kinematics (x, f, r, rb, map=.true., &
          set_momenta=.true.)
-    
+
     write (u, "(A,9(1x,F10.7))")  "r =", r
     write (u, "(A,9(1x,F10.7))")  "rb=", rb
     write (u, "(A,9(1x,F10.7))")  "x =", x
@@ -340,16 +340,16 @@ contains
     write (u, "(A)")
     write (u, "(A)")  "* Evaluate EWA structure function"
     write (u, "(A)")
-    
+
     call sf_int%apply (scale = 100._default)
     call sf_int%write (u)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
-    
+
     call sf_int%final ()
     call model%final ()
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_ewa_3"
 
@@ -367,12 +367,12 @@ contains
     real(default) :: E
     real(default), dimension(:), allocatable :: r, rb, x
     real(default) :: f
-    
+
     write (u, "(A)")  "* Test output: sf_ewa_4"
     write (u, "(A)")  "*   Purpose: initialize and fill &
          &test structure function object"
     write (u, "(A)")
-    
+
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
@@ -386,13 +386,13 @@ contains
     select type (data)
     type is (ewa_data_t)
        call data%init (model, pdg_in, 0.01_default, &
-            500._default, 3000.0_default, .false., .true.)
+            500._default, 3000.0_default, .true., .true.)
        call data%set_id (24)
-    end select    
+    end select
 
     write (u, "(A)")  "* Initialize structure-function object"
     write (u, "(A)")
-        
+
     call data%allocate_sf_int (sf_int)
     call sf_int%init (data)
     call sf_int%set_beam_index ([1])
@@ -404,66 +404,66 @@ contains
     k = vector4_moving (E, sqrt (E**2 - flv%get_mass ()**2), 3)
     call pacify (k, 1e-10_default)
     call vector4_write (k, u)
-    call sf_int%seed_kinematics ([k])        
-    
+    call sf_int%seed_kinematics ([k])
+
     write (u, "(A)")
     write (u, "(A)")  "* Set kinematics for r=0.5/0.5/0.25, with EWA mapping, "
     write (u, "(A)")  "          non-coll., keeping energy"
     write (u, "(A)")
-    
+
     allocate (r (data%get_n_par ()))
     allocate (rb(size (r)))
     allocate (x (size (r)))
-    
+
     r = [0.5_default, 0.5_default, 0.25_default]
     rb = 1 - r
     sf_int%on_shell_mode = KEEP_ENERGY
     call sf_int%complete_kinematics (x, f, r, rb, map=.true.)
     call interaction_pacify_momenta (sf_int%interaction_t, 1e-10_default)
-    
+
     write (u, "(A,9(1x,F10.7))")  "r =", r
     write (u, "(A,9(1x,F10.7))")  "rb=", rb
     write (u, "(A,9(1x,F10.7))")  "x =", x
     write (u, "(A,9(1x,F10.7))")  "f =", f
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Recover x and r from momenta"
     write (u, "(A)")
-    
+
     q = sf_int%get_momenta (outgoing=.true.)
     call sf_int%final ()
     deallocate (sf_int)
-    
+
     call data%allocate_sf_int (sf_int)
     call sf_int%init (data)
     call sf_int%set_beam_index ([1])
     call sf_int%setup_constants ()
-    
+
     call sf_int%seed_kinematics ([k])
     call sf_int%set_momenta (q, outgoing=.true.)
     call sf_int%recover_x (x)
     call sf_int%inverse_kinematics (x, f, r, rb, map=.true., &
-         set_momenta=.true.)    
+         set_momenta=.true.)
     call interaction_pacify_momenta (sf_int%interaction_t, 1e-10_default)
-    
+
     write (u, "(A,9(1x,F10.7))")  "r =", r
     write (u, "(A,9(1x,F10.7))")  "rb=", rb
     write (u, "(A,9(1x,F10.7))")  "x =", x
     write (u, "(A,9(1x,F10.7))")  "f =", f
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Evaluate EWA structure function"
     write (u, "(A)")
-    
+
     call sf_int%apply (scale = 1500._default)
-    call sf_int%write (u, testflag = .true.)    
-    
+    call sf_int%write (u, testflag = .true.)
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
-    
+
     call sf_int%final ()
     call model%final ()
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_ewa_4"
 
@@ -480,12 +480,12 @@ contains
     real(default) :: E
     real(default), dimension(:), allocatable :: r, rb, x
     real(default) :: f
-    
+
     write (u, "(A)")  "* Test output: sf_ewa_5"
     write (u, "(A)")  "*   Purpose: initialize and fill &
          &test structure function object"
     write (u, "(A)")
-    
+
     write (u, "(A)")  "* Initialize configuration data"
     write (u, "(A)")
 
@@ -494,23 +494,23 @@ contains
     pdg_in = [1, 2, -1, -2]
 
     call reset_interaction_counter ()
-    
+
     allocate (ewa_data_t :: data)
     select type (data)
     type is (ewa_data_t)
        call data%init (model, pdg_in, 0.01_default, &
-            500._default, 3000._default, .false., .false.)
+            500._default, 3000._default, .false., .true.)
        call data%set_id (24)
     end select
-       
+
     write (u, "(A)")  "* Initialize structure-function object"
     write (u, "(A)")
-    
+
     call data%allocate_sf_int (sf_int)
     call sf_int%init (data)
     call sf_int%set_beam_index ([1])
     call sf_int%setup_constants ()
-    
+
     call sf_int%write (u)
 
     write (u, "(A)")
@@ -521,41 +521,41 @@ contains
     call pacify (k, 1e-10_default)
     call vector4_write (k, u)
     call sf_int%seed_kinematics ([k])
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Set kinematics for r=0.4, no EWA mapping, collinear"
     write (u, "(A)")
-    
+
     allocate (r (data%get_n_par ()))
     allocate (rb(size (r)))
     allocate (x (size (r)))
-    
+
     r = 0.4_default
     rb = 1 - r
     call sf_int%complete_kinematics (x, f, r, rb, map=.false.)
-    
+
     write (u, "(A,9(1x,F10.7))")  "r =", r
     write (u, "(A,9(1x,F10.7))")  "rb=", rb
     write (u, "(A,9(1x,F10.7))")  "x =", x
     write (u, "(A,9(1x,F10.7))")  "f =", f
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Evaluate EWA structure function"
     write (u, "(A)")
-    
+
     call sf_int%apply (scale = 100._default)
     call sf_int%write (u)
-        
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
-    
+
     call sf_int%final ()
     call model%final ()
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: sf_ewa_5"
-    
+
   end subroutine sf_ewa_5
-  
+
 
 end module sf_ewa_uti
