@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -34,13 +34,13 @@
 ! to the source 'whizard.nw'
 
 module sf_isr
- 
+
   use kinds, only: default
   use iso_varying_string, string_t => varying_string
   use io_units
   use constants, only: pi
   use format_defs, only: FMT_15, FMT_19
-  use unit_tests, only: vanishes
+  use numeric_utils
   use diagnostics
   use physics_defs, only: PHOTON
   use lorentz
@@ -50,8 +50,8 @@ module sf_isr
   use flavors
   use colors
   use quantum_numbers
-  use state_matrices
   use polarizations
+  use sf_aux
   use sf_mappings
   use sf_base
 
@@ -81,6 +81,7 @@ module sf_isr
      real(default) :: eps = 0
      real(default) :: log = 0
      logical :: recoil = .false.
+     logical :: keep_energy = .true.
      integer :: order = 3
      integer :: error = NONE
    contains
@@ -91,7 +92,7 @@ module sf_isr
      procedure :: get_n_par => isr_data_get_n_par
      procedure :: get_pdg_out => isr_data_get_pdg_out
      procedure :: get_eps => isr_data_get_eps
-     procedure :: allocate_sf_int => isr_data_allocate_sf_int  
+     procedure :: allocate_sf_int => isr_data_allocate_sf_int
   end type isr_data_t
 
   type, extends (sf_int_t) :: isr_t
@@ -108,13 +109,13 @@ module sf_isr
      procedure :: inverse_kinematics => isr_inverse_kinematics
      procedure :: init => isr_init
      procedure :: apply => isr_apply
-  end type isr_t 
-  
+  end type isr_t
+
 
 contains
 
-  subroutine isr_data_init &
-       (data, model, pdg_in, alpha, q_max, mass, order, recoil)
+  subroutine isr_data_init (data, model, pdg_in, alpha, q_max, &
+       mass, order, recoil, keep_energy)
     class(isr_data_t), intent(out) :: data
     class(model_data_t), intent(in), target :: model
     type(pdg_array_t), intent(in) :: pdg_in
@@ -123,6 +124,7 @@ contains
     real(default), intent(in), optional :: mass
     integer, intent(in), optional :: order
     logical, intent(in), optional :: recoil
+    logical, intent(in), optional :: keep_energy
     integer :: i, n_flv
     real(default) :: charge
     data%model => model
@@ -139,6 +141,9 @@ contains
     if (present (recoil)) then
        data%recoil = recoil
     end if
+    if (present (keep_energy)) then
+       data%keep_energy = keep_energy
+    end if
     data%real_mass = data%flv_in(1)%get_mass ()
     if (present (mass)) then
        if (mass > 0) then
@@ -205,7 +210,7 @@ contains
     end select
   end subroutine isr_data_check
 
-  subroutine isr_data_write (data, unit, verbose) 
+  subroutine isr_data_write (data, unit, verbose)
     class(isr_data_t), intent(in) :: data
     integer, intent(in), optional :: unit
     logical, intent(in), optional :: verbose
@@ -218,16 +223,17 @@ contains
           if (i > 1)  write (u, "(',',1x)", advance="no")
           call data%flv_in(i)%write (u)
        end do
-       write (u, *)    
-       write (u, "(3x,A," // FMT_19 // ")") "  alpha  = ", data%alpha
-       write (u, "(3x,A," // FMT_19 // ")") "  q_max  = ", data%q_max
-       write (u, "(3x,A," // FMT_19 // ")") "  mass   = ", data%mass
-       write (u, "(3x,A," // FMT_19 // ")") "  eps    = ", data%eps
-       write (u, "(3x,A," // FMT_19 // ")") "  log    = ", data%log
-       write (u, "(3x,A,I2)")      "  order  = ", data%order
-       write (u, "(3x,A,L2)")      "  recoil = ", data%recoil
+       write (u, *)
+       write (u, "(3x,A," // FMT_19 // ")") "  alpha    = ", data%alpha
+       write (u, "(3x,A," // FMT_19 // ")") "  q_max    = ", data%q_max
+       write (u, "(3x,A," // FMT_19 // ")") "  mass     = ", data%mass
+       write (u, "(3x,A," // FMT_19 // ")") "  eps      = ", data%eps
+       write (u, "(3x,A," // FMT_19 // ")") "  log      = ", data%log
+       write (u, "(3x,A,I2)")      "  order    = ", data%order
+       write (u, "(3x,A,L2)")      "  recoil   = ", data%recoil
+       write (u, "(3x,A,L2)")      "  keep en. = ", data%keep_energy
     else
-       write (u, "(3x,A)") "[undefined]"       
+       write (u, "(3x,A)") "[undefined]"
     end if
   end subroutine isr_data_write
 
@@ -240,13 +246,13 @@ contains
        n = 1
     end if
   end function isr_data_get_n_par
-  
+
   subroutine isr_data_get_pdg_out (data, pdg_out)
     class(isr_data_t), intent(in) :: data
     type(pdg_array_t), dimension(:), intent(inout) :: pdg_out
     pdg_out(1) = data%flv_in%get_pdg ()
   end subroutine isr_data_get_pdg_out
-  
+
   function isr_data_get_eps (data) result (eps)
     class(isr_data_t), intent(in) :: data
     real(default) :: eps
@@ -258,17 +264,17 @@ contains
     class(sf_int_t), intent(inout), allocatable :: sf_int
     allocate (isr_t :: sf_int)
   end subroutine isr_data_allocate_sf_int
-  
+
   function isr_type_string (object) result (string)
     class(isr_t), intent(in) :: object
     type(string_t) :: string
     if (associated (object%data)) then
-       string = "ISR: e+ e- ISR spectrum" 
+       string = "ISR: e+ e- ISR spectrum"
     else
        string = "ISR: [undefined]"
     end if
   end function isr_type_string
-  
+
   subroutine isr_write (object, unit, testflag)
     class(isr_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -287,7 +293,7 @@ contains
        write (u, "(1x,A)")  "ISR data: [undefined]"
     end if
   end subroutine isr_write
-    
+
   subroutine isr_set_order (object, order)
     class(isr_t), intent(inout) :: object
     integer, intent(in) :: order
@@ -333,7 +339,7 @@ contains
     sf_int%x  = x(1)
     sf_int%xb = 1 - x(1)
   end subroutine sf_isr_recover_x
-  
+
   subroutine isr_inverse_kinematics (sf_int, x, f, r, rb, map, set_momenta)
     class(isr_t), intent(inout) :: sf_int
     real(default), dimension(:), intent(in) :: x
@@ -377,49 +383,54 @@ contains
     class(sf_data_t), intent(in), target :: data
     type(quantum_numbers_mask_t), dimension(3) :: mask
     integer, dimension(3) :: hel_lock
-    type(polarization_t) :: pol
-    type(quantum_numbers_t), dimension(1) :: qn_fc, qn_hel
+    type(polarization_t), target :: pol
+    type(quantum_numbers_t), dimension(1) :: qn_fc
     type(flavor_t) :: flv_photon
     type(color_t) :: col_photon
-    type(quantum_numbers_t) :: qn_photon, qn
-    type(state_iterator_t) :: it_hel
+    type(quantum_numbers_t) :: qn_hel, qn_photon, qn
+    type(polarization_iterator_t) :: it_hel
     real(default) :: m2
     integer :: i
     mask = quantum_numbers_mask (.false., .false., &
          mask_h = [.false., .true., .false.])
     hel_lock = [3, 0, 1]
     select type (data)
-    type is (isr_data_t)   
+    type is (isr_data_t)
        m2 = data%mass**2
        call sf_int%base_init (mask, [m2], [0._default], [m2], &
             hel_lock = hel_lock)
-       sf_int%data => data              
+       sf_int%data => data
        call flv_photon%init (PHOTON, data%model)
        call col_photon%init ()
        call qn_photon%init (flv_photon, col_photon)
        call qn_photon%tag_radiated ()
        do i = 1, size (data%flv_in)
-          call polarization_init_generic (pol, data%flv_in(i))
+          call pol%init_generic (data%flv_in(i))
           call qn_fc(1)%init (&
                flv = data%flv_in(i), &
                col = color_from_flavor (data%flv_in(i), 1))
-          call it_hel%init (pol%state)
+          call it_hel%init (pol)
           do while (it_hel%is_valid ())
              qn_hel = it_hel%get_quantum_numbers ()
-             qn = qn_hel(1) .merge. qn_fc(1)
+             qn = qn_hel .merge. qn_fc(1)
              call sf_int%add_state ([qn, qn_photon, qn])
              call it_hel%advance ()
           end do
-          call polarization_final (pol)
+          ! call pol%final ()  !!! Obsolete
        end do
        call sf_int%freeze ()
+       if (data%keep_energy) then
+          sf_int%on_shell_mode = KEEP_ENERGY
+       else
+          sf_int%on_shell_mode = KEEP_MOMENTUM
+       end if
        call sf_int%set_incoming ([1])
        call sf_int%set_radiated ([2])
        call sf_int%set_outgoing ([3])
        sf_int%status = SF_INITIAL
     end select
   end subroutine isr_init
-    
+
   subroutine isr_apply (sf_int, scale)
     class(isr_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
@@ -432,7 +443,7 @@ contains
     real(default), parameter :: &
          & g1 = 3._default / 4._default, &
          & g2 = (27 - 8*pi**2) / 96._default, &
-         & g3 = (27 - 24*pi**2 + 128*zeta3) / 384._default    
+         & g3 = (27 - 24*pi**2 + 128*zeta3) / 384._default
     associate (data => sf_int%data)
       eps = sf_int%data%eps
       x = sf_int%x
@@ -442,7 +453,7 @@ contains
          f = 1 / finv
       else
          f = 0
-      end if   
+      end if
       if (f > 0 .and. data%order > 0) then
          f = f * (1 + g1 * eps)
          x_2 = x*x
@@ -471,7 +482,7 @@ contains
          end if
       end if
     end associate
-    call sf_int%set_matrix_element (cmplx (f, kind=default))    
+    call sf_int%set_matrix_element (cmplx (f, kind=default))
     sf_int%status = SF_EVALUATED
   end subroutine isr_apply
 
