@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -40,7 +40,7 @@ module blha_config
   use io_units
   use constants
   use string_utils
-  use system_defs, only: EOF
+  use variables, only: var_list_t
   use diagnostics
   use md5
   use model_data
@@ -60,6 +60,7 @@ module blha_config
 
   public :: blha_configuration_t
   public :: blha_cfg_process_node_t
+  public :: ew_scheme_string_to_int
   public :: blha_flv_state_t
   public :: blha_master_t
   public :: blha_configuration_init
@@ -69,41 +70,40 @@ module blha_config
   public :: blha_configuration_write
 
   integer, public, parameter :: &
-       BLHA_CT_QCD=1, BLHA_CT_EW=2, BLHA_CT_QED=3, BLHA_CT_OTHER=4
+       BLHA_CT_QCD = 1, BLHA_CT_EW = 2, BLHA_CT_QED = 3, BLHA_CT_OTHER = 4
   integer, public, parameter :: &
-       BLHA_IRREG_CDR=1, BLHA_IRREG_DRED=2, BLHA_IRREG_THV=3, &
-       BLHA_IRREG_MREG=4, BLHA_IRREG_OTHER=5
+       BLHA_IRREG_CDR = 1, BLHA_IRREG_DRED = 2, BLHA_IRREG_THV = 3, &
+       BLHA_IRREG_MREG = 4, BLHA_IRREG_OTHER = 5
   integer, public, parameter :: &
-       BLHA_MPS_ONSHELL=1, BLHA_MPS_OTHER=2
+       BLHA_MPS_ONSHELL = 1, BLHA_MPS_OTHER = 2
   integer, public, parameter :: &
-       BLHA_MODE_GOSAM=1, BLHA_MODE_FEYNARTS = 2, BLHA_MODE_GENERIC=3, &
-       BLHA_MODE_OPENLOOPS=4
+       BLHA_MODE_GOSAM = 1, BLHA_MODE_FEYNARTS = 2, BLHA_MODE_GENERIC = 3, &
+       BLHA_MODE_OPENLOOPS = 4
   integer, public, parameter :: &
        BLHA_VERSION_1 = 1, BLHA_VERSION_2 = 2
   integer, public, parameter :: &
        BLHA_AMP_LOOP = 1, BLHA_AMP_CC = 2, BLHA_AMP_SC = 3, &
        BLHA_AMP_TREE = 4, BLHA_AMP_LOOPINDUCED = 5
   integer, public, parameter :: &
+       BLHA_EW_QED = 0, &
        BLHA_EW_GF = 1, BLHA_EW_MZ = 2, BLHA_EW_MSBAR = 3, &
-       BLHA_EW_0 = 4, BLHA_EW_RUN = 5, BLHA_EW_DEFAULT = 6
+       BLHA_EW_0 = 4, BLHA_EW_RUN = 5
   integer, public, parameter :: &
        BLHA_WIDTH_COMPLEX = 1, BLHA_WIDTH_FIXED = 2, &
        BLHA_WIDTH_RUNNING = 3, BLHA_WIDTH_POLE = 4, &
-       BLHA_WIDTH_DEFAULT = 5 
+       BLHA_WIDTH_DEFAULT = 5
 
-  integer, parameter, public :: OLP_N_MASSIVE_PARTICLES = 10
+  integer, parameter, public :: OLP_N_MASSIVE_PARTICLES = 12
   integer, dimension(OLP_N_MASSIVE_PARTICLES), public :: &
-    OLP_MASSIVE_PARTICLES = [5,-5,6,-6,15,-15,23,24,-24,25]
+    OLP_MASSIVE_PARTICLES = [5, -5, 6, -6, 13, -13, 15, -15, 23, 24, -24, 25]
   integer, parameter :: OLP_HEL_UNPOLARIZED = 0
-  integer, parameter :: OLP_HEL_LEFT = -1
-  integer, parameter :: OLP_HEL_RIGHT = 1
-  integer, parameter :: OLP_HEL_LONG = 2
 
+  integer, parameter :: N_KNOWN_SPECIAL_OL_METHODS = 3
 
   type :: blha_particle_string_element_t
      integer :: pdg = 0
      integer :: hel = OLP_HEL_UNPOLARIZED
-     logical :: polarized = .false. 
+     logical :: polarized = .false.
   contains
     generic :: init => init_default
     generic :: init => init_polarized
@@ -153,10 +153,13 @@ module blha_config
      type(string_t) :: model_file
      logical :: subdivide_subprocesses = .false.
      integer :: alphas_power = -1, alpha_power = -1
-     integer :: ew_scheme = BLHA_EW_DEFAULT
+     integer :: ew_scheme = BLHA_EW_GF
      integer :: width_scheme = BLHA_WIDTH_DEFAULT
+     logical :: openloops_use_cms = .false.
      integer :: openloops_phs_tolerance = 0
-     logical :: openloops_top_signal = .false.
+     type(string_t) :: openloops_extra_cmd
+     integer :: openloops_stability_log = 0
+     logical :: openloops_use_collier = .false.
   end type blha_configuration_t
 
   type:: blha_flv_state_t
@@ -167,26 +170,29 @@ module blha_config
 
   type :: blha_master_t
     integer, dimension(4) :: blha_mode
-    integer :: n_in, n_out
     logical :: compute_borns = .false.
     logical :: compute_real_trees = .false.
     logical :: compute_loops = .true.
     logical :: compute_correlations = .false.
-    integer :: alpha_power, alphas_power
-    type(string_t) :: basename
+    integer :: ew_scheme
     type(string_t), dimension(:), allocatable :: suffix
     type(blha_configuration_t), dimension(:), allocatable :: blha_cfg
     integer :: n_files = 0
   contains
     procedure :: set_methods => blha_master_set_methods
     procedure :: allocate_config_files => blha_master_allocate_config_files
-    procedure :: init => blha_master_init 
+    procedure :: set_ew_scheme => blha_master_set_ew_scheme
+    procedure :: generate => blha_master_generate
+    procedure :: generate_loop => blha_master_generate_loop
+    procedure :: generate_correlation => blha_master_generate_correlation
+    procedure :: generate_real_tree => blha_master_generate_real_tree
+    procedure :: generate_born => blha_master_generate_born
     procedure :: setup_additional_features => blha_master_setup_additional_features
     procedure :: set_gosam => blha_master_set_gosam
     procedure :: set_openloops => blha_master_set_openloops
     procedure :: reset_olp_modes => blha_master_reset_olp_modes
     procedure :: set_polarization => blha_master_set_polarization
-    procedure :: generate => blha_master_generate
+    procedure :: write_olp => blha_master_write_olp
     procedure :: final => blha_master_final
   end type blha_master_t
 
@@ -197,6 +203,27 @@ module blha_config
 
 contains
 
+  subroutine check_extra_cmd (extra_cmd)
+    type(string_t), intent(in) :: extra_cmd
+    type(string_t), dimension(N_KNOWN_SPECIAL_OL_METHODS) :: known_methods
+    integer :: i
+    logical :: found
+    known_methods(1) = 'top'
+    known_methods(2) = 'not'
+    known_methods(3) = 'stop'
+    if (extra_cmd == var_str ("")) return
+    found = .false.
+    do i = 1, N_KNOWN_SPECIAL_OL_METHODS
+       found = found .or. (extra_cmd == var_str ('extra approx ') // known_methods(i))
+    end do
+    if (.not. found) &
+      call msg_fatal ("The given extra OpenLoops method is not kown ", &
+         [var_str ("Available commands are: "), &
+          var_str ("extra approx top (only WbWb signal),"), &
+          var_str ("extra approx stop (only WbWb singletop),"), &
+          var_str ("extra approx not (no top in WbWb).")])
+  end subroutine check_extra_cmd
+
   subroutine blha_particle_string_element_init_default (blha_p, id)
     class(blha_particle_string_element_t), intent(out) :: blha_p
     integer, intent(in) :: id
@@ -239,15 +266,60 @@ contains
     write (c, '(A1,I0,A1)') '(', blha_p%hel, ')'
   end subroutine blha_particle_string_element_write_helicity_character
 
-  subroutine blha_master_set_methods (master, cmp_born, &
-     cmp_real, cmp_loop, cmp_corr)
+  function ew_scheme_string_to_int (ew_scheme_str) result (ew_scheme_int)
+    integer :: ew_scheme_int
+    type(string_t), intent(in) :: ew_scheme_str
+    select case (char (ew_scheme_str))
+    case ('GF', 'Gmu')
+       ew_scheme_int = BLHA_EW_GF
+    case ('alpha_qed')
+       ew_scheme_int = BLHA_EW_QED
+    case ('alpha_mz')
+       ew_scheme_int = BLHA_EW_MZ
+    case ('alpha_0', 'alpha_thompson')
+       ew_scheme_int = BLHA_EW_0
+    case default
+       call msg_fatal ("ew_scheme: " // char (ew_scheme_str) // &
+            " not supported. Try 'Gmu', 'alpha_qed', 'alpha_mz' or 'alpha_0'.")
+    end select
+  end function ew_scheme_string_to_int
+
+  subroutine blha_master_set_methods (master, is_nlo, var_list)
     class(blha_master_t), intent(inout) :: master
-    logical, intent(in) :: cmp_born, cmp_loop, cmp_corr, cmp_real
+    logical, intent(in) :: is_nlo
+    type(var_list_t), intent(in) :: var_list
+    type(string_t) :: born_me_method, real_tree_me_method
+    type(string_t) :: loop_me_method, correlation_me_method
+    logical :: cmp_born, cmp_real
+    logical :: cmp_loop, cmp_corr
+    born_me_method = var_list%get_sval (var_str ("$born_me_method"))
+    real_tree_me_method = var_list%get_sval (var_str ("$real_tree_me_method"))
+    loop_me_method = var_list%get_sval (var_str ("$loop_me_method"))
+    correlation_me_method = var_list%get_sval (var_str ("$correlation_me_method"))
+    cmp_born = born_me_method /= 'omega'
+    cmp_real = is_nlo .and. (real_tree_me_method /= 'omega')
+    cmp_loop = is_nlo .and. (loop_me_method /= 'omega')
+    cmp_corr = is_nlo .and. (correlation_me_method /= 'omega')
+    call set_me_method (1, loop_me_method)
+    call set_me_method (2, correlation_me_method)
+    call set_me_method (3, real_tree_me_method)
+    call set_me_method (4, born_me_method)
     master%n_files = count ([cmp_born, cmp_real, cmp_loop, cmp_corr])
     master%compute_borns = cmp_born
     master%compute_real_trees = cmp_real
     master%compute_loops = cmp_loop
     master%compute_correlations = cmp_corr
+  contains
+    subroutine set_me_method (i, me_method)
+      integer, intent(in) :: i
+      type(string_t) :: me_method
+      select case (char (me_method))
+      case ('gosam')
+         call master%set_gosam (i)
+      case ('openloops')
+         call master%set_openloops (i)
+      end select
+    end subroutine set_me_method
   end subroutine blha_master_set_methods
 
   subroutine blha_master_allocate_config_files (master)
@@ -255,32 +327,45 @@ contains
     allocate (master%blha_cfg (master%n_files))
     allocate (master%suffix (master%n_files))
   end subroutine blha_master_allocate_config_files
+  subroutine blha_master_set_ew_scheme (master, ew_scheme)
+    class(blha_master_t), intent(inout) :: master
+    type(string_t), intent(in) :: ew_scheme
+    master%ew_scheme = ew_scheme_string_to_int (ew_scheme)
+  end subroutine blha_master_set_ew_scheme
 
-  subroutine blha_master_init (master, basename, model, &
+  subroutine blha_master_generate (master, basename, model, &
        n_in, alpha_power, alphas_power, flv_born, flv_real)
     class(blha_master_t), intent(inout) :: master
     type(string_t), intent(in) :: basename
     class(model_data_t), intent(in), target :: model
     integer, intent(in) :: n_in
     integer, intent(in) :: alpha_power, alphas_power
-    integer, dimension(:,:), allocatable, intent(in) :: &
-         flv_born, flv_real
-    integer :: n_proc_real, n_flv
-    type(blha_flv_state_t), dimension(:), allocatable :: blha_flavor
-    integer :: i_flv, i_file
-    integer :: n_flv_born 
-
+    integer, intent(in), dimension(:,:), allocatable :: flv_born, flv_real
+    integer :: i_file
     if (master%n_files < 1) &
-       call msg_fatal ("Attempting to generate OLP-files, but none are specified!")
-    n_flv = 1; n_proc_real = 0
-    n_flv_born = size (flv_born, 2)
-    if (master%compute_real_trees) then
-       if (allocated (flv_real)) then
-         n_proc_real = size (flv_real, 2)
-         n_flv = n_flv + n_proc_real
-       end if
-    end if
-    i_file = 1          
+         call msg_fatal ("Attempting to generate OLP-files, but none are specified!")
+    i_file = 1
+    call master%generate_loop (basename, model, n_in, alpha_power, &
+         alphas_power, flv_born, i_file)
+    call master%generate_correlation (basename, model, n_in, alpha_power, &
+         alphas_power, flv_born, i_file)
+    call master%generate_real_tree (basename, model, n_in, alpha_power, &
+         alphas_power, flv_real, i_file)
+    call master%generate_born (basename, model, n_in, alpha_power, &
+         alphas_power, flv_born, i_file)
+  end subroutine blha_master_generate
+
+  subroutine blha_master_generate_loop (master, basename, model, n_in, &
+         alpha_power, alphas_power, flv_born, i_file)
+    class(blha_master_t), intent(inout) :: master
+    type(string_t), intent(in) :: basename
+    class(model_data_t), intent(in), target :: model
+    integer, intent(in) :: n_in
+    integer, intent(in) :: alpha_power, alphas_power
+    integer, dimension(:,:), allocatable, intent(in) :: flv_born
+    integer, intent(inout) :: i_file
+    type(blha_flv_state_t), dimension(:), allocatable :: blha_flavor
+    integer :: i_flv
     if (master%compute_loops) then
        if (allocated (flv_born)) then
           allocate (blha_flavor (size (flv_born, 2)))
@@ -291,7 +376,7 @@ contains
           end do
           master%suffix(i_file) = "_LOOP"
           call blha_init_virtual (master%blha_cfg(i_file), blha_flavor, &
-               n_in, alpha_power, alphas_power, &
+               n_in, alpha_power, alphas_power, master%ew_scheme, &
                basename, model, master%blha_mode(1))
           i_file = i_file + 1
         else
@@ -299,7 +384,19 @@ contains
                            // "Born flavor not existing")
         end if
     end if
-    if (allocated (blha_flavor)) deallocate (blha_flavor)
+  end subroutine blha_master_generate_loop
+
+  subroutine blha_master_generate_correlation (master, basename, model, n_in, &
+         alpha_power, alphas_power, flv_born, i_file)
+    class(blha_master_t), intent(inout) :: master
+    type(string_t), intent(in) :: basename
+    class(model_data_t), intent(in), target :: model
+    integer, intent(in) :: n_in
+    integer, intent(in) :: alpha_power, alphas_power
+    integer, dimension(:,:), allocatable, intent(in) :: flv_born
+    integer, intent(inout) :: i_file
+    type(blha_flv_state_t), dimension(:), allocatable :: blha_flavor
+    integer :: i_flv
     if (master%compute_correlations) then
        if (allocated (flv_born)) then
           allocate (blha_flavor (size (flv_born, 2)))
@@ -310,7 +407,7 @@ contains
           end do
           master%suffix(i_file) = "_SUB"
           call blha_init_subtraction (master%blha_cfg(i_file), blha_flavor, &
-               n_in, alpha_power, alphas_power, &
+               n_in, alpha_power, alphas_power, master%ew_scheme, &
                basename, model, master%blha_mode(2))
           i_file = i_file + 1
        else
@@ -318,7 +415,19 @@ contains
                            // "Born flavor not existing")
        end if
     end if
-    if (allocated (blha_flavor)) deallocate (blha_flavor)
+  end subroutine blha_master_generate_correlation
+
+  subroutine blha_master_generate_real_tree (master, basename, model, n_in, &
+         alpha_power, alphas_power, flv_real, i_file)
+    class(blha_master_t), intent(inout) :: master
+    type(string_t), intent(in) :: basename
+    class(model_data_t), intent(in), target :: model
+    integer, intent(in) :: n_in
+    integer, intent(in) :: alpha_power, alphas_power
+    integer, dimension(:,:), allocatable, intent(in) :: flv_real
+    integer, intent(inout) :: i_file
+    type(blha_flv_state_t), dimension(:), allocatable :: blha_flavor
+    integer :: i_flv
     if (master%compute_real_trees) then
        if (allocated (flv_real)) then
           allocate (blha_flavor (size (flv_real, 2)))
@@ -329,7 +438,7 @@ contains
           end do
           master%suffix(i_file) = "_REAL"
           call blha_init_real (master%blha_cfg(i_file), blha_flavor, &
-               n_in, alpha_power, alphas_power, &
+               n_in, alpha_power, alphas_power, master%ew_scheme, &
                basename, model, master%blha_mode(3))
           i_file = i_file + 1
        else
@@ -337,55 +446,85 @@ contains
                            // "Real flavor not existing")
        end if
     end if
-    if (allocated (blha_flavor)) deallocate (blha_flavor)
+  end subroutine blha_master_generate_real_tree
+
+subroutine blha_master_generate_born (master, basename, model, n_in, &
+         alpha_power, alphas_power, flv_born, i_file)
+    class(blha_master_t), intent(inout) :: master
+    type(string_t), intent(in) :: basename
+    class(model_data_t), intent(in), target :: model
+    integer, intent(in) :: n_in
+    integer, intent(in) :: alpha_power, alphas_power
+    integer, dimension(:,:), allocatable, intent(in) :: flv_born
+    integer, intent(inout) :: i_file
+    type(blha_flv_state_t), dimension(:), allocatable :: blha_flavor
+    integer :: i_flv
     if (master%compute_borns) then
        if (allocated (flv_born)) then
-          allocate (blha_flavor (n_flv_born))
-          do i_flv = 1, n_flv_born
+          allocate (blha_flavor (size (flv_born, 2)))
+          do i_flv = 1, size (flv_born, 2)
              allocate (blha_flavor(i_flv)%flavors (size (flv_born(:,i_flv))))
              blha_flavor(i_flv)%flavors = flv_born(:,i_flv)
              blha_flavor(i_flv)%flv_mult = 1
           end do
           master%suffix(i_file) = "_BORN"
           call blha_init_born (master%blha_cfg(i_file), blha_flavor, &
-               n_in, alpha_power, alphas_power, &
+               n_in, alpha_power, alphas_power, master%ew_scheme, &
                basename, model, master%blha_mode(4))
        end if
     end if
-  end subroutine blha_master_init
+  end subroutine blha_master_generate_born
 
-  subroutine blha_master_setup_additional_features (master, phs_tolerance, top_signal, beam_structure)
+  subroutine blha_master_setup_additional_features (master, &
+     phs_tolerance, use_cms, stability_log, use_collier, extra_cmd, beam_structure)
      class(blha_master_t), intent(inout) :: master
      integer, intent(in) :: phs_tolerance
-     logical, intent(in), optional :: top_signal
+     logical, intent(in) :: use_cms
+     type(string_t), intent(in), optional :: extra_cmd
+     integer, intent(in) :: stability_log
+     logical, intent(in) :: use_collier
      type(beam_structure_t), intent(in), optional :: beam_structure
      integer :: i_file
      logical :: polarized
-     logical :: yorn
 
-     yorn = .false.; if (present (top_signal)) yorn = top_signal     
      polarized = .false.
      if (present (beam_structure)) polarized = beam_structure%has_polarized_beams ()
 
+     if (use_cms .and. any (master%blha_mode /= BLHA_MODE_OPENLOOPS)) &
+        call cms_warning ()
      do i_file = 1, master%n_files
         if (phs_tolerance > 0) then
            select case (master%blha_mode(i_file))
            case (BLHA_MODE_GOSAM)
               if (polarized) &
-                 call gosam_error_message () 
+                 call gosam_error_message ()
            case (BLHA_MODE_OPENLOOPS)
+              master%blha_cfg(i_file)%openloops_use_cms = use_cms
               master%blha_cfg(i_file)%openloops_phs_tolerance = phs_tolerance
               master%blha_cfg(i_file)%polarized = polarized
+              if (present (extra_cmd)) then
+                master%blha_cfg(i_file)%openloops_extra_cmd = extra_cmd
+              else
+                master%blha_cfg(i_file)%openloops_extra_cmd = var_str ('')
+              end if
+              master%blha_cfg(i_file)%openloops_stability_log = stability_log
+              master%blha_cfg(i_file)%openloops_use_collier = use_collier
            end select
         end if
-        master%blha_cfg(i_file)%openloops_top_signal = yorn
      end do
   contains
+     subroutine cms_warning ()
+        call msg_warning ("You have set ?openloops_use_cms = true, but not all matrix ", &
+             [var_str ("element methods are set to OpenLoops. Note that other "), &
+              var_str ("methods might not necessarily support the complex mass "), &
+              var_str ("scheme. This can yield inconsistencies in your NLO results!")])
+     end subroutine cms_warning
+
      subroutine gosam_error_message ()
         call msg_fatal ("You are trying to evaluate a process at NLO ", &
-           [var_str ("which involves polarized beams using GoSam. "), &
-            var_str ("This feature is not supported yet. "), &
-            var_str ("Please use OpenLoops instead")])
+             [var_str ("which involves polarized beams using GoSam. "), &
+              var_str ("This feature is not supported yet. "), &
+              var_str ("Please use OpenLoops instead")])
      end subroutine gosam_error_message
   end subroutine blha_master_setup_additional_features
 
@@ -414,16 +553,17 @@ contains
   end subroutine blha_master_set_polarization
 
   subroutine blha_init_born (blha_cfg, blha_flavor, n_in, &
-        ap, asp, basename, model, blha_mode)
+        ap, asp, ew_scheme, basename, model, blha_mode)
     type(blha_configuration_t), intent(inout) :: blha_cfg
     type(blha_flv_state_t), intent(in), dimension(:) :: blha_flavor
     integer, intent(in) :: n_in
     integer, intent(in) :: ap, asp
+    integer, intent(in) :: ew_scheme
     type(string_t), intent(in) :: basename
     type(model_data_t), intent(in), target :: model
     integer, intent(in) :: blha_mode
     integer, dimension(:), allocatable :: amp_type
-    integer :: i, ew_scheme
+    integer :: i
 
     allocate (amp_type (size (blha_flavor)))
     do i = 1, size (blha_flavor)
@@ -433,12 +573,6 @@ contains
          model, blha_mode)
     call blha_configuration_append_processes (blha_cfg, n_in, &
          blha_flavor, amp_type)
-    select case (blha_cfg%mode)
-    case (BLHA_MODE_GOSAM)
-       ew_scheme = BLHA_EW_0
-    case (BLHA_MODE_OPENLOOPS)
-       ew_scheme = BLHA_EW_0
-    end select 
     call blha_configuration_set (blha_cfg, BLHA_VERSION_2, &
          correction_type = BLHA_CT_QCD, &
          irreg = BLHA_IRREG_CDR, alphas_power = asp, &
@@ -447,32 +581,27 @@ contains
   end subroutine blha_init_born
 
   subroutine blha_init_virtual (blha_cfg, blha_flavor, n_in, &
-     ap, asp, basename, model, blha_mode)
+     ap, asp, ew_scheme, basename, model, blha_mode)
     type(blha_configuration_t), intent(inout) :: blha_cfg
     type(blha_flv_state_t), intent(in), dimension(:) :: blha_flavor
     integer, intent(in) :: n_in
     integer, intent(in) :: ap, asp
+    integer, intent(in) :: ew_scheme
     type(string_t), intent(in) :: basename
     type(model_data_t), intent(in), target :: model
     integer, intent(in) :: blha_mode
     integer, dimension(:), allocatable :: amp_type
-    integer :: i, ew_scheme
+    integer :: i
 
-    allocate (amp_type (size (blha_flavor)*2))
+    allocate (amp_type (size (blha_flavor) * 2))
     do i = 1, size (blha_flavor)
-       amp_type(2*i-1) = BLHA_AMP_LOOP
-       amp_type(2*i) = BLHA_AMP_CC
+       amp_type(2 * i - 1) = BLHA_AMP_LOOP
+       amp_type(2 * i) = BLHA_AMP_CC
     end do
     call blha_configuration_init (blha_cfg, basename // "_LOOP" , &
          model, blha_mode)
     call blha_configuration_append_processes (blha_cfg, n_in, &
          blha_flavor, amp_type)
-    select case (blha_cfg%mode)
-    case (BLHA_MODE_GOSAM)
-       ew_scheme = BLHA_EW_0
-    case (BLHA_MODE_OPENLOOPS)
-       ew_scheme = BLHA_EW_0
-    end select 
     call blha_configuration_set (blha_cfg, BLHA_VERSION_2, &
          correction_type = BLHA_CT_QCD, &
          irreg = BLHA_IRREG_CDR, &
@@ -483,33 +612,28 @@ contains
   end subroutine blha_init_virtual
 
   subroutine blha_init_subtraction (blha_cfg, blha_flavor, n_in, &
-     ap, asp, basename, model, blha_mode)
+     ap, asp, ew_scheme, basename, model, blha_mode)
     type(blha_configuration_t), intent(inout) :: blha_cfg
     type(blha_flv_state_t), intent(in), dimension(:) :: blha_flavor
     integer, intent(in) :: n_in
     integer, intent(in) :: ap, asp
+    integer, intent(in) :: ew_scheme
     type(string_t), intent(in) :: basename
     type(model_data_t), intent(in), target :: model
     integer, intent(in) :: blha_mode
     integer, dimension(:), allocatable :: amp_type
-    integer :: i, ew_scheme
+    integer :: i
 
-    allocate (amp_type (size (blha_flavor)*3))
+    allocate (amp_type (size (blha_flavor) * 3))
     do i = 1, size (blha_flavor)
-       amp_type(3*i-2) = BLHA_AMP_TREE
-       amp_type(3*i-1) = BLHA_AMP_CC
-       amp_type(3*i) = BLHA_AMP_SC
+       amp_type(3 * i - 2) = BLHA_AMP_TREE
+       amp_type(3 * i - 1) = BLHA_AMP_CC
+       amp_type(3 * i) = BLHA_AMP_SC
     end do
     call blha_configuration_init (blha_cfg, basename // "_SUB" , &
          model, blha_mode)
     call blha_configuration_append_processes (blha_cfg, n_in, &
          blha_flavor, amp_type)
-    select case (blha_cfg%mode)
-    case (BLHA_MODE_GOSAM)
-       ew_scheme = BLHA_EW_0
-    case (BLHA_MODE_OPENLOOPS)
-       ew_scheme = BLHA_EW_0
-    end select 
     call blha_configuration_set (blha_cfg, BLHA_VERSION_2, &
          correction_type = BLHA_CT_QCD, &
          irreg = BLHA_IRREG_CDR, &
@@ -519,17 +643,18 @@ contains
          debug = blha_mode == BLHA_MODE_GOSAM)
   end subroutine blha_init_subtraction
 
-  subroutine blha_init_real (blha_cfg, blha_flavor, n_in, & 
-     ap, asp, basename, model, blha_mode)
+  subroutine blha_init_real (blha_cfg, blha_flavor, n_in, &
+     ap, asp, ew_scheme, basename, model, blha_mode)
     type(blha_configuration_t), intent(inout) :: blha_cfg
     type(blha_flv_state_t), intent(in), dimension(:) :: blha_flavor
     integer, intent(in) :: n_in
     integer, intent(in) :: ap, asp
+    integer, intent(in) :: ew_scheme
     type(string_t), intent(in) :: basename
     type(model_data_t), intent(in), target :: model
     integer, intent(in) :: blha_mode
     integer, dimension(:), allocatable :: amp_type
-    integer :: i, ew_scheme
+    integer :: i
 
     allocate (amp_type (size (blha_flavor)))
     do i = 1, size (blha_flavor)
@@ -539,23 +664,16 @@ contains
          model, blha_mode)
     call blha_configuration_append_processes (blha_cfg, n_in, &
          blha_flavor, amp_type)
-    select case (blha_cfg%mode)
-    case (BLHA_MODE_GOSAM)
-       ew_scheme = BLHA_EW_0
-    case (BLHA_MODE_OPENLOOPS)
-       ew_scheme = BLHA_EW_0
-    end select 
     call blha_configuration_set (blha_cfg, BLHA_VERSION_2, &
          correction_type = BLHA_CT_QCD, &
          irreg = BLHA_IRREG_CDR, &
-         alphas_power = asp+1, &
+         alphas_power = asp + 1, &
          alpha_power = ap, &
          ew_scheme = ew_scheme, &
-         ! debug = .true.)
          debug = blha_mode == BLHA_MODE_GOSAM)
   end subroutine blha_init_real
 
-  subroutine blha_master_generate (master, basename)
+  subroutine blha_master_write_olp (master, basename)
     class(blha_master_t), intent(in) :: master
     type(string_t), intent(in) :: basename
     integer :: unit
@@ -568,7 +686,7 @@ contains
        call blha_configuration_write (master%blha_cfg(i_file), unit)
        close (unit)
     end do
-  end subroutine blha_master_generate
+  end subroutine blha_master_write_olp
 
   subroutine blha_master_final (master)
     class(blha_master_t), intent(inout) :: master
@@ -597,7 +715,7 @@ contains
     integer, parameter :: max_particles = 10
     integer, dimension(max_particles) :: i_massive_tmp
     integer, dimension(max_particles) :: checked
-    type(blha_cfg_process_node_t), pointer :: current_process 
+    type(blha_cfg_process_node_t), pointer :: current_process
     integer :: k
     integer :: n_massive
     n_massive = 0; k = 1
@@ -619,7 +737,7 @@ contains
        else
           exit
        end if
-    end do       
+    end do
     if (n_massive > 0) then
        allocate (i_massive (n_massive))
        i_massive = i_massive_tmp (1:n_massive)
@@ -635,13 +753,13 @@ contains
        do i = 1, size (pdg_list)
           i_pdg = abs (pdg_list(i))
           call flv%init (i_pdg, cfg%model)
-          if (flv%get_mass () > 0._default) then 
+          if (flv%get_mass () > 0._default) then
              !!! Avoid duplicates in output
              if (.not. any (checked == i_pdg)) then
                 i_massive_tmp(k) = i_pdg
-                checked(k) = i_pdg  
-                k=k+1
-                n_massive=n_massive+1
+                checked(k) = i_pdg
+                k = k + 1
+                n_massive = n_massive + 1
              end if
           end if
        end do
@@ -659,7 +777,6 @@ contains
     integer, dimension(:), allocatable :: pdg_in, pdg_out
     integer, dimension(:), allocatable :: flavor_state
     integer :: proc_offset, n_proc_tot
-    integer :: h1, h2
     proc_offset = 0; n_proc_tot = 0
     do i_flv = 1, size (flavor)
        n_proc_tot = n_proc_tot + flavor(i_flv)%flv_mult
@@ -673,22 +790,22 @@ contains
        allocate (flavor_state (n_tot))
        flavor_state = flavor(i_flv)%flavors
        do i_process = 1, flavor(i_flv)%flv_mult
-          pdg_in = flavor_state (1:n_in)
-          pdg_out = flavor_state (n_in+1:)
+          pdg_in = flavor_state (1 : n_in)
+          pdg_out = flavor_state (n_in + 1 : )
           if (cfg%polarized) then
              select case (cfg%mode)
              case (BLHA_MODE_OPENLOOPS)
                 call allocate_and_init_pdg_and_helicities (current_node, &
-                   pdg_in, pdg_out, amp_type (proc_offset+i_process))
+                   pdg_in, pdg_out, amp_type (proc_offset + i_process))
              case (BLHA_MODE_GOSAM)
                 !!! Nothing special for GoSam yet. This exception is already caught
                 !!! in blha_master_setup_additional_features
              end select
           else
              call allocate_and_init_pdg (current_node, pdg_in, pdg_out, &
-                amp_type (proc_offset + i_process)) 
+                amp_type (proc_offset + i_process))
           end if
-          if (proc_offset+i_process /= n_proc_tot) then
+          if (proc_offset + i_process /= n_proc_tot) then
             allocate (current_node%next)
             current_node => current_node%next
           end if
@@ -698,7 +815,9 @@ contains
        deallocate (pdg_in, pdg_out)
        deallocate (flavor_state)
     end do
+
   contains
+
     subroutine allocate_and_init_pdg (node, pdg_in, pdg_out, amp_type)
       type(blha_cfg_process_node_t), intent(inout), pointer :: node
       integer, intent(in), dimension(:), allocatable :: pdg_in, pdg_out
@@ -707,7 +826,7 @@ contains
       allocate (node%pdg_out (size (pdg_out)))
       node%pdg_in%pdg = pdg_in
       node%pdg_out%pdg = pdg_out
-      node%amplitude_type = amp_type  
+      node%amplitude_type = amp_type
     end subroutine allocate_and_init_pdg
 
     subroutine allocate_and_init_pdg_and_helicities (node, pdg_in, pdg_out, amp_type)
@@ -715,22 +834,33 @@ contains
       integer, intent(in), dimension(:), allocatable :: pdg_in, pdg_out
       integer, intent(in) :: amp_type
       integer :: h1, h2
-      do h1 = -1, 1
-         do h2 = -1, 1
-            if (h1 == 0 .or. h2 == 0) cycle 
-            call allocate_and_init_pdg (current_node, pdg_in, pdg_out, amp_type) 
+      if (size (pdg_in) == 2) then
+         do h1 = -1, 1, 2
+            do h2 = -1, 1, 2
+               call allocate_and_init_pdg (current_node, pdg_in, pdg_out, amp_type)
+               current_node%pdg_in(1)%polarized = .true.
+               current_node%pdg_in(2)%polarized = .true.
+               current_node%pdg_in(1)%hel = h1
+               current_node%pdg_in(2)%hel = h2
+               if (h1 + h2 /= 2) then !!! not end of loop
+                  allocate (current_node%next)
+                  current_node => current_node%next
+               end if
+            end do
+         end do
+      else
+         do h1 = -1, 1, 2
+            call allocate_and_init_pdg (current_node, pdg_in, pdg_out, amp_type)
             current_node%pdg_in(1)%polarized = .true.
-            current_node%pdg_in(2)%polarized = .true.
             current_node%pdg_in(1)%hel = h1
-            current_node%pdg_in(2)%hel = h2
-            if (h1 + h2 /= 2) then
+            if (h1 /= 1) then !!! not end of loop
                allocate (current_node%next)
                current_node => current_node%next
             end if
          end do
-      end do
+      end if
     end subroutine allocate_and_init_pdg_and_helicities
-      
+
   end subroutine blha_configuration_append_processes
 
   subroutine blha_configuration_set (cfg, &
@@ -792,9 +922,8 @@ contains
     character(4) :: hel_char
     character(len=25), parameter :: pad = ""
     logical :: write_process, no_v
-    if (present (no_version))  no_v = no_version
-    
-    u = given_output_unit (unit)
+    no_v = .false. ; if (present (no_version))  no_v = no_version
+    u = given_output_unit (unit); if (u < 0) return
     full = .true.; if (present (internal)) full = .not. internal
     if (full .and. cfg%dirty) call msg_bug ( &
        "BUG: attempted to write out a dirty BLHA configuration")
@@ -802,7 +931,7 @@ contains
        if (no_v) then
           write (u, "(A)") "# BLHA order written by WHIZARD [version]"
        else
-          write (u, "(A)") "# BLHA order written by WHIZARD 2.2.8"
+          write (u, "(A)") "# BLHA order written by WHIZARD 2.4.1"
        end if
        write (u, "(A)")
     end if
@@ -853,26 +982,33 @@ contains
          "AlphasPower" // pad, int2char (cfg%alphas_power)
       if (cfg%alpha_power >= 0) write (u,'(A25,A)') &
          "AlphaPower " // pad, int2char (cfg%alpha_power)
-    case (2) 
+    case (2)
       if (cfg%alphas_power >= 0) write (u,'(A25,A)') &
          "CouplingPower QCD " // pad, int2char (cfg%alphas_power)
       if (cfg%alpha_power >= 0) write (u, '(A25,A)') &
          "CouplingPower QED " // pad, int2char (cfg%alpha_power)
     end select
-    select case (cfg%ew_scheme)
-       case (BLHA_EW_GF); buf = "alphaGF"
-       case (BLHA_EW_MZ); buf = "alphaMZ"
-       case (BLHA_EW_MSBAR); buf = "alphaMSbar"
-       case (BLHA_EW_0); buf = "alpha0"
-       case (BLHA_EW_RUN); buf = "alphaRUN"
-       case (BLHA_EW_DEFAULT); buf = "OLPDefined"
-    end select
-    select case (cfg%mode)
-    case (BLHA_MODE_GOSAM)
-!       write (u, '(A25, A)') "EWScheme " // pad, char (buf)
-    case (BLHA_MODE_OPENLOOPS)
-!       write (u, '(A25, A)') "ewscheme " // pad, char (buf)
-    endselect
+    if (cfg%ew_scheme > BLHA_EW_QED) then
+       select case (cfg%mode)
+       case (BLHA_MODE_GOSAM)
+          select case (cfg%ew_scheme)
+             case (BLHA_EW_GF); buf = "alphaGF"
+             case (BLHA_EW_MZ); buf = "alphaMZ"
+             case (BLHA_EW_MSBAR); buf = "alphaMSbar"
+             case (BLHA_EW_0); buf = "alpha0"
+             case (BLHA_EW_RUN); buf = "alphaRUN"
+          end select
+          write (u, '(A25, A)') "EWScheme " // pad, char (buf)
+       case (BLHA_MODE_OPENLOOPS)
+          select case (cfg%ew_scheme)
+             case (BLHA_EW_0); buf = "alpha0"
+             case (BLHA_EW_GF); buf = "Gmu"
+             case default
+                call msg_fatal ("OpenLoops input: Only supported EW schemes are 'alpha0' and 'Gmu'")
+          end select
+          write (u, '(A25, A)') "ewscheme " // pad, char (buf)
+       end select
+    end if
     select case (cfg%mode)
     case (BLHA_MODE_GOSAM)
        write (u, '(A25)', advance='no') "MassiveParticles " // pad
@@ -880,16 +1016,24 @@ contains
           if (OLP_MASSIVE_PARTICLES(i) > 0) &
              write (u, '(I2,1X)', advance='no') OLP_MASSIVE_PARTICLES(i)
        end do
-       write (u,*) 
+       write (u,*)
     case (BLHA_MODE_OPENLOOPS)
-       write (u, '(A25,I1)') "extra use_cms " // pad, 0
+       if (cfg%openloops_use_cms) then
+          write (u, '(A25,I1)') "extra use_cms " // pad, 1
+       else
+          write (u, '(A25,I1)') "extra use_cms " // pad, 0
+       end if
        write (u, '(A25,I1)') "extra me_cache " // pad, 0
        if (cfg%openloops_phs_tolerance > 0) then
           write (u, '(A25,A4,I0)') "extra psp_tolerance " // pad, "10e-", &
              cfg%openloops_phs_tolerance
        end if
-       if (cfg%openloops_top_signal) &
-          write (u, '(A)') "extra approx top"
+       call check_extra_cmd (cfg%openloops_extra_cmd)
+       write (u, '(A)') char (cfg%openloops_extra_cmd)
+       if (cfg%openloops_stability_log > 0) &
+          write (u, '(A25,I1)') "extra stability_log " // pad, cfg%openloops_stability_log
+       if (cfg%openloops_use_collier) &
+          write (u, '(A25,I1)') "extra preset " // pad, 2
     end select
     if (full) then
        write (u, "(A)")
@@ -905,15 +1049,17 @@ contains
        select case (node%amplitude_type)
          case (BLHA_AMP_LOOP); buf = "Loop"
          case (BLHA_AMP_CC); buf = "ccTree"
-         case (BLHA_AMP_SC) 
-            buf = "scTree"
-            if (cfg%mode == BLHA_MODE_OPENLOOPS) write_process = .false. 
+         case (BLHA_AMP_SC)
+            if (cfg%mode == BLHA_MODE_OPENLOOPS) then
+               buf = "sctree_polvect"
+            else
+               buf = "scTree"
+            end if
          case (BLHA_AMP_TREE); buf = "Tree"
          case (BLHA_AMP_LOOPINDUCED); buf = "LoopInduced"
        end select
        if (write_process) then
           write (u, '(A25, A)') "AmplitudeType " // pad, char (buf)
-   
           buf = ""
           do i = 1, size (node%pdg_in)
              call node%pdg_in(i)%write_pdg (pdg_char)
