@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -42,7 +42,7 @@ module eio_lhef_uti
   use event_base
   use eio_data
   use eio_base
-  
+
   use eio_lhef
 
   use eio_base_ut, only: eio_prepare_test, eio_cleanup_test
@@ -74,7 +74,7 @@ contains
     write (u, "(A)")
 
     write (u, "(A)")  "* Initialize test process"
- 
+
     call eio_prepare_test (event, unweighted = .false.)
 
     call data%init (1)
@@ -92,15 +92,15 @@ contains
     write (u, "(A)")
     write (u, "(A)")  "* Generate and write an event"
     write (u, "(A)")
- 
+
     sample = "eio_lhef_1"
-    
+
     allocate (eio_lhef_t :: eio)
     select type (eio)
     type is (eio_lhef_t)
        call eio%set_parameters ()
     end select
-    
+
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
 
@@ -122,18 +122,18 @@ contains
        write (u, "(A)") trim (buffer)
     end do
     close (u_file)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Reset data"
     write (u, "(A)")
- 
+
     deallocate (eio)
     allocate (eio_lhef_t :: eio)
     select type (eio)
     type is (eio_lhef_t)
        call eio%set_parameters ()
     end select
-    
+
     select type (eio)
     type is (eio_lhef_t)
        call eio%set_parameters (keep_beams = .true.)
@@ -142,17 +142,16 @@ contains
 
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
- 
+
     call eio_cleanup_test (event)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_lhef_1"
-    
+
   end subroutine eio_lhef_1
-  
+
   subroutine eio_lhef_2 (u)
     integer, intent(in) :: u
-    type(model_data_t), target :: model
     class(generic_event_t), pointer :: event
     type(event_sample_data_t) :: data
     class(eio_t), allocatable :: eio
@@ -165,7 +164,7 @@ contains
     write (u, "(A)")
 
     write (u, "(A)")  "* Initialize test process"
- 
+
     call eio_prepare_test (event, unweighted = .false.)
 
     call data%init (1)
@@ -183,15 +182,15 @@ contains
     write (u, "(A)")
     write (u, "(A)")  "* Generate and write an event"
     write (u, "(A)")
- 
+
     sample = "eio_lhef_2"
-    
+
     allocate (eio_lhef_t :: eio)
     select type (eio)
     type is (eio_lhef_t)
        call eio%set_parameters (version = "2.0", write_sqme_prc = .true.)
     end select
-    
+
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
 
@@ -214,17 +213,17 @@ contains
        write (u, "(A)") trim (buffer)
     end do
     close (u_file)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
- 
+
     call eio_cleanup_test (event)
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_lhef_2"
-    
+
   end subroutine eio_lhef_2
-  
+
   subroutine eio_lhef_3 (u)
     integer, intent(in) :: u
     class(generic_event_t), pointer :: event
@@ -239,7 +238,7 @@ contains
     write (u, "(A)")
 
     write (u, "(A)")  "* Initialize test process"
- 
+
     call eio_prepare_test (event, unweighted = .false.)
 
     call data%init (1)
@@ -257,15 +256,15 @@ contains
     write (u, "(A)")
     write (u, "(A)")  "* Generate and write an event"
     write (u, "(A)")
- 
+
     sample = "eio_lhef_3"
-    
+
     allocate (eio_lhef_t :: eio)
     select type (eio)
     type is (eio_lhef_t)
        call eio%set_parameters (version = "3.0", write_sqme_prc = .true.)
     end select
-    
+
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
 
@@ -288,17 +287,17 @@ contains
        write (u, "(A)") trim (buffer)
     end do
     close (u_file)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
- 
+
     call eio_cleanup_test (event)
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_lhef_3"
-    
+
   end subroutine eio_lhef_3
-  
+
   subroutine eio_lhef_4 (u)
     integer, intent(in) :: u
     class(model_data_t), pointer :: fallback_model
@@ -314,12 +313,12 @@ contains
 
     write (u, "(A)")  "* Write a LHEF data file"
     write (u, "(A)")
- 
+
     u_file = free_unit ()
     sample = "eio_lhef_4"
     open (u_file, file = char (sample // ".lhe"), &
          status = "replace", action = "readwrite")
-    
+
     write (u_file, "(A)")  '<LesHouchesEvents version="1.0">'
     write (u_file, "(A)")  '<header>'
     write (u_file, "(A)")  '  <arbitrary_tag opt="foo">content</arbitrary_tag>'
@@ -350,7 +349,7 @@ contains
     write (u_file, "(A)")  '</event>'
     write (u_file, "(A)")  '</LesHouchesEvents>'
     close (u_file)
-    
+
 
     write (u, "(A)")  "* Initialize test process"
     write (u, "(A)")
@@ -364,7 +363,7 @@ contains
        call eio%set_parameters (recover_beams = .false.)
     end select
     call eio%set_fallback_model (fallback_model)
-    
+
     call data%init (1)
     data%n_beam = 2
     data%unweighted = .true.
@@ -380,9 +379,9 @@ contains
 
     call eio%init_in (sample, data)
     call eio%write (u)
-    
+
     write (u, *)
-    
+
     select type (eio)
     type is (eio_lhef_t)
        call eio%tag_lhef%write (u);  write (u, *)
@@ -396,13 +395,13 @@ contains
     write (u, "(A)")
 
     call eio%input_i_prc (i_prc, iostat)
-    
+
     select type (eio)
     type is (eio_lhef_t)
        write (u, "(A,I0,A,I0)")  "Found process #", i_prc, &
             " with ID = ", eio%proc_num_id(i_prc)
     end select
-   
+
     call eio%input_event (event, iostat)
 
     call event%write (u)
@@ -413,10 +412,10 @@ contains
 
     call eio%input_i_prc (i_prc, iostat)
     write (u, "(A,I0)")  "iostat = ", iostat
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
- 
+
     call eio%final ()
 
     call eio_cleanup_test (event)
@@ -424,9 +423,9 @@ contains
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_lhef_4"
-    
+
   end subroutine eio_lhef_4
-  
+
   subroutine eio_lhef_5 (u)
     integer, intent(in) :: u
     class(model_data_t), pointer :: fallback_model
@@ -442,12 +441,12 @@ contains
 
     write (u, "(A)")  "* Write a LHEF data file"
     write (u, "(A)")
- 
+
     u_file = free_unit ()
     sample = "eio_lhef_5"
     open (u_file, file = char (sample // ".lhe"), &
          status = "replace", action = "readwrite")
-    
+
     write (u_file, "(A)")  '<LesHouchesEvents version="2.0">'
     write (u_file, "(A)")  '<header>'
     write (u_file, "(A)")  '</header>'
@@ -492,7 +491,7 @@ contains
        call eio%set_parameters (version = "2.0", recover_beams = .false.)
     end select
     call eio%set_fallback_model (fallback_model)
-    
+
     call data%init (1)
     data%unweighted = .false.
     data%norm_mode = NORM_SIGMA
@@ -508,9 +507,9 @@ contains
 
     call eio%init_in (sample, data)
     call eio%write (u)
-    
+
     write (u, *)
-    
+
     select type (eio)
     type is (eio_lhef_t)
        call eio%tag_lhef%write (u);  write (u, *)
@@ -524,13 +523,13 @@ contains
     write (u, "(A)")
 
     call eio%input_i_prc (i_prc, iostat)
-    
+
     select type (eio)
     type is (eio_lhef_t)
        write (u, "(A,I0,A,I0)")  "Found process #", i_prc, &
             " with ID = ", eio%proc_num_id(i_prc)
     end select
-   
+
     call eio%input_event (event, iostat)
 
     call event%write (u)
@@ -541,20 +540,20 @@ contains
 
     call eio%input_i_prc (i_prc, iostat)
     write (u, "(A,I0)")  "iostat = ", iostat
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
- 
+
     call eio%final ()
 
     call eio_cleanup_test (event)
     call eio_cleanup_fallback_model (fallback_model)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_lhef_5"
-    
+
   end subroutine eio_lhef_5
-  
+
   subroutine eio_lhef_6 (u)
     integer, intent(in) :: u
     class(model_data_t), pointer :: fallback_model
@@ -570,12 +569,12 @@ contains
 
     write (u, "(A)")  "* Write a LHEF data file"
     write (u, "(A)")
- 
+
     u_file = free_unit ()
     sample = "eio_lhef_6"
     open (u_file, file = char (sample // ".lhe"), &
          status = "replace", action = "readwrite")
-    
+
     write (u_file, "(A)")  '<LesHouchesEvents version="3.0">'
     write (u_file, "(A)")  '<header>'
     write (u_file, "(A)")  '</header>'
@@ -621,7 +620,7 @@ contains
        call eio%set_parameters (version = "3.0", recover_beams = .false.)
     end select
     call eio%set_fallback_model (fallback_model)
-    
+
     call data%init (1)
     data%unweighted = .false.
     data%norm_mode = NORM_SIGMA
@@ -637,9 +636,9 @@ contains
 
     call eio%init_in (sample, data)
     call eio%write (u)
-    
+
     write (u, *)
-    
+
     select type (eio)
     type is (eio_lhef_t)
        call eio%tag_lhef%write (u);  write (u, *)
@@ -653,13 +652,13 @@ contains
     write (u, "(A)")
 
     call eio%input_i_prc (i_prc, iostat)
-    
+
     select type (eio)
     type is (eio_lhef_t)
        write (u, "(A,I0,A,I0)")  "Found process #", i_prc, &
             " with ID = ", eio%proc_num_id(i_prc)
     end select
-   
+
     call eio%input_event (event, iostat)
 
     call event%write (u)
@@ -670,10 +669,10 @@ contains
 
     call eio%input_i_prc (i_prc, iostat)
     write (u, "(A,I0)")  "iostat = ", iostat
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
- 
+
     call eio%final ()
 
     call eio_cleanup_test (event)
@@ -681,8 +680,8 @@ contains
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: eio_lhef_6"
-    
+
   end subroutine eio_lhef_6
-  
+
 
 end module eio_lhef_uti
