@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -34,7 +34,7 @@
 ! to the source 'whizard.nw'
 
 module event_base
-  
+
   use system_defs, only: LF
   use kinds, only: default
   use kinds, only: i64
@@ -55,7 +55,6 @@ module event_base
   public :: event_callback_t
   public :: event_callback_nop_t
 
-  integer, parameter :: N_SUPPORTED_EVENT_FORMATS = 2
   integer, parameter, public :: NORM_UNDEFINED = 0
   integer, parameter, public :: NORM_UNIT = 1
   integer, parameter, public :: NORM_N_EVT = 2
@@ -63,16 +62,6 @@ module event_base
   integer, parameter, public :: NORM_S_N = 4
 
 
-  type :: nlo_event_info_t
-    logical :: nlo_event = .false.
-    type(string_t), dimension(N_SUPPORTED_EVENT_FORMATS) :: &
-       supported_event_formats
-  contains
-    procedure :: init_sample_formats => nlo_event_info_init_sample_formats
-    procedure :: check_supported_sample_formats &
-       => nlo_event_info_check_supported_sample_formats
-  end type nlo_event_info_t
-
   type, abstract :: generic_event_t
      !private
      logical :: particle_set_is_valid = .false.
@@ -92,7 +81,6 @@ module event_base
      real(default), dimension(:), allocatable :: sqme_alt
      logical :: weight_alt_known = .false.
      real(default), dimension(:), allocatable :: weight_alt
-     type(nlo_event_info_t) :: nlo_info
    contains
      procedure :: base_init => generic_event_init
      procedure :: has_valid_particle_set => generic_event_has_valid_particle_set
@@ -132,13 +120,18 @@ module event_base
      procedure (generic_event_write), deferred :: write
      procedure (generic_event_generate), deferred :: generate
      procedure (generic_event_set_hard_particle_set), deferred :: &
-          set_hard_particle_set 
+          set_hard_particle_set
      procedure (generic_event_handler), deferred :: evaluate_expressions
      procedure (generic_event_select), deferred :: select
      procedure (generic_event_get_model_ptr), deferred :: get_model_ptr
      procedure (generic_event_get_index), deferred :: get_index
      procedure (generic_event_get_fac_scale), deferred :: get_fac_scale
      procedure (generic_event_get_alpha_s), deferred :: get_alpha_s
+     procedure (generic_event_get_sqrts), deferred :: get_sqrts
+     procedure (generic_event_get_polarization), deferred :: get_polarization
+     procedure (generic_event_get_beam_file), deferred :: get_beam_file
+     procedure (generic_event_get_process_name), deferred :: &
+          get_process_name
      procedure (generic_event_set_alpha_qcd_forced), deferred :: &
           set_alpha_qcd_forced
      procedure (generic_event_set_scale_forced), deferred :: &
@@ -146,27 +139,22 @@ module event_base
      procedure :: reset => generic_event_reset
      procedure :: base_reset => generic_event_reset
      procedure :: pacify_particle_set => generic_event_pacify_particle_set
-     procedure :: set_nlo_event => generic_event_set_nlo_event
-     procedure :: is_nlo_event => generic_event_is_nlo_event
-     procedure :: init_sample_formats => generic_event_init_sample_formats
-     procedure :: check_supported_sample_formats &
-        => generic_event_check_supported_sample_formats
   end type generic_event_t
-  
+
   type, abstract :: event_callback_t
      private
    contains
      procedure(event_callback_write), deferred :: write
      procedure(event_callback_proc), deferred :: proc
   end type event_callback_t
-  
+
   type, extends (event_callback_t) :: event_callback_nop_t
      private
    contains
      procedure :: write => event_callback_nop_write
      procedure :: proc => event_callback_nop
   end type event_callback_nop_t
-  
+
 
   abstract interface
      subroutine generic_event_write (object, unit, &
@@ -192,7 +180,7 @@ module event_base
        integer, intent(in), optional :: i_nlo
      end subroutine generic_event_generate
   end interface
-     
+
   abstract interface
      subroutine generic_event_set_hard_particle_set (event, particle_set)
        import
@@ -207,7 +195,7 @@ module event_base
        class(generic_event_t), intent(inout) :: event
      end subroutine generic_event_handler
   end interface
-  
+
   abstract interface
      subroutine generic_event_select (event,  i_mci, i_term, channel)
        import
@@ -215,7 +203,7 @@ module event_base
        integer, intent(in) :: i_mci, i_term, channel
      end subroutine generic_event_select
   end interface
-  
+
   abstract interface
      function generic_event_get_model_ptr (event) result (model)
        import
@@ -223,7 +211,7 @@ module event_base
        class(model_data_t), pointer :: model
      end function generic_event_get_model_ptr
   end interface
-  
+
   abstract interface
      function generic_event_get_index (event) result (index)
        import
@@ -231,7 +219,7 @@ module event_base
        integer :: index
      end function generic_event_get_index
   end interface
-    
+
   abstract interface
      function generic_event_get_fac_scale (event) result (fac_scale)
        import
@@ -239,7 +227,7 @@ module event_base
        real(default) :: fac_scale
      end function generic_event_get_fac_scale
   end interface
-    
+
   abstract interface
      function generic_event_get_alpha_s (event) result (alpha_s)
        import
@@ -248,6 +236,38 @@ module event_base
      end function generic_event_get_alpha_s
   end interface
 
+  abstract interface
+     function generic_event_get_sqrts (event) result (sqrts)
+       import
+       class(generic_event_t), intent(in) :: event
+       real(default) :: sqrts
+     end function generic_event_get_sqrts
+  end interface
+
+  abstract interface
+     function generic_event_get_polarization (event) result (pol)
+       import
+       class(generic_event_t), intent(in) :: event
+       real(default), dimension(2) :: pol
+     end function generic_event_get_polarization
+  end interface
+
+  abstract interface
+     function generic_event_get_beam_file (event) result (file)
+       import
+       class(generic_event_t), intent(in) :: event
+       type(string_t) :: file
+     end function generic_event_get_beam_file
+  end interface
+
+  abstract interface
+     function generic_event_get_process_name (event) result (name)
+       import
+       class(generic_event_t), intent(in) :: event
+       type(string_t) :: name
+     end function generic_event_get_process_name
+  end interface
+
   abstract interface
      subroutine generic_event_set_alpha_qcd_forced (event, alpha_qcd)
        import
@@ -271,7 +291,7 @@ module event_base
        integer, intent(in), optional :: unit
      end subroutine event_callback_write
   end interface
-  
+
   abstract interface
      subroutine event_callback_proc (event_callback, i, event)
        import
@@ -280,32 +300,9 @@ module event_base
        class(generic_event_t), intent(in) :: event
      end subroutine event_callback_proc
   end interface
-  
+
 
 contains
-  
-  subroutine nlo_event_info_init_sample_formats (nlo_info)
-    class(nlo_event_info_t), intent(inout) :: nlo_info
-    nlo_info%supported_event_formats(1) = var_str ("hepmc")
-    nlo_info%supported_event_formats(2) = var_str ("debug")
-  end subroutine nlo_event_info_init_sample_formats
-
-  subroutine nlo_event_info_check_supported_sample_formats (nlo_info, format)
-    class(nlo_event_info_t), intent(in) :: nlo_info
-    type(string_t), intent(in) :: format
-    logical :: value
-    type(string_t) :: err_message
-    integer :: i
-    value = any (nlo_info%supported_event_formats == format)
-    if (.not. value) then
-      err_message = var_str ("NLO event setup: sample_format ") // format // &
-            var_str (" is not supported yet. Please choose one of the following: ") // LF
-      do i = 1, N_SUPPORTED_EVENT_FORMATS
-         err_message = err_message // nlo_info%supported_event_formats (i) // LF
-      end do
-      call msg_fatal (char (err_message))
-    end if
-  end subroutine nlo_event_info_check_supported_sample_formats
 
   subroutine generic_event_init (event, n_alt)
     class(generic_event_t), intent(out) :: event
@@ -314,29 +311,29 @@ contains
     allocate (event%sqme_alt (n_alt))
     allocate (event%weight_alt (n_alt))
   end subroutine generic_event_init
-  
+
   function generic_event_has_valid_particle_set (event) result (flag)
     class(generic_event_t), intent(in) :: event
     logical :: flag
     flag = event%particle_set_is_valid
   end function generic_event_has_valid_particle_set
-  
+
   subroutine generic_event_accept_particle_set (event)
     class(generic_event_t), intent(inout) :: event
     event%particle_set_is_valid = .true.
   end subroutine generic_event_accept_particle_set
-  
+
   subroutine generic_event_discard_particle_set (event)
     class(generic_event_t), intent(inout) :: event
     event%particle_set_is_valid = .false.
   end subroutine generic_event_discard_particle_set
-  
+
   function generic_event_get_particle_set_ptr (event) result (ptr)
     class(generic_event_t), intent(in) :: event
     type(particle_set_t), pointer :: ptr
     ptr => event%particle_set
   end function generic_event_get_particle_set_ptr
-  
+
   subroutine generic_event_link_particle_set (event, particle_set)
     class(generic_event_t), intent(inout) :: event
     type(particle_set_t), intent(in), target :: particle_set
@@ -391,7 +388,7 @@ contains
     integer :: n
     n = event%n_alt
   end function generic_event_get_n_alt
-  
+
   function generic_event_get_sqme_prc (event) result (sqme)
     class(generic_event_t), intent(in) :: event
     real(default) :: sqme
@@ -539,7 +536,7 @@ contains
     end if
     if (present (sqme_ref)) then
        call event%set_sqme_ref (sqme_ref)
-    end if 
+    end if
     if (present (sqme_alt)) then
        call event%set_sqme_alt (sqme_alt)
     end if
@@ -556,7 +553,7 @@ contains
        call event%set_excess_prc (excess_prc)
     end if
   end subroutine generic_event_set
-  
+
   subroutine generic_event_reset (event)
     class(generic_event_t), intent(inout) :: event
     call event%discard_particle_set ()
@@ -568,35 +565,12 @@ contains
     event%weight_alt_known = .false.
     event%excess_prc_known = .false.
   end subroutine generic_event_reset
-  
+
   subroutine generic_event_pacify_particle_set (event)
     class(generic_event_t), intent(inout) :: event
     if (event%has_valid_particle_set ())  call pacify (event%particle_set)
   end subroutine generic_event_pacify_particle_set
 
-  subroutine generic_event_set_nlo_event (event, value)
-    class(generic_event_t), intent(inout) :: event
-    logical, intent(in) :: value
-    event%nlo_info%nlo_event = value
-  end subroutine generic_event_set_nlo_event
-
-  function generic_event_is_nlo_event (event) result (value)
-    logical :: value
-    class(generic_event_t), intent(in) :: event
-    value = event%nlo_info%nlo_event
-  end function generic_event_is_nlo_event
-
-  subroutine generic_event_init_sample_formats (event)
-    class(generic_event_t), intent(inout) :: event
-    call event%nlo_info%init_sample_formats ()
-  end subroutine generic_event_init_sample_formats
-
-  subroutine generic_event_check_supported_sample_formats (event, format)
-    class(generic_event_t), intent(in) :: event
-    type(string_t), intent(in) :: format
-    call event%nlo_info%check_supported_sample_formats (format)
-  end subroutine generic_event_check_supported_sample_formats
-
   function event_normalization_mode (string, unweighted) result (mode)
     integer :: mode
     type(string_t), intent(in) :: string
@@ -621,7 +595,7 @@ contains
             // char (string) // "'")
     end select
   end function event_normalization_mode
-  
+
   function event_normalization_string (norm_mode) result (string)
     integer, intent(in) :: norm_mode
     type(string_t) :: string
@@ -634,7 +608,7 @@ contains
     case default;          string = "???"
     end select
   end function event_normalization_string
-  
+
   subroutine event_normalization_update (weight, sigma, n, mode_new, mode_old)
     real(default), intent(inout) :: weight
     real(default), intent(in) :: sigma
@@ -661,7 +635,7 @@ contains
       end select
     end function factor
   end subroutine event_normalization_update
-  
+
   subroutine event_callback_nop_write (event_callback, unit)
     class(event_callback_nop_t), intent(in) :: event_callback
     integer, intent(in), optional :: unit
@@ -669,12 +643,12 @@ contains
     u = given_output_unit (unit)
     write (u, "(1x,A)")  "NOP"
   end subroutine event_callback_nop_write
-  
+
   subroutine event_callback_nop (event_callback, i, event)
     class(event_callback_nop_t), intent(in) :: event_callback
     integer(i64), intent(in) :: i
     class(generic_event_t), intent(in) :: event
   end subroutine event_callback_nop
-  
+
 
 end module event_base
