@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -42,7 +42,7 @@ module prc_gosam
   use iso_varying_string, string_t => varying_string
   use io_units
   use constants
-  use unit_tests, only: vanishes
+  use numeric_utils
   use system_defs, only: TAB
   use system_dependencies
   use file_utils
@@ -66,18 +66,10 @@ module prc_gosam
 
   use blha_config
   use blha_olp_interfaces
-  use loop_archive
 
   implicit none
   private
 
-  character(10), dimension(5), parameter ::  &
-             lib_suffix = [character(10) :: &
-             '.a', '.la', '.so', '.so.0', '.so.0.0.0']
-
-  integer, parameter :: Q_TO_QG = 1
-  integer, parameter :: G_TO_GG = 2
-  integer, parameter :: G_TO_QQ = 3
 
 
   public :: gosam_def_t
@@ -90,7 +82,7 @@ module prc_gosam
     type(string_t) :: ninja_dir
     type(string_t) :: form_dir
     type(string_t) :: qgraf_dir
-    type(string_t), dimension(2) :: filter
+    type(string_t) :: filter_lo, filter_nlo
     type(string_t) :: symmetries
     integer :: form_threads
     integer :: form_workspace
@@ -119,7 +111,6 @@ module prc_gosam
     type(string_t) :: olc_file
     type(string_t) :: olp_dir
     type(string_t) :: olp_lib
-    type(loop_archive_t) :: loop_archive
   contains
     procedure, nopass :: type_name => gosam_driver_type_name
     procedure :: init_gosam => gosam_driver_init_gosam
@@ -136,18 +127,17 @@ module prc_gosam
     logical :: initialized = .false.
   contains
     procedure :: prepare_library => prc_gosam_prepare_library
-    procedure :: search_for_existing_library => &
-                        prc_gosam_search_for_existing_library
+    procedure :: create_and_load_extra_libraries => &
+         prc_gosam_create_and_load_extra_libraries
     procedure :: write_makefile => prc_gosam_write_makefile
     procedure :: execute_makefile => prc_gosam_execute_makefile
     procedure :: create_olp_library => prc_gosam_create_olp_library
     procedure :: load_driver => prc_gosam_load_driver
     procedure :: start => prc_gosam_start
     procedure :: write => prc_gosam_write
+    procedure :: write_name => prc_gosam_write_name
     procedure :: init_driver => prc_gosam_init_driver
     procedure :: set_initialized => prc_gosam_set_initialized
-    procedure :: compute_sqme_born => prc_gosam_compute_sqme_born
-    procedure :: compute_sqme_real => prc_gosam_compute_sqme_real
     procedure :: compute_sqme_sc => prc_gosam_compute_sqme_sc
     procedure :: allocate_workspace => prc_gosam_allocate_workspace
   end type prc_gosam_t
@@ -163,10 +153,10 @@ module prc_gosam
 contains
 
   subroutine gosam_def_init (object, basename, model_name, &
-                             prt_in, prt_out, nlo_type, var_list)
+     prt_in, prt_out, nlo_type, var_list)
     class(gosam_def_t), intent(inout) :: object
-    type(string_t), intent(in) :: model_name
     type(string_t), intent(in) :: basename
+    type(string_t), intent(in) :: model_name
     type(string_t), dimension(:), intent(in) :: prt_in, prt_out
     integer, intent(in) :: nlo_type
     type(var_list_t), intent(in) :: var_list
@@ -185,9 +175,8 @@ contains
     select type (writer => object%writer)
     type is (gosam_writer_t)
       call writer%init (model_name, prt_in, prt_out)
-      writer%filter = &
-           [var_list%get_sval (var_str ("$gosam_filter_lo")), &
-            var_list%get_sval (var_str ("$gosam_filter_nlo"))]
+      writer%filter_lo = var_list%get_sval (var_str ("$gosam_filter_lo"))
+      writer%filter_nlo = var_list%get_sval (var_str ("$gosam_filter_nlo"))
       writer%symmetries = &
            var_list%get_sval (var_str ("$gosam_symmetries"))
       writer%form_threads = &
@@ -239,7 +228,7 @@ contains
     string = "gosam"
   end function gosam_writer_type_name
 
-  subroutine gosam_writer_init (writer, model_name, prt_in, prt_out, restrictions)
+  pure subroutine gosam_writer_init (writer, model_name, prt_in, prt_out, restrictions)
     class(gosam_writer_t), intent(inout) :: writer
     type(string_t), intent(in) :: model_name
     type(string_t), dimension(:), intent(in) :: prt_in, prt_out
@@ -268,7 +257,6 @@ contains
     object%contract_file = olc_file
     object%olp_dir = olp_dir
     object%olp_lib = olp_lib
-    call object%loop_archive%activate (var_str ('Generated_Loops'), os_data)
   end subroutine gosam_driver_init_gosam
 
   subroutine gosam_driver_init_dlaccess_to_library &
@@ -344,10 +332,10 @@ contains
       !!! write (unit, "(A)") "zero=mU,mD,mC,mS,mB"
       !!! This is covered by the BLHA2 interface
       write (unit, "(A)") "PSP_check=False"
-      if (char (object%filter(1)) /= "") &
-         write (unit, "(A)") "filter.lo=" // char (object%filter(1))
-      if (char (object%filter(2)) /= "") &
-         write (unit, "(A)") "filter.nlo=" // char (object%filter(2))
+      if (char (object%filter_lo) /= "") &
+         write (unit, "(A)") "filter.lo=" // char (object%filter_lo)
+      if (char (object%filter_nlo) /= "") &
+         write (unit, "(A)") "filter.nlo=" // char (object%filter_nlo)
       if (char (object%symmetries) /= "") &
          write (unit, "(A)") "symmetries=" // char(object%symmetries)
       write (unit, "(A,I0)") "form.threads=", object%form_threads
@@ -363,22 +351,18 @@ contains
     write (unit, "(2A)")  "OLP_FILE = ", char (object%olp_file)
     write (unit, "(2A)")  "OLP_DIR = ", char (object%olp_dir)
     write (unit, "(A)")
-    write (unit, "(A)")   "all: autogen"
-    write (unit, "(4A)")   TAB, "make ", &
-         char (object%loop_archive%os_data%makeflags), &
-         " -C $(OLP_DIR) install"
+    write (unit, "(A)")   "all: $(OLP_DIR)/config.log"
+    write (unit, "(2A)")  TAB, "make -C $(OLP_DIR) install"
     write (unit, "(A)")
-    write (unit, "(3A)")   "autogen: ", char (libname)
-    write (unit, "(3A)")  TAB, "cd $(OLP_DIR); ./autogen.sh --prefix=", &
-         "$(dir $(abspath $(lastword $(MAKEFILE_LIST))))"
-    write (unit, "(A)")
-    write (unit, "(2A)")  char (libname), ":"
+    write (unit, "(3A)")  "$(OLP_DIR)/config.log: "
     write (unit, "(4A)")  TAB, char (object%gosam_dir // "/bin/gosam.py "), &
                              "--olp $(OLP_FILE) --destination=$(OLP_DIR)", &
                              " -f -z"
+    write (unit, "(3A)")  TAB, "cd $(OLP_DIR); ./autogen.sh --prefix=", &
+         "$(dir $(abspath $(lastword $(MAKEFILE_LIST))))"
   end subroutine gosam_driver_write_makefile
   subroutine gosam_driver_set_alpha_s (driver, alpha_s)
-     class(gosam_driver_t), intent(inout) :: driver
+     class(gosam_driver_t), intent(in) :: driver
      real(default), intent(in) :: alpha_s
      integer :: ierr
      call driver%blha_olp_set_parameter &
@@ -399,7 +383,7 @@ contains
   subroutine gosam_driver_set_GF (driver, GF)
     class(gosam_driver_t), intent(inout) :: driver
     real(default), intent(in) :: GF
-    integer :: ierr 
+    integer :: ierr
     call driver%blha_olp_set_parameter &
        (c_char_'GF'//c_null_char, &
         dble(GF), 0._double, ierr)
@@ -409,7 +393,7 @@ contains
   subroutine gosam_driver_set_weinberg_angle (driver, sw2)
     class(gosam_driver_t), intent(inout) :: driver
     real(default), intent(in) :: sw2
-    integer :: ierr 
+    integer :: ierr
     call driver%blha_olp_set_parameter &
        (c_char_'sw2'//c_null_char, &
         dble(sw2), 0._double, ierr)
@@ -425,27 +409,31 @@ contains
     class(prc_gosam_t), intent(inout) :: object
     type(os_data_t), intent(in) :: os_data
     type(string_t), intent(in) :: libname
-    logical :: lib_found
     select type (writer => object%def%writer)
     type is (gosam_writer_t)
        call writer%write_config ()
     end select
-    call object%search_for_existing_library (os_data, lib_found)
-    call object%create_olp_library (libname, lib_found)
-    call object%load_driver (os_data, .not. lib_found)
+    call object%create_olp_library (libname)
+    call object%load_driver (os_data)
   end subroutine prc_gosam_prepare_library
 
-  subroutine prc_gosam_search_for_existing_library (object, os_data, found)
-    class(prc_gosam_t), intent(inout) :: object
+  subroutine prc_gosam_create_and_load_extra_libraries ( &
+         core, flv_states, var_list, os_data, libname, model, i_core)
+    class(prc_gosam_t), intent(inout) :: core
+    integer, intent(in), dimension(:,:), allocatable :: flv_states
+    type(var_list_t), intent(in) :: var_list
     type(os_data_t), intent(in) :: os_data
-    logical, intent(out) :: found
-    select type (driver => object%driver)
-    type is (gosam_driver_t)
-       call driver%loop_archive%search ([driver%olp_file, &
-            var_str ('golem.in'), driver%olp_dir // &
-            '/.libs/libgolem_olp.' // os_data%shrlib_ext], found)
-    end select
-  end subroutine prc_gosam_search_for_existing_library
+    type(string_t), intent(in) :: libname
+    type(model_data_t), intent(in), target :: model
+    integer, intent(in) :: i_core
+    core%sqme_tree_pos = 4
+    call core%prepare_library (os_data, libname)
+    call core%start ()
+    call core%read_contract_file (flv_states)
+    call core%set_particle_properties (model)
+    call core%set_electroweak_parameters (model)
+    call core%print_parameter_file (i_core)
+  end subroutine prc_gosam_create_and_load_extra_libraries
 
   subroutine prc_gosam_write_makefile (object, unit, libname)
     class(prc_gosam_t), intent(in) :: object
@@ -462,60 +450,47 @@ contains
     type(string_t), intent(in) :: libname
     select type (driver => object%driver)
     type is (gosam_driver_t)
-       call os_system_call ("make " // &
-            driver%loop_archive%os_data%makeflags // " -f " // &
-            char (libname // "_gosam.makefile"))
+       call os_system_call ("make -f " // &
+          libname // "_gosam.makefile")
     end select
   end subroutine prc_gosam_execute_makefile
 
-  subroutine prc_gosam_create_olp_library (object, libname, lib_exists)
+  subroutine prc_gosam_create_olp_library (object, libname)
     class(prc_gosam_t), intent(inout) :: object
     type(string_t), intent(in) :: libname
-    logical, intent(in) :: lib_exists
     integer :: unit
     select type (driver => object%driver)
     type is (gosam_driver_t)
-       if (lib_exists) then
-          call driver%loop_archive%restore (driver%olp_file, driver%contract_file, &
-                                            driver%olp_dir)
-       else
-          unit = free_unit ()
-          open (unit, file = char (libname // "_gosam.makefile"), &
-               status = "replace", action= "write")
-          call object%write_makefile (unit, libname)
-          close (unit)
-          call object%execute_makefile (libname)
-       end if
+       unit = free_unit ()
+       open (unit, file = char (libname // "_gosam.makefile"), &
+            status = "replace", action= "write")
+       call object%write_makefile (unit, libname)
+       close (unit)
+       call object%execute_makefile (libname)
     end select
   end subroutine prc_gosam_create_olp_library
 
-  subroutine prc_gosam_load_driver (object, os_data, store)
+  subroutine prc_gosam_load_driver (object, os_data)
     class(prc_gosam_t), intent(inout) :: object
     type(os_data_t), intent(in) :: os_data
-    logical, intent(in) :: store
     logical :: dl_success
-    type(string_t) :: libname
 
     select type (driver => object%driver)
     type is (gosam_driver_t)
        call driver%load (os_data, dl_success)
        if (.not. dl_success) &
-          call msg_fatal ("Error: GoSam Libraries could not be loaded")
-       if (store .and. dl_success) then
-          libname = driver%olp_dir // '/.libs/libgolem_olp.' // &
-             os_data%shrlib_ext
-          call driver%loop_archive%record (driver%olp_file, driver%contract_file, &
-               var_str ('golem.in'), libname)
-       end if
+          call msg_fatal ("GoSam Libraries could not be loaded")
     end select
   end subroutine prc_gosam_load_driver
 
   subroutine prc_gosam_start (object)
     class(prc_gosam_t), intent(inout) :: object
     integer :: ierr
+    if (object%includes_polarization())  &
+         call msg_fatal ('GoSam does not support polarized beams!')
     select type (driver => object%driver)
     type is (gosam_driver_t)
-       call driver%blha_olp_start (char (driver%contract_file), ierr)
+       call driver%blha_olp_start (string_f2c (driver%contract_file), ierr)
     end select
   end subroutine prc_gosam_start
 
@@ -525,24 +500,35 @@ contains
     call msg_message (unit = unit, string = "GOSAM")
   end subroutine prc_gosam_write
 
+  subroutine prc_gosam_write_name (object, unit)
+    class(prc_gosam_t), intent(in) :: object
+    integer, intent(in), optional :: unit
+    integer :: u
+    u = given_output_unit (unit)
+    write (u,"(1x,A)") "Core: GoSam"
+  end subroutine prc_gosam_write_name
+
   subroutine prc_gosam_init_driver (object, os_data)
     class(prc_gosam_t), intent(inout) :: object
     type(os_data_t), intent(in) :: os_data
     type(string_t) :: olp_file, olc_file, olp_dir
+    type(string_t) :: suffix
 
     select type (def => object%def)
     type is (gosam_def_t)
-      olp_file = def%basename // def%suffix // '.olp'
-      olc_file = def%basename // def%suffix // '.olc'
-      olp_dir = def%basename // def%suffix // '_olp_modules'
+       suffix = def%suffix
+       olp_file = def%basename // suffix // '.olp'
+       olc_file = def%basename // suffix // '.olc'
+       olp_dir = def%basename // suffix // '_olp_modules'
     class default
-      call msg_bug ("prc_gosam_init_driver: core_def should be of gosam-type")
+       call msg_bug ("prc_gosam_init_driver: core_def should be of gosam-type")
     end select
 
     select type(driver => object%driver)
     type is (gosam_driver_t)
-      call driver%init_gosam (os_data, olp_file, olc_file, olp_dir, &
-           var_str ("libgolem_olp"))
+       driver%nlo_suffix = suffix
+       call driver%init_gosam (os_data, olp_file, olc_file, olp_dir, &
+            var_str ("libgolem_olp"))
     end select
   end subroutine prc_gosam_init_driver
 
@@ -551,84 +537,13 @@ contains
     prc_gosam%initialized = .true.
   end subroutine prc_gosam_set_initialized
 
-  subroutine prc_gosam_compute_sqme_born &
-         (object, i_born, p, mu, sqme, bad_point)
-    class(prc_gosam_t), intent(inout) :: object
-    integer, intent(in) :: i_born
-    type(vector4_t), dimension(:), intent(in) :: p
-    real(default), intent(in) :: mu
-    real(default), intent(out) :: sqme
-    logical, intent(out) :: bad_point
-    real(double), dimension(5*object%n_particles) :: mom
-    real(default) :: acc_born
-    real(double), dimension(OLP_RESULTS_LIMIT) :: r
-
-    real(double) :: mu_dble
-    real(double) :: acc_dble
-    real(default) :: alpha_s
-
-    mom = object%create_momentum_array (p)
-    mu_dble = dble(mu)
-    alpha_s = object%qcd%alpha%get (mu)
-
-    select type (driver => object%driver)
-    type is (gosam_driver_t)
-       call driver%set_alpha_s (alpha_s)
-       if (allocated (object%i_born)) then
-          call driver%blha_olp_eval2 (object%i_born(i_born), mom, mu_dble, r, acc_dble)
-          sqme = r(4)
-       else
-          sqme = 0._default
-          acc_dble = 0._default
-       end if
-    end select
-    acc_born = acc_dble
-    bad_point = acc_born > object%maximum_accuracy
-  end subroutine prc_gosam_compute_sqme_born
-
-  subroutine prc_gosam_compute_sqme_real &
-         (object, i_flv, p, ren_scale, sqme, bad_point)
-    class(prc_gosam_t), intent(inout) :: object
-    integer, intent(in) :: i_flv
-    type(vector4_t), intent(in), dimension(:) :: p
-    real(default), intent(in) :: ren_scale
-    real(default), intent(out) :: sqme
-    logical, intent(out) :: bad_point
-    real(double), dimension(5*object%n_particles) :: mom
-    real(double), dimension(OLP_RESULTS_LIMIT) :: r
-    real(double) :: mu_dble
-    real(default) :: mu
-    real(double) :: acc_dble
-    real(default) :: acc
-    real(default) :: alpha_s
-
-    mom = object%create_momentum_array (p)
-    if (vanishes (ren_scale)) then
-       mu = sqrt (two * p(1)* p(2))
-    else
-      mu = ren_scale
-    end if
-    mu_dble = dble(mu)
-    alpha_s = object%qcd%alpha%get (mu)
-    select type (driver => object%driver)
-    type is (gosam_driver_t)
-       call driver%set_alpha_s (alpha_s)
-       call driver%blha_olp_eval2 (object%i_real(i_flv), mom, &
-                                    mu_dble, r, acc_dble)
-       sqme = r(4)
-    end select
-    acc = acc_dble
-    if (acc > object%maximum_accuracy) bad_point = .true.
-  end subroutine prc_gosam_compute_sqme_real
-
   subroutine prc_gosam_compute_sqme_sc (object, &
-                i_flv, em, p, ren_scale_in, &
-            me_sc, bad_point)
+       i_flv, em, p, ren_scale, me_sc, bad_point)
     class(prc_gosam_t), intent(inout) :: object
     integer, intent(in) :: i_flv
     integer, intent(in) :: em
     type(vector4_t), intent(in), dimension(:) :: p
-    real(default), intent(in) :: ren_scale_in
+    real(default), intent(in) :: ren_scale
     complex(default), intent(out) :: me_sc
     logical, intent(out) :: bad_point
     real(double), dimension(5*object%n_particles) :: mom
@@ -637,37 +552,35 @@ contains
     integer :: i, igm1, n
     integer :: pos_real, pos_imag
     real(double) :: acc_dble
-    real(default) :: acc, ren_scale
-    real(default) :: alpha_s
+    real(default) :: acc, alpha_s
+    if (object%i_sc(i_flv) > 0) then
+       me_sc = cmplx (zero ,zero, kind=default)
+       mom = object%create_momentum_array (p)
+       if (vanishes (ren_scale)) &
+          call msg_fatal ("prc_gosam_compute_sqme_sc: ren_scale vanishes")
+       alpha_s = object%qcd%alpha%get (ren_scale)
+       ren_scale_dble = dble (ren_scale)
+       select type (driver => object%driver)
+       type is (gosam_driver_t)
+          call driver%set_alpha_s (alpha_s)
+          call driver%blha_olp_eval2 (object%i_sc(i_flv), &
+               mom, ren_scale_dble, r, acc_dble)
+       end select
+       igm1 = em - 1
+       n = size(p)
+       do i = 0, n - 1
+          pos_real = 2 * igm1 + 2 * n * i + 1
+          pos_imag = pos_real + 1
+          me_sc = me_sc + cmplx (r(pos_real), r(pos_imag), default)
+       end do
 
-    me_sc = cmplx (zero ,zero)
-    mom = object%create_momentum_array (p)
-    if (vanishes (ren_scale_in)) then
-      ren_scale = sqrt (two * p(1) * p(2))
+       me_sc = - conjg(me_sc) / CA
+
+       acc = acc_dble
+       if (acc > object%maximum_accuracy) bad_point = .true.
     else
-      ren_scale = ren_scale_in
+       r = 0._double
     end if
-    alpha_s = object%qcd%alpha%get (ren_scale)
-    ren_scale_dble = dble (ren_scale)
-    select type (driver => object%driver)
-    type is (gosam_driver_t)
-       call driver%set_alpha_s (alpha_s)
-       call driver%blha_olp_eval2 (object%i_sc(i_flv), &
-            mom, ren_scale_dble, r, acc_dble)
-    end select
-
-    igm1 = em - 1
-    n = size(p)
-    do i = 0, n - 1
-      pos_real = 2 * igm1 + 2 * n*i + 1
-      pos_imag = pos_real + 1
-      me_sc = me_sc + cmplx (r(pos_real), r(pos_imag), default)
-    end do
-
-    me_sc = - conjg(me_sc) / CA
-
-    acc = acc_dble
-    if (acc > object%maximum_accuracy) bad_point = .true.
   end subroutine prc_gosam_compute_sqme_sc
 
   subroutine prc_gosam_allocate_workspace (object, core_state)
