@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -34,9 +34,9 @@
 ! to the source 'whizard.nw'
 
 module prc_omega
-  
+
   use, intrinsic :: iso_c_binding !NODEP!
-  
+
   use kinds
   use iso_varying_string, string_t => varying_string
   use io_units
@@ -64,6 +64,7 @@ module prc_omega
   public :: omega_driver_t
   public :: omega_make_process_component
   public :: prc_omega_t
+  public :: omega_state_t
 
   type, abstract, extends (prc_core_def_t) :: omega_def_t
    contains
@@ -75,17 +76,17 @@ module prc_omega
      procedure, nopass :: get_features => omega_def_get_features
      procedure :: connect => omega_def_connect
   end type omega_def_t
-  
+
   type, extends (omega_def_t) :: omega_omega_def_t
    contains
      procedure, nopass :: type_string => omega_omega_def_type_string
   end type omega_omega_def_t
-  
+
   type, extends (omega_def_t) :: omega_ovm_def_t
    contains
      procedure, nopass :: type_string => omega_ovm_def_type_string
   end type omega_ovm_def_t
-  
+
   type, extends (prc_writer_f_module_t), abstract :: omega_writer_t
      type(string_t) :: model_name
      type(string_t) :: process_mode
@@ -95,6 +96,7 @@ module prc_omega
      logical :: report_progress = .false.
      logical :: diags = .false.
      logical :: diags_color = .false.
+     logical :: complex_mass_scheme = .false.
      type(string_t) :: extra_options
    contains
      procedure, nopass :: get_module_name => omega_writer_get_module_name
@@ -136,48 +138,48 @@ module prc_omega
 
   type, extends (prc_core_t) :: prc_omega_t
      real(default), dimension(:), allocatable :: par
+     integer :: scheme = 0
      type(helicity_selection_t) :: helicity_selection
      type(qcd_t) :: qcd
    contains
      procedure :: allocate_workspace => prc_omega_allocate_workspace
      procedure :: write => prc_omega_write
+     procedure :: write_name => prc_omega_write_name
      procedure :: set_parameters => prc_omega_set_parameters
      procedure :: init => prc_omega_init
      procedure :: activate_parameters => prc_omega_activate_parameters
-     procedure :: needs_mcset => prc_omega_needs_mcset
-     procedure :: get_n_terms => prc_omega_get_n_terms
      procedure :: is_allowed => prc_omega_is_allowed
      procedure :: compute_hard_kinematics => prc_omega_compute_hard_kinematics
      procedure :: compute_eff_kinematics => prc_omega_compute_eff_kinematics
-     procedure :: recover_kinematics => prc_omega_recover_kinematics
      procedure :: reset_helicity_selection => prc_omega_reset_helicity_selection
      procedure :: compute_amplitude => prc_omega_compute_amplitude
      procedure :: get_alpha_s => prc_omega_get_alpha_s
   end type prc_omega_t
-  
+
   type, extends (prc_core_state_t) :: omega_state_t
      logical :: new_kinematics = .true.
      real(default) :: alpha_qcd = -1
    contains
-     procedure :: write => omega_state_write
-     procedure :: reset_new_kinematics => omega_state_reset_new_kinematics
+    procedure :: write => omega_state_write
+    procedure :: reset_new_kinematics => omega_state_reset_new_kinematics
   end type omega_state_t
-  
+
 
   abstract interface
-     subroutine init_t (par) bind(C)
+     subroutine init_t (par, scheme) bind(C)
        import
        real(c_default_float), dimension(*), intent(in) :: par
+       integer(c_int), intent(in) :: scheme
      end subroutine init_t
   end interface
-  
+
   abstract interface
      subroutine update_alpha_s_t (alpha_s) bind(C)
        import
        real(c_default_float), intent(in) :: alpha_s
      end subroutine update_alpha_s_t
   end interface
-  
+
   abstract interface
      subroutine reset_helicity_selection_t (threshold, cutoff) bind(C)
        import
@@ -200,7 +202,7 @@ module prc_omega
        real(c_default_float), dimension(0:3,*), intent(in) :: p
      end subroutine new_event_t
   end interface
-  
+
   abstract interface
      subroutine get_amplitude_t (flv, hel, col, amp) bind(C)
        import
@@ -211,7 +213,7 @@ module prc_omega
 
 
 contains
-  
+
   function omega_omega_def_type_string () result (string)
     type(string_t) :: string
     string = "omega"
@@ -223,12 +225,13 @@ contains
   end function omega_ovm_def_type_string
 
   subroutine omega_def_init (object, model_name, prt_in, prt_out, &
-       restrictions, openmp_support, report_progress, extra_options, &
-       diags, diags_color)
+       restrictions, cms_scheme, openmp_support, report_progress, &
+       extra_options, diags, diags_color)
     class(omega_def_t), intent(out) :: object
     type(string_t), intent(in) :: model_name
     type(string_t), dimension(:), intent(in) :: prt_in
     type(string_t), dimension(:), intent(in) :: prt_out
+    logical, intent(in), optional :: cms_scheme
     type(string_t), intent(in), optional :: restrictions
     logical, intent(in), optional :: openmp_support
     logical, intent(in), optional :: report_progress
@@ -240,16 +243,16 @@ contains
        select type (writer => object%writer)
        type is (omega_omega_writer_t)
           call writer%init (model_name, prt_in, prt_out, &
-               restrictions, openmp_support, report_progress, &
-               extra_options, diags, diags_color)
+               restrictions, cms_scheme, openmp_support, &
+               report_progress, extra_options, diags, diags_color)
        end select
     type is (omega_ovm_def_t)
        allocate (omega_ovm_writer_t :: object%writer)
        select type (writer => object%writer)
        type is (omega_ovm_writer_t)
           call writer%init (model_name, prt_in, prt_out, &
-               restrictions, openmp_support, report_progress, &
-               extra_options, diags, diags_color)
+               restrictions, cms_scheme, openmp_support, &
+               report_progress, extra_options, diags, diags_color)
        end select
     end select
   end subroutine omega_def_init
@@ -264,25 +267,25 @@ contains
        call writer%write (unit)
     end select
   end subroutine omega_def_write
-  
+
   subroutine omega_def_read (object, unit)
     class(omega_def_t), intent(out) :: object
     integer, intent(in) :: unit
     call msg_bug ("O'Mega process definition: input not supported yet")
   end subroutine omega_def_read
-  
+
   subroutine omega_def_allocate_driver (object, driver, basename)
     class(omega_def_t), intent(in) :: object
     class(prc_core_driver_t), intent(out), allocatable :: driver
     type(string_t), intent(in) :: basename
     allocate (omega_driver_t :: driver)
   end subroutine omega_def_allocate_driver
-  
+
   function omega_def_needs_code () result (flag)
     logical :: flag
     flag = .true.
   end function omega_def_needs_code
-  
+
   subroutine omega_def_get_features (features)
     type(string_t), dimension(:), allocatable, intent(out) :: features
     allocate (features (6))
@@ -357,21 +360,24 @@ contains
     write (unit, "(5x,A,L1)")  "Report progress   = ", object%report_progress
     write (unit, "(5x,A,A)")  "Extra options     = ", &
          '"' // char (object%extra_options) // '"'
-    write (unit, "(5x,A,L1)")  "Write diagrams    = ", object%diags    
+    write (unit, "(5x,A,L1)")  "Write diagrams    = ", object%diags
     write (unit, "(5x,A,L1)")  "Write color diag. = ", object%diags_color
+    write (unit, "(5x,A,L1)")  "Complex Mass S.   = ", &
+         object%complex_mass_scheme
   end subroutine omega_writer_write
 
   subroutine omega_writer_init (writer, model_name, prt_in, prt_out, &
-       restrictions, openmp_support, report_progress, extra_options, &
-       diags, diags_color)
+       restrictions, cms_scheme, openmp_support, report_progress, &
+       extra_options, diags, diags_color)
     class(omega_writer_t), intent(out) :: writer
     type(string_t), intent(in) :: model_name
     type(string_t), dimension(:), intent(in) :: prt_in
     type(string_t), dimension(:), intent(in) :: prt_out
+    logical, intent(in), optional :: cms_scheme
     type(string_t), intent(in), optional :: restrictions
     logical, intent(in), optional :: openmp_support
     logical, intent(in), optional :: report_progress
-    logical, intent(in), optional :: diags, diags_color    
+    logical, intent(in), optional :: diags, diags_color
     type(string_t), intent(in), optional :: extra_options
     integer :: i
     writer%model_name = model_name
@@ -382,6 +388,7 @@ contains
     end if
     if (present (openmp_support))  writer%openmp_support = openmp_support
     if (present (report_progress))  writer%report_progress = report_progress
+    if (present (cms_scheme))  writer%complex_mass_scheme = cms_scheme
     if (present (diags))  writer%diags = diags
     if (present (diags_color))  writer%diags_color = diags_color
     if (present (extra_options)) then
@@ -419,6 +426,7 @@ contains
     type(string_t) :: kmatrix_string
     type(string_t) :: progress_string
     type(string_t) :: diagrams_string
+    type(string_t) :: cms_string
     logical :: escape_hyperref
     escape_hyperref = .false.
     if (present (testflag))  escape_hyperref = testflag
@@ -459,14 +467,19 @@ contains
           diagrams_string = " -diagrams " // char(id) // &
                "_diags -diagrams_LaTeX"
        end if
-    else 
+    else
        if (writer%diags_color) then
           diagrams_string = " -diagrams:c " // char(id) // &
-               "_diags -diagrams_LaTeX"        
+               "_diags -diagrams_LaTeX"
        else
           diagrams_string = ""
        end if
     end if
+    if (writer%complex_mass_scheme) then
+       cms_string = " -model:cms_width"
+    else
+       cms_string = ""
+    end if
     select case (char (writer%model_name))
     case ("SM_rx", "SSC", "NoH_rx", "AltH")
        kmatrix_string = " -target:kmatrix_2_write"
@@ -481,23 +494,24 @@ contains
        write (unit, "(5A)")  "SOURCES += ", char (id), ".hbc"
     end select
     if (writer%diags .or. writer%diags_color) then
-       write (unit, "(5A)")  "TEX_SOURCES += ", char (id), "_diags.tex"    
+       write (unit, "(5A)")  "TEX_SOURCES += ", char (id), "_diags.tex"
        if (os_data%event_analysis_pdf) then
           write (unit, "(5A)")  "TEX_OBJECTS += ", char (id), "_diags.pdf"
        else
           write (unit, "(5A)")  "TEX_OBJECTS += ", char (id), "_diags.ps"
        end if
     end if
-    write (unit, "(5A)")  "OBJECTS += ", char (id), ".lo"    
+    write (unit, "(5A)")  "OBJECTS += ", char (id), ".lo"
     select type (writer)
-    type is (omega_omega_writer_t)       
-       write (unit, "(5A)")  char (id), ".f90:"       
+    type is (omega_omega_writer_t)
+       write (unit, "(5A)")  char (id), ".f90:"
        write (unit, "(99A)")  TAB, char (omega_path), &
             " -o ", char (id), ".f90", &
             " -target:whizard", &
             " -target:parameter_module parameters_", char (writer%model_name), &
             " -target:module opr_", char (id), &
             " -target:md5sum '", writer%md5sum, "'", &
+            char (cms_string), &
             char (openmp_string), &
             char (progress_string), &
             char (kmatrix_string), &
@@ -509,6 +523,7 @@ contains
        write (unit, "(99A)")  TAB, char (omega_path), &
             " -o ", char (id), ".hbc", &
             char (progress_string), &
+            char (cms_string), &
             char (writer%process_mode), char (writer%process_string), &
             char (restrictions_string), char (diagrams_string), &
             char (writer%extra_options)
@@ -518,7 +533,7 @@ contains
             " -target:whizard ", &
             " -target:bytecode_file ", char (id), ".hbc", &
             " -target:wrapper_module opr_", char (id), &
-            " -target:parameter_module_external parameters_", &           
+            " -target:parameter_module_external parameters_", &
             char (writer%model_name), &
             " -target:md5sum '", writer%md5sum, "'", &
             char (openmp_string)
@@ -529,7 +544,7 @@ contains
     write (unit, "(5A)")  TAB, "rm -f ", char (id), ".f90"
     write (unit, "(5A)")  TAB, "rm -f opr_", char (id), ".mod"
     write (unit, "(5A)")  TAB, "rm -f ", char (id), ".lo"
-    write (unit, "(5A)")  "CLEAN_SOURCES += ", char (id), ".f90"    
+    write (unit, "(5A)")  "CLEAN_SOURCES += ", char (id), ".f90"
     select type (writer)
     type is (omega_ovm_writer_t)
        write (unit, "(5A)")  "CLEAN_SOURCES += ", char (id), ".hbc"
@@ -537,26 +552,26 @@ contains
     if (writer%diags .or. writer%diags_color) then
        write (unit, "(5A)")  "CLEAN_SOURCES += ", char (id), "_diags.tex"
     end if
-    write (unit, "(5A)")  "CLEAN_OBJECTS += opr_", char (id), ".mod"       
+    write (unit, "(5A)")  "CLEAN_OBJECTS += opr_", char (id), ".mod"
     write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), ".lo"
-    if (writer%diags .or. writer%diags_color) then    
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.aux"  
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.log"         
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.dvi"                
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.toc"                       
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.out"       
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.[1-9]"       
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.[1-9][0-9]"    
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.[1-9][0-9][0-9]"   
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.t[1-9]"       
+    if (writer%diags .or. writer%diags_color) then
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.aux"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.log"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.dvi"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.toc"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.out"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.[1-9]"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.[1-9][0-9]"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.[1-9][0-9][0-9]"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.t[1-9]"
        write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.t[1-9][0-9]"
        write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.t[1-9][0-9][0-9]"
        write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.mp"
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.log"       
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.dvi"              
-       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.ps"                     
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags-fmf.log"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.dvi"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.ps"
        if (os_data%event_analysis_pdf) &
-            write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.pdf" 
+            write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), "_diags.pdf"
     end if
     write (unit, "(5A)")  char (id), ".lo: ", char (id), ".f90"
     write (unit, "(5A)")  TAB, "$(LTFCOMPILE) $<"
@@ -568,7 +583,7 @@ contains
              write (unit, "(5A)")  char (id), "_diags.ps: ", char (id), "_diags.tex"
           end if
           if (escape_hyperref) then
-             write (unit, "(5A)")  TAB, "-cat ", char (id), "_diags.tex | \" 
+             write (unit, "(5A)")  TAB, "-cat ", char (id), "_diags.tex | \"
              write (unit, "(5A)")  TAB, "   sed -e" // &
                 "'s/\\usepackage\[colorlinks\]{hyperref}.*/%\\usepackage" // &
                 "\[colorlinks\]{hyperref}/' > \"
@@ -581,7 +596,7 @@ contains
           write (unit, "(5A)")  TAB, "MPINPUTS=$(MP_FLAGS) $(MPOST) " // &
                char (id) // "_diags-fmf.mp"
           write (unit, "(5A)")  TAB, "TEXINPUTS=$(TEX_FLAGS) $(LATEX) " // &
-               char (id) // "_diags.tex"  
+               char (id) // "_diags.tex"
           write (unit, "(5A)")  TAB, "$(DVIPS) -o " // char (id) // "_diags.ps " // &
                char (id) // "_diags.dvi"
           if (os_data%event_analysis_pdf) then
@@ -614,7 +629,7 @@ contains
        name = feature
     end select
   end function omega_writer_get_procname
-  
+
   subroutine omega_write_interface (writer, unit, id, feature)
     class(omega_writer_t), intent(in) :: writer
     integer, intent(in) :: unit
@@ -625,10 +640,12 @@ contains
     write (unit, "(2x,9A)")  "interface"
     select case (char (feature))
     case ("init")
-       write (unit, "(5x,9A)")  "subroutine ", char (name), " (par) bind(C)"
+       write (unit, "(5x,9A)")  "subroutine ", char (name), &
+            " (par, scheme) bind(C)"
        write (unit, "(7x,9A)")  "import"
        write (unit, "(7x,9A)")  "real(c_default_float), dimension(*), &
             &intent(in) :: par"
+       write (unit, "(7x,9A)")  "integer(c_int), intent(in) :: scheme"
        write (unit, "(5x,9A)")  "end subroutine ", char (name)
     case ("update_alpha_s")
        write (unit, "(5x,9A)")  "subroutine ", char (name), " (alpha_s) bind(C)"
@@ -647,7 +664,7 @@ contains
             &(flv, hel, col, flag) bind(C)"
        write (unit, "(7x,9A)")  "import"
        write (unit, "(7x,9A)")  "integer(c_int), intent(in) :: flv, hel, col"
-       write (unit, "(7x,9A)")  "logical(c_bool), intent(out) :: flag"    
+       write (unit, "(7x,9A)")  "logical(c_bool), intent(out) :: flag"
        write (unit, "(5x,9A)")  "end subroutine ", char (name)
     case ("new_event")
        write (unit, "(5x,9A)")  "subroutine ", char (name), " (p) bind(C)"
@@ -661,7 +678,7 @@ contains
        write (unit, "(7x,9A)")  "import"
        write (unit, "(7x,9A)")  "integer(c_int), intent(in) :: flv, hel, col"
        write (unit, "(7x,9A)")  "complex(c_default_complex), intent(out) &
-            &:: amp"    
+            &:: amp"
        write (unit, "(5x,9A)")  "end subroutine ", char (name)
     end select
     write (unit, "(2x,9A)")  "end interface"
@@ -676,14 +693,16 @@ contains
     write (unit, *)
     select case (char (feature))
     case ("init")
-       write (unit, "(9A)")  "subroutine ", char (name), " (par) bind(C)"
+       write (unit, "(9A)")  "subroutine ", char (name), &
+            " (par, scheme) bind(C)"
        write (unit, "(2x,9A)")  "use iso_c_binding"
        write (unit, "(2x,9A)")  "use kinds"
        write (unit, "(2x,9A)")  "use opr_", char (id)
        write (unit, "(2x,9A)")  "real(c_default_float), dimension(*), &
             &intent(in) :: par"
-       if (c_default_float == default) then
-          write (unit, "(2x,9A)")  "call ", char (feature), " (par)"
+       write (unit, "(2x,9A)")  "integer(c_int), intent(in) :: scheme"
+       if (c_default_float == default .and. c_int == kind(1)) then
+          write (unit, "(2x,9A)")  "call ", char (feature), " (par, scheme)"
        end if
        write (unit, "(9A)")  "end subroutine ", char (name)
     case ("update_alpha_s")
@@ -718,7 +737,7 @@ contains
        write (unit, "(2x,9A)")  "use kinds"
        write (unit, "(2x,9A)")  "use opr_", char (id)
        write (unit, "(2x,9A)")  "integer(c_int), intent(in) :: flv, hel, col"
-       write (unit, "(2x,9A)")  "logical(c_bool), intent(out) :: flag"    
+       write (unit, "(2x,9A)")  "logical(c_bool), intent(out) :: flag"
        write (unit, "(2x,9A)")  "flag = ", char (feature), &
             " (int (flv), int (hel), int (col))"
        write (unit, "(9A)")  "end subroutine ", char (name)
@@ -741,7 +760,7 @@ contains
        write (unit, "(2x,9A)")  "use opr_", char (id)
        write (unit, "(2x,9A)")  "integer(c_int), intent(in) :: flv, hel, col"
        write (unit, "(2x,9A)")  "complex(c_default_complex), intent(out) &
-            &:: amp"    
+            &:: amp"
        write (unit, "(2x,9A)")  "amp = ", char (feature), &
             " (int (flv), int (hel), int (col))"
        write (unit, "(9A)")  "end subroutine ", char (name)
@@ -755,12 +774,13 @@ contains
 
   subroutine omega_make_process_component (entry, component_index, &
          model_name, prt_in, prt_out, restrictions, openmp_support, &
-         report_progress, extra_options, diags, diags_color)
+         cms_scheme, report_progress, extra_options, diags, diags_color)
     class(process_def_entry_t), intent(inout) :: entry
     integer, intent(in) :: component_index
     type(string_t), intent(in) :: model_name
     type(string_t), dimension(:), intent(in) :: prt_in
     type(string_t), dimension(:), intent(in) :: prt_out
+    logical, intent(in), optional :: cms_scheme
     type(string_t), intent(in), optional :: restrictions
     logical, intent(in), optional :: openmp_support
     logical, intent(in), optional :: report_progress
@@ -771,8 +791,8 @@ contains
     select type (def)
     type is (omega_omega_def_t)
        call def%init (model_name, prt_in, prt_out, &
-            restrictions, openmp_support, report_progress, &
-            extra_options, diags, diags_color)
+            restrictions, cms_scheme, openmp_support, &
+            report_progress, extra_options, diags, diags_color)
     end select
     call entry%process_def_t%import_component (component_index, &
          n_out = size (prt_out), &
@@ -781,7 +801,7 @@ contains
          method = var_str ("omega"), &
          variant = def)
   end subroutine omega_make_process_component
-    
+
   subroutine omega_state_write (object, unit)
     class(omega_state_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -790,7 +810,7 @@ contains
     write (u, "(3x,A,L1)")  "O'Mega state: new kinematics = ", &
          object%new_kinematics
   end subroutine omega_state_write
-  
+
   subroutine omega_state_reset_new_kinematics (object)
     class(omega_state_t), intent(inout) :: object
     object%new_kinematics = .true.
@@ -801,7 +821,7 @@ contains
     class(prc_core_state_t), intent(inout), allocatable :: core_state
     allocate (omega_state_t :: core_state)
   end subroutine prc_omega_allocate_workspace
-  
+
   subroutine prc_omega_write (object, unit)
     class(prc_omega_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -822,7 +842,15 @@ contains
     call object%helicity_selection%write (u)
     call object%qcd%write (u)
   end subroutine prc_omega_write
-  
+
+  subroutine prc_omega_write_name (object, unit)
+    class(prc_omega_t), intent(in) :: object
+    integer, intent(in), optional :: unit
+    integer :: u
+    u = given_output_unit (unit)
+    write (u,"(1x,A)") "Core: O'Mega"
+  end subroutine prc_omega_write_name
+
   subroutine prc_omega_set_parameters (prc_omega, model, &
        helicity_selection, qcd, use_color_factors)
     class(prc_omega_t), intent(inout) :: prc_omega
@@ -833,7 +861,8 @@ contains
     if (present (model)) then
        if (.not. allocated (prc_omega%par)) &
             allocate (prc_omega%par (model%get_n_real ()))
-       call model%real_parameters_to_c_array (prc_omega%par)
+       call model%real_parameters_to_array (prc_omega%par)
+       prc_omega%scheme = model%get_scheme_num ()
     end if
     if (present (helicity_selection)) then
        prc_omega%helicity_selection = helicity_selection
@@ -845,7 +874,7 @@ contains
        prc_omega%use_color_factors = use_color_factors
     end if
   end subroutine prc_omega_set_parameters
-  
+
   subroutine prc_omega_init (object, def, lib, id, i_component)
     class(prc_omega_t), intent(inout) :: object
     class(prc_core_def_t), intent(in), target :: def
@@ -855,14 +884,16 @@ contains
     call object%base_init (def, lib, id, i_component)
     call object%activate_parameters ()
   end subroutine prc_omega_init
-    
+
   subroutine prc_omega_activate_parameters (object)
     class (prc_omega_t), intent(inout) :: object
     if (allocated (object%driver)) then
        if (allocated (object%par)) then
           select type (driver => object%driver)
           type is (omega_driver_t)
-             if (associated (driver%init))  call driver%init (object%par)
+             if (associated (driver%init)) then
+                call driver%init (object%par, object%scheme)
+             end if
           end select
        else
           call msg_bug ("prc_omega_activate: parameter set is not allocated")
@@ -872,18 +903,6 @@ contains
        call msg_bug ("prc_omega_activate: driver is not allocated")
     end if
   end subroutine prc_omega_activate_parameters
-    
-  function prc_omega_needs_mcset (object) result (flag)
-    class(prc_omega_t), intent(in) :: object
-    logical :: flag
-    flag = .true.
-  end function prc_omega_needs_mcset
-  
-  function prc_omega_get_n_terms (object) result (n)
-    class(prc_omega_t), intent(in) :: object
-    integer :: n
-    n = 1
-  end function prc_omega_get_n_terms
 
  function prc_omega_is_allowed (object, i_term, f, h, c) result (flag)
     class(prc_omega_t), intent(in) :: object
@@ -896,7 +915,7 @@ contains
        flag = cflag
     end select
   end function prc_omega_is_allowed
- 
+
   subroutine prc_omega_compute_hard_kinematics &
        (object, p_seed, i_term, int_hard, core_state)
     class(prc_omega_t), intent(in) :: object
@@ -906,7 +925,7 @@ contains
     class(prc_core_state_t), intent(inout), allocatable :: core_state
     call int_hard%set_momenta (p_seed)
   end subroutine prc_omega_compute_hard_kinematics
-  
+
   subroutine prc_omega_compute_eff_kinematics &
        (object, i_term, int_hard, int_eff, core_state)
     class(prc_omega_t), intent(in) :: object
@@ -915,20 +934,7 @@ contains
     type(interaction_t), intent(inout) :: int_eff
     class(prc_core_state_t), intent(inout), allocatable :: core_state
   end subroutine prc_omega_compute_eff_kinematics
-  
-  subroutine prc_omega_recover_kinematics &
-       (object, p_seed, int_hard, int_eff, core_state)
-    class(prc_omega_t), intent(in) :: object
-    type(vector4_t), dimension(:), intent(inout) :: p_seed
-    type(interaction_t), intent(inout) :: int_hard
-    type(interaction_t), intent(inout) :: int_eff
-    class(prc_core_state_t), intent(inout), allocatable :: core_state
-    integer :: n_in
-    n_in = int_eff%get_n_in ()
-    call int_eff%set_momenta (p_seed(1:n_in), outgoing = .false.)
-    p_seed(n_in+1:) = int_eff%get_momenta (outgoing = .true.)
-  end subroutine prc_omega_recover_kinematics
-    
+
   subroutine prc_omega_reset_helicity_selection (object)
     class(prc_omega_t), intent(inout) :: object
     select type (driver => object%driver)
@@ -946,7 +952,7 @@ contains
        end if
     end select
   end subroutine prc_omega_reset_helicity_selection
-  
+
   function prc_omega_compute_amplitude &
        (object, j, p, f, h, c, fac_scale, ren_scale, alpha_qcd_forced, &
        core_state)  result (amp)
@@ -992,7 +998,7 @@ contains
                 end if
              end if
           end if
-          n_tot = object%data%n_in + object%data%n_out
+          n_tot = object%data%get_n_tot ()
           allocate (parray (0:3, n_tot))
           do i = 1, n_tot
              parray(:,i) = vector4_get_components (p(i))
@@ -1008,7 +1014,7 @@ contains
        end if
     end select
   end function prc_omega_compute_amplitude
-    
+
   function prc_omega_get_alpha_s (object, core_state) result (alpha)
     class(prc_omega_t), intent(in) :: object
     class(prc_core_state_t), intent(in), allocatable :: core_state
@@ -1021,6 +1027,6 @@ contains
        end select
     end if
   end function prc_omega_get_alpha_s
-  
+
 
 end module prc_omega
