@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -47,17 +47,19 @@ module prc_user_defined
   use lorentz
   use interactions
   use sm_qcd
+  use variables, only: var_list_t
 
   use model_data
   use prclib_interfaces
   use prc_core_def
   use prc_core
+  use prc_omega, only: omega_state_t
 
   use sf_base
   use sf_pdf_builtin, only: pdf_builtin_t
   use sf_lhapdf, only: lhapdf_t
   use pdg_arrays, only: is_gluon, is_quark
- 
+
   implicit none
   private
 
@@ -91,6 +93,7 @@ module prc_user_defined
     real(default) :: alpha_qcd = -1
   contains
     procedure :: reset_new_kinematics => user_defined_state_reset_new_kinematics
+    procedure :: get_alpha_qcd => user_defined_state_get_alpha_qcd
   end type user_defined_state_t
 
   type, abstract, extends (prc_core_driver_t) :: user_defined_driver_t
@@ -106,31 +109,38 @@ module prc_user_defined
     type(qcd_t) :: qcd
     integer :: n_flv = 1
     real(default), dimension(:), allocatable :: par
+    integer :: scheme = 0
     type(sf_handler_t) :: sf_handler
+    real(default) :: maximum_accuracy = 10000.0
   contains
+    procedure :: get_n_flvs => prc_user_defined_base_get_n_flvs
     procedure :: get_flv_state => prc_user_defined_base_get_flv_state
     procedure :: compute_sqme => prc_user_defined_base_compute_sqme
     procedure :: compute_sqme_virt => prc_user_defined_base_compute_sqme_virt
     procedure :: compute_sqme_cc => prc_user_defined_base_compute_sqme_cc
+    procedure :: compute_alpha_s => prc_user_defined_base_compute_alpha_s
     procedure :: get_alpha_s => prc_user_defined_base_get_alpha_s
-    procedure :: needs_mcset => prc_user_defined_base_needs_mcset
-    procedure :: get_n_terms => prc_user_defined_base_get_n_terms
     procedure :: is_allowed => prc_user_defined_base_is_allowed
     procedure :: get_nflv => prc_user_defined_base_get_nflv
     procedure :: compute_hard_kinematics => prc_user_defined_base_compute_hard_kinematics
     procedure :: compute_eff_kinematics => prc_user_defined_base_compute_eff_kinematics
-    procedure :: recover_kinematics => prc_user_defined_base_recover_kinematics
     procedure :: set_parameters => prc_user_defined_base_set_parameters
     procedure :: update_alpha_s => prc_user_defined_base_update_alpha_s
     procedure :: init_sf_handler => prc_user_defined_base_init_sf_handler
     procedure :: init_sf_handler_dummy => prc_user_defined_base_init_sf_handler_dummy
     procedure :: apply_structure_functions => prc_user_defined_base_apply_structure_functions
     procedure :: get_sf_value => prc_user_defined_base_get_sf_value
+    procedure :: get_helicity_list_base => prc_user_defined_base_get_helicity_list
+    procedure(prc_user_defined_base_includes_polarization), deferred :: &
+      includes_polarization
+    procedure(prc_user_defined_base_create_and_load_extra_libraries), deferred :: &
+      create_and_load_extra_libraries
   end type prc_user_defined_base_t
 
   type, abstract, extends (prc_core_def_t) :: user_defined_def_t
     type(string_t) :: basename
   contains
+    procedure :: set_active_writer => user_defined_def_set_active_writer
     procedure, nopass :: get_features => user_defined_def_get_features
     procedure :: connect => user_defined_def_connect
     procedure :: omega_connect => user_defined_def_connect
@@ -142,9 +152,10 @@ module prc_user_defined
     type(string_t) :: process_mode
     type(string_t) :: process_string
     type(string_t) :: restrictions
+    logical :: active = .true.
   contains
-    procedure :: init => user_defined_writer_init 
-    procedure :: base_init => user_defined_writer_init 
+    procedure :: init => user_defined_writer_init
+    procedure :: base_init => user_defined_writer_init
     procedure, nopass :: get_module_name => prc_user_defined_writer_get_module_name
     procedure :: write_wrapper => prc_user_defined_writer_write_wrapper
     procedure :: write_interface => prc_user_defined_writer_write_interface
@@ -181,18 +192,44 @@ module prc_user_defined
   type, extends (prc_user_defined_base_t) :: prc_user_defined_test_t
   contains
     procedure :: write => prc_user_defined_test_write
+    procedure :: write_name => prc_user_defined_test_write_name
     procedure :: compute_amplitude => prc_user_defined_test_compute_amplitude
     procedure :: allocate_workspace => prc_user_defined_test_allocate_workspace
+    procedure :: includes_polarization => prc_user_defined_test_includes_polarization
+    procedure :: create_and_load_extra_libraries => &
+         prc_user_defined_test_create_and_load_extra_libraries
   end type prc_user_defined_test_t
 
 
+  abstract interface
+    function prc_user_defined_base_includes_polarization (object) result (polarized)
+      import
+      logical :: polarized
+      class(prc_user_defined_base_t), intent(in) :: object
+    end function prc_user_defined_base_includes_polarization
+  end interface
+
+  abstract interface
+    subroutine prc_user_defined_base_create_and_load_extra_libraries ( &
+         core, flv_states, var_list, os_data, libname, model, i_core)
+      import
+      class(prc_user_defined_base_t), intent(inout) :: core
+      integer, intent(in), dimension(:,:), allocatable :: flv_states
+      type(var_list_t), intent(in) :: var_list
+      type(os_data_t), intent(in) :: os_data
+      type(string_t), intent(in) :: libname
+      type(model_data_t), intent(in), target :: model
+      integer, intent(in) :: i_core
+    end subroutine prc_user_defined_base_create_and_load_extra_libraries
+  end interface
+
   abstract interface
      subroutine omega_update_alpha_s (alpha_s) bind(C)
        import
        real(c_default_float), intent(in) :: alpha_s
      end subroutine omega_update_alpha_s
   end interface
-  
+
   abstract interface
      subroutine omega_is_allowed (flv, hel, col, flag) bind(C)
        import
@@ -208,7 +245,7 @@ contains
     class(sf_handler_t), intent(out) :: sf_handler
     type(sf_chain_instance_t), intent(in) :: sf_chain
     integer :: i
-    sf_handler%n_sf = size (sf_chain%sf) 
+    sf_handler%n_sf = size (sf_chain%sf)
     if (sf_handler%n_sf == 0) then
        sf_handler%initial_state_type = LEPTONS
     else
@@ -260,7 +297,7 @@ contains
      integer :: k
      real(default), dimension(:), allocatable :: ff
      integer, parameter :: n_flv_light = 6
-     
+
      call sf_chain%get_matrix_elements (i, ff)
 
      if (is_gluon (flavor)) then
@@ -279,6 +316,19 @@ contains
     object%new_kinematics = .true.
   end subroutine user_defined_state_reset_new_kinematics
 
+  function user_defined_state_get_alpha_qcd (object) result (alpha_qcd)
+    real(default) :: alpha_qcd
+    class(user_defined_state_t), intent(in) :: object
+    alpha_qcd = object%alpha_qcd
+  end function user_defined_state_get_alpha_qcd
+
+  pure function prc_user_defined_base_get_n_flvs (object, i_flv) result (n)
+    integer :: n
+    class(prc_user_defined_base_t), intent(in) :: object
+    integer, intent(in) :: i_flv
+    n = size (object%data%flv_state (:,i_flv))
+  end function prc_user_defined_base_get_n_flvs
+
   function prc_user_defined_base_get_flv_state (object, i_flv) result (flv)
     integer, dimension(:), allocatable :: flv
     class(prc_user_defined_base_t), intent(in) :: object
@@ -287,22 +337,27 @@ contains
     flv = object%data%flv_state (:,i_flv)
   end function prc_user_defined_base_get_flv_state
 
-  function prc_user_defined_base_compute_sqme (object, i_flv, p) result (sqme)
-     real(default) :: sqme
+  subroutine prc_user_defined_base_compute_sqme (object, i_flv, p, &
+         ren_scale, sqme, bad_point)
      class(prc_user_defined_base_t), intent(in) :: object
      integer, intent(in) :: i_flv
      type(vector4_t), dimension(:), intent(in) :: p
+     real(default), intent(in) :: ren_scale
+     real(default), intent(out) :: sqme
+     logical, intent(out) :: bad_point
      sqme = one
-  end function prc_user_defined_base_compute_sqme
+     bad_point = .false.
+  end subroutine prc_user_defined_base_compute_sqme
 
   subroutine prc_user_defined_base_compute_sqme_virt (object, i_flv, &
      p, ren_scale, sqme, bad_point)
-    class(prc_user_defined_base_t), intent(inout) :: object
+    class(prc_user_defined_base_t), intent(in) :: object
     integer, intent(in) :: i_flv
     type(vector4_t), dimension(:), intent(in) :: p
     real(default), intent(in) :: ren_scale
     logical, intent(out) :: bad_point
     real(default), dimension(4), intent(out) :: sqme
+    call msg_debug2 (D_ME_METHODS, "prc_user_defined_base_compute_sqme_virt")
     sqme(1) = 0.001_default
     sqme(2) = 0.001_default
     sqme(3) = 0.001_default
@@ -311,24 +366,38 @@ contains
   end subroutine prc_user_defined_base_compute_sqme_virt
 
   subroutine prc_user_defined_base_compute_sqme_cc (object, i_flv, p, &
-     ren_scale, born_out, born_cc, bad_point) 
+     ren_scale, born_cc, bad_point, born_out)
     class(prc_user_defined_base_t), intent(inout) :: object
     integer, intent(in) :: i_flv
     type(vector4_t), intent(in), dimension(:) :: p
     real(default), intent(in) :: ren_scale
-    real(default), intent(out), optional :: born_out
     real(default), intent(inout), dimension(:,:) :: born_cc
     logical, intent(out) :: bad_point
-
-    born_out = 0.0015_default
-    born_cc = 0._default
-    born_cc(3,3) = -CF*born_out
-    born_cc(4,4) = -CF*born_out
-    born_cc(3,4) = CF*born_out
-    born_cc(4,3) = born_cc(3,4)
-    bad_point = .false.
+    real(default), intent(out), optional :: born_out
+    call msg_debug2 (D_ME_METHODS, "prc_user_defined_base_compute_sqme_cc")
+    if (size (p) == 4) then
+       if (present (born_out)) then
+          born_out = 0.0015_default
+          born_cc = zero
+          born_cc(3,3) = - CF * born_out
+          born_cc(4,4) = - CF * born_out
+          born_cc(3,4) = CF * born_out
+          born_cc(4,3) = born_cc(3,4)
+          bad_point = .false.
+       end if
+    else
+       if (present (born_out)) born_out = zero
+       born_cc = zero
+    end if
   end subroutine prc_user_defined_base_compute_sqme_cc
 
+  subroutine prc_user_defined_base_compute_alpha_s (object, core_state, fac_scale)
+    class(prc_user_defined_base_t), intent(in) :: object
+    class(user_defined_state_t), intent(inout) :: core_state
+    real(default), intent(in) :: fac_scale
+    core_state%alpha_qcd = object%qcd%alpha%get (fac_scale)
+  end subroutine prc_user_defined_base_compute_alpha_s
+
   function prc_user_defined_base_get_alpha_s (object, core_state) result (alpha)
     class(prc_user_defined_base_t), intent(in) :: object
     class(prc_core_state_t), intent(in), allocatable :: core_state
@@ -336,27 +405,17 @@ contains
     if (allocated (core_state)) then
       select type (core_state)
       class is (user_defined_state_t)
-        alpha = core_state%alpha_qcd
+         alpha = core_state%alpha_qcd
+      type is (omega_state_t)
+         alpha = core_state%alpha_qcd
       class default
-        alpha = zero
+         alpha = zero
       end select
     else
-      alpha = zero
+       alpha = zero
     end if
   end function prc_user_defined_base_get_alpha_s
 
-  function prc_user_defined_base_needs_mcset (object) result (flag)
-    class(prc_user_defined_base_t), intent(in) :: object
-    logical :: flag
-    flag = .true.
-  end function prc_user_defined_base_needs_mcset
-
-  function prc_user_defined_base_get_n_terms (object) result (n)
-    class(prc_user_defined_base_t), intent(in) :: object
-    integer :: n
-    n = 1
-  end function prc_user_defined_base_get_n_terms
-
   function prc_user_defined_base_is_allowed (object, i_term, f, h, c) result (flag)
     class(prc_user_defined_base_t), intent(in) :: object
     integer, intent(in) :: i_term, f, h, c
@@ -364,10 +423,10 @@ contains
     logical(c_bool) :: cflag
     select type (driver => object%driver)
     class is (user_defined_driver_t)
-       call driver%is_allowed (f, h, c, cflag) 
+       call driver%is_allowed (f, h, c, cflag)
        flag = cflag
     class default
-       call msg_fatal & 
+       call msg_fatal &
           ("Driver does not fit to user_defined_base_t")
     end select
   end function prc_user_defined_base_is_allowed
@@ -384,7 +443,7 @@ contains
     type(vector4_t), dimension(:), intent(in) :: p_seed
     integer, intent(in) :: i_term
     type(interaction_t), intent(inout) :: int_hard
-    class(prc_core_state_t), intent(inout), allocatable :: core_state 
+    class(prc_core_state_t), intent(inout), allocatable :: core_state
     call int_hard%set_momenta (p_seed)
     if (allocated (core_state)) then
       select type (core_state)
@@ -402,34 +461,20 @@ contains
     class(prc_core_state_t), intent(inout), allocatable :: core_state
   end subroutine prc_user_defined_base_compute_eff_kinematics
 
-  subroutine prc_user_defined_base_recover_kinematics &
-       (object, p_seed, int_hard, int_eff, core_state)
-    class(prc_user_defined_base_t), intent(in) :: object
-    type(vector4_t), dimension(:), intent(inout) :: p_seed
-    type(interaction_t), intent(inout) :: int_hard, int_eff
-    class(prc_core_state_t), intent(inout), allocatable :: core_state
-    integer :: n_in
-    n_in = int_eff%get_n_in ()
-    call int_eff%set_momenta (p_seed(1:n_in), outgoing = .false.)
-    p_seed(n_in+1:) = int_eff%get_momenta (outgoing = .true.)
-  end subroutine prc_user_defined_base_recover_kinematics
-
-  subroutine prc_user_defined_base_set_parameters (object, qcd, &
-    use_color_factors, model)
+  subroutine prc_user_defined_base_set_parameters (object, qcd, model)
     class(prc_user_defined_base_t), intent(inout) :: object
     type(qcd_t), intent(in) :: qcd
-    logical, intent(in) :: use_color_factors
     class(model_data_t), intent(in), target, optional :: model
     object%qcd = qcd
-    object%use_color_factors = use_color_factors
     if (present (model)) then
        if (.not. allocated (object%par)) &
             allocate (object%par (model%get_n_real ()))
-       call model%real_parameters_to_c_array (object%par)
+       call model%real_parameters_to_array (object%par)
+       object%scheme = model%get_scheme_num ()
     end if
   end subroutine prc_user_defined_base_set_parameters
 
-  subroutine prc_user_defined_base_update_alpha_s (object, core_state, fac_scale) 
+  subroutine prc_user_defined_base_update_alpha_s (object, core_state, fac_scale)
     class(prc_user_defined_base_t), intent(in) :: object
     class(prc_core_state_t), intent(inout), allocatable :: core_state
     real(default), intent(in) :: fac_scale
@@ -439,10 +484,12 @@ contains
        select type (driver => object%driver)
        class is (user_defined_driver_t)
           call driver%update_alpha_s (alpha_qcd)
-       end select 
+       end select
        select type (core_state)
        class is (user_defined_state_t)
           core_state%alpha_qcd = alpha_qcd
+       type is (omega_state_t)
+          core_state%alpha_qcd = alpha_qcd
        end select
     end if
   end subroutine prc_user_defined_base_update_alpha_s
@@ -465,7 +512,7 @@ contains
   subroutine prc_user_defined_base_apply_structure_functions (core, sf_chain, flavors)
     class(prc_user_defined_base_t), intent(inout) :: core
     type(sf_chain_instance_t), intent(in) :: sf_chain
-    integer, dimension(2), intent(in) :: flavors 
+    integer, dimension(2), intent(in) :: flavors
     call core%sf_handler%apply_structure_functions (sf_chain, flavors)
   end subroutine prc_user_defined_base_apply_structure_functions
 
@@ -475,6 +522,22 @@ contains
     val = core%sf_handler%val
   end function prc_user_defined_base_get_sf_value
 
+  function prc_user_defined_base_get_helicity_list (object, i) result (i_out)
+    integer :: i_out
+    class(prc_user_defined_base_t), intent(in) :: object
+    integer, intent(in) :: i
+    i_out = i
+  end function prc_user_defined_base_get_helicity_list
+
+  subroutine user_defined_def_set_active_writer (def, active)
+    class(user_defined_def_t), intent(inout) :: def
+    logical, intent(in) :: active
+    select type (writer => def%writer)
+    class is (prc_user_defined_writer_t)
+       writer%active = active
+    end select
+  end subroutine user_defined_def_set_active_writer
+
   subroutine user_defined_def_get_features (features)
     type(string_t), dimension(:), allocatable, intent(out) :: features
     allocate (features (6))
@@ -485,7 +548,7 @@ contains
          var_str ("is_allowed"), &
          var_str ("new_event"), &
          var_str ("get_amplitude")]
-  end subroutine user_defined_def_get_features 
+  end subroutine user_defined_def_get_features
 
   subroutine user_defined_def_connect (def, lib_driver, i, proc_driver)
     class(user_defined_def_t), intent(in) :: def
@@ -495,7 +558,7 @@ contains
     integer :: pid, fid
     type(c_funptr) :: fptr
     select type (proc_driver)
-    class is (user_defined_driver_t)       
+    class is (user_defined_driver_t)
        pid = i
        fid = 2
        call lib_driver%get_fptr (pid, fid, fptr)
@@ -511,7 +574,7 @@ contains
     flag = .true.
   end function user_def_needs_code
 
-  subroutine user_defined_writer_init &
+  pure subroutine user_defined_writer_init &
        (writer, model_name, prt_in, prt_out, restrictions)
     class(prc_user_defined_writer_t), intent(inout) :: writer
     type(string_t), intent(in) :: model_name
@@ -529,7 +592,7 @@ contains
        case(2); writer%process_mode = " -scatter"
     end select
     associate (s => writer%process_string)
-      s = " '" 
+      s = " '"
       do i = 1, size (prt_in)
          if (i > 1) s = s // " "
          s = s // prt_in(i)
@@ -551,20 +614,22 @@ contains
   subroutine prc_user_defined_writer_write_wrapper (writer, unit, id, feature)
     class(prc_user_defined_writer_t), intent(in) :: writer
     integer, intent(in) :: unit
-    type(string_t), intent(in) :: id, feature    
+    type(string_t), intent(in) :: id, feature
     type(string_t) :: name
     name = writer%get_c_procname (id, feature)
     write (unit, *)
     select case (char (feature))
     case ("init")
-       write (unit, "(9A)")  "subroutine ", char (name), " (par) bind(C)"
+       write (unit, "(9A)")  "subroutine ", char (name), &
+            " (par, scheme) bind(C)"
        write (unit, "(2x,9A)")  "use iso_c_binding"
        write (unit, "(2x,9A)")  "use kinds"
        write (unit, "(2x,9A)")  "use opr_", char (id)
        write (unit, "(2x,9A)")  "real(c_default_float), dimension(*), &
             &intent(in) :: par"
-       if (c_default_float == default) then
-          write (unit, "(2x,9A)")  "call ", char (feature), " (par)"
+       write (unit, "(2x,9A)")  "integer(c_int), intent(in) :: scheme"
+       if (c_default_float == default .and. c_int == kind (1)) then
+          write (unit, "(2x,9A)")  "call ", char (feature), " (par, scheme)"
        end if
        write (unit, "(9A)")  "end subroutine ", char (name)
     case ("update_alpha_s")
@@ -599,7 +664,7 @@ contains
        write (unit, "(2x,9A)")  "use kinds"
        write (unit, "(2x,9A)")  "use opr_", char (id)
        write (unit, "(2x,9A)")  "integer(c_int), intent(in) :: flv, hel, col"
-       write (unit, "(2x,9A)")  "logical(c_bool), intent(out) :: flag"    
+       write (unit, "(2x,9A)")  "logical(c_bool), intent(out) :: flag"
        write (unit, "(2x,9A)")  "flag = ", char (feature), &
             " (int (flv), int (hel), int (col))"
        write (unit, "(9A)")  "end subroutine ", char (name)
@@ -622,7 +687,7 @@ contains
        write (unit, "(2x,9A)")  "use opr_", char (id)
        write (unit, "(2x,9A)")  "integer(c_int), intent(in) :: flv, hel, col"
        write (unit, "(2x,9A)")  "complex(c_default_complex), intent(out) &
-            &:: amp"    
+            &:: amp"
        write (unit, "(2x,9A)")  "amp = ", char (feature), &
             " (int (flv), int (hel), int (col))"
        write (unit, "(9A)")  "end subroutine ", char (name)
@@ -640,10 +705,12 @@ contains
     write (unit, "(2x,9A)")  "interface"
     select case (char (feature))
     case ("init")
-       write (unit, "(5x,9A)")  "subroutine ", char (name), " (par) bind(C)"
+       write (unit, "(5x,9A)")  "subroutine ", char (name), &
+            " (par, scheme) bind(C)"
        write (unit, "(7x,9A)")  "import"
        write (unit, "(7x,9A)")  "real(c_default_float), dimension(*), &
             &intent(in) :: par"
+       write (unit, "(7x,9A)")  "integer(c_int), intent(in) :: scheme"
        write (unit, "(5x,9A)")  "end subroutine ", char (name)
     case ("update_alpha_s")
        write (unit, "(5x,9A)")  "subroutine ", char (name), " (alpha_s) bind(C)"
@@ -662,7 +729,7 @@ contains
             &(flv, hel, col, flag) bind(C)"
        write (unit, "(7x,9A)")  "import"
        write (unit, "(7x,9A)")  "integer(c_int), intent(in) :: flv, hel, col"
-       write (unit, "(7x,9A)")  "logical(c_bool), intent(out) :: flag"    
+       write (unit, "(7x,9A)")  "logical(c_bool), intent(out) :: flag"
        write (unit, "(5x,9A)")  "end subroutine ", char (name)
     case ("new_event")
        write (unit, "(5x,9A)")  "subroutine ", char (name), " (p) bind(C)"
@@ -676,7 +743,7 @@ contains
        write (unit, "(7x,9A)")  "import"
        write (unit, "(7x,9A)")  "integer(c_int), intent(in) :: flv, hel, col"
        write (unit, "(7x,9A)")  "complex(c_default_complex), intent(out) &
-            &:: amp"    
+            &:: amp"
        write (unit, "(5x,9A)")  "end subroutine ", char (name)
     end select
     write (unit, "(2x,9A)")  "end interface"
@@ -717,8 +784,8 @@ contains
     write (unit, "(5A)")  TAB, "rm -f ", char (id), ".f90"
     write (unit, "(5A)")  TAB, "rm -f opr_", char (id), ".mod"
     write (unit, "(5A)")  TAB, "rm -f ", char (id), ".lo"
-    write (unit, "(5A)")  "CLEAN_SOURCES += ", char (id), ".f90"    
-    write (unit, "(5A)")  "CLEAN_OBJECTS += opr_", char (id), ".mod"       
+    write (unit, "(5A)")  "CLEAN_SOURCES += ", char (id), ".f90"
+    write (unit, "(5A)")  "CLEAN_OBJECTS += opr_", char (id), ".mod"
     write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), ".lo"
     write (unit, "(5A)")  char (id), ".lo: ", char (id), ".f90"
     write (unit, "(5A)")  TAB, "$(LTFCOMPILE) $<"
@@ -800,6 +867,14 @@ contains
     call msg_message ("Test user-defined matrix elements")
   end subroutine prc_user_defined_test_write
 
+  subroutine prc_user_defined_test_write_name (object, unit)
+    class(prc_user_defined_test_t), intent(in) :: object
+    integer, intent(in), optional :: unit
+    integer :: u
+    u = given_output_unit (unit)
+    write (u,"(1x,A)") "Core: user defined test"
+  end subroutine prc_user_defined_test_write_name
+
   function prc_user_defined_test_compute_amplitude &
        (object, j, p, f, h, c, fac_scale, ren_scale, alpha_qcd_forced, &
        core_state)  result (amp)
@@ -824,5 +899,22 @@ contains
     allocate (user_defined_test_state_t :: core_state)
   end subroutine prc_user_defined_test_allocate_workspace
 
+  function prc_user_defined_test_includes_polarization (object) result (polarized)
+    logical :: polarized
+    class(prc_user_defined_test_t), intent(in) :: object
+    polarized = .false.
+  end function prc_user_defined_test_includes_polarization
+
+  subroutine prc_user_defined_test_create_and_load_extra_libraries ( &
+         core, flv_states, var_list, os_data, libname, model, i_core)
+    class(prc_user_defined_test_t), intent(inout) :: core
+    integer, intent(in), dimension(:,:), allocatable :: flv_states
+    type(var_list_t), intent(in) :: var_list
+    type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in) :: libname
+    type(model_data_t), intent(in), target :: model
+    integer, intent(in) :: i_core
+  end subroutine prc_user_defined_test_create_and_load_extra_libraries
+
 
 end module prc_user_defined
