@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -41,6 +41,8 @@ module auto_components
   use diagnostics
   use model_data
   use pdg_arrays
+  use physics_defs, only: PHOTON, GLUON, Z_BOSON, W_BOSON
+  use numeric_utils, only: extend_integer_array
 
   implicit none
   private
@@ -48,11 +50,13 @@ module auto_components
   public :: split_constraints_t
   public :: constrain_n_tot
   public :: constrain_n_loop
+  public :: constrain_splittings
   public :: constrain_insert
   public :: constrain_require
   public :: constrain_radiation
   public :: constrain_mass_sum
   public :: constrain_in_state
+  public :: constrain_couplings
   public :: ps_table_t
   public :: ds_table_t
   public :: fs_table_t
@@ -64,16 +68,16 @@ module auto_components
 
 
   type, abstract :: split_constraint_t
-   contains
-     procedure :: check_before_split  => split_constraint_check_before_split
-     procedure :: check_before_insert => split_constraint_check_before_insert
-     procedure :: check_before_record => split_constraint_check_before_record
+  contains
+    procedure :: check_before_split  => split_constraint_check_before_split
+    procedure :: check_before_insert => split_constraint_check_before_insert
+    procedure :: check_before_record => split_constraint_check_before_record
   end type split_constraint_t
 
   type :: split_constraint_wrap_t
      class(split_constraint_t), allocatable :: c
   end type split_constraint_wrap_t
-  
+
   type :: split_constraints_t
      class(split_constraint_wrap_t), dimension(:), allocatable :: cc
    contains
@@ -83,7 +87,7 @@ module auto_components
      procedure :: check_before_insert => split_constraints_check_before_insert
      procedure :: check_before_record => split_constraints_check_before_record
   end type split_constraints_t
-  
+
   type, extends (split_constraint_t) :: constraint_n_tot
      private
      integer :: n_max = 0
@@ -91,35 +95,42 @@ module auto_components
      procedure :: check_before_split => constraint_n_tot_check_before_split
      procedure :: check_before_record => constraint_n_tot_check_before_record
   end type constraint_n_tot
-  
+
   type, extends (split_constraint_t) :: constraint_n_loop
      private
      integer :: n_loop_max = 0
    contains
      procedure :: check_before_record => constraint_n_loop_check_before_record
   end type constraint_n_loop
-  
+
+  type, extends (split_constraint_t) :: constraint_splittings
+     private
+     type(pdg_list_t) :: pl_match, pl_excluded_gauge_splittings
+   contains
+     procedure :: check_before_insert => constraint_splittings_check_before_insert
+  end type constraint_splittings
+
   type, extends (split_constraint_t) :: constraint_insert
      private
      type(pdg_list_t) :: pl_match
    contains
      procedure :: check_before_insert => constraint_insert_check_before_insert
   end type constraint_insert
-  
+
   type, extends (split_constraint_t) :: constraint_require
      private
      type(pdg_list_t) :: pl
    contains
      procedure :: check_before_record => constraint_require_check_before_record
   end type constraint_require
-  
+
   type, extends (split_constraint_t) :: constraint_radiation
      private
    contains
      procedure :: check_before_insert => &
           constraint_radiation_check_before_insert
   end type constraint_radiation
-  
+
   type, extends (split_constraint_t) :: constraint_mass_sum
      private
      real(default) :: mass_limit = 0
@@ -128,7 +139,7 @@ module auto_components
    contains
      procedure :: check_before_record => constraint_mass_sum_check_before_record
   end type constraint_mass_sum
-  
+
   type, extends (split_constraint_t) :: constraint_in_state
      private
      type(pdg_list_t) :: pl
@@ -136,6 +147,16 @@ module auto_components
      procedure :: check_before_record => constraint_in_state_check_before_record
   end type constraint_in_state
 
+  type, extends (split_constraint_t) :: constraint_coupling_t
+    private
+    logical :: qed = .false.
+    logical :: qcd = .true.
+    logical :: ew = .false.
+    integer :: n_nlo_correction_types
+  contains
+    procedure :: check_before_insert => constraint_coupling_check_before_insert
+  end type constraint_coupling_t
+
   type, extends (pdg_list_t) :: ps_entry_t
      integer :: n_loop = 0
      integer :: n_rad = 0
@@ -166,7 +187,7 @@ module auto_components
      procedure :: get_emitters => ps_table_get_emitters
      procedure :: get_pdg_out => ps_table_get_pdg_out
   end type ps_table_t
-     
+
   type, extends (ps_table_t) :: ds_table_t
      private
      integer :: pdg_in = 0
@@ -209,7 +230,7 @@ contains
     logical, intent(out) :: passed
     passed = .true.
   end subroutine split_constraint_check_before_split
-  
+
   subroutine split_constraint_check_before_insert (c, table, pa, pl, passed)
     class(split_constraint_t), intent(in) :: c
     class(ps_table_t), intent(in) :: table
@@ -218,7 +239,7 @@ contains
     logical, intent(out) :: passed
     passed = .true.
   end subroutine split_constraint_check_before_insert
-  
+
   subroutine split_constraint_check_before_record (c, table, pl, n_loop, passed)
     class(split_constraint_t), intent(in) :: c
     class(ps_table_t), intent(in) :: table
@@ -227,20 +248,20 @@ contains
     logical, intent(out) :: passed
     passed = .true.
   end subroutine split_constraint_check_before_record
-  
+
   subroutine split_constraints_init (constraints, n)
     class(split_constraints_t), intent(out) :: constraints
     integer, intent(in) :: n
     allocate (constraints%cc (n))
   end subroutine split_constraints_init
-  
+
   subroutine split_constraints_set (constraints, i, c)
     class(split_constraints_t), intent(inout) :: constraints
     integer, intent(in) :: i
     class(split_constraint_t), intent(in) :: c
     allocate (constraints%cc(i)%c, source = c)
   end subroutine split_constraints_set
-  
+
   subroutine split_constraints_check_before_split &
        (constraints, table, pl, k, passed)
     class(split_constraints_t), intent(in) :: constraints
@@ -255,7 +276,7 @@ contains
        if (.not. passed)  return
     end do
   end subroutine split_constraints_check_before_split
-    
+
   subroutine split_constraints_check_before_insert &
        (constraints, table, pa, pl, passed)
     class(split_constraints_t), intent(in) :: constraints
@@ -270,7 +291,7 @@ contains
        if (.not. passed)  return
     end do
   end subroutine split_constraints_check_before_insert
-    
+
   subroutine split_constraints_check_before_record &
        (constraints, table, pl, n_loop, passed)
     class(split_constraints_t), intent(in) :: constraints
@@ -285,13 +306,13 @@ contains
        if (.not. passed)  return
     end do
   end subroutine split_constraints_check_before_record
-    
+
   function constrain_n_tot (n_max) result (c)
     integer, intent(in) :: n_max
     type(constraint_n_tot) :: c
     c%n_max = n_max
   end function constrain_n_tot
-  
+
   subroutine constraint_n_tot_check_before_split (c, table, pl, k, passed)
     class(constraint_n_tot), intent(in) :: c
     class(ps_table_t), intent(in) :: table
@@ -326,12 +347,47 @@ contains
     passed = n_loop <= c%n_loop_max
   end subroutine constraint_n_loop_check_before_record
 
+  function constrain_splittings (pl_match, pl_excluded_gauge_splittings) result (c)
+    type(pdg_list_t), intent(in) :: pl_match
+    type(pdg_list_t), intent(in) :: pl_excluded_gauge_splittings
+    type(constraint_splittings) :: c
+    c%pl_match = pl_match
+    c%pl_excluded_gauge_splittings = pl_excluded_gauge_splittings
+  end function constrain_splittings
+
+  subroutine constraint_splittings_check_before_insert (c, table, pa, pl, passed)
+    class(constraint_splittings), intent(in) :: c
+    class(ps_table_t), intent(in) :: table
+    type(pdg_array_t), intent(in) :: pa
+    type(pdg_list_t), intent(inout) :: pl
+    logical, intent(out) :: passed
+    logical :: has_massless_vector
+    integer :: i
+    has_massless_vector = .false.
+    do i = 1, pa%get_length ()
+       if (is_massless_vector(pa%get(i))) then
+          has_massless_vector = .true.
+          exit
+       end if
+    end do
+    passed = .false.
+    if (has_massless_vector .and. count (is_fermion(pl%a%get ())) == 2) then
+       do i = 1, c%pl_excluded_gauge_splittings%get_size ()
+          if (pl .match. c%pl_excluded_gauge_splittings%a(i)) return
+       end do
+       call pl%match_replace (c%pl_match, passed)
+       passed = .true.
+    else
+       call pl%match_replace (c%pl_match, passed)
+    end if
+  end subroutine constraint_splittings_check_before_insert
+
   function constrain_insert (pl_match) result (c)
     type(pdg_list_t), intent(in) :: pl_match
     type(constraint_insert) :: c
     c%pl_match = pl_match
   end function constrain_insert
-  
+
   subroutine constraint_insert_check_before_insert (c, table, pa, pl, passed)
     class(constraint_insert), intent(in) :: c
     class(ps_table_t), intent(in) :: table
@@ -346,7 +402,7 @@ contains
     type(constraint_require) :: c
     c%pl = pl
   end function constrain_require
-  
+
   subroutine constraint_require_check_before_record &
        (c, table, pl, n_loop, passed)
     class(constraint_require), intent(in) :: c
@@ -382,7 +438,7 @@ contains
   function constrain_radiation () result (c)
     type(constraint_radiation) :: c
   end function constrain_radiation
-  
+
   subroutine constraint_radiation_check_before_insert (c, table, pa, pl, passed)
     class(constraint_radiation), intent(in) :: c
     class(ps_table_t), intent(in) :: table
@@ -402,7 +458,7 @@ contains
        c%margin = margin
     end if
   end function constrain_mass_sum
-  
+
   subroutine constraint_mass_sum_check_before_record &
        (c, table, pl, n_loop, passed)
     class(constraint_mass_sum), intent(in) :: c
@@ -455,7 +511,59 @@ contains
     end select
     passed = .true.
   end subroutine constraint_in_state_check_before_record
-  
+
+  function constrain_couplings (qcd, qed, n_nlo_correction_types) result (c)
+    type(constraint_coupling_t) :: c
+    logical, intent(in) :: qcd, qed
+    integer, intent(in) :: n_nlo_correction_types
+    c%qcd = qcd; c%qed = qed
+    c%n_nlo_correction_types = n_nlo_correction_types
+  end function constrain_couplings
+
+  subroutine constraint_coupling_check_before_insert (c, table, pa, pl, passed)
+    class(constraint_coupling_t), intent(in) :: c
+    class(ps_table_t), intent(in) :: table
+    type(pdg_array_t), intent(in) :: pa
+    type(pdg_list_t), intent(inout) :: pl
+    logical, intent(out) :: passed
+    type(pdg_list_t) :: pl_vertex
+    type(pdg_array_t) :: pdg_gluon, pdg_photon, pdg_W_Z, pdg_gauge_bosons
+    integer :: i, j
+    pdg_gluon = GLUON; pdg_photon = PHOTON
+    pdg_W_Z = [W_BOSON,-W_BOSON, Z_BOSON]
+    if (c%qcd) pdg_gauge_bosons = pdg_gauge_bosons // pdg_gluon
+    if (c%qed) pdg_gauge_bosons = pdg_gauge_bosons // pdg_photon
+    if (c%ew) pdg_gauge_bosons = pdg_gauge_bosons // pdg_W_Z
+    do j = 1, pa%get_length ()
+       call pl_vertex%init (pl%get_size () + 1)
+       call pl_vertex%set (1, pa%get(j))
+       do i = 1, pl%get_size ()
+          call pl_vertex%set (i + 1, pl%get(i))
+       end do
+       if (is_massless_vector(pa%get(j))) then
+          if (.not. table%model%check_vertex &
+               (pl_vertex%a(1)%get (), pl_vertex%a(2)%get (), pl_vertex%a(3)%get ())) then
+             passed = .false.
+             cycle
+          end if
+       else if (.not. table%model%check_vertex &
+            (- pl_vertex%a(1)%get (), pl_vertex%a(2)%get (), pl_vertex%a(3)%get ())) then
+          passed = .false.
+          cycle
+       end if
+       if (.not. (pl_vertex .match. pdg_gauge_bosons)) then
+          passed = .false.
+          cycle
+       end if
+       if (.not. c%ew .and. (pl_vertex .match. pdg_W_Z)) then
+          passed = .false.
+          cycle
+       end if
+       passed = .true.
+       exit
+    end do
+  end subroutine constraint_coupling_check_before_insert
+
   subroutine ps_table_final (object)
     class(ps_table_t), intent(inout) :: object
     type(ps_entry_t), pointer :: current
@@ -466,7 +574,7 @@ contains
     end do
     nullify (object%last)
   end subroutine ps_table_final
-  
+
   subroutine ps_table_base_write (object, unit, n_in)
     class(ps_table_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -513,7 +621,7 @@ contains
        entry => entry%next
     end do
   end subroutine ps_table_base_write
-          
+
   subroutine ds_table_write (object, unit)
     class(ds_table_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -525,7 +633,7 @@ contains
          char (prt%get_name (object%pdg_in < 0))
     call object%base_write (u)
   end subroutine ds_table_write
-          
+
   subroutine fs_table_write (object, unit)
     class(fs_table_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -534,7 +642,7 @@ contains
     write (u, "(1x,A)")  "Table of final states:"
     call object%base_write (u)
   end subroutine fs_table_write
-          
+
   subroutine if_table_write (object, unit)
     class(if_table_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -548,7 +656,7 @@ contains
        call object%base_write (u, n_in = 2)
      end select
   end subroutine if_table_write
-          
+
   subroutine ps_table_get_particle_string (object, index, prt_in, prt_out)
     class(ps_table_t), intent(in) :: object
     integer, intent(in) :: index
@@ -558,7 +666,7 @@ contains
     type(ps_entry_t), pointer :: entry
     integer, dimension(:), allocatable :: pdg
     integer :: n0
-    integer :: i, j 
+    integer :: i, j
     entry => object%first
     i = 1
     do while (i < index)
@@ -604,7 +712,7 @@ contains
       end do
     end do
   end subroutine ps_table_get_particle_string
-                        
+
   subroutine ps_table_init (table, model, pl, constraints, n_in)
     class(ps_table_t), intent(out) :: table
     class(model_data_t), intent(in), target :: model
@@ -617,11 +725,11 @@ contains
 
     if (present (n_in)) then
        select case (n_in)
-       case (1) 
+       case (1)
           table%proc_type = PROC_DECAY
-       case (2) 
+       case (2)
           table%proc_type = PROC_SCATTER
-       case default 
+       case default
           table%proc_type = PROC_UNDEFINED
        end select
     else
@@ -631,11 +739,11 @@ contains
     do i = 1, size (pl)
        call table%record (pl(i), 0, 0, constraints, passed)
        if (.not. passed) then
-          call msg_fatal ("Registering process components: constraint failed")
+          call msg_fatal ("ps_table: Registering process components failed")
        end if
     end do
   end subroutine ps_table_init
-    
+
   subroutine if_table_init (table, model, pl_in, pl_out, constraints)
     class(if_table_t), intent(out) :: table
     class(model_data_t), intent(in), target :: model
@@ -668,12 +776,12 @@ contains
     n_in = size (pl_in(1)%a)
     call table%init (model, pl, constraints, n_in)
   end subroutine if_table_init
-    
+
   subroutine ps_table_enable_loops (table)
     class(ps_table_t), intent(inout) :: table
     table%loops = .true.
   end subroutine ps_table_enable_loops
-    
+
   subroutine ds_table_make (table, model, pdg_in, constraints)
     class(ds_table_t), intent(out) :: table
     class(model_data_t), intent(in), target :: model
@@ -687,7 +795,7 @@ contains
     call pl_in%set (1, [pdg_in])
     call table%split (pl_in, 0, constraints)
   end subroutine ds_table_make
-    
+
   subroutine fs_table_radiate (table, constraints)
     class(fs_table_t), intent(inout) :: table
     type(split_constraints_t) :: constraints
@@ -707,7 +815,7 @@ contains
     type(split_constraints_t), intent(in) :: constraints
     logical, intent(in), optional :: record
     integer :: n_loop, i
-    logical :: passed
+    logical :: passed, save_pdg_index
     type(vertex_iterator_t) :: vit
     integer, dimension(:), allocatable :: pdg1
     integer, dimension(:), allocatable :: pdg2
@@ -722,11 +830,17 @@ contains
           end do INCR_LOOPS
        end if
     end if
+    select type (table)
+    type is (if_table_t)
+       save_pdg_index = .true.
+    class default
+       save_pdg_index = .false.
+    end select
     do i = 1, pl%get_size ()
        call constraints%check_before_split (table, pl, i, passed)
        if (passed) then
           pdg1 = pl%get (i)
-          call vit%init (table%model, pdg1)
+          call vit%init (table%model, pdg1, save_pdg_index)
           SCAN_VERTICES: do
              call vit%get_next_match (pdg2)
              if (allocated (pdg2)) then
@@ -738,7 +852,7 @@ contains
        end if
     end do
   end subroutine ps_table_split
-    
+
   recursive subroutine ps_table_insert &
        (table, pl, n_rad, i, pdg, constraints, n_in)
     class(ps_table_t), intent(inout) :: table
@@ -756,12 +870,10 @@ contains
        call pl_insert%set (k, pdg(k))
     end do
     call constraints%check_before_insert (table, pl%get (i), pl_insert, passed)
-    if (passed) then
-       call table%split (pl%replace (i, pl_insert, n_in), n_rad + s - 1, &
+    if (passed) call table%split (pl%replace (i, pl_insert, n_in), n_rad + s - 1, &
             constraints, record = .true.)
-    end if
   end subroutine ps_table_insert
-    
+
   recursive subroutine if_table_insert  &
        (table, pl, n_rad, i, pdg, constraints, n_in)
     class(if_table_t), intent(inout) :: table
@@ -801,7 +913,7 @@ contains
     logical, intent(out) :: passed
     call table%record (pl%sort_abs (), n_loop, n_rad, constraints, passed)
   end subroutine ps_table_record_sorted
-  
+
   subroutine if_table_record_sorted &
        (table, pl, n_loop, n_rad, constraints, passed)
     class(if_table_t), intent(inout) :: table
@@ -819,9 +931,15 @@ contains
     type(split_constraints_t), intent(in) :: constraints
     logical, intent(out) :: passed
     type(ps_entry_t), pointer :: current
-    if (.not. pl%is_regular ())  return
+    passed = .false.
+    if (.not. pl%is_regular ()) then
+       call msg_warning ("Record ps_table entry: Irregular pdg-list encountered!")
+       return
+    end if
     call constraints%check_before_record (table, pl, n_loop, passed)
-    if (.not. passed)  return
+    if (.not. passed)  then
+       return
+    end if
     current => table%first
     do while (associated (current))
        if (pl == current) then
@@ -860,7 +978,7 @@ contains
       end if
     end subroutine insert
   end subroutine ps_table_record
-    
+
   function mass_sum (pl, n1, n2, model) result (m)
     type(pdg_list_t), intent(in) :: pl
     integer, intent(in) :: n1, n2
@@ -876,7 +994,7 @@ contains
        m = m + prt%get_mass ()
     end do
   end function mass_sum
-  
+
   function invert_pdg_array (pa, model) result (pa_inv)
     type(pdg_array_t), intent(in) :: pa
     class(model_data_t), intent(in), target :: model
@@ -890,7 +1008,7 @@ contains
        if (prt%has_antiparticle ())  call pa_inv%set (i, -pdg)
     end do
   end function invert_pdg_array
-          
+
   function ps_table_get_length (ps_table) result (n)
     class(ps_table_t), intent(in) :: ps_table
     integer :: n
@@ -903,39 +1021,43 @@ contains
     end do
   end function ps_table_get_length
 
-  function ps_table_get_emitters (table, constraints) result (emitters)
-     integer, dimension(:), allocatable :: emitters
+  subroutine ps_table_get_emitters (table, constraints, emitters)
      class(ps_table_t), intent(in) :: table
      type(split_constraints_t), intent(in) :: constraints
+     integer, dimension(:), allocatable, intent(out) :: emitters
      class(pdg_list_t), pointer :: pl
      integer :: i
      logical :: passed
      type(vertex_iterator_t) :: vit
      integer, dimension(:), allocatable :: pdg1, pdg2
      integer :: n_emitters
-     integer, dimension(20) :: emitters_tmp
-
+     integer, dimension(:), allocatable :: emitters_tmp
+     integer, parameter :: buf0 = 6
      n_emitters = 0
      pl => table%first
+     allocate (emitters_tmp (buf0))
      do i = 1, pl%get_size ()
         call constraints%check_before_split (table, pl, i, passed)
         if (passed) then
            pdg1 = pl%get(i)
-           call vit%init (table%model, pdg1)
-           do 
-               call vit%get_next_match(pdg2)
-               if (allocated (pdg2)) then
-                  emitters_tmp (n_emitters+1) = pdg1(1)
-                  n_emitters = n_emitters + 1
-               else
-                  exit
-               end if
+           call vit%init (table%model, pdg1, .false.)
+           do
+              call vit%get_next_match(pdg2)
+              if (allocated (pdg2)) then
+                 if (n_emitters + 1 > size (emitters_tmp)) &
+                      call extend_integer_array (emitters_tmp, 10)
+                 emitters_tmp (n_emitters + 1) = pdg1(1)
+                 n_emitters = n_emitters + 1
+              else
+                 exit
+              end if
            end do
         end if
      end do
      allocate (emitters (n_emitters))
      emitters = emitters_tmp (1:n_emitters)
-  end function ps_table_get_emitters
+     deallocate (emitters_tmp)
+  end subroutine ps_table_get_emitters
 
   subroutine ps_table_get_pdg_out (ps_table, i, pa_out, n_loop, n_rad)
     class(ps_table_t), intent(in) :: ps_table
@@ -960,6 +1082,6 @@ contains
        entry => entry%next
     end do FIND_ENTRY
   end subroutine ps_table_get_pdg_out
-  
+
 
 end module auto_components
