@@ -31,8 +31,9 @@ subroutine init_external_parameters (par) bind (C)
 
   real(c_default_float), dimension(*), intent(inout) :: par
   real(default) :: m1s, Vtb, wt_inv, alphas, mZ, &
-                   nloop, sh, sf, FF, v1, v2, match, mpole, &
-                   aemi, sw, mW, mb, wtop, sqrts_min, sqrts_max, sqrts_it
+                   nrqcd_order, sh, sf, FF, offshell_strategy, v1, v2, mpole, &
+                   aemi, sw, mW, mb, wtop, sqrts_min, sqrts_max, sqrts_it, &
+                   top_helicity_selection
   logical :: mpole_fixed
   call msg_debug (D_THRESHOLD, "init_external_parameters")
   mZ     = par(1)
@@ -43,20 +44,24 @@ subroutine init_external_parameters (par) bind (C)
   m1s    = par(20)
   Vtb    = par(21)
   wt_inv = par(22)
-  nloop  = par(23)
+  nrqcd_order  = par(23)
   sh     = par(24)
   sf     = par(25)
   FF     = par(26)
-  v1     = par(27)
-  v2     = par(28)
-  sqrts_min = par(29)
-  sqrts_max = par(30)
-  sqrts_it = par(31)
-  mpole_fixed = par(34) > 0.0_default
-  sw     = par(37)
+  offshell_strategy = par(27)
+  top_helicity_selection = par(28)
+  v1     = par(29)
+  v2     = par(30)
+  sqrts_min = par(31)
+  sqrts_max = par(32)
+  sqrts_it = par(33)
+  mpole_fixed = par(36) > 0.0_default
+  sw     = par(39)
   call init_parameters (mpole, wtop, m1s, Vtb, wt_inv, aemi, &
-                        sw, alphas, mZ, mW, mb, sh, sf, nloop, FF, v1, v2, &
-                        sqrts_min, sqrts_max, sqrts_it, mpole_fixed)
-  par(39) = mpole
-  par(40) = wtop
+                        sw, alphas, mZ, mW, mb, sh, sf, nrqcd_order, FF, &
+                        offshell_strategy, v1, v2, &
+                        sqrts_min, sqrts_max, sqrts_it, mpole_fixed, &
+                        top_helicity_selection)
+  par(41) = mpole
+  par(42) = wtop
 end subroutine init_external_parameters
