@@ -1,6 +1,6 @@
 ! $Id: parameters.NMSSM.f90,v 1.1 2005/10/25 09:21:48 reuter Exp $
 !
-! Copyright (C) 1999-2015 by 
+! Copyright (C) 1999-2017 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -264,8 +264,9 @@ module parameters_nmssm
   integer :: i,j,sfm1,sfm2
 
 contains
-  subroutine import_from_whizard (par_array)
+  subroutine import_from_whizard (par_array, scheme)
     real(default), dimension(168), intent(in) :: par_array
+    integer, intent(in) :: scheme
     type :: parameter_set
        real(default) :: gf
        real(default) :: mZ
