@@ -1,6 +1,6 @@
 ! $Id: parameters.NoH_rx.f90,v 1.0 2014/02/21 13:31:48 msekulla Exp $
 !
-! Copyright (C) 1999-2015 by 
+! Copyright (C) 1999-2017 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -62,8 +62,9 @@ module parameters_noh_rx
   public :: import_from_whizard, model_update_alpha_s
 
 contains
-  subroutine import_from_whizard (par_array)
+  subroutine import_from_whizard (par_array, scheme)
     real(default), dimension(64), intent(in) :: par_array
+    integer, intent(in) :: scheme
     type :: parameter_set
        real(default) :: gf
        real(default) :: mZ
