@@ -1,6 +1,6 @@
 ! $Id: parameters.QED.omega.f90,v 1.1 2004/03/11 04:21:17 kilian Exp $
 !
-! Copyright (C) 1999-2015 by 
+! Copyright (C) 1999-2017 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -34,8 +34,9 @@ module parameters_qed
   complex(default), public :: qlep
 
 contains
-  subroutine import_from_whizard (par_array)
+  subroutine import_from_whizard (par_array, scheme)
     real(default), dimension(4), intent(in) :: par_array
+    integer, intent(in) :: scheme
     real(default) :: e, qelep
     type :: parameter_set
        real(default) :: ee
