@@ -79,8 +79,9 @@ module parameters_ssc_altt
   public :: import_from_whizard, model_update_alpha_s
 
 contains
-  subroutine import_from_whizard (par_array)
+  subroutine import_from_whizard (par_array, scheme)
     real(default), dimension(72), intent(in) :: par_array
+    integer, intent(in) :: scheme
     type :: parameter_set
        real(default) :: gf
        real(default) :: mZ
