@@ -1,6 +1,6 @@
 ! $Id: parameters.Zprime.f90,v 1.1 2004/05/04 14:37:17 reuter Exp $
 !
-! Copyright (C) 1999-2015 by 
+! Copyright (C) 1999-2017 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -46,8 +46,9 @@ module parameters_Zprime
   real(default) :: glep_l, glep_r, gneu_l, gup_l, gup_r, gdwn_l, gdwn_r
   public :: import_from_whizard, model_update_alpha_s
 contains
-  subroutine import_from_whizard (par_array)
+  subroutine import_from_whizard (par_array, scheme)
     real(default), dimension(40), intent(in) :: par_array
+    integer, intent(in) :: scheme
     type :: parameter_set
        real(default) :: GF
        real(default) :: mZ
