@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -41,8 +41,7 @@ module prc_openloops
   use io_units
   use iso_varying_string, string_t => varying_string
   use constants
-  use unit_tests, only: vanishes
-  use system_defs, only: TAB
+  use numeric_utils
   use diagnostics
   use system_dependencies
   use physics_defs
@@ -51,6 +50,7 @@ module prc_openloops
   use lorentz
   use interactions
   use sm_qcd
+  use sm_physics, only: top_width_sm_lo, top_width_sm_qcd_nlo_jk
   use model_data
 
   use prclib_interfaces
@@ -64,6 +64,7 @@ module prc_openloops
   private
 
   public :: openloops_def_t
+  public :: openloops_state_t
   public :: prc_openloops_t
 
   real(default), parameter :: openloops_default_bmass = 0._default
@@ -85,6 +86,7 @@ module prc_openloops
   end type openloops_writer_t
 
   type, extends (blha_def_t) :: openloops_def_t
+     integer :: verbosity
   contains
     procedure :: init => openloops_def_init
     procedure, nopass :: type_string => openloops_def_type_string
@@ -93,11 +95,11 @@ module prc_openloops
     procedure :: allocate_driver => openloops_def_allocate_driver
   end type openloops_def_t
 
-  type, extends (blha_driver_t) :: openloops_driver_t 
+  type, extends (blha_driver_t) :: openloops_driver_t
     integer :: n_external = 0
     type(string_t) :: olp_file
-    procedure(ol_evaluate_sc), nopass, pointer :: &
-       evaluate_spin_correlations => null ()
+    procedure(ol_evaluate_scpowheg), nopass, pointer :: &
+         evaluate_spin_correlations_powheg => null ()
   contains
     procedure :: init_dlaccess_to_library => openloops_driver_init_dlaccess_to_library
     procedure :: set_alpha_s => openloops_driver_set_alpha_s
@@ -107,10 +109,22 @@ module prc_openloops
     procedure :: print_alpha_s => openloops_driver_print_alpha_s
     procedure, nopass :: type_name => openloops_driver_type_name
     procedure :: load_sc_procedure => openloops_driver_load_sc_procedure
-  end type openloops_driver_t 
+  end type openloops_driver_t
+
+  type :: openloops_threshold_data_t
+    logical :: nlo = .true.
+    real(default) :: alpha_ew
+    real(default) :: sinthw
+    real(default) :: m_b, m_W
+    real(default) :: vtb
+  contains
+    procedure :: compute_top_width => openloops_threshold_data_compute_top_width
+  end type openloops_threshold_data_t
 
   type, extends (blha_state_t) :: openloops_state_t
+    type(openloops_threshold_data_t), allocatable :: threshold_data
   contains
+    procedure :: init_threshold => openloops_state_init_threshold
     procedure :: write => openloops_state_write
   end type openloops_state_t
 
@@ -119,48 +133,72 @@ module prc_openloops
     procedure :: allocate_workspace => prc_openloops_allocate_workspace
     procedure :: init_driver => prc_openloops_init_driver
     procedure :: write => prc_openloops_write
+    procedure :: write_name => prc_openloops_write_name
     procedure :: prepare_library => prc_openloops_prepare_library
     procedure :: load_driver => prc_openloops_load_driver
     procedure :: start => prc_openloops_start
     procedure :: set_n_external => prc_openloops_set_n_external
     procedure :: reset_parameters => prc_openloops_reset_parameters
     procedure :: set_verbosity => prc_openloops_set_verbosity
-    procedure :: compute_sqme_born => prc_openloops_compute_sqme_born
-    procedure :: compute_sqme_real => prc_openloops_compute_sqme_real
+    procedure :: create_and_load_extra_libraries => &
+         prc_openloops_create_and_load_extra_libraries
     procedure :: compute_sqme_sc => prc_openloops_compute_sqme_sc
   end type prc_openloops_t
 
 
   abstract interface
-     subroutine omega_update_alpha_s (alpha_s) bind(C)
+     subroutine ol_evaluate_scpowheg (id, pp, emitter, res, resmunu) bind(C)
        import
-       real(c_default_float), intent(in) :: alpha_s
-     end subroutine omega_update_alpha_s
-  end interface
-
-  abstract interface
-     subroutine ol_evaluate_sc (id, pp, emitter, polvect, res) bind(C)
-       import
-       integer(kind=c_int), value :: id, emitter
-       real(kind=c_double), intent(in) :: pp(5*N_EXTERNAL), polvect(4)
-       real(kind=c_double), intent(out) :: res(N_EXTERNAL)
-     end subroutine ol_evaluate_sc
+       integer(kind = c_int), value :: id, emitter
+       real(kind = c_double), intent(in) :: pp(5 * N_EXTERNAL)
+       real(kind = c_double), intent(out) :: res(N_EXTERNAL), resmunu(16)
+     end subroutine ol_evaluate_scpowheg
   end interface
 
 
 contains
 
+  function openloops_threshold_data_compute_top_width (data, mtop, alpha_s) result (wtop)
+    real(default) :: wtop
+    class(openloops_threshold_data_t), intent(in) :: data
+    real(default), intent(in) :: mtop, alpha_s
+    if (data%nlo) then
+       wtop = top_width_sm_qcd_nlo_jk (data%alpha_ew, data%sinthw, mtop, &
+              data%m_W, data%m_b, alpha_s)
+    else
+       wtop = top_width_sm_lo (data%alpha_ew, data%sinthw, data%vtb, &
+              mtop, data%m_W, data%m_b)
+    end if
+  end function openloops_threshold_data_compute_top_width
+
+  subroutine openloops_state_init_threshold (object, model)
+    class(openloops_state_t), intent(inout) :: object
+    type(model_data_t), intent(in) :: model
+    if (model%get_name () == "SM_tt_threshold") then
+       allocate (object%threshold_data)
+       associate (data => object%threshold_data)
+          data%nlo = btest (int (model%get_real (var_str ('offshell_strategy'))), 0)
+          data%alpha_ew = one / model%get_real (var_str ('alpha_em_i'))
+          data%sinthw = model%get_real (var_str ('sw'))
+          data%m_b = model%get_real (var_str ('mb'))
+          data%m_W = model%get_real (var_str ('mW'))
+          data%vtb = model%get_real (var_str ('Vtb'))
+       end associate
+    end if
+  end subroutine openloops_state_init_threshold
+
   function openloops_writer_type_name () result (string)
     type(string_t) :: string
     string = "openloops"
   end function openloops_writer_type_name
 
   subroutine openloops_def_init (object, basename, model_name, &
-                                 prt_in, prt_out, nlo_type)
+     prt_in, prt_out, nlo_type, var_list)
     class(openloops_def_t), intent(inout) :: object
     type(string_t), intent(in) :: basename, model_name
     type(string_t), dimension(:), intent(in) :: prt_in, prt_out
-    integer :: nlo_type
+    integer, intent(in) :: nlo_type
+    type(var_list_t), intent(in) :: var_list
     object%basename = basename
     allocate (openloops_writer_t :: object%writer)
     select case (nlo_type)
@@ -172,11 +210,16 @@ contains
        object%suffix = '_LOOP'
     case (NLO_SUBTRACTION)
        object%suffix = '_SUB'
+    case (NLO_MISMATCH)
+       object%suffix = '_MISMATCH'
+    case (NLO_DGLAP)
+       object%suffix = '_DGLAP'
     end select
     select type (writer => object%writer)
     class is (prc_blha_writer_t)
        call writer%init (model_name, prt_in, prt_out)
     end select
+    object%verbosity = var_list%get_ival (var_str ("openloops_verbosity"))
   end subroutine openloops_def_init
 
   function openloops_def_type_string () result (string)
@@ -212,15 +255,19 @@ contains
        call msg_fatal ("Link OpenLoops: library not found")
     end if
     success = .not. dlaccess_has_error (dlaccess)
-  end subroutine openloops_driver_init_dlaccess_to_library 
+  end subroutine openloops_driver_init_dlaccess_to_library
 
   subroutine openloops_driver_set_alpha_s (driver, alpha_s)
-    class(openloops_driver_t), intent(inout) :: driver
+    class(openloops_driver_t), intent(in) :: driver
     real(default), intent(in) :: alpha_s
     integer :: ierr
-    call driver%blha_olp_set_parameter &
-       (c_char_'alphas'//c_null_char, &
-        dble (alpha_s), 0._double, ierr)
+    if (associated (driver%blha_olp_set_parameter)) then
+       call driver%blha_olp_set_parameter &
+            (c_char_'alphas'//c_null_char, &
+             dble (alpha_s), 0._double, ierr)
+    else
+       call msg_fatal ("blha_olp_set_parameter not associated!")
+    end if
     if (ierr == 0) call parameter_error_message (var_str ('alphas'))
   end subroutine openloops_driver_set_alpha_s
 
@@ -230,24 +277,24 @@ contains
     integer :: ierr
     call driver%blha_olp_set_parameter &
        (c_char_'alpha_qed'//c_null_char, &
-        dble (alpha), 0._double, ierr) 
+        dble (alpha), 0._double, ierr)
     if (ierr == 0) call parameter_error_message (var_str ('alpha_qed'))
   end subroutine openloops_driver_set_alpha_qed
 
   subroutine openloops_driver_set_GF (driver, GF)
     class(openloops_driver_t), intent(inout) :: driver
     real(default), intent(in) :: GF
-    integer :: ierr 
+    integer :: ierr
     call driver%blha_olp_set_parameter &
-       (c_char_'GF'//c_null_char, &
+       (c_char_'Gmu'//c_null_char, &
         dble(GF), 0._double, ierr)
-    if (ierr == 0) call parameter_error_message (var_str ('GF'))
+    if (ierr == 0) call parameter_error_message (var_str ('Gmu'))
   end subroutine openloops_driver_set_GF
 
   subroutine openloops_driver_set_weinberg_angle (driver, sw2)
     class(openloops_driver_t), intent(inout) :: driver
     real(default), intent(in) :: sw2
-    integer :: ierr 
+    integer :: ierr
     call driver%blha_olp_set_parameter &
        (c_char_'sw2'//c_null_char, &
         dble(sw2), 0._double, ierr)
@@ -273,14 +320,16 @@ contains
     logical :: init_success
 
     call object%init_dlaccess_to_library (os_data, dlaccess, init_success)
-  
-    c_fptr = dlaccess_get_c_funptr (dlaccess, var_str ("ol_evaluate_sc"))
-    call c_f_procpointer (c_fptr, object%evaluate_spin_correlations)
-    if (dlaccess_has_error (dlaccess)) &
-       call msg_fatal ("Could not load Openloops spin correlations!")
 
-    success = .true.
-  end subroutine openloops_driver_load_sc_procedure 
+    c_fptr = dlaccess_get_c_funptr (dlaccess, var_str ("ol_evaluate_scpowheg"))
+    call c_f_procpointer (c_fptr, object%evaluate_spin_correlations_powheg)
+    if (dlaccess_has_error (dlaccess)) then
+       call msg_fatal ("Could not load Openloops-powheg spin correlations!")
+    else
+       success = .true.
+    end if
+
+  end subroutine openloops_driver_load_sc_procedure
 
   subroutine openloops_def_read (object, unit)
     class(openloops_def_t), intent(out) :: object
@@ -309,11 +358,13 @@ contains
     class(prc_openloops_t), intent(inout) :: object
     type(os_data_t), intent(in) :: os_data
     type(string_t) :: olp_file, olc_file
+    type(string_t) :: suffix
 
     select type (def => object%def)
     type is (openloops_def_t)
-       olp_file = def%basename // def%suffix // '.olp'
-       olc_file = def%basename // def%suffix // '.olc'
+       suffix = def%suffix
+       olp_file = def%basename // suffix // '.olp'
+       olc_file = def%basename // suffix // '.olc'
     class default
        call msg_bug ("prc_openloops_init_driver: core_def should be openloops-type")
     end select
@@ -322,6 +373,7 @@ contains
     type is (openloops_driver_t)
        driver%olp_file = olp_file
        driver%contract_file = olc_file
+       driver%nlo_suffix = suffix
     end select
   end subroutine prc_openloops_init_driver
 
@@ -331,19 +383,26 @@ contains
     call msg_message (unit = unit, string = "OpenLoops")
   end subroutine prc_openloops_write
 
-  subroutine prc_openloops_prepare_library (object, os_data, model, var_list)
+  subroutine prc_openloops_write_name (object, unit)
+    class(prc_openloops_t), intent(in) :: object
+    integer, intent(in), optional :: unit
+    integer :: u
+    u = given_output_unit (unit)
+    write (u,"(1x,A)") "Core: OpenLoops"
+  end subroutine prc_openloops_write_name
+
+  subroutine prc_openloops_prepare_library (object, os_data, model)
     class(prc_openloops_t), intent(inout) :: object
     type(os_data_t), intent(in) :: os_data
     type(model_data_t), intent(in), target :: model
-    type(var_list_t), intent(in) :: var_list
-    integer :: verbosity
     call object%load_driver (os_data)
     call object%reset_parameters ()
     call object%set_particle_properties (model)
-    !!!call object%set_alpha_qed (model)
     call object%set_electroweak_parameters (model)
-    verbosity = var_list%get_ival (var_str ("openloops_verbosity"))
-    call object%set_verbosity (verbosity)
+    select type(def => object%def)
+    type is (openloops_def_t)
+       call object%set_verbosity (def%verbosity)
+    end select
   end subroutine prc_openloops_prepare_library
 
   subroutine prc_openloops_load_driver (object, os_data)
@@ -361,7 +420,7 @@ contains
     class(prc_openloops_t), intent(inout) :: object
     integer :: ierr
     select type (driver => object%driver)
-    type is (openloops_driver_t) 
+    type is (openloops_driver_t)
        call driver%blha_olp_start (char (driver%olp_file)//c_null_char, ierr)
     end select
   end subroutine prc_openloops_start
@@ -378,23 +437,23 @@ contains
     select type (driver => object%driver)
     type is (openloops_driver_t)
        call driver%blha_olp_set_parameter ('mass(5)'//c_null_char, &
-          dble(openloops_default_bmass), 0._double, ierr) 
+          dble(openloops_default_bmass), 0._double, ierr)
        call driver%blha_olp_set_parameter ('mass(6)'//c_null_char, &
-          dble(openloops_default_topmass), 0._double, ierr)        
+          dble(openloops_default_topmass), 0._double, ierr)
        call driver%blha_olp_set_parameter ('width(6)'//c_null_char, &
-          dble(openloops_default_topwidth), 0._double, ierr) 
+          dble(openloops_default_topwidth), 0._double, ierr)
        call driver%blha_olp_set_parameter ('mass(23)'//c_null_char, &
           dble(openloops_default_zmass), 0._double, ierr)
        call driver%blha_olp_set_parameter ('width(23)'//c_null_char, &
-          dble(openloops_default_zwidth), 0._double, ierr) 
+          dble(openloops_default_zwidth), 0._double, ierr)
        call driver%blha_olp_set_parameter ('mass(24)'//c_null_char, &
-          dble(openloops_default_wmass), 0._double, ierr) 
+          dble(openloops_default_wmass), 0._double, ierr)
        call driver%blha_olp_set_parameter ('width(24)'//c_null_char, &
-          dble(openloops_default_wwidth), 0._double, ierr) 
+          dble(openloops_default_wwidth), 0._double, ierr)
        call driver%blha_olp_set_parameter ('mass(25)'//c_null_char, &
-          dble(openloops_default_higgsmass), 0._double, ierr) 
+          dble(openloops_default_higgsmass), 0._double, ierr)
        call driver%blha_olp_set_parameter ('width(25)'//c_null_char, &
-          dble(openloops_default_higgswidth), 0._double, ierr) 
+          dble(openloops_default_higgswidth), 0._double, ierr)
     end select
   end subroutine prc_openloops_reset_parameters
 
@@ -405,122 +464,60 @@ contains
     select type (driver => object%driver)
     type is (openloops_driver_t)
        call driver%blha_olp_set_parameter ('verbose'//c_null_char, &
-          dble(verbose), 0._double, ierr)   
+          dble(verbose), 0._double, ierr)
     end select
   end subroutine prc_openloops_set_verbosity
 
-  subroutine prc_openloops_compute_sqme_born &
-         (object, i_born, p, mu, sqme, bad_point)
-    class(prc_openloops_t), intent(inout) :: object
-    integer, intent(in) :: i_born
-    type(vector4_t), dimension(:), intent(in) :: p
-    real(default), intent(in) :: mu
-    real(default), intent(out) :: sqme
-    logical, intent(out) :: bad_point
-    real(double), dimension(5*object%n_particles) :: mom
-    real(default) :: acc_born 
-    real(double), dimension(blha_result_array_size (object%n_particles, &
-                                                        BLHA_AMP_TREE)) :: r
-    real(double) :: mu_dble
-    real(double) :: acc_dble
-    real(default) :: alpha_s
-    
-    mom = object%create_momentum_array (p) 
-    mu_dble = dble(mu)    
-    alpha_s = object%qcd%alpha%get (mu)
-
-    select type (driver => object%driver)
-    type is (openloops_driver_t)
-       call driver%set_alpha_s (alpha_s)
-       if (allocated (object%i_born)) then
-          call driver%blha_olp_eval2 (object%i_born(i_born), mom, mu_dble, r, acc_dble)
-          sqme = r(1)
-       else
-          sqme = 0._default
-          acc_dble = 0._default
-       end if
-    end select
-    acc_born = acc_dble
-    bad_point = acc_born > object%maximum_accuracy
-  end subroutine prc_openloops_compute_sqme_born
-
-  subroutine prc_openloops_compute_sqme_real &
-         (object, i_flv, p, ren_scale, sqme, bad_point)
-    class(prc_openloops_t), intent(inout) :: object
-    integer, intent(in) :: i_flv
-    type(vector4_t), intent(in), dimension(:) :: p
-    real(default), intent(in) :: ren_scale
-    real(default), intent(out) :: sqme
-    logical, intent(out) :: bad_point
-    real(default) :: mu
-    real(double), dimension(5*object%n_particles) :: mom
-    real(double), dimension(blha_result_array_size (object%n_particles, &
-                                                    BLHA_AMP_TREE)) :: r
-    real(double) :: mu_dble
-    real(double) :: acc_dble
-    real(default) :: acc
-    real(default) :: alpha_s
- 
-    mom = object%create_momentum_array (p)
-    if (vanishes (ren_scale)) then
-       mu = sqrt (two * p(1) * p(2))
-    else
-       mu = ren_scale
-    end if
-    mu_dble = dble (mu)
-
-    alpha_s = object%qcd%alpha%get (mu)
-    select type (driver => object%driver)
-    type is (openloops_driver_t)
-       call driver%set_alpha_s (alpha_s)
-       call driver%blha_olp_eval2 (object%i_real(i_flv), mom, &
-                                    mu_dble, r, acc_dble)
-       sqme = r(1)
-    end select
-    acc = acc_dble
-    if (acc > object%maximum_accuracy) bad_point = .true.
-  end subroutine prc_openloops_compute_sqme_real
+  subroutine prc_openloops_create_and_load_extra_libraries ( &
+         core, flv_states, var_list, os_data, libname, model, i_core)
+    class(prc_openloops_t), intent(inout) :: core
+    integer, intent(in), dimension(:,:), allocatable :: flv_states
+    type(var_list_t), intent(in) :: var_list
+    type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in) :: libname
+    type(model_data_t), intent(in), target :: model
+    integer, intent(in) :: i_core
+    core%sqme_tree_pos = 1
+    call core%set_n_external (core%data%get_n_tot ())
+    call core%prepare_library (os_data, model)
+    call core%start ()
+    call core%read_contract_file (flv_states)
+    call core%print_parameter_file (i_core)
+    call core%reset_helicity_list ()
+  end subroutine prc_openloops_create_and_load_extra_libraries
 
   subroutine prc_openloops_compute_sqme_sc (object, &
-                i_flv, em, p, ren_scale_in, pol_vects, &
-            me_sc, bad_point)
+       i_flv, em, p, ren_scale, sqme_sc, bad_point)
     class(prc_openloops_t), intent(inout) :: object
     integer, intent(in) :: i_flv
     integer, intent(in) :: em
     type(vector4_t), intent(in), dimension(:) :: p
-    real(default), intent(in) :: ren_scale_in
-    type(vector4_t), dimension(:) :: pol_vects
-    complex(default), intent(out) :: me_sc
+    real(default), intent(in) :: ren_scale
+    real(default), intent(out), dimension(0:3, 0:3) :: sqme_sc
     logical, intent(out) :: bad_point
     real(double), dimension(5*N_EXTERNAL) :: mom
-    real(double), dimension(N_EXTERNAL) :: r
-    real(default) :: ren_scale, alpha_s
-    real(double), dimension(4) :: polvect
-    integer :: i
-    
-    mom = object%create_momentum_array (p)
-    me_sc = zero
-    if (vanishes (ren_scale_in)) then
-       ren_scale = sqrt (two * p(1) * p(2))
+    real(double), dimension(N_EXTERNAL) :: res
+    real(double), dimension(16) :: res_munu
+    real(default) :: alpha_s
+    if (object%i_sc(i_flv) > 0) then
+       mom = object%create_momentum_array (p)
+       sqme_sc = zero
+       if (vanishes (ren_scale)) &
+            call msg_fatal ("prc_openloops_compute_sqme_sc: ren_scale vanishes")
+       alpha_s = object%qcd%alpha%get (ren_scale)
+
+       select type (driver => object%driver)
+       type is (openloops_driver_t)
+          call driver%set_alpha_s (alpha_s)
+          call driver%evaluate_spin_correlations_powheg &
+               (object%i_sc(i_flv), mom, em, res, res_munu)
+       end select
+       sqme_sc = reshape (res_munu, (/4,4/))
+       bad_point = .false.
+       if (object%includes_polarization ()) sqme_sc = object%n_hel * sqme_sc
     else
-       ren_scale = ren_scale_in
+       sqme_sc = zero
     end if
-    alpha_s = object%qcd%alpha%get (ren_scale)
-
-    forall(i=1:4) polvect(i) = pol_vects(em)%p(i-1)
-
-    select type (driver => object%driver)
-    type is (openloops_driver_t)
-       call driver%set_alpha_s (alpha_s)
-       call driver%evaluate_spin_correlations (1, mom, em, polvect, r)
-    end select
-    do i = 1, N_EXTERNAL
-       if (i /= em) me_sc = me_sc + r(i)
-    end do
-
-    me_sc = me_sc / CA
-    bad_point = .false.
-
   end subroutine prc_openloops_compute_sqme_sc
 
 
