@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -57,7 +57,7 @@ module xml
      procedure :: get_record => cstream_get_record
      procedure :: revert_record => cstream_revert_record
   end type cstream_t
-  
+
   type :: attribute_t
      type(string_t) :: name
      type(string_t) :: value
@@ -65,30 +65,30 @@ module xml
    contains
      procedure :: write => attribute_write
      procedure :: set_value => attribute_set_value
-     procedure :: get_value => attribute_get_value     
+     procedure :: get_value => attribute_get_value
   end type attribute_t
-  
+
   type :: xml_tag_t
      type(string_t) :: name
      type(attribute_t), dimension(:), allocatable :: attribute
      logical :: has_content = .false.
    contains
      generic :: init => init_no_attributes
-     procedure :: init_no_attributes => tag_init_no_attributes
+     procedure :: init_no_attributes => xml_tag_init_no_attributes
      generic :: init => init_with_attributes
-     procedure :: init_with_attributes => tag_init_with_attributes
-     procedure :: set_attribute => tag_set_attribute
-     procedure :: get_attribute => tag_get_attribute
+     procedure :: init_with_attributes => xml_tag_init_with_attributes
+     procedure :: set_attribute => xml_tag_set_attribute
+     procedure :: get_attribute => xml_tag_get_attribute
      generic :: write => write_without_content
-     procedure :: write_without_content => tag_write
-     procedure :: close => tag_close
+     procedure :: write_without_content => xml_tag_write
+     procedure :: close => xml_tag_close
      generic :: write => write_with_content
-     procedure :: write_with_content => tag_write_with_content
-     procedure :: read => tag_read
-     procedure :: read_attribute => tag_read_attribute
-     procedure :: read_content => tag_read_content
+     procedure :: write_with_content => xml_tag_write_with_content
+     procedure :: read => xml_tag_read
+     procedure :: read_attribute => xml_tag_read_attribute
+     procedure :: read_content => xml_tag_read_content
   end type xml_tag_t
-  
+
 
 contains
 
@@ -97,7 +97,7 @@ contains
     stream%cache_is_empty = .true.
     call stream%stream_t%final ()
   end subroutine cstream_final
-  
+
   subroutine cstream_get_record (cstream, string, iostat)
     class(cstream_t), intent(inout) :: cstream
     type(string_t), intent(out) :: string
@@ -110,7 +110,7 @@ contains
        iostat = 0
     end if
   end subroutine cstream_get_record
-  
+
   subroutine cstream_revert_record (cstream, string)
     class(cstream_t), intent(inout) :: cstream
     type(string_t), intent(in) :: string
@@ -121,7 +121,7 @@ contains
        call msg_bug ("CStream: attempt to revert twice")
     end if
   end subroutine cstream_revert_record
-  
+
   subroutine attribute_write (object, unit)
     class(attribute_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -134,7 +134,7 @@ contains
        write (u, "('?')", advance = "no")
     end if
   end subroutine attribute_write
-    
+
   function xml_attribute (name, value) result (attribute)
     type(string_t), intent(in) :: name
     type(string_t), intent(in), optional :: value
@@ -154,7 +154,7 @@ contains
     attribute%value = value
     attribute%known = .true.
   end subroutine attribute_set_value
-  
+
   function attribute_get_value (attribute) result (value)
     class(attribute_t), intent(in) :: attribute
     type(string_t) :: value
@@ -164,17 +164,17 @@ contains
        value = "?"
     end if
   end function attribute_get_value
-  
-  subroutine tag_init_no_attributes (tag, name, has_content)
+
+  subroutine xml_tag_init_no_attributes (tag, name, has_content)
     class(xml_tag_t), intent(out) :: tag
     type(string_t), intent(in) :: name
     logical, intent(in), optional :: has_content
     tag%name = name
     allocate (tag%attribute (0))
     if (present (has_content))  tag%has_content = has_content
-  end subroutine tag_init_no_attributes
-  
-  subroutine tag_init_with_attributes (tag, name, attribute, has_content)
+  end subroutine xml_tag_init_no_attributes
+
+  subroutine xml_tag_init_with_attributes (tag, name, attribute, has_content)
     class(xml_tag_t), intent(out) :: tag
     type(string_t), intent(in) :: name
     type(attribute_t), dimension(:), intent(in) :: attribute
@@ -183,23 +183,23 @@ contains
     allocate (tag%attribute (size (attribute)))
     tag%attribute = attribute
     if (present (has_content))  tag%has_content = has_content
-  end subroutine tag_init_with_attributes
-  
-  subroutine tag_set_attribute (tag, i, value)
+  end subroutine xml_tag_init_with_attributes
+
+  subroutine xml_tag_set_attribute (tag, i, value)
     class(xml_tag_t), intent(inout) :: tag
     integer, intent(in) :: i
     type(string_t), intent(in) :: value
     call tag%attribute(i)%set_value (value)
-  end subroutine tag_set_attribute
-  
-  function tag_get_attribute (tag, i) result (value)
+  end subroutine xml_tag_set_attribute
+
+  function xml_tag_get_attribute (tag, i) result (value)
     class(xml_tag_t), intent(in) :: tag
     integer, intent(in) :: i
     type(string_t) :: value
     value = tag%attribute(i)%get_value ()
-  end function tag_get_attribute
-  
-  subroutine tag_write (tag, unit)
+  end function xml_tag_get_attribute
+
+  subroutine xml_tag_write (tag, unit)
     class(xml_tag_t), intent(in) :: tag
     integer, intent(in), optional :: unit
     integer :: u, i
@@ -214,17 +214,17 @@ contains
     else
        write (u, "(' />')", advance = "no")
     end if
-  end subroutine tag_write
-  
-  subroutine tag_close (tag, unit)
+  end subroutine xml_tag_write
+
+  subroutine xml_tag_close (tag, unit)
     class(xml_tag_t), intent(in) :: tag
     integer, intent(in), optional :: unit
     integer :: u
     u = given_output_unit (unit)
     write (u, "('</',A,'>')", advance = "no")  char (tag%name)
-  end subroutine tag_close
-    
-  subroutine tag_write_with_content (tag, content, unit)
+  end subroutine xml_tag_close
+
+  subroutine xml_tag_write_with_content (tag, content, unit)
     class(xml_tag_t), intent(in) :: tag
     type(string_t), intent(in) :: content
     integer, intent(in), optional :: unit
@@ -233,9 +233,9 @@ contains
     call tag%write (u)
     write (u, "(A)", advance = "no")  char (content)
     call tag%close (u)
-  end subroutine tag_write_with_content
-  
-  subroutine tag_read (tag, cstream, success)
+  end subroutine xml_tag_write_with_content
+
+  subroutine xml_tag_read (tag, cstream, success)
     class(xml_tag_t), intent(inout) :: tag
     type(cstream_t), intent(inout) :: cstream
     logical, intent(out) :: success
@@ -256,7 +256,7 @@ contains
        ! Look for comment beginning
        p2 = p1 + 3
        if (extract (string, p1, p2) /= "<!--")  exit FIND_NON_COMMENT
-       
+
        ! Look for comment end, then restart
        string = extract (string, p2 + 1)
        FIND_COMMENT_END: do
@@ -312,7 +312,7 @@ contains
           end if
 
           ! Return trailing text to the stream
-          string = extract (string, p2 + 1) 
+          string = extract (string, p2 + 1)
           if (string /= "")  call cstream%revert_record (string)
           success = .true.
 
@@ -332,23 +332,23 @@ contains
       end select
       success = .false.
     end subroutine err_io
-    
+
     subroutine err_incomplete ()
       call msg_fatal ("XML: Error reading tag '" // char (tag%name) &
            // "': tag incomplete")
       success = .false.
     end subroutine err_incomplete
-    
-  end subroutine tag_read
 
-  subroutine tag_read_attribute (tag, string, done)
+  end subroutine xml_tag_read
+
+  subroutine xml_tag_read_attribute (tag, string, done)
     class(xml_tag_t), intent(inout) :: tag
     type(string_t), intent(inout) :: string
     logical, intent(out) :: done
     character(2), parameter :: WS = BLANK // TAB
     type(string_t) :: name, value
     integer :: p1, p2, i
-    
+
     p1 = verify (string, WS);  if (p1 == 0)  call err ()
     p2 = p1
 
@@ -365,7 +365,7 @@ contains
           p2 = scan (string, '=')
           if (p2 == 0)  call err ()
           name = trim (extract (string, p1, p2 - 1))
-          
+
           ! Look for '"'
           string = extract (string, p2 + 1)
           p1 = verify (string, WS);  if (p1 == 0)  call err ()
@@ -385,22 +385,22 @@ contains
                 exit SCAN_KNOWN_ATTRIBUTES
              end if
           end do SCAN_KNOWN_ATTRIBUTES
-          
+
           string = extract (string, p2 + 1)
           done = .false.
        end if
     end if
 
   contains
-    
+
     subroutine err ()
       call msg_fatal ("XML: Error reading attributes of '" // char (tag%name) &
            // "': syntax error")
     end subroutine err
-    
-  end subroutine tag_read_attribute
-    
-  subroutine tag_read_content (tag, cstream, content, closing)
+
+  end subroutine xml_tag_read_attribute
+
+  subroutine xml_tag_read_content (tag, cstream, content, closing)
     class(xml_tag_t), intent(in) :: tag
     type(cstream_t), intent(inout) :: cstream
     type(string_t), intent(out) :: content
@@ -424,7 +424,7 @@ contains
           p1 = verify (string, WS);  if (p1 == 0)  call err_incomplete ()
           p2 = p1 + len (tag%name) - 1
           if (extract (string, p1, p2) == tag%name) then
-             
+
              ! Tag name matches: look for final >
              string = extract (string, p2 + 1)
              p1 = verify (string, WS);  if (p1 == 0)  call err_incomplete ()
@@ -441,9 +441,9 @@ contains
           end if
        end if
     end do FIND_CLOSING
-    
+
   contains
-    
+
     subroutine err_io ()
       select case (iostat)
       case (:-1)
@@ -455,14 +455,14 @@ contains
       end select
       closing = .false.
     end subroutine err_io
-    
+
     subroutine err_incomplete ()
       call msg_fatal ("XML: Error reading content '" // char (tag%name) &
            // "': closing tag incomplete")
       closing = .false.
     end subroutine err_incomplete
-    
-  end subroutine tag_read_content
-          
+
+  end subroutine xml_tag_read_content
+
 
 end module xml
