@@ -1,27 +1,27 @@
-!$Id: pdf_builtin.f90 7369 2015-11-16 18:03:59Z jr_reuter $
+!$Id: pdf_builtin.f90 7783 2017-01-03 09:28:52Z jr_reuter $
 
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 !
-! Copyright (C) 1999-2015 by 
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
+!     Christian Speckner <cnspeckn@googlemail.com>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -36,13 +36,13 @@ module pdf_builtin
   use kinds, only: default, double
   use iso_varying_string, string_t => varying_string
   use string_utils, only: upper_case
-  use constants, only: PI 
+  use constants, only: PI
   use diagnostics
   use mrst2004qed
   use cteq6pdf
   use mstwpdf
   use ct10pdf
-  use CJ12
+  use CJ_pdf
   use ct14pdf
 
   implicit none
@@ -51,14 +51,15 @@ module pdf_builtin
   private
 
 ! The available sets
-  integer, parameter :: nsets = 20
+  integer, parameter :: nsets = 22
   integer, parameter, public :: &
        CTEQ6M = 1, CTEQ6D = 2, CTEQ6L = 3, CTEQ6L1 = 4, &
        MRST2004QEDp = 5, MRST2004QEDn = 6, MSTW2008LO = 7, MSTW2008NLO = 8, &
        MSTW2008NNLO = 9, CT10 = 10, CJ12_max = 11, CJ12_mid = 12, &
        CJ12_min = 13, MMHT2014LO = 14, MMHT2014NLO = 15, MMHT2014NNLO = 16, &
-       CT14LL = 17, CT14L = 18, CT14N = 19, CT14NN = 20
-  
+       CT14LL = 17, CT14L = 18, CT14N = 19, CT14NN = 20, CJ15_LO = 21, &
+       CJ15_NLO = 22
+
 ! Limits
   real(kind=default), parameter :: &
        cteq6_q_min = 1.3, cteq6_q_max = 10.E3, &
@@ -69,18 +70,18 @@ module pdf_builtin
        mstw2008_x_min = 1.01E-6, mstw2008_x_max = 1., &
        ct10_q_min = 1.3, ct10_q_max = 1.E5, &
        ct10_x_min = 1.E-8, ct10_x_max = 1., &
-       cj12_q_min = 1.69, cj12_q_max = 3.E5, &
-       cj12_x_min = 2.E-5, cj12_x_max = 0.9, &
+       cj12_q_min = 1.3, cj12_q_max = 1.E5, &
+       cj12_x_min = 1.E-6, cj12_x_max = 1., &
        mmht2014_q_min = mstw2008_q_min, mmht2014_q_max = mstw2008_q_max, &
        mmht2014_x_min = mstw2008_x_min, mmht2014_x_max = mstw2008_x_max, &
        ct14_q_min = 1.3, ct14_q_max = 1.E5, ct14_x_min = 1.E-9, &
        ct14_x_max = 1.
 
 ! Lambda_QCD and quark masses
-  
+
   real(kind=double), dimension(6), parameter, public :: alam_ct10 = &
        [  0.37219423568859566_double, 0.37219423568859566_double, &
-          0.37219423568859566_double, 0.37219423568859566_double, &      
+          0.37219423568859566_double, 0.37219423568859566_double, &
           0.32560730624033124_double, 0.22600000000000001_double ]
 
   real(kind=double), dimension(6), parameter, public :: alam_cteq6m = &
@@ -97,9 +98,9 @@ module pdf_builtin
        [  0.24560434095679567_double, 0.24560434095679567_double, &
           0.24560434095679567_double, 0.24560434095679567_double, &
           0.21495099184302788_double, 0.16499885346541945_double ]
-  
+
   real(kind=double), dimension(0:5), parameter :: amhat = &
-       [  0.0000000000000000_double,  0.0000000000000000_double, &        
+       [  0.0000000000000000_double,  0.0000000000000000_double, &
           0.0000000000000000_double,  0.0000000000000000_double, &
           1.3000000000000000_double,  4.500000000000000_double ]
 
@@ -109,7 +110,7 @@ module pdf_builtin
 
   integer, parameter, private :: nhq = 2
 
-  real(kind=double), parameter :: &             
+  real(kind=double), parameter :: &
        ca = 3.D0, cf = 4./3.D0, tr = 0.5D0
   real(kind=double), parameter :: &
        b00 =  11./3.d0 * ca, b01 =  -4./3.d0 * tr, b10 =  34./3.d0 * ca**2, &
@@ -122,6 +123,7 @@ module pdf_builtin
   logical :: ct10_initialized = .false.
   integer :: cj12_initialized = -1
   integer :: ct14_initialized = -1
+  integer :: cj15_initialized = -1
   logical :: &
        mrst2004qedp_initialized =  .false., &
        mrst2004qedn_initialized =  .false.
@@ -140,7 +142,7 @@ module pdf_builtin
   public :: pdf_getmass
 
 contains
-  
+
 ! Get PDF name
   function pdf_get_name (pdftype) result (name)
     integer, intent(in) :: pdftype
@@ -172,6 +174,10 @@ contains
        name = var_str ("CJ12_mid")
     case (CJ12_min)
        name = var_str ("CJ12_min")
+    case (CJ15_LO)
+       name = var_str ("CJ15LO")
+    case (CJ15_NLO)
+       name = var_str ("CJ15NLO")
     case (MMHT2014LO)
        name = var_str ("MMHT2014LO")
     case (MMHT2014NLO)
@@ -185,7 +191,7 @@ contains
     case (CT14N)
        name = var_str ("CT14N")
     case (CT14NN)
-       name = var_str ("CT14NN")       
+       name = var_str ("CT14NN")
     case default
        call msg_fatal ("pdf_builtin: internal: invalid PDF set!")
     end select
@@ -200,7 +206,7 @@ contains
     end do
     id = -1
   end function pdf_get_id
-  
+
 ! Query whether a PDF supplies a photon distribution
   function pdf_provides_photon (pdftype) result (flag)
     integer, intent(in) :: pdftype
@@ -216,6 +222,8 @@ contains
        flag = .false.
     case (CJ12_max, CJ12_mid, CJ12_min)
        flag = .false.
+    case (CJ15_LO, CJ15_NLO)
+       flag = .false.
     case (MMHT2014LO, MMHT2014NLO, MMHT2014NNLO)
        flag = .false.
     case (CT14LL, CT14L, CT14N, CT14NN)
@@ -275,6 +283,14 @@ contains
        if (cj12_initialized == pdftype) return
        call setCJ (char (mprefix), 100)
        cj12_initialized = pdftype
+    case (CJ15_LO)
+       if (cj15_initialized == pdftype) return
+       call setCJ (char (mprefix), 400)
+       cj15_initialized = pdftype
+    case (CJ15_NLO)
+       if (cj15_initialized == pdftype) return
+       call setCJ (char (mprefix), 500)
+       cj15_initialized = pdftype
     case (MMHT2014LO, MMHT2014NLO, MMHT2014NNLO)
        if (mmht2014_initialized == pdftype) return
        mmht2014_initialized = pdftype
@@ -285,15 +301,15 @@ contains
        call setct14 (char (mprefix), 1)
     case (CT14L)
        if (ct14_initialized == pdftype) return
-       ct14_initialized = pdftype       
+       ct14_initialized = pdftype
        call setct14 (char (mprefix), 2)
     case (CT14N)
        if (ct14_initialized == pdftype) return
-       ct14_initialized = pdftype       
+       ct14_initialized = pdftype
        call setct14 (char (mprefix), 3)
     case (CT14NN)
        if (ct14_initialized == pdftype) return
-       ct14_initialized = pdftype       
+       ct14_initialized = pdftype
        call setct14 (char (mprefix), 4)
     case default
        call msg_fatal ("pdf_builtin: internal: invalid PDF set!")
@@ -302,7 +318,7 @@ contains
          char (pdf_get_name (pdftype)))
     !!! Up to now only proton
   end subroutine pdf_init
-  
+
 ! Evolve PDF
   subroutine pdf_evolve (pdftype, x, q, f, fphoton)
     integer, intent(in) :: pdftype
@@ -416,6 +432,24 @@ contains
             CJpdf ( 4, mx, mq), CJpdf ( 5, mx, mq), 0._double ]
        if (present (fphoton)) call msg_fatal ("photon pdf requested for " // &
             "CJ12 which does not provide it!")
+    case (CJ15_LO, CJ15_NLO)
+       if (cj15_initialized < 0) &
+            call msg_fatal ("pdf_builtin: internal: PDF set " // &
+            char (pdf_get_name (pdftype)) // " requested without initialization!")
+       if (cj15_initialized /= pdftype) &
+            call msg_fatal ( &
+            "PDF sets " // char (pdf_get_name (pdftype)) // " and " // &
+            char (pdf_get_name (cj15_initialized)) // &
+            " cannot be used simultaneously")
+       mx = max (min (x, cj12_x_max), cj12_x_min)
+       mq = max (min (q, cj12_q_max), cj12_q_min)
+       if (present (f)) f = [ 0._double, &
+            CJpdf (-5, mx, mq), CJpdf (-4, mx, mq), CJpdf (-3, mx, mq), &
+            CJpdf (-2, mx, mq), CJpdf (-1, mx, mq), CJpdf ( 0, mx, mq), &
+            CJpdf ( 1, mx, mq), CJpdf ( 2, mx, mq), CJpdf ( 3, mx, mq), &
+            CJpdf ( 4, mx, mq), CJpdf ( 5, mx, mq), 0._double ]
+       if (present (fphoton)) call msg_fatal ("photon pdf requested for " // &
+            "CJ15 which does not provide it!")
     case (MMHT2014LO, MMHT2014NLO, MMHT2014NNLO)
        if (mmht2014_initialized < 0) &
             call msg_fatal ("pdf_builtin: internal: PDF set " // &
@@ -483,7 +517,7 @@ contains
     call pdf_evolve (set, dx, dq, f)
     ff = f
   end subroutine pdf_evolve_LHAPDF
-  
+
 ! PDF-specific running alphas
   function pdf_alphas (pdftype, q) result (alphas)
     real(kind=double) :: as
@@ -507,9 +541,9 @@ contains
           case (CTEQ6M, CTEQ6D)
              as = PI*pdf_builtin_alpi (qdummy,2,5,alam_cteq6m)
           case (CTEQ6L)
-             as = PI*pdf_builtin_alpi (qdummy,2,5,alam_cteq6l)      
+             as = PI*pdf_builtin_alpi (qdummy,2,5,alam_cteq6l)
           case (CTEQ6L1)
-             as = PI*pdf_builtin_alpi (qdummy,1,5,alam_cteq6ll)             
+             as = PI*pdf_builtin_alpi (qdummy,1,5,alam_cteq6ll)
           case default
              call msg_fatal ("Unrecognized PDF setup in evolution of alphas.")
        end select
@@ -517,7 +551,7 @@ contains
        if (.not. ct10_initialized) call msg_fatal ( &
             "pdf_builtin: internal: PDF set CT10 requested without " // &
             "initialization!")
-        as = PI*pdf_builtin_alpi (qdummy,2,5,alam_ct10)     
+        as = PI*pdf_builtin_alpi (qdummy,2,5,alam_ct10)
     case (MRST2004QEDp)
        if (.not. mrst2004qedp_initialized) call msg_fatal ( &
             "pdf_builtin: internal: PDF set MRST2004QEDp requested without " // &
@@ -541,7 +575,7 @@ contains
           case (MSTW2008lo)
              call pdf_builtin_alfa (as,qdummy,0)
           case (MSTW2008nlo)
-             call pdf_builtin_alfa (as,qdummy,1)  
+             call pdf_builtin_alfa (as,qdummy,1)
           case (MSTW2008nnlo)
              call pdf_builtin_alfa (as,qdummy,2)
        end select
@@ -552,9 +586,24 @@ contains
        if (cj12_initialized /= pdftype) &
             call msg_fatal ( &
             "PDF sets " // char (pdf_get_name (pdftype)) // " and " // &
-            char (pdf_get_name (cteq6_initialized)) // &
+            char (pdf_get_name (cj12_initialized)) // &
             " cannot be used simultaneously")
        as = PI*pdf_builtin_alpi (qdummy,2,5,alam_cteq6m)
+    case (CJ15_LO, CJ15_NLO)
+       if (cj15_initialized < 0) &
+            call msg_fatal ("pdf_builtin: internal: PDF set " // &
+            char (pdf_get_name (pdftype)) // " requested without initialization!")
+       if (cj15_initialized /= pdftype) &
+            call msg_fatal ( &
+            "PDF sets " // char (pdf_get_name (pdftype)) // " and " // &
+            char (pdf_get_name (cj15_initialized)) // &
+            " cannot be used simultaneously")
+       select case (pdftype)
+       case (CJ15_NLO)
+          as = PI*pdf_builtin_alpi (qdummy,2,5,alam_cteq6m)
+       case (CJ15_LO)
+          as = PI*pdf_builtin_alpi (qdummy,2,5,alam_cteq6l)
+       end select
     case (MMHT2014LO, MMHT2014NLO, MMHT2014NNLO)
        if (mmht2014_initialized < 0) &
             call msg_fatal ("pdf_builtin: internal: PDF set " // &
@@ -568,7 +617,7 @@ contains
           case (MMHT2014lo)
              call pdf_builtin_alfa (as,qdummy,0)
           case (MMHT2014nlo)
-             call pdf_builtin_alfa (as,qdummy,1)  
+             call pdf_builtin_alfa (as,qdummy,1)
           case (MMHT2014nnlo)
              call pdf_builtin_alfa (as,qdummy,2)
        end select
@@ -611,16 +660,16 @@ contains
        end select
   end function pdf_getmass
 
-  function pdf_builtin_alpi (AMU,NORDER,NF,alam)     
+  function pdf_builtin_alpi (AMU,NORDER,NF,alam)
     integer, intent(in) :: nf, norder
     real(kind=double), intent(in) :: amu
     real(kind=double), dimension(6), intent(in) :: alam
     real(kind=double) :: alm
     real(kind=double) :: pdf_builtin_alpi
     integer :: irt, n_eff
-    n_eff = num_flavor (AMU,NF)     
-    alm  = alam (n_eff+1) 
-    pdf_builtin_alpi = pdf_builtin_alpqcd (norder, n_eff, amu/alm, irt) 
+    n_eff = num_flavor (AMU,NF)
+    alm  = alam (n_eff+1)
+    pdf_builtin_alpi = pdf_builtin_alpqcd (norder, n_eff, amu/alm, irt)
     select case (irt)
        case (1)
           call msg_warning ("AMU < ALAM in pdf_builtin_alpi")
@@ -628,53 +677,53 @@ contains
           call msg_warning ("pdf_builtin_alpi > 3; Be aware!")
     end select
   end function pdf_builtin_alpi
-  
-  double precision function pdf_builtin_ALPQCD (IRDR, NF, RML, IRT) 
+
+  double precision function pdf_builtin_ALPQCD (IRDR, NF, RML, IRT)
     real(kind=double) :: b0, b1
     integer, intent(in) :: irdr, nf
     integer :: irt
     real(kind=double), intent(in) :: rml
     real(kind=double) :: al, aln
     real(kind=double) :: rm2
-    IRT = 0 
-    if (IRDR .LT. 1 .OR. IRDR .GT. 2) THEN 
+    IRT = 0
+    if (IRDR .LT. 1 .OR. IRDR .GT. 2) THEN
        print *,                                                        &
-            &  'Order out of range in pdf_builtin_ALPQCD: IRDR = ', IRDR               
-       STOP 
+            &  'Order out of range in pdf_builtin_ALPQCD: IRDR = ', IRDR
+       STOP
     end if
-    b0 = (11.d0*CA  - 2.* NF) / 3.d0 
-    b1 = (34.d0*CA**2 - 10.d0*CA*NF - 6.d0*CF*NF) / 3.d0 
-    rm2 = rml**2 
-    if (RM2 .LE. 1.) THEN 
-       IRT = 1 
-       pdf_builtin_ALPQCD = 99. 
-       RETURN 
+    b0 = (11.d0*CA  - 2.* NF) / 3.d0
+    b1 = (34.d0*CA**2 - 10.d0*CA*NF - 6.d0*CF*NF) / 3.d0
+    rm2 = rml**2
+    if (RM2 .LE. 1.) THEN
+       IRT = 1
+       pdf_builtin_ALPQCD = 99.
+       RETURN
     end if
-    aln = log (RM2) 
+    aln = log (RM2)
     al = 4.d0/ B0 / aln
-    IF (IRDR .GE. 2) AL = AL * (1.d0-B1*log(ALN) / ALN / B0**2) 
-    if (AL .GE. 3.) THEN 
-       IRT = 2 
+    IF (IRDR .GE. 2) AL = AL * (1.d0-B1*log(ALN) / ALN / B0**2)
+    if (AL .GE. 3.) THEN
+       IRT = 2
     end if
-    pdf_builtin_ALPQCD = AL 
+    pdf_builtin_ALPQCD = AL
   end function pdf_builtin_ALPQCD
 
-  function num_flavor (amu,nf) 
+  function num_flavor (amu,nf)
     real(kind=double), intent(in) :: amu
     integer, intent(in) :: nf
     integer :: num_flavor, i
     num_flavor = nf - nhq
-    IF ((num_flavor .EQ. nf) .OR. (amu .LE. amn)) GOTO 20 
+    IF ((num_flavor .EQ. nf) .OR. (amu .LE. amn)) GOTO 20
     do 10 I = nf - NHQ + 1, nf
-       if (amu .GE. amhat(I)) THEN 
-          num_flavor = I 
+       if (amu .GE. amhat(I)) THEN
+          num_flavor = I
        else
-          goto 20 
+          goto 20
        end if
 10     continue
 20     return
   end function num_flavor
-           
+
   subroutine pdf_builtin_alfa (alfas,Qalfa,norder)
     integer, intent(in) :: norder
     real(kind=double), intent(in) :: Qalfa
@@ -714,7 +763,7 @@ contains
     real(kind=double) :: ast
     nff = 4
     select case (norder)
-    case (0) 
+    case (0)
        as0 = 5.4258307424173556E-002_double
        asc = 4.0838232991033577E-002_double
        asb = 2.2297506503539639E-002_double
@@ -736,29 +785,29 @@ contains
        r2t = m2t * r2/m2
        asi = ast
        asf = mstw_alphas (r2,r2t,ast,nf,norder)
-    else 
+    else
        if (m2 .gt. m2b) then
           nf = 5
           r2b = m2b * r2/m2
           asi = asb
           asf = mstw_alphas (r2,r2b,asb,nf,norder)
-       else 
+       else
           if (m2 .gt. m2c) then
              nf = 4
              r2c = m2c * r2/m2
              asi = asc
              asf = mstw_alphas (r2,r2c,asc,nf,norder)
-          else                
+          else
              nf = 3
              r20 = m20 * r2/m2
              asi = as0
-             asf = mstw_alphas (r2,r20,as0,nf,norder)       
+             asf = mstw_alphas (r2,r20,as0,nf,norder)
           end if
        end if
     end if
     alfas = 4.D0*PI*ASF
   end function pdf_builtin_mstw_alphas
-     
+
   function mstw_alphas (r2, r20, as0, nf, naord) result (as)
     real(kind=double) :: as
     real(kind=double), intent(in) :: r2, r20, as0
@@ -770,8 +819,8 @@ contains
     as = as0
     lrrat = log (r2/r20)
     dlr = lrrat / nastps
-    ! ..Solution of the evolution equation depending on  NAORD                                          
-    !   (fourth-order Runge-Kutta beyond the leading order)                                             
+    ! ..Solution of the evolution equation depending on  NAORD
+    !   (fourth-order Runge-Kutta beyond the leading order)
     select case (naord)
     case (0)
        as = as0 / (1.+ beta0(nf) * as0 * lrrat)
@@ -801,49 +850,49 @@ contains
        end do
     end select
   end function mstw_alphas
-  
+
   double precision function beta0 (nf)
     integer, intent(in) :: nf
     beta0 = b00 + b01 * nf
   end function beta0
-  
+
   double precision function beta1 (nf)
     integer, intent(in) :: nf
     beta1 = b10 + b11 * nf
   end function beta1
-  
+
   double precision function beta2 (nf)
     integer, intent(in) :: nf
     beta2 = 1428.50 - 279.611 * nf + 6.01852 * nf**2
   end function beta2
-  
+
   double precision function beta3 (nf)
     integer, intent(in) :: nf
     beta3 = 29243.0 - 6946.30 * nf + 405.089 * nf**2 + &
          1.49931 * nf**3
   end function beta3
-  
-  ! ..The beta functions FBETAn at N^nLO for n = 1, 2, and 3    
-  double precision function fbeta1 (a, nf) 
+
+  ! ..The beta functions FBETAn at N^nLO for n = 1, 2, and 3
+  double precision function fbeta1 (a, nf)
     real(kind=double), intent(in) :: a
     integer, intent(in) :: nf
     fbeta1 = - a**2 * ( beta0(nf) + a * beta1(nf) )
   end function fbeta1
-  
-  double precision function fbeta2 (a, nf) 
+
+  double precision function fbeta2 (a, nf)
     real(kind=double), intent(in) :: a
     integer, intent(in) :: nf
     fbeta2 = - a**2 * ( beta0(nf) + a * ( beta1(nf) &
          + a * beta2(nf) ) )
   end function fbeta2
-  
-  double precision function fbeta3 (a, nf) 
+
+  double precision function fbeta3 (a, nf)
     real(kind=double), intent(in) :: a
     integer, intent(in) :: nf
     fbeta3 = - a**2 * ( beta0(nf) + a * ( beta1(nf) &
          + a * ( beta2(nf) + a * beta3(nf)) ) )
   end function fbeta3
- 
+
   subroutine mrst_alphas (alpha, q)
     real(kind=double), intent(in) :: q
     real(kind=double), intent(out) :: alpha
@@ -860,7 +909,7 @@ contains
     idir = +1
 100  continue
     alambda = alambda + idir*astep
-    call mrst_lambda (nflav,alpha,qz,alambda)    
+    call mrst_lambda (nflav,alpha,qz,alambda)
     if (idir*(alphas-alpha).gt.0.0) then
        goto 200
     else
@@ -870,14 +919,14 @@ contains
     end if
 200  continue
     if(abs(alpha-alphas).gt.tol2) goto 100
-    ! alambda found  -- save it !!!!                                                        
-    ! next call mrst_lambda to get alphas at q with the correct alambda                       
+    ! alambda found  -- save it !!!!
+    ! next call mrst_lambda to get alphas at q with the correct alambda
     call mrst_lambda (nflav,alpha,q,alambda)
     RETURN
   end subroutine mrst_alphas
 
   subroutine mrst_lambda (nflav,alpha,Q,alambda)
-    integer, intent(in) :: nflav    
+    integer, intent(in) :: nflav
     integer :: ith
     real(kind=double), intent(in) :: q, alambda
     real(kind=double), intent(out) :: alpha
@@ -887,16 +936,16 @@ contains
     real(kind=double) :: qsdt, qsdtt, qsct, qsctt, qs, q2
     real(kind=double) :: b0, b1, del, f, fp, as, as2
     real(kind=double) :: flav
-    !  The value of Lambda required corresponds to nflav=4                                  
-    qsdt=8.18    !!  This is the value of 4m_c^2                                            
-    qsct=74.0    !!  This is the value of 4m_b^2                                            
+    !  The value of Lambda required corresponds to nflav=4
+    qsdt=8.18    !!  This is the value of 4m_c^2
+    qsct=74.0    !!  This is the value of 4m_b^2
     al2=alambda*alambda
     q2=q*q
-    t = log (q2/al2)    
-    ! CHECK: explicitly initialising ALFQC{3,4,5} (by AB)                                   
+    t = log (q2/al2)
+    ! CHECK: explicitly initialising ALFQC{3,4,5} (by AB)
     alfqc3 = 0
     alfqc4 = 0
-    alfqc5 = 0    
+    alfqc5 = 0
     ith=0
     tt=t
     qsdtt=qsdt/4.
@@ -904,12 +953,12 @@ contains
     al = alambda
     al2=al*al
     flav=4.
-    qs=al2*exp(T)    
-    if (qs.lt.0.5d0) then   !!  running stops below 0.5                                      
+    qs=al2*exp(T)
+    if (qs.lt.0.5d0) then   !!  running stops below 0.5
        qs=0.5d0
        t = log (qs/al2)
        tt= t
-    end if    
+    end if
     if (qs.lt.qsdtt .and. nflav.gt.3) GOTO 312
 11  continue
     b0 = 11._double - 2./3._double * flav
@@ -927,13 +976,13 @@ contains
 51  continue
     IF(ITH.EQ.0) RETURN
     GOTO (13,14,15) ITH
-    ! GOTO 5                                                                                
+    ! GOTO 5
 12  ITH=1
     T=log(QSCTT/AL2)
     GOTO 11
 13  ALFQC4=ALPHA
     FLAV=5.
-    ITH=2    
+    ITH=2
     GOTO 11
 14  ALFQC5=ALPHA
     ITH=3
@@ -955,7 +1004,7 @@ contains
     AS2=AS-F/FP
     DEL=ABS(F/FP/AS)
     IF((DEL-tol).GT.0.) goto 35
-    ALPHA=AS2    
+    ALPHA=AS2
 351 continue
     IF(ITH.EQ.0) RETURN
     GOTO (313,314,315) ITH
