@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -39,8 +39,9 @@ module cascades
   use kinds, only: TC, i8, i32
   use iso_varying_string, string_t => varying_string
   use io_units
+  use constants, only: one
   use format_defs, only: FMT_12, FMT_19
-  use unit_tests
+  use numeric_utils
   use diagnostics
   use hashes
   use sorting
@@ -48,12 +49,22 @@ module cascades
   use physics_defs, only: UNDEFINED
   use model_data
   use flavors
+  use lorentz
+
+  use resonances, only: resonance_info_t
+  use resonances, only: resonance_history_t
+  use resonances, only: resonance_history_set_t
   use phs_forests
 
   implicit none
   private
 
+  public :: hash_entry_init
   public :: cascade_set_t
+  interface cascade_set_init
+     module procedure cascade_set_init_base
+     module procedure cascade_set_init_from_cascade
+  end interface
   public :: cascade_set_is_valid
   public :: cascade_set_final
   public :: cascade_set_write_process_bincode_format
@@ -61,6 +72,9 @@ module cascades
   public :: cascade_set_write_graph_format
   public :: cascade_set_write
   public :: cascade_set_generate
+  public :: assignment(=)
+  public :: cascade_set_get_n_trees
+  public :: cascade_set_get_resonance_histories
 
   integer, parameter :: &
        & EXTERNAL_PRT = -1, &
@@ -114,6 +128,8 @@ module cascades
      type(cascade_t), pointer :: mother => null ()
      ! next in list
      type(cascade_t), pointer :: next => null ()
+   contains
+     procedure :: extract_resonance_history => cascade_extract_resonance_history
   end type cascade_t
 
   type :: cascade_p
@@ -167,12 +183,12 @@ module cascades
      module procedure cascade_set_add_outgoing1
      module procedure cascade_set_add_outgoing2
   end interface
- 
+
   interface cascade_set_add_incoming
      module procedure cascade_set_add_incoming0
      module procedure cascade_set_add_incoming1
   end interface
- 
+
 
 contains
 
@@ -204,7 +220,6 @@ contains
     integer, intent(in), optional :: unit
     type(flavor_t) :: flv
     integer :: u, i
-1   format(3x,A,1x,40(1x,I4))
 2   format(3x,A,1x,I3,1x,A,1x,I7,1x,'!',1x,A)
     u = given_output_unit (unit);  if (u < 0)  return
     call write_reduced (cascade%tree, u)
@@ -215,7 +230,7 @@ contains
        case (NO_MAPPING, EXTERNAL_PRT)
        case (S_CHANNEL)
           write(u,2) 'map', &
-               cascade%tree(i), 's_channel', abs (cascade%tree_pdg(i)), &
+               cascade%tree(i), 's_channel', cascade%tree_pdg(i), &
                char (flv%get_name ())
        case (T_CHANNEL)
           write(u,2) 'map', &
@@ -227,19 +242,19 @@ contains
                char (flv%get_name ())
        case (RADIATION)
           write(u,2) 'map', &
-               cascade%tree(i), 'radiation', abs (cascade%tree_pdg(i)), &
+               cascade%tree(i), 'radiation', cascade%tree_pdg(i), &
                char (flv%get_name ())
        case (COLLINEAR)
           write(u,2) 'map', &
-               cascade%tree(i), 'collinear', abs (cascade%tree_pdg(i)), &
+               cascade%tree(i), 'collinear', cascade%tree_pdg(i), &
                char (flv%get_name ())
        case (INFRARED)
           write(u,2) 'map', &
-               cascade%tree(i), 'infrared ',  abs (cascade%tree_pdg(i)), &
+               cascade%tree(i), 'infrared ', cascade%tree_pdg(i), &
                char (flv%get_name ())
        case (ON_SHELL)
           write(u,2) 'map', &
-               cascade%tree(i), 'on_shell ', abs (cascade%tree_pdg(i)), &
+               cascade%tree(i), 'on_shell ', cascade%tree_pdg(i), &
                char (flv%get_name ())
        case default
           call msg_bug (" Impossible mapping mode encountered")
@@ -468,7 +483,7 @@ contains
     call cascade_init (cascade, 1)
     cascade%bincode = ibset (0_TC, pos-1)
     cascade%flv = flv
-    cascade%pdg = abs (cascade%flv%get_pdg ())
+    cascade%pdg = cascade%flv%get_pdg ()
     cascade%is_vector = flv%get_spin_type () == VECTOR
     cascade%m_min = flv%get_mass ()
     cascade%m_rea = cascade%m_min
@@ -492,7 +507,7 @@ contains
     cascade%incoming = .true.
     cascade%bincode = ibset (0_TC, pos-1)
     cascade%flv = flv%anti ()
-    cascade%pdg = abs (flv%get_pdg ())
+    cascade%pdg = cascade%flv%get_pdg ()
     cascade%is_vector = flv%get_spin_type () == VECTOR
     cascade%m_min = flv%get_mass ()
     cascade%m_rea = cascade%m_min
@@ -520,12 +535,33 @@ contains
     cascade%res_hash = hash (transfer &
          (concat (sort (pack (cascade%tree_pdg, &
                               cascade%tree_resonant)), &
-                  sort (pack (cascade%tree_pdg, &
+                  sort (pack (abs (cascade%tree_pdg), &
                               cascade%tree_mapping == T_CHANNEL .or. &
                               cascade%tree_mapping == U_CHANNEL))), &
           mold))
   end subroutine cascade_assign_resonance_hash
 
+  subroutine hash_entry_init (entry, entry_in)
+    type(hash_entry_t), intent(out) :: entry
+    type(hash_entry_t), intent(in) :: entry_in
+    type(cascade_p), pointer :: casc_iter, casc_copy
+    entry%hashval = entry_in%hashval
+    entry%key = entry_in%key
+    casc_iter => entry_in%first
+    do while (associated (casc_iter))
+       allocate (casc_copy)
+       casc_copy = casc_iter
+       casc_copy%next => null ()
+       if (associated (entry%first)) then
+          entry%last%next => casc_copy
+       else
+          entry%first => casc_copy
+       end if
+       entry%last => casc_copy
+       casc_iter => casc_iter%next
+    end do
+  end subroutine hash_entry_init
+
   subroutine hash_entry_final (hash_entry)
     type(hash_entry_t), intent(inout) :: hash_entry
     type(cascade_p), pointer :: current
@@ -569,7 +605,7 @@ contains
     current%cascade => cascade
     if (associated (hash_entry%last)) then
        hash_entry%last%next => current
-    else 
+    else
        hash_entry%first => current
     end if
     hash_entry%last => current
@@ -630,6 +666,48 @@ contains
     end select
   end function pdg_match
 
+  subroutine cascade_set_init_from_cascade (cascade_set, cascade_set_in)
+    type(cascade_set_t), intent(out) :: cascade_set
+    type(cascade_set_t), intent(in), target :: cascade_set_in
+    type(cascade_t), pointer :: casc_iter, casc_copy
+    cascade_set%model => cascade_set_in%model
+    cascade_set%n_in = cascade_set_in%n_in
+    cascade_set%n_out = cascade_set_in%n_out
+    cascade_set%n_tot = cascade_set_in%n_tot
+    cascade_set%flv = cascade_set_in%flv
+    cascade_set%depth_out = cascade_set_in%depth_out
+    cascade_set%depth_tot = cascade_set_in%depth_tot
+    cascade_set%sqrts = cascade_set_in%sqrts
+    cascade_set%m_threshold_s = cascade_set_in%m_threshold_s
+    cascade_set%m_threshold_t = cascade_set_in%m_threshold_t
+    cascade_set%off_shell = cascade_set_in%off_shell
+    cascade_set%t_channel = cascade_set_in%t_channel
+    cascade_set%keep_nonresonant = cascade_set_in%keep_nonresonant
+    cascade_set%n_groves = cascade_set_in%n_groves
+
+    casc_iter => cascade_set_in%first
+    do while (associated (casc_iter))
+       allocate (casc_copy)
+       casc_copy = casc_iter
+       casc_copy%next => null ()
+       if (associated (cascade_set%first)) then
+          cascade_set%last%next => casc_copy
+       else
+          cascade_set%first => casc_copy
+       end if
+       cascade_set%last => casc_copy
+       casc_iter => casc_iter%next
+    end do
+
+    cascade_set%n_entries = cascade_set_in%n_entries
+    cascade_set%fill_ratio = cascade_set_in%fill_ratio
+    cascade_set%n_entries_max = cascade_set_in%n_entries_max
+    cascade_set%mask = cascade_set_in%mask
+    cascade_set%fatal_beam_decay = cascade_set_in%fatal_beam_decay
+    allocate (cascade_set%entry (0:cascade_set%mask))
+    cascade_set%entry = cascade_set_in%entry
+  end subroutine cascade_set_init_from_cascade
+
   function cascade_set_is_valid (cascade_set) result (flag)
     logical :: flag
     type(cascade_set_t), intent(in) :: cascade_set
@@ -645,7 +723,7 @@ contains
     end do
   end function cascade_set_is_valid
 
-  subroutine cascade_set_init (cascade_set, model, n_in, n_out, phs_par, &
+  subroutine cascade_set_init_base (cascade_set, model, n_in, n_out, phs_par, &
         fatal_beam_decay, flv)
     type(cascade_set_t), intent(out) :: cascade_set
     class(model_data_t), intent(in), target :: model
@@ -684,8 +762,8 @@ contains
     cascade_set%mask = size_guess - 1
     allocate (cascade_set%entry (0:cascade_set%mask))
     cascade_set%fatal_beam_decay = fatal_beam_decay
-  end subroutine cascade_set_init
-    
+  end subroutine cascade_set_init_base
+
   subroutine cascade_set_final (cascade_set)
     type(cascade_set_t), intent(inout), target :: cascade_set
     type(cascade_t), pointer :: current
@@ -728,7 +806,7 @@ contains
     do i = n_in, 1, -1
        bincode(i) = bc
        bc = 2 * bc
-    end do       
+    end do
     do i = 1, n_tot
        write (str, "(I0)")  bincode(i)
        field_width(i) = len_trim (str)
@@ -748,7 +826,7 @@ contains
           fmt_head = fmt_head // ",1x,'  '"
        end if
     end do
-    !!! !!! !!! Workaround for standard-semantics ifort 16.0 bug              
+    !!! !!! !!! Workaround for standard-semantics ifort 16.0 bug
     do i = 1, n_tot
        fmt_proc(i) = fmt_proc(i) // ")"
     end do
@@ -790,7 +868,7 @@ contains
        else
           write (u, "(A)")  ""
        end if
-    end do       
+    end do
     write (u, "(A)")  "\end{align*}"
   end subroutine cascade_set_write_process_tex_format
 
@@ -890,7 +968,7 @@ contains
     call cascade_set_write_process_tex_format (cascade_set, u)
     write (u, *)
     write (u, '(A)') "\noindent" // &
-         & "\textbf{Note:} These are pseudo Feynman graphs that " 
+         & "\textbf{Note:} These are pseudo Feynman graphs that "
     write (u, '(A)') "visualize phase-space parameterizations " // &
          & "(``integration channels'').  "
     write (u, '(A)') "They do \emph{not} indicate Feynman graphs used for the " // &
@@ -1275,7 +1353,7 @@ contains
     call cascade_init (cascade3, cascade1%depth + cascade2%depth + 1)
     cascade3%bincode = ior (cascade1%bincode, cascade2%bincode)
     cascade3%flv = flv%anti ()
-    cascade3%pdg = abs (cascade3%flv%get_pdg ())
+    cascade3%pdg = cascade3%flv%get_pdg ()
     cascade3%is_vector = flv%get_spin_type () == VECTOR
     cascade3%m_min = cascade1%m_min + cascade2%m_min
     cascade3%m_rea = flv%get_mass ()
@@ -1351,7 +1429,7 @@ contains
     if (keep) then
        cascade3%on_shell = cascade3%resonant .or. cascade3%log_enhanced
        if (cascade3%resonant) then
-          cascade3%pdg = abs (cascade3%flv%get_pdg ())
+          cascade3%pdg = cascade3%flv%get_pdg ()
           if (cascade_set%keep_nonresonant) then
              allocate (cascade4)
              cascade4 = cascade3
@@ -1360,7 +1438,7 @@ contains
              cascade4%mapping = NO_MAPPING
              cascade4%resonant = .false.
              cascade4%on_shell = .false.
-          end if      
+          end if
           cascade3%m_min = cascade3%m_rea
           call cascade_fusion (cascade_set, cascade1, cascade2, cascade3)
           if (cascade_set%keep_nonresonant) then
@@ -1434,7 +1512,7 @@ contains
          .and. abs (cascade1%m_eff - cascade3%m_eff) &
                < cascade_set%m_threshold_t) &
          then
-       cascade3%pdg = abs (flv%get_pdg ())
+       cascade3%pdg = flv%get_pdg ()
        cascade3%log_enhanced = .true.
        cascade3%mapping = RADIATION
     end if
@@ -1461,7 +1539,7 @@ contains
          call msg_fatal (" Phase space: Initial beam particle can decay")
       else
          call msg_warning (" Phase space: Initial beam particle can decay")
-      end if 
+      end if
     end subroutine beam_decay
   end subroutine cascade_combine_t
 
@@ -1569,7 +1647,7 @@ contains
             cascade1%n_log_enhanced + cascade2%n_log_enhanced
     end if
     if (cascade3%resonant) then
-       cascade3%n_off_shell = 0    
+       cascade3%n_off_shell = 0
     else if (cascade3%log_enhanced) then
        cascade3%n_off_shell = cascade1%n_off_shell + cascade2%n_off_shell
     else
@@ -1761,7 +1839,7 @@ contains
        cascade_seed => cascade_seed%next
     end do LOOP_SEED
   end subroutine cascade_set_generate_t
-                   
+
   subroutine cascade_set_generate_decay (cascade_set)
     type(cascade_set_t), intent(inout), target :: cascade_set
     type(cascade_t), pointer :: cascade1, cascade2
@@ -1839,7 +1917,7 @@ contains
        cascade_seed => cascade_seed%next
     end do LOOP_SEED
   end subroutine cascade_set_generate_scattering
-    
+
   subroutine cascade_set_assign_resonance_hash (cascade_set)
     type(cascade_set_t), intent(inout) :: cascade_set
     type(cascade_t), pointer :: cascade
@@ -1918,7 +1996,7 @@ contains
           cascade => cascade%next
        end do
        call cascade_set_final (cset(i))
-    end do           
+    end do
     cascade_set%first_k => cascade_set%first
     call cascade_set_assign_resonance_hash (cascade_set)
     call cascade_set_assign_groves (cascade_set)
@@ -1992,6 +2070,84 @@ contains
        flag = .true.;  return
     end if
   end function phase_space_vanishes
-       
+
+  subroutine cascade_extract_resonance_history &
+       (cascade, res_hist, model, n_out)
+    class(cascade_t), intent(in), target :: cascade
+    type(resonance_history_t), intent(out) :: res_hist
+    class(model_data_t), intent(in), target :: model
+    integer, intent(in) :: n_out
+    type(resonance_info_t) :: resonance
+    integer :: i, mom_id, pdg
+    call msg_debug2 (D_PHASESPACE, "cascade_extract_resonance_history")
+    if (cascade%n_resonances > 0) then
+       if (cascade%has_children) then
+          call msg_debug2 (D_PHASESPACE, "cascade has resonances and children")
+          do i = 1, size(cascade%tree_resonant)
+             if (cascade%tree_resonant (i)) then
+                mom_id = cascade%tree (i)
+                pdg = cascade%tree_pdg (i)
+                call resonance%init (mom_id, pdg, model, n_out)
+                if (debug2_active (D_PHASESPACE)) then
+                   print *, 'D: Adding resonance'
+                   call resonance%write ()
+                end if
+                call res_hist%add_resonance (resonance)
+             end if
+          end do
+       end if
+    end if
+  end subroutine cascade_extract_resonance_history
+
+  function cascade_set_get_n_trees (cascade_set) result (n)
+    type(cascade_set_t), intent(in), target :: cascade_set
+    integer :: n
+    type(cascade_t), pointer :: cascade
+    integer :: grove
+    call msg_debug (D_PHASESPACE, "cascade_set_get_n_trees")
+    n = 0
+    do grove = 1, cascade_set%n_groves
+       cascade => cascade_set%first_k
+       do while (associated (cascade))
+          if (cascade%active .and. cascade%complete) then
+             if (cascade%grove == grove) then
+                n = n + 1
+             end if
+          end if
+          cascade => cascade%next
+       end do
+    end do
+    call msg_debug (D_PHASESPACE, "n", n)
+  end function cascade_set_get_n_trees
+
+  subroutine cascade_set_get_resonance_histories (cascade_set, n_filter, res_hists)
+    type(cascade_set_t), intent(in), target :: cascade_set
+    integer, intent(in), optional :: n_filter
+    type(resonance_history_t), dimension(:), allocatable, intent(out) :: res_hists
+    type(resonance_history_t), dimension(:), allocatable :: tmp
+    type(cascade_t), pointer :: cascade
+    type(resonance_history_t) :: res_hist
+    type(resonance_history_set_t) :: res_hist_set
+    integer :: grove, i, n_hists
+    logical :: included, add_to_list
+    call msg_debug (D_PHASESPACE, "cascade_set_get_resonance_histories")
+    call res_hist_set%init (n_filter = n_filter)
+    do grove = 1, cascade_set%n_groves
+       cascade => cascade_set%first_k
+       do while (associated (cascade))
+          if (cascade%active .and. cascade%complete) then
+             if (cascade%grove == grove) then
+                call msg_debug2 (D_PHASESPACE, "grove", grove)
+                call cascade%extract_resonance_history &
+                     (res_hist, cascade_set%model, cascade_set%n_out)
+                call res_hist_set%enter (res_hist)
+             end if
+          end if
+          cascade => cascade%next
+       end do
+    end do
+    call res_hist_set%to_array (res_hists)
+  end subroutine cascade_set_get_resonance_histories
+
 
 end module cascades
