@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -36,6 +36,7 @@
 module lorentz
 
   use kinds, only: default, double
+  use numeric_utils
   use io_units
   use constants, only: pi, twopi, degree, zero, one, two, eps0, tiny_07
   use format_defs, only: FMT_11, FMT_13, FMT_15, FMT_19
@@ -65,16 +66,26 @@ module lorentz
   public :: vector4_set_component
   public :: vector4_get_component
   public :: vector4_get_components
+  public :: vector4_invert_direction
   public :: assignment (=)
+  public :: vector4
   public :: vector4_to_c_prt
+  public :: phs_point_t
   public :: lorentz_transformation_t
   public :: lorentz_transformation_write
   public :: lorentz_transformation_get_components
   public :: identity
   public :: space_reflection
+  public :: compute_resonance_mass
+  public :: get_resonance_momentum
+  public :: create_two_particle_decay
+  public :: create_three_particle_decay
+  public :: evaluate_one_to_two_splitting_special
+  public :: generate_on_shell_decay
   public :: vector_set_reshuffle
   public :: vector_set_is_cms
   public :: vector4_write_set
+  public :: vector4_check_momentum_conservation
   public :: spinor_product
 
   public :: operator(==), operator(/=)
@@ -137,6 +148,15 @@ module lorentz
     procedure :: write => vector4_write
     procedure :: to_pythia6 => vector4_to_pythia6
   end type vector4_t
+  type :: phs_point_t
+     type(vector4_t), dimension(:), allocatable :: p
+     integer :: n_momenta = 0
+  contains
+    procedure :: final => phs_point_final
+    procedure :: write => phs_point_write
+    procedure :: get_x => phs_point_get_x
+  end type phs_point_t
+
   type :: lorentz_transformation_t
      private
      real(default), dimension(0:3, 0:3) :: L
@@ -267,6 +287,16 @@ module lorentz
   interface assignment (=)
      module procedure vector4_from_c_prt, c_prt_from_vector4
   end interface
+  interface operator(==)
+     module procedure phs_point_eq
+  end interface
+  interface operator(*)
+     module procedure prod_LT_phs_point
+  end interface
+  interface assignment(=)
+     module procedure phs_point_from_n, phs_point_from_vector4, &
+                      phs_point_from_phs_point
+  end interface
   interface azimuthal_angle
      module procedure vector3_azimuthal_angle
      module procedure vector4_azimuthal_angle
@@ -358,6 +388,18 @@ module lorentz
   interface inverse
      module procedure lorentz_transformation_inverse
   end interface
+  abstract interface
+     subroutine evaluate_one_to_two_splitting_special (p_origin, &
+          p1_in, p2_in, p1_out, p2_out, msq_in, jac)
+       import
+       type(vector4_t), intent(in) :: p_origin
+       type(vector4_t), intent(in) :: p1_in, p2_in
+       type(vector4_t), intent(inout) :: p1_out, p2_out
+       real(default), intent(in), optional :: msq_in
+       real(default), intent(inout), optional :: jac
+     end subroutine evaluate_one_to_two_splitting_special
+  end interface
+
   interface boost
      module procedure boost_from_rest_frame
      module procedure boost_from_rest_frame_vector3
@@ -587,8 +629,8 @@ contains
     if (comp) then
        write (u, "(4(F12.3,1X))", advance="no")  p%p(0:3)
     else
-       write(u, "(1x,A,1x," // fmt // ")") 'E = ', p%p(0)
-       write(u, "(1x,A,3(1x," // fmt // "))") 'P = ', p%p(1:)
+       write (u, "(1x,A,1x," // fmt // ")") 'E = ', p%p(0)
+       write (u, "(1x,A,3(1x," // fmt // "))") 'P = ', p%p(1:)
        if (sm) then
           m = p**1
           if (tf)  call pacify (m, tolerance = 1E-6_default)
@@ -707,11 +749,11 @@ contains
     real(default) :: s
     type(vector4_t), intent(in) :: p
     integer, intent(in) :: e
-    s = p*p
-    if (e/=2) then
-       if (mod(e,2)==0) then
-          s = s**(e/2)
-       elseif (s>=0) then
+    s = p * p
+    if (e /= 2) then
+       if (mod(e, 2) == 0) then
+          s = s**(e / 2)
+       else if (s >= 0) then
           s = sqrt(s)**e
        else
           s = -(sqrt(abs(s))**e)
@@ -790,6 +832,11 @@ contains
     end if
   end function vector4_get_direction
 
+  elemental subroutine vector4_invert_direction (p)
+    type(vector4_t), intent(inout) :: p
+    p%p(1:3) = -p%p(1:3)
+  end subroutine vector4_invert_direction
+
   pure subroutine array_from_vector4_1 (a, p)
     real(default), dimension(:), intent(out) :: a
     type(vector4_t), intent(in) :: p
@@ -832,6 +879,12 @@ contains
     p%p(1:3) = a
   end subroutine vector3_from_array
 
+  pure function vector4 (a) result (p)
+    type(vector4_t) :: p
+    real(default), intent(in), dimension(4) :: a
+    p%p = a
+  end function vector4
+
   pure function vector4_to_pythia6 (vector4, m) result (p)
     real(double), dimension(1:5) :: p
     class(vector4_t), intent(in) :: vector4
@@ -879,6 +932,69 @@ contains
     end if
   end function vector4_to_c_prt
 
+  elemental function phs_point_eq (phs_point_1, phs_point_2) result (eq)
+    logical :: eq
+    type(phs_point_t), intent(in) :: phs_point_1, phs_point_2
+    eq = all (phs_point_1%p == phs_point_2%p)
+  end function phs_point_eq
+
+  elemental function prod_LT_phs_point (L, phs_point) result (phs_point_LT)
+    type(phs_point_t) :: phs_point_LT
+    type(lorentz_transformation_t), intent(in) :: L
+    type(phs_point_t), intent(in) :: phs_point
+    phs_point_LT = size (phs_point%p)
+    phs_point_LT%p = L * phs_point%p
+  end function prod_LT_phs_point
+
+  pure subroutine phs_point_from_n (phs_point, n_particles)
+    type(phs_point_t), intent(out) :: phs_point
+    integer, intent(in) :: n_particles
+    allocate (phs_point%p (n_particles))
+    phs_point%n_momenta = n_particles
+    phs_point%p = vector4_null
+  end subroutine phs_point_from_n
+
+  pure subroutine phs_point_from_vector4 (phs_point, p)
+    type(phs_point_t), intent(out) :: phs_point
+    type(vector4_t), intent(in), dimension(:) :: p
+    phs_point%n_momenta = size (p)
+    allocate (phs_point%p (phs_point%n_momenta), source = p)
+  end subroutine phs_point_from_vector4
+
+  pure subroutine phs_point_from_phs_point (phs_point, phs_point_in)
+    type(phs_point_t), intent(out) :: phs_point
+    type(phs_point_t), intent(in) :: phs_point_in
+    phs_point%n_momenta = phs_point_in%n_momenta
+    allocate (phs_point%p (phs_point%n_momenta))
+    phs_point%p = phs_point_in%p
+  end subroutine phs_point_from_phs_point
+
+  subroutine phs_point_final (phs_point)
+    class(phs_point_t), intent(inout) :: phs_point
+    deallocate (phs_point%p)
+    phs_point%n_momenta = 0
+  end subroutine phs_point_final
+
+  subroutine phs_point_write (phs_point, unit, show_mass, testflag, &
+      check_conservation, ultra, n_in)
+    class(phs_point_t), intent(in) :: phs_point
+    integer, intent(in), optional :: unit
+    logical, intent(in), optional :: show_mass
+    logical, intent(in), optional :: testflag, ultra
+    logical, intent(in), optional :: check_conservation
+    integer, intent(in), optional :: n_in
+    call vector4_write_set (phs_point%p, unit = unit, show_mass = show_mass, &
+         testflag = testflag, check_conservation = check_conservation, &
+         ultra = ultra, n_in = n_in)
+  end subroutine phs_point_write
+
+  function phs_point_get_x (phs_point, E_beam) result (x)
+    real(default), dimension(2) :: x
+    class(phs_point_t), intent(in) :: phs_point
+    real(default), intent(in) :: E_beam
+    x = phs_point%p(1:2)%p(0) / E_beam
+  end function phs_point_get_x
+
   elemental function vector3_azimuthal_angle (p) result (phi)
     real(default) :: phi
     type(vector3_t), intent(in) :: p
@@ -1160,18 +1276,26 @@ contains
          + azimuthal_distance (p, q)**2)
   end function eta_phi_distance_vector4
 
-  subroutine lorentz_transformation_write (L, unit)
+  subroutine lorentz_transformation_write (L, unit, testflag, ultra)
     class(lorentz_transformation_t), intent(in) :: L
     integer, intent(in), optional :: unit
-    integer :: u
-    integer :: i
+    logical, intent(in), optional :: testflag, ultra
+    integer :: u, i
+    logical :: ult
+    character(len=7) :: fmt
+    ult = .false.; if (present (ultra)) ult = ultra
+    if (ult) then
+       call pac_fmt (fmt, FMT_19, FMT_11, ultra)
+    else
+       call pac_fmt (fmt, FMT_19, FMT_13, testflag)
+    end if
     u = given_output_unit (unit);  if (u < 0)  return
-    write (u, "(1x,A,3(1x," // FMT_19 // "))")  "L00 = ", L%L(0,0)
-    write (u, "(1x,A,3(1x," // FMT_19 // "))")  "L0j = ", L%L(0,1:3)
+    write (u, "(1x,A,3(1x," // fmt // "))")  "L00 = ", L%L(0,0)
+    write (u, "(1x,A,3(1x," // fmt // "))")  "L0j = ", L%L(0,1:3)
     do i = 1, 3
-       write (u, "(1x,A,I0,A,3(1x," // FMT_19 // "))")  &
+       write (u, "(1x,A,I0,A,3(1x," // fmt // "))")  &
             "L", i, "0 = ", L%L(i,0)
-       write (u, "(1x,A,I0,A,3(1x," // FMT_19 // "))")  &
+       write (u, "(1x,A,I0,A,3(1x," // fmt // "))")  &
             "L", i, "j = ", L%L(i,1:3)
     end do
   end subroutine lorentz_transformation_write
@@ -1211,19 +1335,193 @@ contains
   function create_unit_vector (p_in) result (p_out)
     type(vector4_t), intent(in) :: p_in
     type(vector3_t) :: p_out
-    real(default) :: abs
-    abs = space_part_norm (p_in)
-    p_out%p(1) = p_in%p(1)/abs
-    p_out%p(2) = p_in%p(2)/abs
-    p_out%p(3) = p_in%p(3)/abs
+    p_out%p = p_in%p(1:3) / space_part_norm (p_in)
   end function create_unit_vector
 
-  subroutine normalize(p)
-    type(vector3_t), intent(inout) :: p
+  function normalize(p) result (p_norm)
+    type(vector3_t) :: p_norm
+    type(vector3_t), intent(in) :: p
     real(default) :: abs
     abs = sqrt (p%p(1)**2 + p%p(2)**2 + p%p(3)**2)
-    p = p/abs
-  end subroutine normalize
+    p_norm = p / abs
+  end function normalize
+
+  pure function compute_resonance_mass (p, i_res_born, i_gluon) result (m)
+    real(default) :: m
+    type(vector4_t), intent(in), dimension(:) :: p
+    integer, intent(in), dimension(:) :: i_res_born
+    integer, intent(in), optional :: i_gluon
+    type(vector4_t) :: p_res
+    p_res = get_resonance_momentum (p, i_res_born, i_gluon)
+    m = p_res**1
+  end function compute_resonance_mass
+
+  pure function get_resonance_momentum (p, i_res_born, i_gluon) result (p_res)
+    type(vector4_t) :: p_res
+    type(vector4_t), intent(in), dimension(:) :: p
+    integer, intent(in), dimension(:) :: i_res_born
+    integer, intent(in), optional :: i_gluon
+    integer :: i
+    p_res = vector4_null
+    do i = 1, size (i_res_born)
+       p_res = p_res + p (i_res_born(i))
+    end do
+    if (present (i_gluon)) p_res = p_res + p (i_gluon)
+  end function get_resonance_momentum
+
+  function create_two_particle_decay (s, p1, p2) result (p_rest)
+    type(vector4_t), dimension(3) :: p_rest
+    real(default), intent(in) :: s
+    type(vector4_t), intent(in) :: p1, p2
+    real(default) :: m1_sq, m2_sq
+    real(default) :: E1, E2, p
+    m1_sq = p1**2; m2_sq = p2**2
+    p = sqrt (lambda (s, m1_sq, m2_sq)) / (two * sqrt (s))
+    E1 = sqrt (m1_sq + p**2); E2 = sqrt (m2_sq + p**2)
+    p_rest(1)%p = [sqrt (s), zero, zero, zero]
+    p_rest(2)%p(0) = E1
+    p_rest(2)%p(1:3) = p * p1%p(1:3) / space_part_norm (p1)
+    p_rest(3)%p(0) = E2; p_rest(3)%p(1:3) = -p_rest(2)%p(1:3)
+  end function create_two_particle_decay
+
+  function create_three_particle_decay (p1, p2, p3) result (p_rest)
+    type(vector4_t), dimension(4) :: p_rest
+    type(vector4_t), intent(in) :: p1, p2, p3
+    real(default) :: E1, E2, E3
+    real(default) :: pr1, pr2, pr3
+    real(default) :: s, s1, s2, s3
+    real(default) :: m1_sq, m2_sq, m3_sq
+    real(default) :: cos_theta_12
+    type(vector3_t) :: v3_unit
+    type(lorentz_transformation_t) :: rot
+    m1_sq = p1**2
+    m2_sq = p2**2
+    m3_sq = p3**2
+    s1 = (p1 + p2)**2
+    s2 = (p2 + p3)**2
+    s3 = (p3 + p1)**2
+    s = s1 + s2 + s3 - m1_sq - m2_sq - m3_sq
+    E1 = (s - s2 - m1_sq) / (two * sqrt (s2))
+    E2 = (s2 + m2_sq - m3_sq) / (two * sqrt (s2))
+    E3 = (s2 + m3_sq - m2_sq) / (two * sqrt (s2))
+    pr1 = sqrt (lambda (s, s2, m1_sq)) / (two * sqrt (s2))
+    pr2 = sqrt (lambda (s2, m2_sq, m3_sq)) / (two * sqrt(s2))
+    pr3 = pr2
+    cos_theta_12 = ((s - s2 - m1_sq) * (s2 + m2_sq - m3_sq) + two * s2 * (m1_sq + m2_sq - s1)) / &
+         sqrt (lambda (s, s2, m1_sq) * lambda (s2, m2_sq, m3_sq))
+    v3_unit%p = [zero, zero, one]
+    p_rest(1)%p(0) = E1
+    p_rest(1)%p(1:3) = v3_unit%p * pr1
+    p_rest(2)%p(0) = E2
+    p_rest(2)%p(1:3) = v3_unit%p * pr2
+    p_rest(3)%p(0) = E3
+    p_rest(3)%p(1:3) = v3_unit%p * pr3
+    p_rest(4)%p(0) = (s + s2 - m1_sq) / (2 * sqrt (s2))
+    p_rest(4)%p(1:3) = - p_rest(1)%p(1:3)
+    rot = rotation (cos_theta_12, sqrt (one - cos_theta_12**2), 2)
+    p_rest(2) = rot * p_rest(2)
+    p_rest(3)%p(1:3) = - p_rest(2)%p(1:3)
+  end function create_three_particle_decay
+
+  recursive subroutine generate_on_shell_decay (p_dec, &
+      p_in, p_out, i_real, msq_in, jac, evaluate_special)
+    type(vector4_t), intent(in) :: p_dec
+    type(vector4_t), intent(in), dimension(:) :: p_in
+    type(vector4_t), intent(inout), dimension(:) :: p_out
+    integer, intent(in) :: i_real
+    real(default), intent(in), optional :: msq_in
+    real(default), intent(inout), optional :: jac
+    procedure(evaluate_one_to_two_splitting_special), intent(in), &
+          pointer, optional :: evaluate_special
+    type(vector4_t) :: p_dec_new
+    integer :: n_recoil
+    n_recoil = size (p_in) - 1
+    if (n_recoil > 1) then
+       if (present (evaluate_special)) then
+          call evaluate_special (p_dec, p_in(1), sum (p_in (2 : n_recoil + 1)), &
+               p_out(i_real), p_dec_new)
+          call generate_on_shell_decay (p_dec_new, p_in (2 : ), p_out, &
+               i_real + 1, msq_in, jac, evaluate_special)
+       else
+          call evaluate_one_to_two_splitting (p_dec, p_in(1), &
+               sum (p_in (2 : n_recoil + 1)), p_out(i_real), p_dec_new, msq_in, jac)
+          call generate_on_shell_decay (p_dec_new, p_in (2 : ), p_out, &
+               i_real + 1, msq_in, jac)
+       end if
+    else
+       call evaluate_one_to_two_splitting (p_dec, p_in(1), p_in(2), &
+            p_out(i_real), p_out(i_real + 1), msq_in, jac)
+    end if
+
+  end subroutine generate_on_shell_decay
+
+  subroutine evaluate_one_to_two_splitting (p_origin, &
+      p1_in, p2_in, p1_out, p2_out, msq_in, jac)
+    type(vector4_t), intent(in) :: p_origin
+    type(vector4_t), intent(in) :: p1_in, p2_in
+    type(vector4_t), intent(inout) :: p1_out, p2_out
+    real(default), intent(in), optional :: msq_in
+    real(default), intent(inout), optional :: jac
+    type(lorentz_transformation_t) :: L
+    type(vector4_t) :: p1_rest, p2_rest
+    real(default) :: m, msq, msq1, msq2
+    real(default) :: E1, E2, p
+    real(default) :: lda, rlda_soft
+
+    call get_rest_frame (p1_in, p2_in, p1_rest, p2_rest)
+
+    msq = p_origin**2; m = sqrt(msq)
+    msq1 = p1_in**2; msq2 = p2_in**2
+
+    lda = lambda (msq, msq1, msq2)
+    if (lda < zero) then
+       print *, 'Encountered lambda < 0 in 1 -> 2 splitting! '
+       print *, 'lda: ', lda
+       print *, 'm: ', m, 'msq: ', msq
+       print *, 'm1: ', sqrt (msq1), 'msq1: ', msq1
+       print *, 'm2: ', sqrt (msq2), 'msq2: ', msq2
+       stop
+    end if
+    p = sqrt (lda) / (two * m)
+
+    E1 = sqrt (msq1 + p**2)
+    E2 = sqrt (msq2 + p**2)
+
+    p1_out = shift_momentum (p1_rest, E1, p)
+    p2_out = shift_momentum (p2_rest, E2, p)
+
+    L = boost (p_origin, p_origin**1)
+    p1_out = L  * p1_out
+    p2_out = L  * p2_out
+
+    if (present (jac) .and. present (msq_in)) then
+       jac = jac * sqrt(lda) / msq
+       rlda_soft = sqrt (lambda (msq_in, msq1, msq2))
+       !!! We have to undo the Jacobian which has already been
+       !!! supplied by the Born phase space.
+       jac = jac * msq_in / rlda_soft
+    end if
+
+  contains
+
+   subroutine get_rest_frame (p1_in, p2_in, p1_out, p2_out)
+     type(vector4_t), intent(in) :: p1_in, p2_in
+     type(vector4_t), intent(out) :: p1_out, p2_out
+     type(lorentz_transformation_t) :: L
+     L = inverse (boost (p1_in + p2_in, (p1_in + p2_in)**1))
+     p1_out = L * p1_in; p2_out = L * p2_in
+   end subroutine get_rest_frame
+
+   function shift_momentum (p_in, E, p) result (p_out)
+     type(vector4_t) :: p_out
+     type(vector4_t), intent(in) :: p_in
+     real(default), intent(in) :: E, p
+     type(vector3_t) :: vec
+     vec = p_in%p(1:3) / space_part_norm (p_in)
+     p_out = vector4_moving (E, p * vec)
+   end function shift_momentum
+
+  end subroutine evaluate_one_to_two_splitting
 
   elemental function boost_from_rest_frame (p, m) result (L)
     type(lorentz_transformation_t) :: L
@@ -1249,7 +1547,6 @@ contains
     if (bg2 > eps0) then
        g = sqrt(1 + bg2);  c = (g-1)/bg2
     else
-       !!!L = identity
        g = one + bg2 / two
        c = one / two
     end if
@@ -1338,9 +1635,9 @@ contains
        a = direction (p)
        b = direction (q)
        ab = cross_product(a,b)
-       ct = a*b;  st = ab**1
+       ct = a * b;  st = ab**1
        if (abs(st) > eps0) then
-          R = rotation_generic_cs (ct, st, ab/st)
+          R = rotation_generic_cs (ct, st, ab / st)
        else if (ct < 0) then
           R = space_reflection
        else
@@ -1367,7 +1664,7 @@ contains
        end do
        ct = b%p(k);  st = ab**1
        if (abs(st) > eps0) then
-          R = rotation_generic_cs (ct, st, ab/st)
+          R = rotation_generic_cs (ct, st, ab / st)
        else if (ct < 0) then
           R = space_reflection
        else
@@ -1591,9 +1888,10 @@ contains
     real(default), dimension(0:3) :: p_tot
     character(len=7) :: fmt
     integer :: u
-    logical :: yorn
+    logical :: yorn, is_test
     integer :: n
     extreme = .false.; if (present (ultra))  extreme = ultra
+    is_test = .false.; if (present (testflag)) is_test = testflag
     u = given_output_unit (unit);  if (u < 0)  return
     n = 2; if (present (n_in)) n = n_in
     p_tot = 0
@@ -1606,20 +1904,59 @@ contains
       end if
       call vector4_write (p(i), u, show_mass=show_mass, &
            testflag=testflag, ultra=ultra)
-    end do 
+    end do
     if (extreme) then
        call pac_fmt (fmt, FMT_19, FMT_11, testflag)
     else
        call pac_fmt (fmt, FMT_19, FMT_15, testflag)
     end if
-    if (present (testflag)) then
-       if (testflag)  call pacify (p_tot, 1.E-9_default)
+    if (is_test)  call pacify (p_tot, 1.E-9_default)
+    if (.not. is_test) then
+       write (u, "(A5)") 'Total: '
+       write (u, "(1x,A,1x," // fmt // ")")    "E = ", p_tot(0)
+       write (u, "(1x,A,3(1x," // fmt // "))") "P = ", p_tot(1:)
     end if
-    write (u, "(A5)") 'Total: '
-    write (u, "(1x,A,1x," // fmt // ")")    "E = ", p_tot(0)
-    write (u, "(1x,A,3(1x," // fmt // "))") "P = ", p_tot(1:)
   end subroutine vector4_write_set
 
+  subroutine vector4_check_momentum_conservation (p, n_in, unit, &
+     abs_smallness, rel_smallness, verbose)
+    type(vector4_t), dimension(:), intent(in) :: p
+    integer, intent(in) :: n_in
+    integer, intent(in), optional :: unit
+    real(default), intent(in), optional :: abs_smallness, rel_smallness
+    logical, intent(in), optional :: verbose
+    integer :: u, i
+    type(vector4_t) :: psum_in, psum_out
+    logical, dimension(0:3) :: p_diff
+    logical :: verb
+    u = given_output_unit (unit);  if (u < 0)  return
+    verb = .false.; if (present (verbose)) verb = verbose
+    psum_in = vector4_null
+    do i = 1, n_in
+       psum_in = psum_in + p(i)
+    end do
+    psum_out = vector4_null
+    do i = n_in + 1, size (p)
+       psum_out = psum_out + p(i)
+    end do
+    !!! !!! !!! Workaround for gfortran-4.8.4 bug
+    do i = 0, 3
+       p_diff(i) = vanishes (psum_in%p(i) - psum_out%p(i), &
+            abs_smallness = abs_smallness, rel_smallness = rel_smallness)
+    end do
+    if (.not. all (p_diff)) then
+       call msg_warning ("Momentum conservation: FAIL", unit = u)
+       if (verb) then
+          write (u, "(A)") "Incoming:"
+          call vector4_write (psum_in, u)
+          write (u, "(A)") "Outgoing:"
+          call vector4_write (psum_out, u)
+       end if
+    else
+       write (u, "(A)") "Momentum conservation: CHECK"
+    end if
+  end subroutine vector4_check_momentum_conservation
+
   subroutine spinor_product (p1, p2, prod1, prod2)
     type(vector4_t), intent(in) :: p1, p2
     complex(default), intent(out) :: prod1, prod2
