@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -42,9 +42,10 @@ module process_stacks_uti
   use process_libraries
   use rng_base
   use prc_test, only: prc_test_create_library
-  use processes
+  use process, only: process_t
+  use instances, only: process_instance_t
   use processes_ut, only: prepare_test_process
-  
+
   use process_stacks
 
   use rng_base_ut, only: rng_test_factory_t
@@ -68,12 +69,12 @@ contains
     write (u, "(A)")
 
     call stack%write (u)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: process_stacks_1"
-    
+
   end subroutine process_stacks_1
-  
+
   subroutine process_stacks_2 (u)
     integer, intent(in) :: u
     type(process_stack_t) :: stack
@@ -105,21 +106,21 @@ contains
 
     allocate (process)
     run_id = "run1"
-    call process%init &
-         (procname, run_id, lib, os_data, qcd, rng_factory, model) 
+    call process%init (procname, run_id, &
+         lib, os_data, qcd, rng_factory, model)
     call stack%push (process)
-    
+
     allocate (model)
     call model%init_test ()
 
     allocate (process)
     run_id = "run2"
-    call process%init &
-         (procname, run_id, lib, os_data, qcd, rng_factory, model) 
+    call process%init (procname, run_id, &
+         lib, os_data, qcd, rng_factory, model)
     call stack%push (process)
-    
+
     call stack%write (u)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
 
@@ -127,9 +128,9 @@ contains
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: process_stacks_2"
-    
+
   end subroutine process_stacks_2
-  
+
   subroutine process_stacks_3 (u)
     integer, intent(in) :: u
     type(process_stack_t) :: stack
@@ -161,29 +162,29 @@ contains
 
     allocate (process)
     call prepare_test_process (process%process_t, process_instance, model)
-    call process%integrate (process_instance, 1, 1, 1000)
+    call process_instance%integrate (1, 1, 1000)
     call process_instance%final ()
     call process%final_integration (1)
     call stack%push (process)
-    
+
     write (u, "(A)")  "* Fill process variables"
     write (u, "(A)")
 
     call stack%fill_result_vars (procname)
     call stack%write_var_list (u)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
 
     call stack%final ()
 
     call model%final ()
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: process_stacks_3"
-    
+
   end subroutine process_stacks_3
-  
+
   subroutine process_stacks_4 (u)
     integer, intent(in) :: u
     type(process_library_t), target :: lib
@@ -220,10 +221,10 @@ contains
 
     allocate (process)
     run_id = "run1"
-    call process%init &
-         (procname, run_id, lib, os_data, qcd, rng_factory, model) 
+    call process%init (procname, run_id, &
+         lib, os_data, qcd, rng_factory, model)
     call stack1%push (process)
-    
+
     write (u, "(A)")  "* Initialize second process"
     write (u, "(A)")
 
@@ -237,15 +238,15 @@ contains
 
     allocate (process)
     run_id = "run2"
-    call process%init &
-         (procname, run_id, lib, os_data, qcd, rng_factory, model) 
+    call process%init (procname, run_id, &
+         lib, os_data, qcd, rng_factory, model)
     call stack2%push (process)
-    
+
     write (u, "(A)")  "* Show linked stacks"
     write (u, "(A)")
 
     call stack2%write (u)
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Cleanup"
 
@@ -254,9 +255,9 @@ contains
 
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: process_stacks_4"
-    
+
   end subroutine process_stacks_4
-  
+
 
 end module process_stacks_uti
-  
+
