@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -36,6 +36,7 @@ module subevt_expr
 
   use kinds, only: default
   use iso_varying_string, string_t => varying_string
+  use constants, only: zero, one
   use io_units
   use format_utils, only: write_separator
   use diagnostics
@@ -52,8 +53,6 @@ module subevt_expr
   private
 
   public :: parton_expr_t
-!  public :: interaction_to_subevt
-!  public :: interaction_momenta_to_subevt
   public :: event_expr_t
 
   type, extends (subevt_t), abstract :: subevt_expr_t
@@ -78,7 +77,7 @@ module subevt_expr
      procedure :: base_reset => subevt_expr_reset
      procedure :: base_evaluate => subevt_expr_evaluate
   end type subevt_expr_t
-  
+
   type, extends (subevt_expr_t) :: parton_expr_t
      integer, dimension(:), allocatable :: i_beam
      integer, dimension(:), allocatable :: i_in
@@ -105,7 +104,7 @@ module subevt_expr
      procedure :: get_beam_index => parton_expr_get_beam_index
      procedure :: get_in_index => parton_expr_get_in_index
   end type parton_expr_t
-     
+
   type, extends (subevt_expr_t) :: event_expr_t
      logical :: has_reweight = .false.
      logical :: has_analysis = .false.
@@ -146,7 +145,7 @@ module subevt_expr
      procedure :: fill_subevt => event_expr_fill_subevt
      procedure :: evaluate => event_expr_evaluate
   end type event_expr_t
-     
+
 
   interface interaction_momenta_to_subevt
      module procedure interaction_momenta_to_subevt_id
@@ -155,7 +154,7 @@ module subevt_expr
 
 
 contains
-  
+
   subroutine subevt_expr_write (object, unit, pacified)
     class(subevt_expr_t), intent(in) :: object
     integer, intent(in), optional :: unit
@@ -179,7 +178,7 @@ contains
        write (u, "(1x,A)")  "subevt: [undefined]"
     end if
   end subroutine subevt_expr_write
-    
+
   subroutine subevt_expr_final (object)
     class(subevt_expr_t), intent(inout) :: object
     call object%var_list%final ()
@@ -187,7 +186,7 @@ contains
        call object%selection%final ()
     end if
   end subroutine subevt_expr_final
-  
+
   subroutine subevt_expr_setup_vars (expr, sqrts)
     class(subevt_expr_t), intent(inout), target :: expr
     real(default), intent(in) :: sqrts
@@ -212,7 +211,7 @@ contains
          is_known = expr%subevt_filled, &
          locked = .true., verbose = .false., intrinsic = .true.)
   end subroutine subevt_expr_setup_vars
-    
+
   subroutine subevt_expr_setup_var_self (expr)
     class(subevt_expr_t), intent(inout), target :: expr
     if (.not. expr%var_list%contains (var_str ("@evt"))) then
@@ -223,7 +222,7 @@ contains
             locked = .true., verbose = .false., intrinsic=.true.)
     end if
   end subroutine subevt_expr_setup_var_self
-  
+
   subroutine subevt_expr_link_var_list (expr, var_list)
     class(subevt_expr_t), intent(inout) :: expr
     type(var_list_t), intent(in), target :: var_list
@@ -245,10 +244,10 @@ contains
     class(subevt_expr_t), intent(inout) :: expr
     expr%subevt_filled = .false.
   end subroutine subevt_expr_reset
-  
+
   subroutine subevt_expr_evaluate (expr, passed)
     class(subevt_expr_t), intent(inout) :: expr
-    logical, intent(out) :: passed 
+    logical, intent(out) :: passed
     if (expr%has_selection) then
        call expr%selection%evaluate ()
        if (expr%selection%is_known ()) then
@@ -261,7 +260,7 @@ contains
        passed = .true.
     end if
   end subroutine subevt_expr_evaluate
-  
+
   subroutine parton_expr_final (object)
     class(parton_expr_t), intent(inout) :: object
     call object%base_final ()
@@ -314,7 +313,7 @@ contains
        end if
     end if
   end subroutine parton_expr_write
-    
+
   subroutine parton_expr_setup_vars (expr, sqrts)
     class(parton_expr_t), intent(inout), target :: expr
     real(default), intent(in) :: sqrts
@@ -456,9 +455,9 @@ contains
     expr%sqrts_hat = subevt_get_sqrts_hat (expr%subevt_t)
     expr%subevt_filled = .true.
   end subroutine parton_expr_fill_subevt
-    
+
   subroutine parton_expr_evaluate &
-       (expr, passed, scale, fac_scale, ren_scale, weight, scale_forced)
+       (expr, passed, scale, fac_scale, ren_scale, weight, scale_forced, force_evaluation)
     class(parton_expr_t), intent(inout) :: expr
     logical, intent(out) :: passed
     real(default), intent(out) :: scale
@@ -466,11 +465,13 @@ contains
     real(default), intent(out) :: ren_scale
     real(default), intent(out) :: weight
     real(default), intent(in), allocatable, optional :: scale_forced
-    logical :: force_scale
-    force_scale = .false.
+    logical, intent(in), optional :: force_evaluation
+    logical :: force_scale, force_eval
+    force_scale = .false.; force_eval = .false.
     if (present (scale_forced))  force_scale = allocated (scale_forced)
+    if (present (force_evaluation)) force_eval = force_evaluation
     call expr%base_evaluate (passed)
-    if (passed) then
+    if (passed .or. force_eval) then
        if (force_scale) then
           scale = scale_forced
        else if (expr%has_scale) then
@@ -479,7 +480,7 @@ contains
              scale = expr%scale%get_real ()
           else
              call msg_error ("Evaluate scale expression: result undefined")
-             scale = 0
+             scale = zero
           end if
        else
           scale = expr%sqrts_hat
@@ -493,7 +494,7 @@ contains
           else
              call msg_error ("Evaluate factorization scale expression: &
                   &result undefined")
-             fac_scale = 0
+             fac_scale = zero
           end if
        else
           fac_scale = scale
@@ -507,7 +508,7 @@ contains
           else
              call msg_error ("Evaluate renormalization scale expression: &
                   &result undefined")
-             ren_scale = 0
+             ren_scale = zero
           end if
        else
           ren_scale = scale
@@ -518,26 +519,28 @@ contains
              weight = expr%weight%get_real ()
           else
              call msg_error ("Evaluate weight expression: result undefined")
-             weight = 0
+             weight = zero
           end if
        else
-          weight = 1
+          weight = one
        end if
+    else
+       weight = zero
     end if
   end subroutine parton_expr_evaluate
-  
+
   subroutine parton_expr_get_beam_index (expr, i_beam)
     class(parton_expr_t), intent(in) :: expr
     integer, dimension(:), intent(out) :: i_beam
     i_beam = expr%i_beam
   end subroutine parton_expr_get_beam_index
-  
+
   subroutine parton_expr_get_in_index (expr, i_in)
     class(parton_expr_t), intent(in) :: expr
     integer, dimension(:), intent(out) :: i_in
     i_in = expr%i_in
   end subroutine parton_expr_get_in_index
-  
+
   subroutine event_expr_final (object)
     class(event_expr_t), intent(inout) :: object
     call object%base_final ()
@@ -572,7 +575,7 @@ contains
        end if
     end if
   end subroutine event_expr_write
-    
+
   subroutine event_expr_init (expr, n_alt)
     class(event_expr_t), intent(out) :: expr
     integer, intent(in), optional :: n_alt
@@ -582,7 +585,7 @@ contains
        allocate (expr%weight_alt (n_alt), source = 0._default)
     end if
   end subroutine event_expr_init
-  
+
   subroutine event_expr_setup_vars (expr, sqrts)
     class(event_expr_t), intent(inout), target :: expr
     real(default), intent(in) :: sqrts
@@ -649,14 +652,14 @@ contains
     expr%id = id
     expr%has_id = .true.
   end subroutine event_expr_set_process_id
-    
+
   subroutine event_expr_set_process_num_id (expr, num_id)
     class(event_expr_t), intent(inout) :: expr
     integer, intent(in) :: num_id
     expr%num_id = num_id
     expr%has_num_id = .true.
   end subroutine event_expr_set_process_num_id
-    
+
   subroutine event_expr_reset (expr)
     class(event_expr_t), intent(inout) :: expr
     call expr%base_reset ()
@@ -668,7 +671,7 @@ contains
     expr%has_weight_alt = .false.
     expr%has_excess_prc = .false.
   end subroutine event_expr_reset
-  
+
   subroutine event_expr_set (expr, &
        weight_ref, weight_prc, weight_alt, &
        excess_prc, &
@@ -685,7 +688,7 @@ contains
     if (present (sqme_prc)) then
        expr%has_sqme_prc = .true.
        expr%sqme_prc = sqme_prc
-    end if 
+    end if
     if (present (sqme_alt)) then
        expr%has_sqme_alt = .true.
        expr%sqme_alt = sqme_alt
@@ -707,7 +710,7 @@ contains
        expr%excess_prc = excess_prc
     end if
   end subroutine event_expr_set
-  
+
   subroutine event_expr_fill_subevt (expr, particle_set)
     class(event_expr_t), intent(inout) :: expr
     type(particle_set_t), intent(in) :: particle_set
@@ -724,7 +727,7 @@ contains
        expr%has_index = .true.
     end if
   end subroutine event_expr_fill_subevt
-  
+
   subroutine event_expr_evaluate (expr, passed, reweight, analysis_flag)
     class(event_expr_t), intent(inout) :: expr
     logical, intent(out) :: passed
@@ -758,6 +761,6 @@ contains
        end if
     end if
   end subroutine event_expr_evaluate
-  
+
 
 end module subevt_expr
