@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -36,10 +36,10 @@
 module colors
 
   use kinds, only: default
+  use iso_varying_string, string_t => varying_string
   use io_units
-  use unit_tests
   use diagnostics
-  
+
   implicit none
   private
 
@@ -102,28 +102,28 @@ module colors
      module procedure color_init_from_array2
      module procedure color_init_from_array2g
   end interface color_init_from_array
-  
+
   interface color_write
      module procedure color_write_single
      module procedure color_write_array
   end interface color_write
-  
+
   interface color_get_max_value
      module procedure color_get_max_value0
      module procedure color_get_max_value1
      module procedure color_get_max_value2
   end interface color_get_max_value
-  
+
   interface make_color_map
      module procedure color_make_color_map
   end interface make_color_map
-  
+
   interface color_translate
      module procedure color_translate0
      module procedure color_translate0_offset
      module procedure color_translate1
   end interface color_translate
-  
+
 
 contains
 
@@ -308,7 +308,7 @@ contains
        write (u) col%ghost
     end if
   end subroutine color_write_raw
-    
+
   subroutine color_read_raw (col, u, iostat)
     class(color_t), intent(inout) :: col
     integer, intent(in) :: u
@@ -585,7 +585,6 @@ contains
     integer :: i, j, c
     n_prt = size (col_in)
     call extract_color_line_indices (col_in, c_index, col_pos)
-    ! print *, c_index     !!! Debugging
     n_c_index = size (c_index)
     allocate (map (n_c_index))
     map = 0
@@ -640,7 +639,6 @@ contains
          if (any (entry%col(i)%c1 /= 0) .and. &
               entry%col(i)%c1(1) == - entry%col(i)%c1(2))  return
       end do
-      ! call color_write (entry%col); print *, map     !!! Debugging
       if (associated (list%last)) then
          list%last%next => entry
       else
@@ -714,10 +712,10 @@ contains
     if (col%defined .and. .not. col%ghost) then
        col_tmp = col
        do i = 1, size (map,2)
-          where (abs (col%c1) == map(1,i))  
+          where (abs (col%c1) == map(1,i))
              col_tmp%c1 = sign (map(2,i), col%c1)
           end where
-          where (abs (col%c2) == map(1,i))  
+          where (abs (col%c2) == map(1,i))
              col_tmp%c2 = sign (map(2,i), col%c2)
           end where
        end do
@@ -737,11 +735,11 @@ contains
        mask1 = col%c1 /= 0
        mask2 = col%c2 /= 0
        do i = 1, size (map,2)
-          where (abs (col%c1) == map(1,i))  
+          where (abs (col%c1) == map(1,i))
              col_tmp%c1 = sign (map(2,i), col%c1)
              mask1 = .false.
           end where
-          where (abs (col%c2) == map(1,i))  
+          where (abs (col%c2) == map(1,i))
              col_tmp%c2 = sign (map(2,i), col%c2)
              mask2 = .false.
           end where
@@ -803,21 +801,15 @@ contains
     type(color_t), dimension(:), intent(in) :: col
     type(color_t), dimension(size(col)) :: cc
     integer :: i, n, offset
-    ! print *, "Count color loops:"      !!! Debugging
-    ! call color_write (col); print *    !!! Debugging
     cc = col
     n = size (cc)
     offset = n
     call color_add_offset (cc, offset)
-    ! print *, offset                    !!! Debugging
-    ! call color_write (cc); print *     !!! Debugging
     count = 0
     SCAN_LOOPS: do
        do i = 1, n
-          ! print *, i, ':', cc(i)%c1    !!! Debugging 
           if (color_is_nonzero (cc(i))) then
              if (any (cc(i)%c1 > offset)) then
-                ! print *, 'start', i    !!! Debugging
                 count = count + 1
                 call follow_line1 (pick_new_line (cc(i)%c1, count, 1))
                 cycle SCAN_LOOPS
@@ -846,6 +838,7 @@ contains
          call color_mismatch
       end if
     end function pick_new_line
+
     subroutine reset_line (c, line)
       integer, dimension(:), intent(inout) :: c
       integer, intent(in) :: line
@@ -857,44 +850,43 @@ contains
          end if
       end do
     end subroutine reset_line
+
     recursive subroutine follow_line1 (line)
       integer, intent(in) :: line
       integer :: i
-      ! print *, 'follow line 1:', line     !!! Debugging
-      if (line == count) then
-         ! print *, 'loop closed'           !!! Debugging
-         return
-      end if
+      if (line == count) return
       do i = 1, n
          if (any (cc(i)%c1 == -line)) then
             call reset_line (cc(i)%c1, -line)
-            ! print *, 'found', -line, ' resetting c1:'   !!! Debugging
-            ! call color_write (cc); print *              !!! Debugging
             call follow_line2 (pick_new_line (cc(i)%c2, 0, sign (1, -line)))
             return
          end if
       end do
       call color_mismatch ()
     end subroutine follow_line1
+
     recursive subroutine follow_line2 (line)
       integer, intent(in) :: line
       integer :: i
-      ! print *, 'follow line 2:', line     !!! Debugging   
       do i = 1, n
          if (any (cc(i)%c2 == -line)) then
             call reset_line (cc(i)%c2, -line)
-            ! print *, 'found', -line, ' resetting c2:'   !!! Debugging
-            ! call color_write (cc); print *              !!! Debugging
             call follow_line1 (pick_new_line (cc(i)%c1, 0, sign (1, -line)))
             return
          end if
       end do
       call color_mismatch ()
     end subroutine follow_line2
+
     subroutine color_mismatch ()
       call color_write (col)
       print *
-      call msg_bug (" Color flow mismatch (color loops should be closed)")
+      call msg_fatal ("Color flow mismatch: Non-closed color lines appear during ", &
+         [var_str ("the evaluation of color correlations. This can happen if there "), &
+          var_str ("are different color structures in the initial or final state of "), &
+          var_str ("the process definition. If so, please use separate processes for "), &
+          var_str ("the different initial / final states. In a future WHIZARD version "), &
+          var_str ("this will be fixed.")])
     end subroutine color_mismatch
   end function count_color_loops
 
