@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -52,6 +52,7 @@ module flavors
   private
 
   public :: flavor_t
+  public :: flavor_write_array
   public :: operator(.merge.)
   public :: color_from_flavor
 
@@ -62,18 +63,18 @@ module flavors
      type(field_data_t), pointer :: field_data => null ()
    contains
      generic :: init => &
-          flavor_init0_empty, &
-          flavor_init0, &
-          flavor_init0_field_data, &
-          flavor_init0_model, &
-          flavor_init0_model_alt, &
-          flavor_init0_name_model
-     procedure, private :: flavor_init0_empty
-     procedure, private :: flavor_init0
-     procedure, private :: flavor_init0_field_data
-     procedure, private :: flavor_init0_model
-     procedure, private :: flavor_init0_model_alt
-     procedure, private :: flavor_init0_name_model
+          flavor_init_empty, &
+          flavor_init, &
+          flavor_init_field_data, &
+          flavor_init_model, &
+          flavor_init_model_alt, &
+          flavor_init_name_model
+     procedure, private :: flavor_init_empty
+     procedure, private :: flavor_init
+     procedure, private :: flavor_init_field_data
+     procedure, private :: flavor_init_model
+     procedure, private :: flavor_init_model_alt
+     procedure, private :: flavor_init_name_model
      procedure :: tag_radiated => flavor_tag_radiated
      procedure :: undefine => flavor_undefine
      procedure :: write => flavor_write
@@ -132,42 +133,42 @@ module flavors
      module procedure color_from_flavor0
      module procedure color_from_flavor1
   end interface
-       
+
 contains
 
-  elemental subroutine flavor_init0_empty (flv)
+  elemental subroutine flavor_init_empty (flv)
     class(flavor_t), intent(inout) :: flv
     flv%f = UNDEFINED
     flv%radiated = .false.
     flv%field_data => null ()
-  end subroutine flavor_init0_empty
+  end subroutine flavor_init_empty
 
-  elemental subroutine flavor_init0 (flv, f)
+  elemental subroutine flavor_init (flv, f)
     class(flavor_t), intent(inout) :: flv
     integer, intent(in) :: f
     flv%f = f
     flv%radiated = .false.
     flv%field_data => null ()
-  end subroutine flavor_init0
+  end subroutine flavor_init
 
-  impure elemental subroutine flavor_init0_field_data (flv, field_data)
+  impure elemental subroutine flavor_init_field_data (flv, field_data)
     class(flavor_t), intent(inout) :: flv
     type(field_data_t), intent(in), target :: field_data
     flv%f = field_data%get_pdg ()
     flv%radiated = .false.
     flv%field_data => field_data
-  end subroutine flavor_init0_field_data
+  end subroutine flavor_init_field_data
 
-  impure elemental subroutine flavor_init0_model (flv, f, model)
+  impure elemental subroutine flavor_init_model (flv, f, model)
     class(flavor_t), intent(inout) :: flv
     integer, intent(in) :: f
     class(model_data_t), intent(in), target :: model
     flv%f = f
     flv%radiated = .false.
     flv%field_data => model%get_field_ptr (f, check=.true.)
-  end subroutine flavor_init0_model
+  end subroutine flavor_init_model
 
-  impure elemental subroutine flavor_init0_model_alt (flv, f, model, alt_model)
+  impure elemental subroutine flavor_init_model_alt (flv, f, model, alt_model)
     class(flavor_t), intent(inout) :: flv
     integer, intent(in) :: f
     class(model_data_t), intent(in), target :: model, alt_model
@@ -184,72 +185,22 @@ contains
           call msg_fatal ()
        end if
     end if
-  end subroutine flavor_init0_model_alt
+  end subroutine flavor_init_model_alt
 
-!   subroutine flavor_init1_model (flv, f, model)
-!     type(flavor_t), dimension(:), intent(out) :: flv
-!     integer, dimension(:), intent(in) :: f
-!     class(model_data_t), intent(in), target :: model
-!     integer :: i
-!     do i = 1, size (f)
-!        call flavor_init0_model (flv(i), f(i), model)
-!     end do
-!   end subroutine flavor_init1_model
-! 
-!   subroutine flavor_init1_model_alt (flv, f, model, alt_model)
-!     type(flavor_t), dimension(:), intent(out) :: flv
-!     integer, dimension(:), intent(in) :: f
-!     class(model_data_t), intent(in), target :: model, alt_model
-!     integer :: i
-!     do i = 1, size (f)
-!        call flavor_init0_model_alt (flv(i), f(i), model, alt_model)
-!     end do
-!   end subroutine flavor_init1_model_alt
-! 
-!   subroutine flavor_init2_model (flv, f, model)
-!     type(flavor_t), dimension(:,:), intent(out) :: flv
-!     integer, dimension(:,:), intent(in) :: f
-!     class(model_data_t), intent(in), target :: model
-!     integer :: i
-!     do i = 1, size (f, 2)
-!        call flavor_init1_model (flv(:,i), f(:,i), model)
-!     end do
-!   end subroutine flavor_init2_model
-! 
-!   subroutine flavor_init2_model_alt (flv, f, model, alt_model)
-!     type(flavor_t), dimension(:,:), intent(out) :: flv
-!     integer, dimension(:,:), intent(in) :: f
-!     class(model_data_t), intent(in), target :: model, alt_model
-!     integer :: i
-!     do i = 1, size (f, 2)
-!        call flavor_init1_model_alt (flv(:,i), f(:,i), model, alt_model)
-!     end do
-!   end subroutine flavor_init2_model_alt
-! 
-  impure elemental subroutine flavor_init0_name_model (flv, name, model)
+  impure elemental subroutine flavor_init_name_model (flv, name, model)
     class(flavor_t), intent(inout) :: flv
     type(string_t), intent(in) :: name
     class(model_data_t), intent(in), target :: model
     flv%f = model%get_pdg (name)
     flv%radiated = .false.
     flv%field_data => model%get_field_ptr (name, check=.true.)
-  end subroutine flavor_init0_name_model
-! 
-!   subroutine flavor_init1_name_model (flv, name, model)
-!     type(flavor_t), dimension(:), intent(out) :: flv
-!     type(string_t), dimension(:), intent(in) :: name
-!     class(model_data_t), intent(in), target :: model
-!     integer :: i
-!     do i = 1, size (name)
-!        call flavor_init0_name_model (flv(i), name(i), model)
-!     end do
-!   end subroutine flavor_init1_name_model
+  end subroutine flavor_init_name_model
 
   elemental subroutine flavor_tag_radiated (flv)
     class(flavor_t), intent(inout) :: flv
     flv%radiated = .true.
   end subroutine flavor_tag_radiated
-  
+
   elemental subroutine flavor_undefine (flv)
     class(flavor_t), intent(inout) :: flv
     flv%f = UNDEFINED
@@ -273,6 +224,18 @@ contains
     write (u, "(A)", advance="no")  ")"
   end subroutine flavor_write
 
+  subroutine flavor_write_array (flv, unit)
+    type(flavor_t), intent(in), dimension(:) :: flv
+    integer, intent(in), optional :: unit
+    integer :: u, i_flv
+    u = given_output_unit (unit); if (u < 0) return
+    do i_flv = 1, size (flv)
+       call flv(i_flv)%write (u)
+       if (i_flv /= size (flv)) write (u,"(A)", advance = "no") " / "
+    end do
+    write (u,"(A)")
+  end subroutine flavor_write_array
+
   subroutine flavor_write_raw (flv, u)
     class(flavor_t), intent(in) :: flv
     integer, intent(in) :: u
@@ -513,7 +476,7 @@ contains
        name = "?"
     end if
   end function flavor_get_name
-       
+
   elemental function flavor_get_tex_name (flv) result (name)
     type(string_t) :: name
     class(flavor_t), intent(in) :: flv
@@ -523,7 +486,7 @@ contains
        name = "?"
     end if
   end function flavor_get_tex_name
-       
+
   elemental function flavor_get_spin_type (flv) result (type)
     integer :: type
     class(flavor_t), intent(in) :: flv
@@ -584,11 +547,17 @@ contains
   elemental function flavor_get_charge (flv) result (charge)
     real(default) :: charge
     class(flavor_t), intent(in) :: flv
+    integer :: charge_type
     if (associated (flv%field_data)) then
-       if (flavor_is_antiparticle (flv)) then
-          charge = flv%field_data%get_charge ()
+       charge_type = flv%get_charge_type ()
+       if (charge_type == 0 .or. charge_type == 1) then
+          charge = 0
        else
-          charge = - flv%field_data%get_charge ()
+          if (flavor_is_antiparticle (flv)) then
+             charge = - flv%field_data%get_charge ()
+          else
+             charge = flv%field_data%get_charge ()
+          end if
        end if
     else
        charge = 0
@@ -620,9 +589,9 @@ contains
     class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        if (flavor_is_antiparticle (flv)) then
-          isospin = flv%field_data%get_isospin ()
-       else
           isospin = - flv%field_data%get_isospin ()
+       else
+          isospin = flv%field_data%get_isospin ()
        end if
     else
        isospin = 0
