@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -59,7 +59,6 @@ module interactions
   public :: interaction_get_unstable_particle
   public :: interaction_get_flv_out
   public :: interaction_get_flv_content
-  public :: interaction_set_mask
   public :: interaction_set_flavored_values
   public :: interaction_get_n_children
   public :: interaction_get_n_parents
@@ -70,6 +69,7 @@ module interactions
   public :: interaction_exchange_mask
   public :: interaction_send_momenta
   public :: interaction_pacify_momenta
+  public :: interaction_declare_subtraction
   public :: find_connections
 
   type :: external_link_t
@@ -112,13 +112,28 @@ module interactions
      procedure :: basic_init => interaction_init
      procedure :: final => interaction_final
      procedure :: basic_write => interaction_write
+     procedure :: write_state_matrix => interaction_write_state_matrix
+     procedure :: reduce_state_matrix => interaction_reduce_state_matrix
      procedure :: add_state => interaction_add_state
      procedure :: freeze => interaction_freeze
      procedure :: is_empty => interaction_is_empty
      procedure :: get_n_matrix_elements => &
           interaction_get_n_matrix_elements
+     procedure :: get_n_in_helicities => interaction_get_n_in_helicities
+     procedure :: get_me_size => interaction_get_me_size
      procedure :: get_norm => interaction_get_norm
-     procedure :: get_quantum_numbers => interaction_get_quantum_numbers
+     procedure :: get_n_sub => interaction_get_n_sub
+     generic :: get_quantum_numbers => get_quantum_numbers_single, &
+                                       get_quantum_numbers_all, &
+                                       get_quantum_numbers_all_qn_mask
+     procedure :: get_quantum_numbers_single => &
+        interaction_get_quantum_numbers_single
+     procedure :: get_quantum_numbers_all => &
+        interaction_get_quantum_numbers_all
+     procedure :: get_quantum_numbers_all_qn_mask => &
+        interaction_get_quantum_numbers_all_qn_mask
+     procedure :: get_quantum_numbers_all_sub => interaction_get_quantum_numbers_all_sub
+     procedure :: get_quantum_numbers_mask => interaction_get_quantum_numbers_mask
      generic :: get_matrix_element => get_matrix_element_single
      generic :: get_matrix_element => get_matrix_element_array
      procedure :: get_matrix_element_single => &
@@ -132,13 +147,16 @@ module interactions
           interaction_set_matrix_element_clone
      procedure :: interaction_set_matrix_element_qn
      procedure :: interaction_set_matrix_element_all
-     procedure :: interaction_set_matrix_element_array 
+     procedure :: interaction_set_matrix_element_array
      procedure :: interaction_set_matrix_element_single
      procedure :: interaction_set_matrix_element_clone
+     procedure :: set_only_matrix_element => interaction_set_only_matrix_element
+     procedure :: add_to_matrix_element => interaction_add_to_matrix_element
      procedure :: get_diagonal_entries => interaction_get_diagonal_entries
      procedure :: normalize_by_trace => interaction_normalize_by_trace
      procedure :: normalize_by_max => interaction_normalize_by_max
      procedure :: set_norm => interaction_set_norm
+     procedure :: set_state_matrix => interaction_set_state_matrix
      procedure :: get_max_color_value => &
           interaction_get_max_color_value
      procedure :: factorize => interaction_factorize
@@ -166,6 +184,7 @@ module interactions
      generic :: get_mask => get_mask_all, get_mask_slice
      procedure :: get_mask_all => interaction_get_mask_all
      procedure :: get_mask_slice => interaction_get_mask_slice
+     procedure :: set_mask => interaction_set_mask
      procedure :: reset_momenta => interaction_reset_momenta
      procedure :: set_momenta => interaction_set_momenta
      procedure :: set_momentum => interaction_set_momentum
@@ -241,7 +260,7 @@ contains
        write (u, "(1x,I0)", advance="no")  object%link(i)
     end do
   end subroutine internal_link_list_write
-  
+
   subroutine internal_link_list_append (link_list, link)
     class(internal_link_list_t), intent(inout) :: link_list
     integer, intent(in) :: link
@@ -427,7 +446,7 @@ contains
                   write (u, "(1x,A)", advance="no") "=>"
              write (u, "(1x,A)", advance="no") "X"
              if (int%children(i)%has_entries ()) &
-                  write (u, "(1x,A)", advance="no") "=>" 
+                  write (u, "(1x,A)", advance="no") "=>"
              call int%children(i)%write (u)
              write (u, *)
           end if
@@ -449,23 +468,43 @@ contains
           if (allocated (int%p) .and. show_momentum_sum) then
              write (u, "(1x,A)") "Incoming particles (sum):"
              call vector4_write &
-                  (sum (int%p(1:int%n_in)), u, show_mass = show_mass)
+                  (sum (int%p(1 : int%n_in)), u, show_mass = show_mass)
              write (u, "(1x,A)") "Outgoing particles (sum):"
              call vector4_write &
-                  (sum (int%p(int%n_in+int%n_vir+1:)), u, show_mass = show_mass)
+                  (sum (int%p(int%n_in + int%n_vir + 1 : )), &
+                   u, show_mass = show_mass)
              write (u, *)
           end if
        end if
        if (show_st) then
-          call int%state_matrix%write (write_value_list=verbose, &
-               verbose=verbose, unit=unit, col_verbose=col_verbose, &
-               testflag = testflag)
+          call int%write_state_matrix (write_value_list = verbose, &
+             verbose = verbose, unit = unit, col_verbose = col_verbose, &
+             testflag = testflag)
        end if
     else
        write (u, "(1x,A)") "Interaction: [empty]"
     end if
   end subroutine interaction_write
 
+  subroutine interaction_write_state_matrix (int, unit, write_value_list, &
+     verbose, col_verbose, testflag)
+    class(interaction_t), intent(in) :: int
+    logical, intent(in), optional :: write_value_list, verbose, col_verbose
+    logical, intent(in), optional :: testflag
+    integer, intent(in), optional :: unit
+    call int%state_matrix%write (write_value_list = verbose, &
+       verbose = verbose, unit = unit, col_verbose = col_verbose, &
+       testflag = testflag)
+  end subroutine interaction_write_state_matrix
+
+  subroutine interaction_reduce_state_matrix (int, qn_mask)
+    class(interaction_t), intent(inout) :: int
+    type(quantum_numbers_mask_t), intent(in), dimension(:) :: qn_mask
+    type(state_matrix_t) :: state
+    call int%state_matrix%reduce (qn_mask, state)
+    int%state_matrix = state
+  end subroutine interaction_reduce_state_matrix
+
   subroutine interaction_assign (int_out, int_in)
     type(interaction_t), intent(out) :: int_out
     type(interaction_t), intent(in), target :: int_in
@@ -541,35 +580,138 @@ contains
     end if
   end subroutine interaction_freeze
 
-  function interaction_is_empty (int) result (flag)
-    class(interaction_t), intent(in) :: int
+  pure function interaction_is_empty (int) result (flag)
     logical :: flag
+    class(interaction_t), intent(in) :: int
     flag = int%state_matrix%is_empty ()
   end function interaction_is_empty
 
-  function interaction_get_n_matrix_elements (int) result (n)
-    class(interaction_t), intent(in) :: int
+  pure function interaction_get_n_matrix_elements (int) result (n)
     integer :: n
+    class(interaction_t), intent(in) :: int
     n = int%state_matrix%get_n_matrix_elements ()
   end function interaction_get_n_matrix_elements
 
-  function interaction_get_norm (int) result (norm)
+  function interaction_get_n_in_helicities (int) result (n_hel)
+    integer :: n_hel
+    class(interaction_t), intent(in) :: int
+    type(interaction_t) :: int_copy
+    type(quantum_numbers_mask_t), dimension(:), allocatable :: qn_mask
+    type(quantum_numbers_t), dimension(:,:), allocatable :: qn
+    integer :: i
+    allocate (qn_mask (int%n_tot))
+    do i = 1, int%n_tot
+       if (i <= int%n_in) then
+          call qn_mask(i)%init (.true., .true., .false.)
+       else
+          call qn_mask(i)%init (.true., .true., .true.)
+       end if
+    end do
+    int_copy = int
+    call int_copy%set_mask (qn_mask)
+    call int_copy%freeze ()
+    allocate (qn (int_copy%state_matrix%get_n_matrix_elements (), &
+         int_copy%state_matrix%get_depth ()))
+    qn = int_copy%get_quantum_numbers ()
+    n_hel = 0
+    do i = 1, size (qn, dim=1)
+       if (all (qn(i,:)%get_subtraction_index () == 0)) n_hel = n_hel + 1
+    end do
+    call int_copy%final ()
+    deallocate (qn_mask)
+    deallocate (qn)
+  end function interaction_get_n_in_helicities
+
+  pure function interaction_get_me_size (int) result (n)
+    integer :: n
+    class(interaction_t), intent(in) :: int
+    n = int%state_matrix%get_me_size ()
+  end function interaction_get_me_size
+
+  pure function interaction_get_norm (int) result (norm)
     real(default) :: norm
     class(interaction_t), intent(in) :: int
     norm = int%state_matrix%get_norm ()
   end function interaction_get_norm
 
-  function interaction_get_quantum_numbers (int, i) result (qn)
-    class(interaction_t), intent(in), target :: int
+  pure function interaction_get_n_sub (int) result (n_sub)
+    integer :: n_sub
+    class(interaction_t), intent(in) :: int
+    n_sub = int%state_matrix%get_n_sub ()
+  end function interaction_get_n_sub
+
+  function interaction_get_quantum_numbers_single (int, i) result (qn)
     type(quantum_numbers_t), dimension(:), allocatable :: qn
+    class(interaction_t), intent(in), target :: int
     integer, intent(in) :: i
     allocate (qn (int%state_matrix%get_depth ()))
-    qn = int%state_matrix%get_quantum_numbers (i)
-  end function interaction_get_quantum_numbers
+    qn = int%state_matrix%get_quantum_number (i)
+  end function interaction_get_quantum_numbers_single
 
-  function interaction_get_matrix_element_single (int, i) result (me)
+  function interaction_get_quantum_numbers_all (int) result (qn)
+    type(quantum_numbers_t), dimension(:,:), allocatable :: qn
+    class(interaction_t), intent(in), target :: int
+    integer :: i
+    allocate (qn (int%state_matrix%get_n_matrix_elements (), &
+       int%state_matrix%get_depth()))
+    do i = 1, int%state_matrix%get_n_matrix_elements ()
+       qn (i, :) = int%state_matrix%get_quantum_number (i)
+    end do
+  end function interaction_get_quantum_numbers_all
+
+  function interaction_get_quantum_numbers_all_qn_mask (int, qn_mask) &
+     result (qn)
+    type(quantum_numbers_t), dimension(:,:), allocatable :: qn
     class(interaction_t), intent(in) :: int
+    type(quantum_numbers_mask_t), intent(in) :: qn_mask
+    integer :: n_redundant, n_all, n_me
+    integer :: i
+    type(quantum_numbers_t), dimension(:,:), allocatable :: qn_all
+    call int%state_matrix%get_quantum_numbers (qn_all)
+    n_redundant = count (qn_all%are_redundant (qn_mask))
+    n_all = size (qn_all)
+    !!! Number of matrix elements = survivors / n_particles
+    n_me = (n_all - n_redundant) / int%state_matrix%get_depth ()
+    allocate (qn (n_me, int%state_matrix%get_depth()))
+    do i = 1, n_me
+       if (.not. any (qn_all(i, :)%are_redundant (qn_mask))) &
+          qn (i, :) = qn_all (i, :)
+    end do
+  end function interaction_get_quantum_numbers_all_qn_mask
+
+  subroutine interaction_get_quantum_numbers_all_sub (int, qn)
+    class(interaction_t), intent(in) :: int
+    type(quantum_numbers_t), dimension(:,:), allocatable, intent(out) :: qn
+    integer :: i
+    allocate (qn (int%state_matrix%get_n_matrix_elements (), &
+       int%state_matrix%get_depth()))
+    do i = 1, int%state_matrix%get_n_matrix_elements ()
+       qn (i, :) = int%state_matrix%get_quantum_number (i)
+    end do
+  end subroutine interaction_get_quantum_numbers_all_sub
+
+  subroutine interaction_get_quantum_numbers_mask (int, qn_mask, qn)
+    class(interaction_t), intent(in) :: int
+    type(quantum_numbers_mask_t), intent(in) :: qn_mask
+    type(quantum_numbers_t), dimension(:,:), allocatable, intent(out) :: qn
+    integer :: n_redundant, n_all, n_me
+    integer :: i
+    type(quantum_numbers_t), dimension(:,:), allocatable :: qn_all
+    call int%state_matrix%get_quantum_numbers (qn_all)
+    n_redundant = count (qn_all%are_redundant (qn_mask))
+    n_all = size (qn_all)
+    !!! Number of matrix elements = survivors / n_particles
+    n_me = (n_all - n_redundant) / int%state_matrix%get_depth ()
+    allocate (qn (n_me, int%state_matrix%get_depth()))
+    do i = 1, n_me
+       if (.not. any (qn_all(i, :)%are_redundant (qn_mask))) &
+          qn (i, :) = qn_all (i, :)
+    end do
+  end subroutine interaction_get_quantum_numbers_mask
+
+  elemental function interaction_get_matrix_element_single (int, i) result (me)
     complex(default) :: me
+    class(interaction_t), intent(in) :: int
     integer, intent(in) :: i
     me = int%state_matrix%get_matrix_element (i)
   end function interaction_get_matrix_element_single
@@ -577,6 +719,7 @@ contains
   function interaction_get_matrix_element_array (int) result (me)
     complex(default), dimension(:), allocatable :: me
     class(interaction_t), intent(in) :: int
+    allocate (me (int%get_n_matrix_elements ()))
     me = int%state_matrix%get_matrix_element ()
   end function interaction_get_matrix_element_array
 
@@ -612,6 +755,22 @@ contains
     call int%state_matrix%set_matrix_element (int1%state_matrix)
   end subroutine interaction_set_matrix_element_clone
 
+  subroutine interaction_set_only_matrix_element (int, i, value)
+    class(interaction_t), intent(inout) :: int
+    integer, intent(in) :: i
+    complex(default), intent(in) :: value
+    call int%set_matrix_element (cmplx (0, 0, default))
+    call int%set_matrix_element (i, value)
+  end subroutine interaction_set_only_matrix_element
+
+  subroutine interaction_add_to_matrix_element (int, qn, value, match_only_flavor)
+    class(interaction_t), intent(inout) :: int
+    type(quantum_numbers_t), dimension(:), intent(in) :: qn
+    complex(default), intent(in) :: value
+    logical, intent(in), optional :: match_only_flavor
+    call int%state_matrix%add_to_matrix_element (qn, value, match_only_flavor)
+  end subroutine interaction_add_to_matrix_element
+
   subroutine interaction_get_diagonal_entries (int, i)
     class(interaction_t), intent(in) :: int
     integer, dimension(:), allocatable, intent(out) :: i
@@ -634,6 +793,12 @@ contains
     call int%state_matrix%set_norm (norm)
   end subroutine interaction_set_norm
 
+  subroutine interaction_set_state_matrix (int, state)
+    class(interaction_t), intent(inout) :: int
+    type(state_matrix_t), intent(in) :: state
+    int%state_matrix = state
+  end subroutine interaction_set_state_matrix
+
   function interaction_get_max_color_value (int) result (cmax)
     class(interaction_t), intent(in) :: int
     integer :: cmax
@@ -718,25 +883,25 @@ contains
     tag = int%tag
   end function interaction_get_tag
 
-  function interaction_get_n_tot (object) result (n_tot)
+  pure function interaction_get_n_tot (object) result (n_tot)
     class(interaction_t), intent(in) :: object
     integer :: n_tot
     n_tot = object%n_tot
   end function interaction_get_n_tot
 
-  function interaction_get_n_in (object) result (n_in)
+  pure function interaction_get_n_in (object) result (n_in)
     class(interaction_t), intent(in) :: object
     integer :: n_in
     n_in = object%n_in
   end function interaction_get_n_in
 
-  function interaction_get_n_vir (object) result (n_vir)
+  pure function interaction_get_n_vir (object) result (n_vir)
     class(interaction_t), intent(in) :: object
     integer :: n_vir
     n_vir = object%n_vir
   end function interaction_get_n_vir
 
-  function interaction_get_n_out (object) result (n_out)
+  pure function interaction_get_n_out (object) result (n_out)
     class(interaction_t), intent(in) :: object
     integer :: n_out
     n_out = object%n_out
@@ -866,7 +1031,7 @@ contains
     if (int%n_in /= 0) then
        s = sum (int%p(:int%n_in)) ** 2
     else
-       s = sum (int%p(int%n_vir+1:)) ** 2
+       s = sum (int%p(int%n_vir + 1 : )) ** 2
     end if
   end function interaction_get_s
 
@@ -923,12 +1088,12 @@ contains
     call it%init (int%get_state_matrix_ptr ())
     do while (it%is_valid ())
        flv_state = it%get_flavor ()
-       flv(:,i) = flv_state(n_in+n_vir+1:)
+       flv(:,i) = flv_state(n_in + n_vir + 1 : )
        i = i + 1
        call it%advance ()
     end do
   end subroutine interaction_get_flv_out
-  
+
   subroutine interaction_get_flv_content (int, state_flv, n_out_hard)
     type(interaction_t), intent(in), target :: int
     type(state_flv_content_t), intent(out) :: state_flv
@@ -937,13 +1102,15 @@ contains
     integer :: n_tot
     n_tot = int%get_n_tot ()
     allocate (mask (n_tot), source = .false.)
-    mask(n_tot-n_out_hard+1:) = .true.
+    mask(n_tot-n_out_hard + 1 : ) = .true.
     call state_flv%fill (int%get_state_matrix_ptr (), mask)
   end subroutine interaction_get_flv_content
-  
+
   subroutine interaction_set_mask (int, mask)
-    type(interaction_t), intent(inout) :: int
+    class(interaction_t), intent(inout) :: int
     type(quantum_numbers_mask_t), dimension(:), intent(in) :: mask
+    if (size (int%mask) /= size (mask)) &
+       call msg_fatal ("Attempting to set mask with unfitting size!")
     int%mask = mask
     int%update_state_matrix = .true.
   end subroutine interaction_set_mask
@@ -1054,7 +1221,7 @@ contains
     logical, intent(in), optional :: resonant
     logical :: reson
     integer :: i, j, i2, k2
-    reson = .false.;  if (present (resonant))  reson = resonant 
+    reson = .false.;  if (present (resonant))  reson = resonant
     do i = 1, size (map_connections)
        k2 = connection_index(i)
        do j = 1, int_in%children(k2)%get_length ()
@@ -1147,7 +1314,7 @@ contains
     int1 => external_link_get_ptr (link)
     i1 = external_link_get_index (link)
   end subroutine interaction_find_source
-    
+
   function interaction_get_ultimate_source (int, i) result (link)
     type(external_link_t) :: link
     type(interaction_t), intent(in) :: int
@@ -1220,6 +1387,35 @@ contains
     end do
   end subroutine interaction_pacify_momenta
 
+  subroutine interaction_declare_subtraction (int, n_sub)
+    type(interaction_t), intent(inout), target :: int
+    integer, intent(in) :: n_sub
+    type(state_iterator_t) :: it
+    type(quantum_numbers_t), dimension(:), allocatable :: qn
+    integer :: i, s
+    integer :: n_me_orig
+    complex(default), dimension(:), allocatable :: me_orig
+    call it%init (int%state_matrix)
+    i = 1; n_me_orig = int%state_matrix%get_n_matrix_elements ()
+    allocate (me_orig (n_me_orig))
+    allocate (qn (it%get_depth ()))
+    do while (it%is_valid () .and. i <= n_me_orig)
+       qn = it%get_quantum_numbers ()
+       me_orig (i) = it%get_matrix_element ()
+       do s = 1, n_sub
+          call qn%set_subtraction_index (s)
+          call int%state_matrix%add_state (qn)
+       end do
+       call it%advance ()
+       i = i + 1
+    end do
+    call int%state_matrix%freeze()
+    do i = 1, n_me_orig
+       call int%state_matrix%set_matrix_element (i, me_orig(i))
+       call int%state_matrix%set_matrix_element (i + n_me_orig, me_orig(i))
+    end do
+  end subroutine interaction_declare_subtraction
+
   subroutine find_connections (int1, int2, n, connection_index)
     class(interaction_t), intent(in) :: int1, int2
     integer, intent(out) :: n
@@ -1285,7 +1481,7 @@ contains
        allocate (ordering (n))
        ordering = order (conn_index_tmp(:,1))
        connection_index = conn_index_tmp(ordering,:)
-    else               
+    else
        connection_index = conn_index_tmp
     end if
   end subroutine find_connections
