@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -60,8 +60,9 @@ module state_matrices
   integer, parameter, public :: FM_IGNORE_HELICITY = 1
   integer, parameter, public :: FM_SELECT_HELICITY = 2
   integer, parameter, public :: FM_FACTOR_HELICITY = 3
+  integer, parameter, public :: FM_CORRELATED_HELICITY = 4
+
 
-       
   type :: node_t
      private
      type(quantum_numbers_t) :: qn
@@ -84,6 +85,7 @@ module state_matrices
      integer :: n_counters = 0
      complex(default), dimension(:), allocatable :: me
      real(default) :: norm = 1
+     integer :: n_sub = 0
    contains
      procedure :: init => state_matrix_init
      procedure :: final => state_matrix_final
@@ -94,10 +96,16 @@ module state_matrices
      procedure :: is_defined => state_matrix_is_defined
      procedure :: is_empty => state_matrix_is_empty
      procedure :: get_n_matrix_elements => state_matrix_get_n_matrix_elements
+     procedure :: get_me_size => state_matrix_get_me_size
+     procedure :: compute_n_sub => state_matrix_compute_n_sub
+     procedure :: get_n_sub => state_matrix_get_n_sub
      procedure :: get_n_leaves => state_matrix_get_n_leaves
      procedure :: get_depth => state_matrix_get_depth
      procedure :: get_norm => state_matrix_get_norm
-     procedure :: get_quantum_numbers => state_matrix_get_quantum_numbers
+     procedure :: get_quantum_number => &
+        state_matrix_get_quantum_number
+     procedure :: get_quantum_numbers => &
+        state_matrix_get_quantum_numbers
      generic :: get_matrix_element => get_matrix_element_single
      generic :: get_matrix_element => get_matrix_element_array
      procedure :: get_matrix_element_single => &
@@ -117,7 +125,7 @@ module state_matrices
      procedure :: set_matrix_element_qn => state_matrix_set_matrix_element_qn
      procedure :: set_matrix_element_all => state_matrix_set_matrix_element_all
      procedure :: set_matrix_element_array => &
-          state_matrix_set_matrix_element_array 
+          state_matrix_set_matrix_element_array
      procedure :: set_matrix_element_single => &
           state_matrix_set_matrix_element_single
      procedure :: set_matrix_element_clone => &
@@ -152,6 +160,8 @@ module state_matrices
      procedure :: is_valid => state_iterator_is_valid
      procedure :: get_me_index => state_iterator_get_me_index
      procedure :: get_me_count => state_iterator_get_me_count
+     procedure :: get_depth => state_iterator_get_depth
+     procedure :: go_to_qn => state_iterator_go_to_qn
      generic :: get_quantum_numbers => get_qn_multi, get_qn_slice, &
           get_qn_range, get_qn_single
      generic :: get_flavor => get_flv_multi, get_flv_slice, &
@@ -178,7 +188,8 @@ module state_matrices
      procedure :: get_hel_single => state_iterator_get_hel_single
      procedure :: set_model => state_iterator_set_model
      procedure :: get_matrix_element => state_iterator_get_matrix_element
-     procedure :: set_matrix_element => state_iterator_set_matrix_element  
+     procedure :: set_matrix_element => state_iterator_set_matrix_element
+     procedure :: add_to_matrix_element => state_iterator_add_to_matrix_element
   end type state_iterator_t
 
   type :: state_flv_content_t
@@ -193,7 +204,7 @@ module state_matrices
      procedure :: fill => state_flv_content_fill
      procedure :: match => state_flv_content_match
   end type state_flv_content_t
-  
+
 
   interface assignment(=)
      module procedure state_matrix_assign
@@ -306,14 +317,14 @@ contains
     current => node%child_first
     do while (associated (current))
        write (u, "(A)", advance="no")  repeat (" ", i)
-       call node_write (current, me_array, verbose=verb, &
-          unit=u, col_verbose=col_verbose, testflag=testflag)
-       call node_write_rec (current, me_array, verbose=verb, &
-          indent=i+2, unit=u, col_verbose=col_verbose, testflag=testflag)
+       call node_write (current, me_array, verbose = verb, &
+          unit = u, col_verbose = col_verbose, testflag = testflag)
+       call node_write_rec (current, me_array, verbose = verb, &
+          indent = i + 2, unit = u, col_verbose = col_verbose, testflag = testflag)
        current => current%next
     end do
   end subroutine node_write_rec
-  
+
   recursive subroutine node_write_raw_rec (node, u)
     type(node_t), intent(in), target :: node
     integer, intent(in) :: u
@@ -393,23 +404,24 @@ contains
     character(len=7) :: fmt
     integer :: u
     integer :: i
-    call pac_fmt (fmt, FMT_19, FMT_17, testflag)        
+    call pac_fmt (fmt, FMT_19, FMT_17, testflag)
     u = given_output_unit (unit);  if (u < 0)  return
     write (u, "(1x,A," // fmt // ")") "State matrix:  norm = ", state%norm
     if (associated (state%root)) then
        if (allocated (state%me)) then
-          call node_write_rec (state%root, state%me, verbose=verbose, &
-             indent=1, unit=u, col_verbose=col_verbose, testflag=testflag)
+          call node_write_rec (state%root, state%me, verbose = verbose, &
+             indent = 1, unit = u, col_verbose = col_verbose, &
+             testflag = testflag)
        else
-          call node_write_rec (state%root, verbose=verbose, indent=1, &
-             unit=u, col_verbose=col_verbose, testflag=testflag)
+          call node_write_rec (state%root, verbose = verbose, indent = 1, &
+             unit = u, col_verbose = col_verbose, testflag = testflag)
        end if
     end if
     if (present (write_value_list)) then
        if (write_value_list .and. allocated (state%me)) then
           do i = 1, size (state%me)
              write (u, "(1x,I0,A)", advance="no")  i, ":"
-             me_dum = state%me(i)             
+             me_dum = state%me(i)
              if (real(state%me(i)) == -real(state%me(i))) then
                 me_dum = &
                      cmplx (0._default, aimag(me_dum), kind=default)
@@ -505,7 +517,7 @@ contains
        call it%advance ()
     end do
   end subroutine state_matrix_set_model
-    
+
   elemental function state_matrix_is_defined (state) result (defined)
     logical :: defined
     class(state_matrix_t), intent(in) :: state
@@ -518,12 +530,42 @@ contains
     flag = state%depth == 0
   end function state_matrix_is_empty
 
-  function state_matrix_get_n_matrix_elements (state) result (n)
+  pure function state_matrix_get_n_matrix_elements (state) result (n)
     integer :: n
     class(state_matrix_t), intent(in) :: state
     n = state%n_matrix_elements
   end function state_matrix_get_n_matrix_elements
 
+  pure function state_matrix_get_me_size (state) result (n)
+    integer :: n
+    class(state_matrix_t), intent(in) :: state
+    if (allocated (state%me)) then
+       n = size (state%me)
+    else
+       n = 0
+    end if
+  end function state_matrix_get_me_size
+
+  subroutine state_matrix_compute_n_sub (state)
+    class(state_matrix_t), intent(inout) :: state
+    type(state_iterator_t) :: it
+    type(quantum_numbers_t), dimension(state%depth) :: qn
+    integer :: sub
+    call it%init (state)
+    do while (it%is_valid ())
+       qn = it%get_quantum_numbers ()
+       sub = qn(1)%get_sub ()
+       if (sub > state%n_sub) state%n_sub = sub
+       call it%advance ()
+    end do
+  end subroutine state_matrix_compute_n_sub
+
+  pure function state_matrix_get_n_sub (state) result (n)
+    integer :: n
+    class(state_matrix_t), intent(in) :: state
+    n = state%n_sub
+  end function state_matrix_get_n_sub
+
   function state_matrix_get_n_leaves (state) result (n)
     integer :: n
     class(state_matrix_t), intent(in) :: state
@@ -536,19 +578,19 @@ contains
     end do
   end function state_matrix_get_n_leaves
 
-  function state_matrix_get_depth (state) result (depth)
+  pure function state_matrix_get_depth (state) result (depth)
     integer :: depth
     class(state_matrix_t), intent(in) :: state
     depth = state%depth
   end function state_matrix_get_depth
 
-  function state_matrix_get_norm (state) result (norm)
+  pure function state_matrix_get_norm (state) result (norm)
     real(default) :: norm
     class(state_matrix_t), intent(in) :: state
     norm = state%norm
   end function state_matrix_get_norm
 
-  function state_matrix_get_quantum_numbers (state, i) result (qn)
+  function state_matrix_get_quantum_number (state, i) result (qn)
     class(state_matrix_t), intent(in), target :: state
     integer, intent(in) :: i
     type(quantum_numbers_t), dimension(state%depth) :: qn
@@ -564,7 +606,18 @@ contains
        end if
        call it%advance ()
     end do
-  end function state_matrix_get_quantum_numbers
+  end function state_matrix_get_quantum_number
+
+  subroutine state_matrix_get_quantum_numbers (state, qn)
+    type(quantum_numbers_t), dimension(:,:), allocatable, intent(out) :: qn
+    class(state_matrix_t), intent(in), target :: state
+    integer :: i
+    allocate (qn (state%get_n_matrix_elements (), &
+       state%get_depth()))
+    do i = 1, state%get_n_matrix_elements ()
+       qn (i, :) = state%get_quantum_number (i)
+    end do
+  end subroutine state_matrix_get_quantum_numbers
 
   elemental function state_matrix_get_matrix_element_single (state, i) result (me)
     complex(default) :: me
@@ -704,6 +757,7 @@ contains
        call red_state%add_state (qn)
        call it%advance ()
     end do
+    call red_state%freeze ()
   end subroutine state_matrix_reduce
 
   subroutine state_matrix_freeze (state)
@@ -713,6 +767,7 @@ contains
        if (allocated (state%me))  deallocate (state%me)
        allocate (state%me (state%n_matrix_elements))
        state%me = 0
+       call state%compute_n_sub ()
     end if
     if (state%leaf_nodes_store_values) then
        call it%init (state)
@@ -725,28 +780,23 @@ contains
   end subroutine state_matrix_freeze
 
   subroutine state_matrix_set_matrix_element_qn (state, qn, value)
-    class(state_matrix_t), intent(inout) :: state
+    class(state_matrix_t), intent(inout), target :: state
     type(quantum_numbers_t), dimension(:), intent(in) :: qn
     complex(default), intent(in) :: value
     type(state_iterator_t) :: it
-    if (.not. allocated (it%state%me)) then
-       allocate (it%state%me (size(qn)))
+    if (.not. allocated (state%me)) then
+       allocate (state%me (size(qn)))
     end if
     call it%init (state)
-    do while (it%is_valid ())
-       if (all (qn == it%get_quantum_numbers ())) then
-          call it%set_matrix_element (value)
-          return
-       end if
-       call it%advance ()
-    end do
+    call it%go_to_qn (qn)
+    call it%set_matrix_element (value)
   end subroutine state_matrix_set_matrix_element_qn
 
   subroutine state_matrix_set_matrix_element_all (state, value)
     class(state_matrix_t), intent(inout) :: state
     complex(default), intent(in) :: value
     if (.not. allocated (state%me)) then
-       allocate (state%me (state%n_matrix_elements))    
+       allocate (state%me (state%n_matrix_elements))
     end if
     state%me = value
   end subroutine state_matrix_set_matrix_element_all
@@ -765,7 +815,7 @@ contains
     integer, intent(in) :: i
     complex(default), intent(in) :: value
     if (.not. allocated (state%me)) then
-       allocate (state%me (state%n_matrix_elements))    
+       allocate (state%me (state%n_matrix_elements))
     end if
     state%me(i) = value
   end subroutine state_matrix_set_matrix_element_single
@@ -778,11 +828,19 @@ contains
     state%me = state1%me
   end subroutine state_matrix_set_matrix_element_clone
 
-  subroutine state_matrix_add_to_matrix_element (state, i, value)
-    class(state_matrix_t), intent(inout) :: state
-    integer, intent(in) :: i
+  subroutine state_matrix_add_to_matrix_element (state, qn, value, match_only_flavor)
+    class(state_matrix_t), intent(inout), target :: state
+    type(quantum_numbers_t), dimension(:), intent(in) :: qn
     complex(default), intent(in) :: value
-    state%me(i) = state%me(i) + value
+    logical, intent(in), optional :: match_only_flavor
+    type(state_iterator_t) :: it
+    call it%init (state)
+    call it%go_to_qn (qn, match_only_flavor)
+    if (it%is_valid ()) then
+       call it%add_to_matrix_element (value)
+    else
+       call msg_fatal ("Cannot add to matrix element - it%node not allocated")
+    end if
   end subroutine state_matrix_add_to_matrix_element
 
   subroutine state_iterator_init (it, state)
@@ -799,7 +857,7 @@ contains
        it%node => null ()
     end if
   end subroutine state_iterator_init
-    
+
   subroutine state_iterator_advance (it)
     class(state_iterator_t), intent(inout) :: it
     call find_next (it%node)
@@ -845,6 +903,35 @@ contains
     end if
   end function state_iterator_get_me_count
 
+  pure function state_iterator_get_depth (state_iterator) result (depth)
+    integer :: depth
+    class(state_iterator_t), intent(in) :: state_iterator
+    depth = state_iterator%depth
+  end function state_iterator_get_depth
+
+  subroutine state_iterator_go_to_qn (it, qn, match_only_flavor)
+    class(state_iterator_t), intent(inout) :: it
+    type(quantum_numbers_t), dimension(:), intent(in) :: qn
+    logical, intent(in), optional :: match_only_flavor
+    logical :: match_flv
+    match_flv = .false.; if (present (match_only_flavor)) match_flv = .true.
+    do while (it%is_valid ())
+       if (match_flv) then
+          if (all (qn .fmatch. it%get_quantum_numbers ())) then
+             return
+          else
+             call it%advance ()
+          end if
+       else
+          if (all (qn == it%get_quantum_numbers ())) then
+             return
+          else
+             call it%advance ()
+          end if
+       end if
+    end do
+  end subroutine state_iterator_go_to_qn
+
   function state_iterator_get_qn_multi (it) result (qn)
     class(state_iterator_t), intent(in) :: it
     type(quantum_numbers_t), dimension(it%depth) :: qn
@@ -1003,7 +1090,7 @@ contains
        node => node%parent
     end do
   end subroutine state_iterator_set_model
-  
+
   function state_iterator_get_matrix_element (it) result (me)
     complex(default) :: me
     class(state_iterator_t), intent(in) :: it
@@ -1019,11 +1106,16 @@ contains
   subroutine state_iterator_set_matrix_element (it, value)
     class(state_iterator_t), intent(inout) :: it
     complex(default), intent(in) :: value
-    if (it%node%me_index /= 0) then
-       it%state%me(it%node%me_index) = value
-    end if
+    if (it%node%me_index /= 0) it%state%me(it%node%me_index) = value
   end subroutine state_iterator_set_matrix_element
 
+  subroutine state_iterator_add_to_matrix_element (it, value)
+    class(state_iterator_t), intent(inout) :: it
+    complex(default), intent(in) :: value
+    if (it%node%me_index /= 0) &
+         it%state%me(it%node%me_index) = it%state%me(it%node%me_index) + value
+  end subroutine state_iterator_add_to_matrix_element
+
   subroutine state_matrix_assign (state_out, state_in)
     type(state_matrix_t), intent(out) :: state_out
     type(state_matrix_t), intent(in), target :: state_in
@@ -1040,6 +1132,7 @@ contains
        allocate (state_out%me (size (state_in%me)))
        state_out%me = state_in%me
     end if
+    state_out%n_sub = state_in%n_sub
   end subroutine state_matrix_assign
 
   subroutine state_matrix_get_diagonal_entries (state, i)
@@ -1095,7 +1188,7 @@ contains
     real(default), intent(in) :: norm
     state%norm = norm
   end subroutine state_matrix_set_norm
-  
+
   function state_matrix_sum (state) result (value)
     complex(default) :: value
     class(state_matrix_t), intent(in) :: state
@@ -1300,7 +1393,7 @@ contains
     logical, dimension(:,:), allocatable :: mask
     ok = .true.
     if (x /= 0) then
-       xt = x * state%trace (qn_in)
+       xt = x * abs (state%trace (qn_in))
     else
        xt = 0
     end if
@@ -1316,14 +1409,7 @@ contains
           end if
        end if
        if (all (qn%are_diagonal ())) then
-          value = it%get_matrix_element ()
-          if (real (value, default) < 0) then
-             call state%write ()
-             print *, value
-             call msg_bug ("Event generation: " &
-                  // "Negative real part of squared matrix element value")
-             value = 0
-          end if
+          value = abs (it%get_matrix_element ())
           s = s + value
           if (s > xt)  exit
        end if
@@ -1345,7 +1431,7 @@ contains
        do i = 1, depth
           call single_state(i)%add_state ([qn(i)], value=value)
        end do
-       if (.not. present (correlated_state)) then       
+       if (.not. present (correlated_state)) then
           do i = 1, size(single_state)
              call single_state(i)%freeze ()
           end do
@@ -1372,7 +1458,8 @@ contains
           end do
           value = it%get_matrix_element ()
           select case (mode)
-          case (FM_IGNORE_HELICITY)  ! trace over diagonal states that match qn
+          case (FM_IGNORE_HELICITY, FM_CORRELATED_HELICITY)
+             ! trace over diagonal states that match qn
              if (all (diagonal)) then
                 do i = 1, depth
                    call single_state(i)%add_state &
@@ -1413,7 +1500,7 @@ contains
        end do
     else
        call msg_fatal ("Polarization state not allocated!")
-    end if 
+    end if
   end function state_matrix_get_polarization_density_matrix
 
   subroutine state_flv_content_write (state_flv, unit)
@@ -1435,18 +1522,18 @@ contains
        write (u, "(' )')")
     end do
   end subroutine state_flv_content_write
-    
+
   subroutine state_flv_content_init (state_flv, n, mask)
     class(state_flv_content_t), intent(out) :: state_flv
     integer, intent(in) :: n
     logical, dimension(:), intent(in) :: mask
     integer :: d, i
-    d = size (mask)    
+    d = size (mask)
     allocate (state_flv%pdg (d, n), source = 0)
     allocate (state_flv%map (d, n), source = spread ([(i, i = 1, d)], 2, n))
     allocate (state_flv%mask (d), source = mask)
   end subroutine state_flv_content_init
-    
+
   subroutine state_flv_content_set_entry (state_flv, i, pdg, map)
     class(state_flv_content_t), intent(inout) :: state_flv
     integer, intent(in) :: i
@@ -1456,7 +1543,7 @@ contains
        state_flv%map(:,i) = map
     end where
   end subroutine state_flv_content_set_entry
-  
+
   subroutine state_flv_content_fill &
        (state_flv, state_full, mask)
     class(state_flv_content_t), intent(out) :: state_flv
@@ -1492,7 +1579,7 @@ contains
        !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug
        !!! pdg = flv%get_pdg ()
        do j = 1, d
-          pdg(j) = flv(j)%get_pdg ()          
+          pdg(j) = flv(j)%get_pdg ()
        end do
        idx_subset = pack (idx, mask)
        pdg_subset = pack (pdg, mask)
@@ -1539,7 +1626,7 @@ contains
        where (pdg == 0)  map = 0
     end if
   end subroutine state_flv_content_match
-    
+
   elemental function pacify_complex (c_in) result (c_pac)
     complex(default), intent(in) :: c_in
     complex(default) :: c_pac
@@ -1553,6 +1640,6 @@ contains
             cmplx (real(c_pac), 0._default, kind=default)
     end if
   end function pacify_complex
-  
+
 
 end module state_matrices
