@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -40,7 +40,7 @@ module decays
   use io_units
   use format_utils, only: write_indent, write_separator
   use format_defs, only: FMT_15
-  use unit_tests, only: nearly_equal, vanishes
+  use numeric_utils
   use diagnostics
   use flavors
   use helicities
@@ -51,7 +51,8 @@ module decays
   use rng_base
   use selectors
   use parton_states
-  use processes
+  use process, only: process_t
+  use instances, only: process_instance_t, pacify
   use process_stacks
   use event_transforms
 
@@ -274,6 +275,7 @@ module decays
      type(decay_root_t) :: decay_root
      type(decay_chain_t) :: decay_chain
    contains
+     procedure :: write_name => evt_decay_write_name
      procedure :: write => evt_decay_write
      procedure :: connect => evt_decay_connect
      procedure :: prepare_new_event => evt_decay_prepare_new_event
@@ -627,13 +629,13 @@ contains
        !!! m_prod = flv(:,1)%get_mass ()
        call flv%set_model (model)
        allocate (m_dec (size (flv(:,1)%get_mass ())))
-       !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug       
+       !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug
        do j = 1, size (flv(:,1)%get_mass ())
           m_dec(j) = flv(j,1)%get_mass ()
        end do
        !!! m_dec = flv(:,1)%get_mass ()
        allocate (stable (size (flv, 1)))
-       !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug       
+       !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug
        do j = 1, size (flv, 1)
           stable(j) = flv(j,1)%is_stable ()
        end do
@@ -956,7 +958,7 @@ contains
     call decay%rng%generate (x)
     decay%selected_mci = decay%config%mci_selector%select (x)
     call decay%process_instance%choose_mci (decay%selected_mci)
-    call decay%process_instance%select_i_term (decay%selected_term)
+    decay%selected_term = decay%process_instance%select_i_term ()
     do i = 1, size (decay%term)
        call decay%term(i)%select_chain ()
     end do
@@ -967,8 +969,7 @@ contains
     type(isolated_state_t), pointer :: isolated_state
     integer :: i
     call decay%process_instance%receive_beam_momenta ()
-    call decay%config%process%generate_unweighted_event &
-         (decay%process_instance, decay%selected_mci)
+    call decay%process_instance%generate_unweighted_event (decay%selected_mci)
     if (signal_is_pending ())  return
     call decay%process_instance%evaluate_event_data ()
     isolated_state => &
@@ -1354,6 +1355,14 @@ contains
     x = real (chain%correlated_trace%get_matrix_element (1))
   end function decay_chain_get_probability
 
+  subroutine evt_decay_write_name (evt, unit)
+    class(evt_decay_t), intent(in) :: evt
+    integer, intent(in), optional :: unit
+    integer :: u
+    u = given_output_unit (unit)
+    write (u, "(1x,A)")  "Event transform: partonic decays"
+  end subroutine evt_decay_write_name
+
   subroutine evt_decay_write (evt, unit, verbose, more_verbose, testflag)
     class(evt_decay_t), intent(in) :: evt
     integer, intent(in), optional :: unit
@@ -1364,7 +1373,7 @@ contains
     verb = .true.;  if (present (verbose))  verb = verbose
     verb2 = .false.;  if (present (more_verbose))  verb2 = more_verbose
     call write_separator (u, 2)
-    write (u, "(1x,A)")  "Event transform: partonic decays"
+    call evt%write_name (u)
     call write_separator (u, 2)
     call evt%base_write (u, testflag = testflag)
     if (verb) then
