@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -40,18 +40,20 @@ module events
   use constants, only: one
   use io_units
   use format_utils, only: pac_fmt, write_separator
-  use format_defs, only: FMT_12, FMT_14, FMT_19
-  use unit_tests
+  use format_defs, only: FMT_12, FMT_19
+  use numeric_utils
   use diagnostics
   use variables
   use expr_base
   use model_data
-  use state_matrices, only: &
-       FM_IGNORE_HELICITY, FM_SELECT_HELICITY, FM_FACTOR_HELICITY
+  use state_matrices, only: FM_IGNORE_HELICITY, &
+       FM_SELECT_HELICITY, FM_FACTOR_HELICITY, FM_CORRELATED_HELICITY
   use particles
   use subevt_expr
   use rng_base
-  use processes
+  use process, only: process_t
+  use instances, only: process_instance_t
+  use pcm, only: pcm_instance_nlo_t
   use process_stacks
   use event_base
   use event_transforms
@@ -99,6 +101,7 @@ module events
      logical :: analysis_flag = .false.
      integer :: i_event = 0
    contains
+     procedure :: clone => event_clone
      procedure :: final => event_final
      procedure :: write => event_write
      procedure :: basic_init => event_init
@@ -114,6 +117,7 @@ module events
      procedure :: evaluate_expressions => event_evaluate_expressions
      procedure :: passed_selection => event_passed_selection
      procedure :: store_alt_values => event_store_alt_values
+     procedure :: is_nlo => event_is_nlo
      procedure :: reset => event_reset
      procedure :: import_instance_results => event_import_instance_results
      procedure :: accept_sqme_ref => event_accept_sqme_ref
@@ -141,6 +145,10 @@ module events
      procedure :: get_index => event_get_index
      procedure :: get_fac_scale => event_get_fac_scale
      procedure :: get_alpha_s => event_get_alpha_s
+     procedure :: get_sqrts => event_get_sqrts
+     procedure :: get_polarization => event_get_polarization
+     procedure :: get_beam_file => event_get_beam_file
+     procedure :: get_process_name => event_get_process_name
      procedure :: get_actual_calls_total => event_get_actual_calls_total
   end type event_t
 
@@ -195,6 +203,39 @@ contains
     end if
   end subroutine event_config_write
 
+  subroutine event_clone (event, event_new)
+    class(event_t), intent(in), target :: event
+    class(event_t), intent(out), target:: event_new
+    type(string_t) :: id
+    integer :: num_id
+    event_new%config = event%config
+    event_new%process => event%process
+    event_new%instance => event%instance
+    if (allocated (event%rng)) &
+         allocate(event_new%rng, source=event%rng)
+    event_new%selected_i_mci = event%selected_i_mci
+    event_new%selected_i_term = event%selected_i_term
+    event_new%selected_channel = event%selected_channel
+    event_new%is_complete = event%is_complete
+    event_new%transform_first => event%transform_first
+    event_new%transform_last => event%transform_last
+    event_new%selection_evaluated = event%selection_evaluated
+    event_new%passed = event%passed
+    if (allocated (event%alpha_qcd_forced)) &
+         allocate(event_new%alpha_qcd_forced, source=event%alpha_qcd_forced)
+    if (allocated (event%scale_forced)) &
+         allocate(event_new%scale_forced, source=event%scale_forced)
+    event_new%reweight = event%reweight
+    event_new%analysis_flag = event%analysis_flag
+    event_new%i_event = event%i_event
+    id = event_new%process%get_id ()
+    if (id /= "")  call event_new%expr%set_process_id (id)
+    num_id = event_new%process%get_num_id ()
+    if (num_id /= 0)  call event_new%expr%set_process_num_id (num_id)
+    call event_new%expr%setup_vars (event_new%process%get_sqrts ())
+    call event_new%expr%link_var_list (event_new%process%get_var_list_ptr ())
+  end subroutine event_clone
+
   subroutine event_final (object)
     class(event_t), intent(inout) :: object
     class(evt_t), pointer :: evt
@@ -219,7 +260,7 @@ contains
     class(evt_t), pointer :: evt
     character(len=7) :: fmt
     integer :: u, i
-    call pac_fmt (fmt, FMT_19, FMT_14, testflag)
+    call pac_fmt (fmt, FMT_19, FMT_12, testflag)
     u = given_output_unit (unit)
     prc = .true.;  if (present (show_process))  prc = show_process
     trans = .true.;  if (present (show_transforms))  trans = show_transforms
@@ -238,7 +279,7 @@ contains
     end if
     if (object%sqme_ref_is_known ()) then
        write (u, "(3x,A," // fmt // ")") &
-            "Squared matrix el. = ", object%get_sqme_ref ()
+            "Squared matrix el. (ref) = ", object%get_sqme_ref ()
        if (object%sqme_alt_is_known ()) then
           do i = 1, object%get_n_alt ()
              write (u, "(5x,A," // fmt // ",1x,I0)")  &
@@ -246,16 +287,24 @@ contains
           end do
        end if
     end if
+    if (object%sqme_prc_is_known ()) &
+       write (u, "(3x,A," // fmt // ")") &
+            "Squared matrix el. (prc) = ", object%get_sqme_prc ()
+
     if (object%weight_ref_is_known ()) then
        write (u, "(3x,A," // fmt // ")") &
-            "Event weight       = ", object%get_weight_ref ()
+            "Event weight (ref)       = ", object%get_weight_ref ()
        if (object%weight_alt_is_known ()) then
           do i = 1, object%get_n_alt ()
              write (u, "(5x,A," // fmt // ",1x,I0)")  &
-                  "alternate weight = ", object%get_weight_alt(i), i
+                  "alternate weight      = ", object%get_weight_alt(i), i
           end do
        end if
     end if
+    if (object%weight_prc_is_known ()) &
+       write (u, "(3x,A," // fmt // ")") &
+            "Event weight (prc)       = ", object%get_weight_prc ()
+
     if (object%selected_i_mci /= 0) then
        call write_separator (u)
        write (u, "(3x,A,I0)")  "Selected MCI group = ", object%selected_i_mci
@@ -311,7 +360,7 @@ contains
     class(event_t), intent(out) :: event
     type(var_list_t), intent(in), optional :: var_list
     integer, intent(in), optional :: n_alt
-    type(string_t) :: norm_string
+    type(string_t) :: norm_string, mode_string
     logical :: polarized_events
     if (present (n_alt)) then
        call event%base_init (n_alt)
@@ -329,7 +378,21 @@ contains
        polarized_events = &
             var_list%get_lval (var_str ("?polarized_events"))
        if (polarized_events) then
-          event%config%factorization_mode = FM_SELECT_HELICITY
+          mode_string = &
+               var_list%get_sval (var_str ("$polarization_mode"))
+          select case (char (mode_string))
+          case ("ignore")
+             event%config%factorization_mode = FM_IGNORE_HELICITY
+          case ("helicity")
+             event%config%factorization_mode = FM_SELECT_HELICITY
+          case ("factorized")
+             event%config%factorization_mode = FM_FACTOR_HELICITY
+          case ("correlated")
+             event%config%factorization_mode = FM_CORRELATED_HELICITY
+          case default
+             call msg_fatal ("Polarization mode " &
+                  // char (mode_string) // " is undefined")
+          end select
        else
           event%config%factorization_mode = FM_IGNORE_HELICITY
        end if
@@ -420,11 +483,12 @@ contains
     class(evt_t), pointer :: evt
     real(default) :: sigma_over_sqme
     integer :: i_term
+    logical :: failed_but_keep = .false.
     call msg_debug (D_TRANSFORMS, "event_evaluate_transforms")
     call event%discard_particle_set ()
     call event%check ()
     if (event%instance%is_complete_event ()) then
-       call event%instance%select_i_term (i_term)
+       i_term = event%instance%select_i_term ()
        event%selected_i_term = i_term
        evt => event%transform_first
        do while (associated (evt))
@@ -437,10 +501,21 @@ contains
        call msg_debug (D_TRANSFORMS, "event%weight_prc", event%weight_prc)
        call msg_debug (D_TRANSFORMS, "event%sqme_prc", event%sqme_prc)
        do while (associated (evt))
+          call print_transform_name_if_debug ()
           if (evt%only_weighted_events) then
-             sigma_over_sqme = event%weight_prc / event%sqme_prc
-             call evt%generate_weighted (event%sqme_prc)
-             event%weight_prc = sigma_over_sqme * event%sqme_prc
+             select type (evt)
+             type is (evt_nlo_t)
+                failed_but_keep = .not. evt%is_valid_event (i_term) .and. evt%keep_failed_events
+                if (.not. evt%is_valid_event (i_term) .and. .not. failed_but_keep) &
+                   return
+             end select
+             if (abs (event%weight_prc) > 0._default) then
+                sigma_over_sqme = event%weight_prc / event%sqme_prc
+                call evt%generate_weighted (event%sqme_prc)
+                event%weight_prc = sigma_over_sqme * event%sqme_prc
+             else
+                if (.not. failed_but_keep) exit
+             end if
           else
              call evt%generate_unweighted ()
           end if
@@ -452,13 +527,18 @@ contains
           evt => evt%next
        end do
        evt => event%transform_last
-       if (associated (evt) .and. evt%particle_set_exists) then
-          if (event%is_nlo_event()) then
+       if ((associated (evt) .and. evt%particle_set_exists) .or. failed_but_keep) then
+          if (event%is_nlo ()) then
              select type (evt)
              type is (evt_nlo_t)
-                call evt%build_radiated_particle_set (event%i_event+1)
+                if (evt%i_evaluation > 0) then
+                   evt%particle_set_radiated (event%i_event + 1) = evt%particle_set
+                else
+                   call evt%keep_and_boost_born_particle_set (event%i_event + 1)
+                end if
+                evt%i_evaluation = evt%i_evaluation + 1
                 call event%link_particle_set &
-                   (evt%particle_set_radiated(event%i_event+1))
+                   (evt%particle_set_radiated(event%i_event + 1))
              end select
           else
              call event%link_particle_set (evt%particle_set)
@@ -467,7 +547,15 @@ contains
        call msg_debug (D_TRANSFORMS, "After event transformations")
        call msg_debug (D_TRANSFORMS, "event%weight_prc", event%weight_prc)
        call msg_debug (D_TRANSFORMS, "event%sqme_prc", event%sqme_prc)
+       call msg_debug (D_TRANSFORMS, "evt%particle_set_exists", evt%particle_set_exists)
     end if
+  contains
+    subroutine print_transform_name_if_debug ()
+       if (debug_active (D_TRANSFORMS)) then
+          print *, 'Current event transform: '
+          call evt%write_name ()
+       end if
+    end subroutine print_transform_name_if_debug
   end subroutine event_evaluate_transforms
 
   subroutine event_evaluate_expressions (event)
@@ -515,6 +603,21 @@ contains
     end if
   end subroutine event_store_alt_values
 
+  function event_is_nlo (event) result (is_nlo)
+    logical :: is_nlo
+    class(event_t), intent(in) :: event
+    if (associated (event%instance)) then
+       select type (pcm => event%instance%pcm)
+       type is (pcm_instance_nlo_t)
+          is_nlo = pcm%is_fixed_order_nlo_events ()
+       class default
+          is_nlo = .false.
+       end select
+    else
+       is_nlo = .false.
+    end if
+  end function event_is_nlo
+
   subroutine event_reset (event)
     class(event_t), intent(inout) :: event
     class(evt_t), pointer :: evt
@@ -630,14 +733,13 @@ contains
     if (generate_new) call event%reset ()
     event%selected_i_mci = i_mci
     if (event%config%unweighted) then
-       call event%process%generate_unweighted_event (event%instance, i_mci)
+       call event%instance%generate_unweighted_event (i_mci)
        if (signal_is_pending ()) return
        call event%instance%evaluate_event_data ()
        call event%instance%normalize_weight ()
     else
-       if (event%is_nlo_event()) &
-          call event%process%deactivate_real_component ()
-       if (generate_new) call event%process%generate_weighted_event (event%instance, i_mci)
+       if (generate_new) &
+          call event%instance%generate_weighted_event (i_mci)
        if (signal_is_pending ()) return
        call event%instance%evaluate_event_data ()
     end if
@@ -740,7 +842,7 @@ contains
           call event%instance%evaluate_event_data &
                (weight = event%get_weight_ref ())
        else
-          call event%process%recover_event (event%instance, i_term)
+          call event%instance%recover_event ()
           if (signal_is_pending ())  return
           call event%instance%evaluate_event_data ()
           if (event%config%unweighted) then
@@ -844,6 +946,30 @@ contains
     alpha_s = event%instance%get_alpha_s (event%selected_i_term)
   end function event_get_alpha_s
 
+  function event_get_sqrts (event) result (sqrts)
+    class(event_t), intent(in) :: event
+    real(default) :: sqrts
+    sqrts = event%instance%get_sqrts ()
+  end function event_get_sqrts
+
+  function event_get_polarization (event) result (pol)
+    class(event_t), intent(in) :: event
+    real(default), dimension(2) :: pol
+    pol = event%instance%get_polarization ()
+  end function event_get_polarization
+
+  function event_get_beam_file (event) result (file)
+    class(event_t), intent(in) :: event
+    type(string_t) :: file
+    file = event%instance%get_beam_file ()
+  end function event_get_beam_file
+
+  function event_get_process_name (event) result (name)
+    class(event_t), intent(in) :: event
+    type(string_t) :: name
+    name = event%instance%get_process_name ()
+  end function event_get_process_name
+
   elemental function event_get_actual_calls_total (event) result (n)
     class(event_t), intent(in) :: event
     integer :: n
@@ -853,7 +979,7 @@ contains
        n = 0
     end if
   end function event_get_actual_calls_total
-  
+
   subroutine pacify_event (event)
     class(event_t), intent(inout) :: event
     class(evt_t), pointer :: evt
