@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -41,12 +41,14 @@ module events_uti
   use model_data
   use particles
   use process_libraries
-  use processes
   use process_stacks
   use event_transforms
   use decays
   use decays_ut, only: prepare_testbed
 
+  use process, only: process_t
+  use instances, only: process_instance_t
+
   use events
 
   implicit none
@@ -77,7 +79,7 @@ contains
   end subroutine events_1
 
   subroutine events_2 (u)
-    use processes_ut, only: prepare_test_process
+    use processes_ut, only: prepare_test_process, cleanup_test_process
     integer, intent(in) :: u
     type(event_t), allocatable, target :: event
     type(process_t), allocatable, target :: process
@@ -107,7 +109,7 @@ contains
     write (u, "(A)")
     write (u, "(A)")  "* Generate test process event"
 
-    call process%generate_weighted_event (process_instance, 1)
+    call process_instance%generate_weighted_event (1)
 
     write (u, "(A)")
     write (u, "(A)")  "* Fill event object"
@@ -135,7 +137,7 @@ contains
   end subroutine events_2
 
   subroutine events_4 (u)
-    use processes_ut, only: prepare_test_process
+    use processes_ut, only: prepare_test_process, cleanup_test_process
     integer, intent(in) :: u
     type(event_t), allocatable, target :: event
     type(process_t), allocatable, target :: process
@@ -232,7 +234,7 @@ contains
   end subroutine events_4
 
   subroutine events_5 (u)
-    use processes_ut, only: prepare_test_process
+    use processes_ut, only: prepare_test_process, cleanup_test_process
     integer, intent(in) :: u
     type(event_t), allocatable, target :: event
     type(process_t), allocatable, target :: process
@@ -440,7 +442,7 @@ contains
     call process_instance%setup_event_data (model)
     call process_instance%init_simulation (1)
 
-    call process%generate_weighted_event (process_instance, 1)
+    call process_instance%generate_weighted_event (1)
     call process_instance%write (u)
 
     call process_instance%final ()
@@ -459,7 +461,7 @@ contains
     call process_instance%setup_event_data (model)
     call process_instance%init_simulation (1)
 
-    call process%generate_weighted_event (process_instance, 1)
+    call process_instance%generate_weighted_event (1)
     call process_instance%write (u)
 
     call process_instance%final ()
@@ -481,7 +483,7 @@ contains
     call process_instance%setup_event_data (model)
     call process_instance%init_simulation (1)
 
-    call process%generate_weighted_event (process_instance, 1)
+    call process_instance%generate_weighted_event (1)
     call process_instance%write (u)
 
     call process_instance%final ()
