@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -57,7 +57,8 @@ module shower
   use event_transforms
   use models
   use hep_common
-  use processes
+  use process, only: process_t
+  use instances, only: process_instance_t
   use process_stacks
 
   implicit none
@@ -77,6 +78,7 @@ module shower
      type(os_data_t) :: os_data
      logical :: is_first_event
    contains
+     procedure :: write_name => evt_shower_write_name
      procedure :: write => evt_shower_write
      procedure :: connect => evt_shower_connect
      procedure :: init => evt_shower_init
@@ -94,6 +96,14 @@ module shower
 
 contains
 
+  subroutine evt_shower_write_name (evt, unit)
+    class(evt_shower_t), intent(in) :: evt
+    integer, intent(in), optional :: unit
+    integer :: u
+    u = given_output_unit (unit)
+    write (u, "(1x,A)")  "Event transform: shower"
+  end subroutine evt_shower_write_name
+
   subroutine evt_shower_write (evt, unit, verbose, more_verbose, testflag)
     class(evt_shower_t), intent(in) :: evt
     integer, intent(in), optional :: unit
@@ -101,7 +111,7 @@ contains
     integer :: u
     u = given_output_unit (unit)
     call write_separator (u, 2)
-    write (u, "(1x,A)")  "Event transform: shower"
+    call evt%write_name (u)
     call write_separator (u)
     call evt%base_write (u, testflag = testflag, show_set = .false.)
     if (evt%particle_set_exists)  call evt%particle_set%write &
@@ -157,9 +167,14 @@ contains
     call msg_debug (D_TRANSFORMS, "evt_shower_first_event")
     associate (settings => evt%shower%settings)
        settings%hadron_collision = .false.
-       if (all (evt%particle_set%prt(1:2)%flv%get_pdg_abs () <= 18)) then
+       !!! !!! !!! Workaround for PGF90 v16.1
+       !!! if (all (evt%particle_set%prt(1:2)%flv%get_pdg_abs () <= 39)) then
+       if (evt%particle_set%prt(1)%flv%get_pdg_abs () <= 39 .and. &
+           evt%particle_set%prt(2)%flv%get_pdg_abs () <= 39) then
           settings%hadron_collision = .false.
-       else if (all (evt%particle_set%prt(1:2)%flv%get_pdg_abs () >= 1000)) then
+       !!! else if (all (evt%particle_set%prt(1:2)%flv%get_pdg_abs () >= 100)) then
+       else if (evt%particle_set%prt(1)%flv%get_pdg_abs () >= 100 .and. &
+                evt%particle_set%prt(2)%flv%get_pdg_abs () >= 100) then
           settings%hadron_collision = .true.
        else
           call msg_fatal ("evt_shower didn't recognize beams setup")
@@ -195,14 +210,19 @@ contains
     class(evt_shower_t), intent(inout) :: evt
     real(default), intent(inout) :: probability
     logical :: valid, vetoed
+    integer :: i_term
+    real(default) :: fac_scale
     call msg_debug (D_TRANSFORMS, "evt_shower_generate_weighted")
     if (signal_is_pending ())  return
+    i_term = 1
     evt%particle_set = evt%previous%particle_set
     valid = .true.;  vetoed = .false.
+    fac_scale = evt%process_instance%get_fac_scale (i_term)
     if (evt%is_first_event) then
        call evt%first_event ()
     end if
-    call evt%shower%import_particle_set (evt%particle_set, evt%os_data)
+    call evt%shower%import_particle_set &
+         (evt%particle_set, evt%os_data, fac_scale)
     if (allocated (evt%matching)) then
        call evt%matching%before_shower (evt%particle_set, vetoed)
        if (msg_level(D_TRANSFORMS) >= DEBUG) then
@@ -265,7 +285,7 @@ contains
      class(evt_shower_t), intent(in) :: evt
      val = .false.
      if (allocated (evt%matching)) &
-        val = evt%matching%get_method () == "POWHEG" 
+        val = evt%matching%get_method () == "POWHEG"
   end function evt_shower_contains_powheg_matching
 
   subroutine evt_shower_disable_powheg_matching (evt)
@@ -273,7 +293,7 @@ contains
      select type (matching => evt%matching)
      type is (powheg_matching_t)
         matching%active = .false.
-     class default 
+     class default
         call msg_fatal ("Trying to disable powheg but no powheg matching is allocated!")
      end select
   end subroutine evt_shower_disable_powheg_matching
@@ -283,7 +303,7 @@ contains
      select type (matching => evt%matching)
      type is (powheg_matching_t)
         matching%active = .true.
-     class default 
+     class default
         call msg_fatal ("Trying to enable powheg but no powheg matching is allocated!")
      end select
   end subroutine evt_shower_enable_powheg_matching
