@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -40,6 +40,7 @@ module shower_uti
   use format_utils, only: write_separator
   use os_interface
   use sm_qcd
+  use physics_defs, only: BORN
   use model_data
   use state_matrices, only: FM_IGNORE_HELICITY
   use process_libraries
@@ -53,8 +54,11 @@ module shower_uti
   use prc_omega
   use variables
   use models
-  use processes
   use event_transforms
+  use tauola_interface !NODEP!
+
+  use process, only: process_t
+  use instances, only: process_instance_t
 
   use pdf
   use shower_base
@@ -137,22 +141,26 @@ contains
     type is (prc_omega_t)
        call core_template%set_parameters (model = model)
     end select
+    call process%core_manager_register (BORN, 1, var_str ("omega"))
+    call process%allocate_cm_arrays (1)
+    call process%allocate_core (1, core_template)
+    call process%init_cores ()
     call process%init_component &
-         (1, core_template, mci_template, phs_config_template)
+       (1, .true., mci_template, phs_config_template)
 
     sqrts = 1000
-    call process%setup_beams_sqrts (sqrts)
+    call process%setup_beams_sqrts (sqrts, i_core = 1)
     call process%configure_phs ()
     call process%setup_mci ()
     call process%setup_terms ()
 
     call process_instance%init (process)
-    call process%integrate (process_instance, 1, 1, 1000)
+    call process_instance%integrate (1, 1, 1000)
     call process%final_integration (1)
 
-    call process_instance%setup_event_data ()
+    call process_instance%setup_event_data (i_core = 1)
     call process_instance%init_simulation (1)
-    call process%generate_weighted_event (process_instance, 1)
+    call process_instance%generate_weighted_event (1)
     call process_instance%evaluate_event_data ()
 
   end subroutine setup_testbed
@@ -172,6 +180,7 @@ contains
     class(evt_t), allocatable, target :: evt_trivial
     class(evt_t), allocatable, target :: evt_shower
     type(shower_settings_t) :: settings
+    type(taudec_settings_t) :: taudec_settings
 
     write (u, "(A)")  "* Test output: shower_1"
     write (u, "(A)")  "*   Purpose: Two-jet event with disabled shower"
@@ -216,7 +225,7 @@ contains
     type is (evt_shower_t)
        call evt_shower%init (model_hadrons, os_data)
        allocate (shower_t :: evt_shower%shower)
-       call evt_shower%shower%init (settings, pdf_data)
+       call evt_shower%shower%init (settings, taudec_settings, pdf_data)
        call evt_shower%connect (process_instance, model)
     end select
 
@@ -265,6 +274,7 @@ contains
     class(evt_t), allocatable, target :: evt_trivial
     class(evt_t), allocatable, target :: evt_shower
     type(shower_settings_t) :: settings
+    type(taudec_settings_t) :: taudec_settings
 
     write (u, "(A)")  "* Test output: shower_2"
     write (u, "(A)")  "*   Purpose: Two-jet event with FSR shower"
@@ -311,7 +321,7 @@ contains
     type is (evt_shower_t)
        call evt_shower%init (model_hadrons, os_data)
        allocate (shower_t :: evt_shower%shower)
-       call evt_shower%shower%init (settings, pdf_data)
+       call evt_shower%shower%init (settings, taudec_settings, pdf_data)
        call evt_shower%connect (process_instance, model)
     end select
 
