@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -36,7 +36,7 @@
 module subevents
 
   use, intrinsic :: iso_c_binding !NODEP!
-  
+
   use kinds, only: default
   use io_units
   use format_defs, only: FMT_14, FMT_19
@@ -94,7 +94,7 @@ module subevents
   public :: subevt_select_pdg_code
   public :: pacify
 
-  integer, parameter, public :: PRT_UNDEFINED = 0 
+  integer, parameter, public :: PRT_UNDEFINED = 0
   integer, parameter, public :: PRT_BEAM = -9
   integer, parameter, public :: PRT_INCOMING = 1
   integer, parameter, public :: PRT_OUTGOING = 2
@@ -151,7 +151,7 @@ module subevents
      module procedure pacify_prt
      module procedure pacify_subevt
   end interface pacify
-  
+
 
 contains
 
@@ -164,7 +164,7 @@ contains
     prt%type = PRT_BEAM
     call prt_set (prt, pdg, - p, p2, src)
   end subroutine prt_init_beam
-    
+
   subroutine prt_init_incoming (prt, pdg, p, p2, src)
     type(prt_t), intent(out) :: prt
     integer, intent(in) :: pdg
@@ -174,7 +174,7 @@ contains
     prt%type = PRT_INCOMING
     call prt_set (prt, pdg, - p, p2, src)
   end subroutine prt_init_incoming
-    
+
   subroutine prt_init_outgoing (prt, pdg, p, p2, src)
     type(prt_t), intent(out) :: prt
     integer, intent(in) :: pdg
@@ -184,7 +184,7 @@ contains
     prt%type = PRT_OUTGOING
     call prt_set (prt, pdg, p, p2, src)
   end subroutine prt_init_outgoing
-    
+
   subroutine prt_init_composite (prt, p, src)
     type(prt_t), intent(out) :: prt
     type(vector4_t), intent(in) :: p
@@ -203,17 +203,18 @@ contains
     call prt_set (prt, 0, p, p**2, src)
   end subroutine prt_init_combine
 
-  subroutine prt_init_pseudojet (prt, jet, src)
+  subroutine prt_init_pseudojet (prt, jet, src, pdg)
     type(prt_t), intent(out) :: prt
     type(pseudojet_t), intent(in) :: jet
     integer, dimension(:), intent(in) :: src
+    integer, intent(in) :: pdg
     type(vector4_t) :: p
     prt%type = PRT_COMPOSITE
     p = vector4_moving (jet%e(), &
          vector3_moving ([jet%px(), jet%py(), jet%pz()]))
-    call prt_set (prt, 0, p, p**2, src)
+    call prt_set (prt, pdg, p, p**2, src)
   end subroutine prt_init_pseudojet
-  
+
   elemental function prt_get_pdg (prt) result (pdg)
     integer :: pdg
     type(prt_t), intent(in) :: prt
@@ -309,7 +310,7 @@ contains
     logical, intent(in), optional :: testflag
     logical :: pacified
     type(prt_t) :: tmp
-    character(len=7) :: fmt   
+    character(len=7) :: fmt
     integer :: u, i
     call pac_fmt (fmt, FMT_19, FMT_14, testflag)
     u = given_output_unit (unit);  if (u < 0)  return
@@ -688,19 +689,19 @@ contains
     end do
     sqrts_hat = p ** 1
   end function subevt_get_sqrts_hat
-    
+
   function subevt_get_n_in (subevt) result (n_in)
     type(subevt_t), intent(in) :: subevt
     integer :: n_in
     n_in = count (subevt%prt(:subevt%n_active)%type == PRT_INCOMING)
   end function subevt_get_n_in
-    
+
   function subevt_get_n_out (subevt) result (n_out)
     type(subevt_t), intent(in) :: subevt
     integer :: n_out
     n_out = count (subevt%prt(:subevt%n_active)%type == PRT_OUTGOING)
   end function subevt_get_n_out
-    
+
   function c_prt_from_subevt (subevt, i) result (c_prt)
     type(c_prt_t) :: c_prt
     type(subevt_t), intent(in) :: subevt
@@ -811,17 +812,18 @@ contains
     end do
   end subroutine subevt_collect
 
-  subroutine subevt_cluster (subevt, pl1, mask1, jet_def)
+  subroutine subevt_cluster (subevt, pl1, mask1, jet_def, keep_jets)
     type(subevt_t), intent(inout) :: subevt
     type(subevt_t), intent(in) :: pl1
     logical, dimension(:), intent(in) :: mask1
     type(jet_definition_t), intent(in) :: jet_def
+    logical, intent(in) :: keep_jets
     integer, dimension(:), allocatable :: src, src_tmp
     integer, dimension(:), allocatable :: map, jet_idx
     type(pseudojet_t), dimension(:), allocatable :: jet_in, jet_out
     type(pseudojet_vector_t) :: jv_in, jv_out
     type(cluster_sequence_t) :: cs
-    integer :: i, j, k, n_src, n_active
+    integer :: i, prt_idx, k, n_src, n_active, this_jet, combined_pdg, pdg, n_quarks
     n_active = 0
     allocate (map (pl1%n_active), source = 0)
     allocate (src (0))
@@ -847,20 +849,33 @@ contains
     allocate (jet_out (jv_out%size ()))
     jet_out = jv_out
     call subevt_reset (subevt, size (jet_out))
-    do i = 1, size (jet_out)
+    do this_jet = 1, size (jet_out)
        src = 0
        n_src = 0
-       do j = 1, size (jet_idx)
-          if (jet_idx(j) == i) then
-             associate (prt => pl1%prt(map(j)))
+       combined_pdg = 0
+       n_quarks = 0
+       do prt_idx = 1, size (jet_idx)
+          if (jet_idx(prt_idx) == this_jet) then
+             associate (prt => pl1%prt(map(prt_idx)))
                do k = 1, size (prt%src)
                   src(n_src + k) = prt%src(k)
                end do
                n_src = n_src + size (prt%src)
+               if (is_quark (prt%pdg)) then
+                  n_quarks = n_quarks + 1
+                  if (combined_pdg == 0) then
+                     combined_pdg = prt%pdg
+                  end if
+               end if
              end associate
           end if
        end do
-       call prt_init_pseudojet (subevt%prt(i), jet_out(i), src(:n_src))
+       if (keep_jets .and. n_quarks == 1) then
+          pdg = combined_pdg
+       else
+          pdg = 0
+       end if
+       call prt_init_pseudojet (subevt%prt(this_jet), jet_out(this_jet), src(:n_src), pdg)
     end do
     do i = 1, size (jet_out)
        call jet_out(i)%final ()
@@ -933,9 +948,18 @@ contains
     type(subevt_t), intent(inout) :: subevt
     type(subevt_t), intent(in) :: pl
     real(default), dimension(:), intent(in) :: rval
+    integer, dimension(size(rval)) :: idx
+    integer :: i
     call subevt_reset (subevt, pl%n_active)
     subevt%n_active = pl%n_active
-    subevt%prt = pl%prt( order (rval) )
+    if (allocated (subevt%prt))  deallocate (subevt%prt)
+    allocate (subevt%prt (size(pl%prt)))
+    !!! !!! !!! Workaround for the ifort 16.0/1
+    !subevt%prt = pl%prt(order (rval))
+    idx = order (rval)
+    do i = 1, size(idx)
+       subevt%prt(i) = pl%prt(idx(i))
+    end do
   end subroutine subevt_sort_real
 
   subroutine subevt_select_pdg_code (subevt, aval, subevt_in, prt_type)
@@ -972,7 +996,7 @@ contains
     call pacify (prt%p, e)
     call pacify (prt%p2, 1E3_default * e)
   end subroutine pacify_prt
-  
+
   subroutine pacify_subevt (subevt)
     class(subevt_t), intent(inout) :: subevt
     integer :: i
