@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -34,7 +34,7 @@
 ! to the source 'whizard.nw'
 
 module event_streams
-  
+
   use iso_varying_string, string_t => varying_string
   use io_units
   use diagnostics
@@ -43,8 +43,8 @@ module event_streams
   use eio_base
   use rt_data
 
-  use dispatch, only: dispatch_eio
-  
+  use dispatch_transforms, only: dispatch_eio
+
   implicit none
   private
 
@@ -53,7 +53,7 @@ module event_streams
   type :: event_stream_entry_t
      class(eio_t), allocatable :: eio
   end type event_stream_entry_t
-  
+
   type :: event_stream_array_t
      type(event_stream_entry_t), dimension(:), allocatable :: entry
      integer :: i_in = 0
@@ -68,7 +68,7 @@ module event_streams
      procedure :: skip_eio_entry => event_stream_array_skip_eio_entry
      procedure :: has_input => event_stream_array_has_input
   end type event_stream_array_t
-  
+
 
 contains
 
@@ -157,16 +157,22 @@ contains
     allocate (es_array%entry (n))
     if (i_checkpoint > 0) then
        call dispatch_eio &
-            (es_array%entry(i_checkpoint)%eio, var_str ("checkpoint"), global)
+            (es_array%entry(i_checkpoint)%eio, var_str ("checkpoint"), &
+            global%var_list, global%fallback_model, &
+            global%event_callback)
        call es_array%entry(i_checkpoint)%eio%init_out (sample, data)
     end if
     if (i_callback > 0) then
        call dispatch_eio &
-            (es_array%entry(i_callback)%eio, var_str ("callback"), global)
+            (es_array%entry(i_callback)%eio, var_str ("callback"), &
+            global%var_list, global%fallback_model, &
+            global%event_callback)
        call es_array%entry(i_callback)%eio%init_out (sample, data)
     end if
     if (i_input > 0) then
-       call dispatch_eio (es_array%entry(i_input)%eio, input, global)
+       call dispatch_eio (es_array%entry(i_input)%eio, input, &
+            global%var_list, global%fallback_model, &
+            global%event_callback)
        if (present (input_data)) then
           call es_array%entry(i_input)%eio%init_in &
                (sample_in, input_data, success)
@@ -194,11 +200,13 @@ contains
        end if
     end if
     do i = 1, n_output
-       call dispatch_eio (es_array%entry(i)%eio, stream_fmt(i), global)
+       call dispatch_eio (es_array%entry(i)%eio, stream_fmt(i), &
+            global%var_list, global%fallback_model, &
+            global%event_callback)
        call es_array%entry(i)%eio%init_out (sample, data)
     end do
   end subroutine event_stream_array_init
-  
+
   subroutine event_stream_array_switch_inout (es_array)
     class(event_stream_array_t), intent(inout) :: es_array
     integer :: n
@@ -210,7 +218,7 @@ contains
        call msg_bug ("Reading events: switch_inout: no input stream selected")
     end if
   end subroutine event_stream_array_switch_inout
-  
+
   subroutine event_stream_array_output (es_array, event, i_prc, &
                                         event_index, passed, pacify)
     class(event_stream_array_t), intent(inout) :: es_array
@@ -239,7 +247,7 @@ contains
        end if
     end do
   end subroutine event_stream_array_output
-  
+
   subroutine event_stream_array_input_i_prc (es_array, i_prc, iostat)
     class(event_stream_array_t), intent(inout) :: es_array
     integer, intent(out) :: i_prc
@@ -252,7 +260,7 @@ contains
        call msg_fatal ("Reading events: no input stream selected")
     end if
   end subroutine event_stream_array_input_i_prc
-  
+
   subroutine event_stream_array_input_event (es_array, event, iostat)
     class(event_stream_array_t), intent(inout) :: es_array
     type(event_t), intent(inout), target :: event
@@ -265,7 +273,7 @@ contains
        call msg_fatal ("Reading events: no input stream selected")
     end if
   end subroutine event_stream_array_input_event
-  
+
   subroutine event_stream_array_skip_eio_entry (es_array, iostat)
     class(event_stream_array_t), intent(inout) :: es_array
     integer, intent(out) :: iostat
@@ -283,6 +291,6 @@ contains
     logical :: flag
     flag = es_array%i_in /= 0
   end function event_stream_array_has_input
-  
+
 
 end module event_streams
