@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -36,7 +36,6 @@
 program main
 
   use iso_varying_string, string_t => varying_string
-  use unit_tests
   use system_dependencies
   use diagnostics
   use ifiles
@@ -50,7 +49,13 @@ program main
 
   integer, parameter :: CMDLINE_ARG_LEN = 1000
 
-  ! Main program variable declarations
+!!! (WK 02/2016) Interface for the separate external routine below
+  interface
+     subroutine print_usage ()
+     end subroutine print_usage
+  end interface
+
+! Main program variable declarations
   character(CMDLINE_ARG_LEN) :: arg
   character(2) :: option
   type(string_t) :: long_option, value
@@ -84,7 +89,7 @@ program main
   model = "SM"
   default_lib = "default_lib"
   library = ""
-  libraries = ""  
+  libraries = ""
   banner = .true.
   logging = .true.
   msg_level = RESULT
@@ -171,7 +176,7 @@ program main
            case ("--no-library")
               call no_option_value (long_option, value)
               default_lib = ""
-              library = ""              
+              library = ""
               libraries = ""
               cycle SCAN_CMDLINE
            case ("--localprefix")
@@ -195,20 +200,16 @@ program main
            case ("--debug")
               call no_option_value (long_option, value)
               area = d_area (get_option_value (i, long_option, value))
-              if (area == D_ALL) then
-                 msg_level = DEBUG
-              else
-                 msg_level(area) = DEBUG
-              end if
+              msg_level(area) = DEBUG
               cycle SCAN_CMDLINE
            case ("--debug2")
               call no_option_value (long_option, value)
               area = d_area (get_option_value (i, long_option, value))
-              if (area == D_ALL) then
-                 msg_level = DEBUG2
-              else
-                 msg_level(area) = DEBUG2
-              end if
+              msg_level(area) = DEBUG2
+              cycle SCAN_CMDLINE
+           case ("--single-event")
+              call no_option_value (long_option, value)
+              single_event = .true.
               cycle SCAN_CMDLINE
            case ("--banner")
               call no_option_value (long_option, value)
@@ -331,7 +332,7 @@ program main
                     else
                        library = trim (arg(j+1:))
                     end if
-                    libraries = libraries // " " // library                    
+                    libraries = libraries // " " // library
                     cycle SCAN_CMDLINE
                  case ("-L")
                     if (j == len_trim (arg)) then
@@ -373,7 +374,7 @@ program main
   ! Overall initialization
   if (logfile /= "")  call logfile_init (logfile)
   if (banner)  call msg_banner ()
-   
+
    allocate (options)
    allocate (whizard_instance)
 
@@ -385,13 +386,13 @@ program main
       options%preload_libraries = libraries
       options%rebuild_library = rebuild_library
       options%recompile_library = recompile_library
-      options%rebuild_user = rebuild_user 
+      options%rebuild_user = rebuild_user
       options%rebuild_phs = rebuild_phs
       options%rebuild_grids = rebuild_grids
       options%rebuild_events = rebuild_events
-      
+
       call whizard_instance%init (options, paths, logfile)
-      
+
       call mask_term_signals ()
 
    end if
@@ -417,7 +418,7 @@ program main
 
       end if
   end if
- 
+
   ! Enter an interactive shell if requested
   if (.not. quit .and. interactive) then
      call whizard_instance%shell (quit_code)
@@ -430,7 +431,7 @@ program main
 
   call whizard_instance%final ()
   deallocate (whizard_instance)
-  
+
   call terminate_now_if_signal ()
   call release_term_signals ()
   call msg_terminate (quit_code = quit_code)
@@ -439,15 +440,22 @@ program main
 contains
 
   subroutine print_version ()
-    print "(A)", "WHIZARD " // WHIZARD_VERSION 
-    print "(A)", "Copyright (C) 1999-2015 Wolfgang Kilian, Thorsten Ohl, Juergen Reuter"
+    print "(A)", "WHIZARD " // WHIZARD_VERSION
+    print "(A)", "Copyright (C) 1999-2017 Wolfgang Kilian, Thorsten Ohl, Juergen Reuter"
     print "(A)", "              ---------------------------------------                "
     print "(A)", "This is free software; see the source for copying conditions.  There is NO"
     print "(A)", "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
     print *
   end subroutine print_version
 
+end program main
+
+!!! (WK 02/2016)
+!!! Separate subroutine, because this becomes a procedure pointer target
+!!! Internal procedures as targets are not supported by some compilers.
+
   subroutine print_usage ()
+    use system_dependencies, only: WHIZARD_VERSION
     print "(A)", "WHIZARD " // WHIZARD_VERSION
     print "(A)", "Usage: whizard [OPTIONS] [FILE]"
     print "(A)", "Run WHIZARD with the command list taken from FILE(s)"
@@ -467,6 +475,7 @@ contains
     print "(A)", "    --debug AREA      switch on debug output for AREA."
     print "(A)", "                      AREA can be one of Whizard's src dirs or 'all'"
     print "(A)", "    --debug2 AREA     switch on more verbose debug output for AREA."
+    print "(A)", "    --single-event    only compute one phase-space point (for debugging)"
     print "(A)", "-e, --execute CMDS    execute SINDARIN CMDS before reading FILE(s)"
     print "(A)", "-i, --interactive     run interactively after reading FILE(s)"
     print "(A)", "-l, --library         preload process library NAME"
@@ -503,4 +512,3 @@ contains
     print "(A)", "With no FILE, read standard input."
   end subroutine print_usage
 
-end program main
