@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -37,21 +37,152 @@ program main_ut
 
   use iso_varying_string, string_t => varying_string
   use unit_tests
+  use io_units
   use system_dependencies
   use diagnostics
   use os_interface
 
   use cmdline_options
 
+  use model_testbed !NODEP!
+
+  use eio_base_ut, only: eio_prepare_test
+  use eio_base_ut, only: eio_cleanup_test
+  use eio_base_ut, only: eio_prepare_fallback_model
+  use eio_base_ut, only: eio_cleanup_fallback_model
+  use dispatch_rng, only: dispatch_rng_factory_extra
+  use dispatch_rng_ut, only: dispatch_rng_factory_test
+  use dispatch_beams, only: dispatch_sf_data_extra
+  use dispatch_ut, only: dispatch_sf_data_test
+  use formats_ut, only: format_test
+  use md5_ut, only: md5_test
+  use os_interface_ut, only: os_interface_test
+  use sorting_ut, only: sorting_test
+  use grids_ut, only: grids_test
+  use solver_ut, only: solver_test
+  use cputime_ut, only: cputime_test
+  use sm_qcd_ut, only: sm_qcd_test
+  use sm_physics_ut, only: sm_physics_test
+  use lexers_ut, only: lexer_test
+  use parser_ut, only: parse_test
+  use xml_ut, only: xml_test
+  use colors_ut, only: color_test
+  use state_matrices_ut, only: state_matrix_test
+  use analysis_ut, only: analysis_test
+  use particles_ut, only: particles_test
+  use models_ut, only: models_test
+  use auto_components_ut, only: auto_components_test
+  use radiation_generator_ut, only: radiation_generator_test
+  use blha_ut, only: blha_test
+  use evaluators_ut, only: evaluator_test
+  use eval_trees_ut, only: expressions_test
+  use resonances_ut, only: resonances_test
+  use phs_trees_ut, only: phs_trees_test
+  use phs_forests_ut, only: phs_forests_test
+  use beams_ut, only: beams_test
+  use su_algebra_ut, only: su_algebra_test
+  use bloch_vectors_ut, only: bloch_vectors_test
+  use polarizations_ut, only: polarizations_test
+  use sf_aux_ut, only: sf_aux_test
+  use sf_mappings_ut, only: sf_mappings_test
+  use sf_base_ut, only: sf_base_test
+  use sf_pdf_builtin_ut, only: sf_pdf_builtin_test
+  use sf_lhapdf_ut, only: sf_lhapdf_test
+  use sf_isr_ut, only: sf_isr_test
+  use sf_epa_ut, only: sf_epa_test
+  use sf_ewa_ut, only: sf_ewa_test
+  use sf_circe1_ut, only: sf_circe1_test
+  use sf_circe2_ut, only: sf_circe2_test
+  use sf_gaussian_ut, only: sf_gaussian_test
+  use sf_beam_events_ut, only: sf_beam_events_test
+  use sf_escan_ut, only: sf_escan_test
+  use phs_base_ut, only: phs_base_test
+  use phs_single_ut, only: phs_single_test
+  use phs_wood_ut, only: phs_wood_test
+  use phs_wood_ut, only: phs_wood_vis_test
+  use phs_fks_ut, only: phs_fks_generator_test
+  use fks_regions_ut, only: fks_regions_test
+  use prc_recola_ut, only: prc_recola_test
+  use nlo_color_data_ut, only: nlo_color_data_test
+  use rng_base_ut, only: rng_base_test
+  use rng_tao_ut, only: rng_tao_test
+  use rng_stream_ut, only: rng_stream_test
+  use selectors_ut, only: selectors_test
+  use vegas_ut, only: vegas_test
+  use vamp2_ut, only: vamp2_test
+  use mci_base_ut, only: mci_base_test
+  use mci_midpoint_ut, only: mci_midpoint_test
+  use mci_vamp_ut, only: mci_vamp_test
+  use mci_vamp2_ut, only: mci_vamp2_test
+  use prclib_interfaces_ut, only: prclib_interfaces_test
+  use particle_specifiers_ut, only: particle_specifiers_test
+  use process_libraries_ut, only: process_libraries_test
+  use prclib_stacks_ut, only: prclib_stacks_test
+  use hepmc_interface_ut, only: hepmc_interface_test
+  use lcio_interface_ut, only: lcio_interface_test
+  use jets_ut, only: jets_test
+  use pdg_arrays_ut, only: pdg_arrays_test
+  use interactions_ut, only: interaction_test
+  use slha_interface_ut, only: slha_test
+  use cascades_ut, only: cascades_test
+  use prc_test_ut, only: prc_test_test
+  use prc_template_me_ut, only: prc_template_me_test
+  use prc_omega_ut, only: prc_omega_test
+  use prc_omega_ut, only: prc_omega_diags_test
+  use parton_states_ut, only: parton_states_test
+  use expr_tests_ut, only: subevt_expr_test
+  use processes_ut, only: processes_test
+  use process_stacks_ut, only: process_stacks_test
+  use event_transforms_ut, only: event_transforms_test
+  use decays_ut, only: decays_test
+  use shower_ut, only: shower_test
+  use events_ut, only: events_test
+  use hep_events_ut, only: hep_events_test
+  use eio_data_ut, only: eio_data_test
+  use eio_base_ut, only: eio_base_test
+  use eio_raw_ut, only: eio_raw_test
+  use eio_checkpoints_ut, only: eio_checkpoints_test
+  use eio_lhef_ut, only: eio_lhef_test
+  use eio_hepmc_ut, only: eio_hepmc_test
+  use eio_lcio_ut, only: eio_lcio_test
+  use eio_stdhep_ut, only: eio_stdhep_test
+  use eio_ascii_ut, only: eio_ascii_test
+  use eio_weights_ut, only: eio_weights_test
+  use eio_dump_ut, only: eio_dump_test
+  use iterations_ut, only: iterations_test
+  use beam_structures_ut, only: beam_structures_test
+  use rt_data_ut, only: rt_data_test
+  use dispatch_ut, only: dispatch_test
+  use dispatch_rng_ut, only: dispatch_rng_test
+  use dispatch_mci_ut, only: dispatch_mci_test
+  use dispatch_phs_ut, only: dispatch_phs_test
+  use dispatch_transforms_ut, only: dispatch_transforms_test
+  use shower_base_ut, only: shower_base_test
+  use process_configurations_ut, only: process_configurations_test
+  use compilations_ut, only: compilations_test
+  use compilations_ut, only: compilations_static_test
+  use integrations_ut, only: integrations_test
+  use integrations_ut, only: integrations_history_test
+  use event_streams_ut, only: event_streams_test
+  use simulations_ut, only: simulations_test
+  use commands_ut, only: commands_test
+  use ttv_formfactors_ut, only: ttv_formfactors_test
+
   implicit none
 
   integer, parameter :: CMDLINE_ARG_LEN = 1000
 
+!!! (WK 02/2016) Interface for the separate external routine below
+  interface
+     subroutine print_usage ()
+     end subroutine print_usage
+  end interface
+
   ! Main program variable declarations
   character(CMDLINE_ARG_LEN) :: arg
   character(2) :: option
   type(string_t) :: long_option, value
-  integer :: i, j, arg_len, arg_status
+  integer :: i, j, arg_len, arg_status, area
   logical :: look_for_options
   logical :: banner
   type(string_t) :: check, checks
@@ -59,7 +190,6 @@ program main_ut
   logical :: success
 
   ! Exit status
-  logical :: quit = .false.
   integer :: quit_code = 0
 
   ! Initial values
@@ -107,6 +237,16 @@ program main_ut
               check = get_option_value (i, long_option, value)
               checks = checks // " " // check
               cycle SCAN_CMDLINE
+           case ("--debug")
+              call no_option_value (long_option, value)
+              area = d_area (get_option_value (i, long_option, value))
+              msg_level(area) = DEBUG
+              cycle SCAN_CMDLINE
+           case ("--debug2")
+              call no_option_value (long_option, value)
+              area = d_area (get_option_value (i, long_option, value))
+              msg_level(area) = DEBUG
+              cycle SCAN_CMDLINE
            case default
               call print_usage ()
               call msg_fatal ("Option '" // trim (arg) // "' not recognized")
@@ -146,7 +286,7 @@ program main_ut
 
   ! Overall initialization
   if (banner)  call msg_banner ()
-  
+
    ! Run any self-checks (and no commands)
    if (checks /= "") then
       checks = trim (adjustl (checks))
@@ -156,7 +296,6 @@ program main_ut
       end do RUN_CHECKS
       call test_results%wrapup (6, success)
       if (.not. success)  quit_code = 7
-      quit = .true.
    end if
 
    call msg_terminate (quit_code = quit_code)
@@ -166,50 +305,39 @@ contains
 
   subroutine print_version ()
     print "(A)", "WHIZARD " // WHIZARD_VERSION // " (unit test driver)"
-    print "(A)", "Copyright (C) 1999-2015 Wolfgang Kilian, Thorsten Ohl, Juergen Reuter"
+    print "(A)", "Copyright (C) 1999-2017 Wolfgang Kilian, Thorsten Ohl, Juergen Reuter"
     print "(A)", "              ---------------------------------------                "
     print "(A)", "This is free software; see the source for copying conditions.  There is NO"
     print "(A)", "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
     print *
   end subroutine print_version
 
-  subroutine print_usage ()
-    print "(A)", "WHIZARD " // WHIZARD_VERSION // " (unit test driver)"
-    print "(A)", "Usage: whizard_ut [OPTIONS] [FILE]"
-    print "(A)", "Run WHIZARD unit tests as given on the command line"
-    print "(A)", "Options:"
-    print "(A)", "-h, --help            display this help and exit"
-    print "(A)", "    --banner          display banner at startup (default)"
-    print "(A)", "    --no-banner       do not display banner at startup"
-    print "(A)", "-V, --version         output version information and exit"
-    print "(A)", "    --check TEST      run unit test TEST"
-  end subroutine print_usage
-
   subroutine prepare_eio_test (event, unweighted, n_alt)
-    use variables
+    use variables, only: var_list_t
     use model_data
-    use processes
+    use process, only: process_t
+    use instances, only: process_instance_t
     use processes_ut, only: prepare_test_process
     use event_base
     use events
-    
+
     class(generic_event_t), intent(inout), pointer :: event
     logical, intent(in), optional :: unweighted
     integer, intent(in), optional :: n_alt
     type(model_data_t), pointer :: model
     type(var_list_t) :: var_list
-    type(process_t), pointer :: process
+    type(process_t), pointer :: proc
     type(process_instance_t), pointer :: process_instance
 
     allocate (model)
     call model%init_test ()
 
-    allocate (process)
+    allocate (proc)
     allocate (process_instance)
 
-    call prepare_test_process (process, process_instance, model)
+    call prepare_test_process (proc, process_instance, model)
     call process_instance%setup_event_data ()
- 
+
     call model%final ()
     deallocate (model)
 
@@ -217,47 +345,49 @@ contains
     select type (event)
     type is (event_t)
        if (present (unweighted)) then
-          call var_list_append_log (var_list, &
+          call var_list%append_log (&
                var_str ("?unweighted"), unweighted, &
                intrinsic = .true.)
        else
-          call var_list_append_log (var_list, &
+          call var_list%append_log (&
                var_str ("?unweighted"), .true., &
                intrinsic = .true.)
        end if
-       call var_list_append_string (var_list, &
+       call var_list%append_string (&
             var_str ("$sample_normalization"), &
             var_str ("auto"), intrinsic = .true.)
        call event%basic_init (var_list, n_alt)
-       call event%connect (process_instance, process%get_model_ptr ())
+       call event%connect (process_instance, proc%get_model_ptr ())
        call var_list%final ()
     end select
 
   end subroutine prepare_eio_test
-    
+
   subroutine cleanup_eio_test (event)
     use model_data
-    use processes
+    use process, only: process_t
+    use instances, only: process_instance_t
+    use processes_ut, only: cleanup_test_process
     use event_base
     use events
-    
+
     class(generic_event_t), intent(inout), pointer :: event
-    type(process_t), pointer :: process
+    type(process_t), pointer :: proc
     type(process_instance_t), pointer :: process_instance
 
     select type (event)
     type is (event_t)
-       process => event%get_process_ptr ()
+       proc => event%get_process_ptr ()
        process_instance => event%get_process_instance_ptr ()
-       call cleanup_test_process (process, process_instance)
+       call cleanup_test_process (proc, process_instance)
        deallocate (process_instance)
-       deallocate (process)
+       deallocate (proc)
        call event%final ()
     end select
     deallocate (event)
 
   end subroutine cleanup_eio_test
-    
+
   subroutine prepare_whizard_model (model, name, vars)
     use iso_varying_string, string_t => varying_string
     use os_interface
@@ -279,7 +409,7 @@ contains
        end if
     end select
   end subroutine prepare_whizard_model
-    
+
   subroutine cleanup_whizard_model (model)
     use model_data
     use models
@@ -288,137 +418,18 @@ contains
     deallocate (model)
     call syntax_model_file_final ()
   end subroutine cleanup_whizard_model
-    
+
   subroutine prepare_fallback_model (model)
     use model_data
     class(model_data_t), intent(inout), pointer :: model
     call prepare_whizard_model (model, var_str ("SM_hadrons"))
   end subroutine prepare_fallback_model
-    
+
   subroutine whizard_check (check, results)
-    use io_units
-    use model_testbed
-    
-    use eio_base_ut, only: eio_prepare_test
-    use eio_base_ut, only: eio_cleanup_test
-    use eio_base_ut, only: eio_prepare_fallback_model
-    use eio_base_ut, only: eio_cleanup_fallback_model
-    use dispatch, only: dispatch_rng_factory_extra
-    use dispatch_ut, only: dispatch_rng_factory_test
-    use dispatch, only: dispatch_sf_data_extra
-    use dispatch_ut, only: dispatch_sf_data_test
-    use formats_ut, only: format_test
-    use md5_ut, only: md5_test
-    use os_interface_ut, only: os_interface_test
-    use sorting_ut, only: sorting_test
-    use codes_ut, only: codes_test
-    use object_base_ut, only: object_base_test
-    use object_builder_ut, only: object_builder_test
-    use object_logical_ut, only: object_logical_test
-    use object_integer_ut, only: object_integer_test
-    use object_container_ut, only: object_container_test
-    use object_comparison_ut, only: object_comparison_test
-    use object_conditional_ut, only: object_conditional_test
-    use sindarin_parser_ut, only: sindarin_parser_test
-    use grids_ut, only: grids_test
-    use solver_ut, only: solver_test
-    use cputime_ut, only: cputime_test
-    use sm_qcd_ut, only: sm_qcd_test
-    use sm_physics_ut, only: sm_physics_test
-    use lexers_ut, only: lexer_test
-    use parser_ut, only: parse_test
-    use xml_ut, only: xml_test
-    use colors_ut, only: color_test
-    use state_matrices_ut, only: state_matrix_test
-    use analysis_ut, only: analysis_test
-    use particles_ut, only: particles_test
-    use models_ut, only: models_test
-    use auto_components_ut, only: auto_components_test
-    use radiation_generator_ut, only: radiation_generator_test
-    use blha_ut, only: blha_test
-    use evaluators_ut, only: evaluator_test
-    use eval_trees_ut, only: expressions_test
-    use phs_forests_ut, only: phs_forests_test
-    use beams_ut, only: beams_test
-    use polarizations_ut, only: polarizations_test
-    use sf_aux_ut, only: sf_aux_test
-    use sf_mappings_ut, only: sf_mappings_test
-    use sf_base_ut, only: sf_base_test
-    use sf_pdf_builtin_ut, only: sf_pdf_builtin_test
-    use sf_lhapdf_ut, only: sf_lhapdf_test
-    use sf_isr_ut, only: sf_isr_test
-    use sf_epa_ut, only: sf_epa_test
-    use sf_ewa_ut, only: sf_ewa_test
-    use sf_circe1_ut, only: sf_circe1_test
-    use sf_circe2_ut, only: sf_circe2_test
-    use sf_gaussian_ut, only: sf_gaussian_test
-    use sf_beam_events_ut, only: sf_beam_events_test
-    use sf_escan_ut, only: sf_escan_test
-    use phs_base_ut, only: phs_base_test
-    use phs_single_ut, only: phs_single_test
-    use phs_wood_ut, only: phs_wood_test
-    use phs_wood_ut, only: phs_wood_vis_test
-    use phs_fks_ut, only: phs_fks_generator_test
-    use rng_base_ut, only: rng_base_test
-    use rng_tao_ut, only: rng_tao_test
-    use selectors_ut, only: selectors_test
-    use mci_base_ut, only: mci_base_test
-    use mci_midpoint_ut, only: mci_midpoint_test
-    use mci_vamp_ut, only: mci_vamp_test
-    use prclib_interfaces_ut, only: prclib_interfaces_test
-    use particle_specifiers_ut, only: particle_specifiers_test
-    use process_libraries_ut, only: process_libraries_test
-    use prclib_stacks_ut, only: prclib_stacks_test
-    use hepmc_interface_ut, only: hepmc_interface_test
-    use lcio_interface_ut, only: lcio_interface_test
-    use jets_ut, only: jets_test
-    use pdg_arrays_ut, only: pdg_arrays_test
-    use interactions_ut, only: interaction_test
-    use slha_interface_ut, only: slha_test
-    use cascades_ut, only: cascades_test
-    use prc_test_ut, only: prc_test_test
-    use prc_template_me_ut, only: prc_template_me_test
-    use prc_omega_ut, only: prc_omega_test
-    use prc_omega_ut, only: prc_omega_diags_test
-    use expr_tests_ut, only: subevt_expr_test
-    use processes_ut, only: processes_test
-    use process_stacks_ut, only: process_stacks_test
-    use event_transforms_ut, only: event_transforms_test
-    use decays_ut, only: decays_test
-    use powheg_matching_ut, only: powheg_test
-    use shower_ut, only: shower_test
-    use events_ut, only: events_test
-    use hep_events_ut, only: hep_events_test
-    use eio_data_ut, only: eio_data_test
-    use eio_base_ut, only: eio_base_test
-    use eio_raw_ut, only: eio_raw_test
-    use eio_checkpoints_ut, only: eio_checkpoints_test
-    use eio_lhef_ut, only: eio_lhef_test
-    use eio_hepmc_ut, only: eio_hepmc_test
-    use eio_lcio_ut, only: eio_lcio_test
-    use eio_stdhep_ut, only: eio_stdhep_test
-    use eio_ascii_ut, only: eio_ascii_test
-    use eio_weights_ut, only: eio_weights_test
-    use eio_dump_ut, only: eio_dump_test
-    use iterations_ut, only: iterations_test
-    use beam_structures_ut, only: beam_structures_test
-    use rt_data_ut, only: rt_data_test
-    use dispatch_ut, only: dispatch_test
-    use process_configurations_ut, only: process_configurations_test
-    use compilations_ut, only: compilations_test
-    use compilations_ut, only: compilations_static_test
-    use integrations_ut, only: integrations_test
-    use integrations_ut, only: integrations_history_test
-    use event_streams_ut, only: event_streams_test
-    use simulations_ut, only: simulations_test
-    use commands_ut, only: commands_test
-    use ttv_formfactors_ut, only: ttv_formfactors_test
-    
     type(string_t), intent(in) :: check
     type(test_results_t), intent(inout) :: results
     type(os_data_t) :: os_data
     integer :: u
-
     call os_data_init (os_data)
     u = free_unit ()
     open (u, file="whizard_check." // char (check) // ".log", &
@@ -443,24 +454,6 @@ contains
        call os_interface_test (u, results)
     case ("sorting")
        call sorting_test (u, results)
-    case ("codes")
-       call codes_test (u, results)
-    case ("object_base")
-       call object_base_test (u, results)
-    case ("object_builder")
-       call object_builder_test (u, results)
-    case ("object_logical")
-       call object_logical_test (u, results)
-    case ("object_integer")
-       call object_integer_test (u, results)
-    case ("object_container")
-       call object_container_test (u, results)
-    case ("object_comparison")
-       call object_comparison_test (u, results)
-    case ("object_conditional")
-       call object_conditional_test (u, results)
-    case ("sindarin_parser")
-       call sindarin_parser_test (u, results)
     case ("grids")
        call grids_test (u, results)
     case ("solver")
@@ -497,10 +490,18 @@ contains
        call evaluator_test (u, results)
     case ("expressions")
        call expressions_test (u, results)
+    case ("resonances")
+       call resonances_test (u, results)
+    case ("phs_trees")
+       call phs_trees_test (u, results)
     case ("phs_forests")
        call phs_forests_test (u, results)
     case ("beams")
        call beams_test (u, results)
+    case ("su_algebra")
+       call su_algebra_test (u, results)
+    case ("bloch_vectors")
+       call bloch_vectors_test (u, results)
     case ("polarizations")
        call polarizations_test (u, results)
     case ("sf_aux")
@@ -539,18 +540,32 @@ contains
        call phs_wood_vis_test (u, results)
     case ("phs_fks_generator")
        call phs_fks_generator_test (u, results)
+    case ("fks_regions")
+       call fks_regions_test (u, results)
+    case ("prc_recola")
+       call prc_recola_test (u, results)
+    case ("nlo_color_data")
+       call nlo_color_data_test (u, results)
     case ("rng_base")
        call rng_base_test (u, results)
     case ("rng_tao")
        call rng_tao_test (u, results)
+    case ("rng_stream")
+       call rng_stream_test (u, results)
     case ("selectors")
        call selectors_test (u, results)
+    case ("vegas")
+       call vegas_test (u, results)
+    case ("vamp2")
+       call vamp2_test (u, results)
     case ("mci_base")
        call mci_base_test (u, results)
     case ("mci_midpoint")
        call mci_midpoint_test (u, results)
     case ("mci_vamp")
        call mci_vamp_test (u, results)
+    case ("mci_vamp2")
+       call mci_vamp2_test (u, results)
     case ("prclib_interfaces")
        call prclib_interfaces_test (u, results)
     case ("particle_specifiers")
@@ -581,6 +596,8 @@ contains
        call prc_omega_test (u, results)
     case ("prc_omega_diags")
        call prc_omega_diags_test (u, results)
+    case ("parton_states")
+       call parton_states_test (u, results)
     case ("subevt_expr")
        call subevt_expr_test (u, results)
     case ("processes")
@@ -591,8 +608,6 @@ contains
        call event_transforms_test (u, results)
     case ("decays")
        call decays_test (u, results)
-    case ("powheg")
-       call powheg_test (u, results)
     case ("shower")
        call shower_test (u, results)
     case ("events")
@@ -629,6 +644,16 @@ contains
        call rt_data_test (u, results)
     case ("dispatch")
        call dispatch_test (u, results)
+    case ("dispatch_rng")
+       call dispatch_rng_test (u, results)
+    case ("dispatch_mci")
+       call dispatch_mci_test (u, results)
+    case ("dispatch_phs")
+       call dispatch_phs_test (u, results)
+    case ("dispatch_transforms")
+       call dispatch_transforms_test (u, results)
+    case ("shower_base")
+       call shower_base_test (u, results)
     case ("process_configurations")
        call process_configurations_test (u, results)
     case ("compilations")
@@ -652,15 +677,6 @@ contains
        call md5_test (u, results)
        call os_interface_test (u, results)
        call sorting_test (u, results)
-       call codes_test (u, results)
-       call object_base_test (u, results)
-       call object_builder_test (u, results)
-       call object_logical_test (u, results)
-       call object_integer_test (u, results)
-       call object_container_test (u, results)
-       call object_comparison_test (u, results)
-       call object_conditional_test (u, results)
-       call sindarin_parser_test (u, results)
        call grids_test (u, results)
        call solver_test (u, results)
        call cputime_test (u, results)
@@ -679,8 +695,12 @@ contains
        call blha_test (u, results)
        call evaluator_test (u, results)
        call expressions_test (u, results)
+       call resonances_test (u, results)
+       call phs_trees_test (u, results)
        call phs_forests_test (u, results)
        call beams_test (u, results)
+       call su_algebra_test (u, results)
+       call bloch_vectors_test (u, results)
        call polarizations_test (u, results)
        call sf_aux_test (u, results)
        call sf_mappings_test (u, results)
@@ -700,9 +720,15 @@ contains
        call phs_wood_test (u, results)
        call phs_wood_vis_test (u, results)
        call phs_fks_generator_test (u, results)
+       call fks_regions_test (u, results)
+       call prc_recola_test (u, results)
+       call nlo_color_data_test (u, results)
        call rng_base_test (u, results)
        call rng_tao_test (u, results)
+       call rng_stream_test (u, results)
        call selectors_test (u, results)
+       call vegas_test (u, results)
+       call vamp2_test (u, results)
        call mci_base_test (u, results)
        call mci_midpoint_test (u, results)
        call mci_vamp_test (u, results)
@@ -721,12 +747,12 @@ contains
        call prc_template_me_test (u, results)
        call prc_omega_test (u, results)
        call prc_omega_diags_test (u, results)
+       call parton_states_test (u, results)
        call subevt_expr_test (u, results)
        call processes_test (u, results)
        call process_stacks_test (u, results)
        call event_transforms_test (u, results)
        call decays_test (u, results)
-       call powheg_test (u, results)
        call shower_test (u, results)
        call events_test (u, results)
        call hep_events_test (u, results)
@@ -745,6 +771,11 @@ contains
        call beam_structures_test (u, results)
        call rt_data_test (u, results)
        call dispatch_test (u, results)
+       call dispatch_rng_test (u, results)
+       call dispatch_mci_test (u, results)
+       call dispatch_phs_test (u, results)
+       call dispatch_transforms_test (u, results)
+       call shower_base_test (u, results)
        call process_configurations_test (u, results)
        call compilations_test (u, results)
        call compilations_static_test (u, results)
@@ -762,3 +793,23 @@ contains
 
 
 end program main_ut
+
+!!! (WK 02/2016)
+!!! Separate subroutine, because this becomes a procedure pointer target
+!!! Internal procedures as targets are not supported by some compilers.
+
+  subroutine print_usage ()
+    use system_dependencies, only: WHIZARD_VERSION
+    print "(A)", "WHIZARD " // WHIZARD_VERSION // " (unit test driver)"
+    print "(A)", "Usage: whizard_ut [OPTIONS] [FILE]"
+    print "(A)", "Run WHIZARD unit tests as given on the command line"
+    print "(A)", "Options:"
+    print "(A)", "-h, --help            display this help and exit"
+    print "(A)", "    --banner          display banner at startup (default)"
+    print "(A)", "    --no-banner       do not display banner at startup"
+    print "(A)", "    --debug AREA      switch on debug output for AREA."
+    print "(A)", "                      AREA can be one of Whizard's src dirs or 'all'"
+    print "(A)", "    --debug2 AREA     switch on more verbose debug output for AREA."
+    print "(A)", "-V, --version         output version information and exit"
+    print "(A)", "    --check TEST      run unit test TEST"
+  end subroutine print_usage
