@@ -1,28 +1,28 @@
-! WHIZARD 2.2.8 Nov 22 2015
-! 
-! Copyright (C) 1999-2015 by 
+! WHIZARD 2.4.1 Mar 24 2017
+!
+! Copyright (C) 1999-2017 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
-!     
+!
 !     with contributions from
 !     Fabian Bach <fabian.bach@t-online.de>
 !     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com> 
-!     Soyoung Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>  
+!     Christian Speckner <cnspeckn@googlemail.com>
+!     So Young Shim <soyoung.shim@desy.de>
+!     Florian Staub <florian.staub@cern.ch>
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam, 
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam,
+!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
-! under the terms of the GNU General Public License as published by 
+! under the terms of the GNU General Public License as published by
 ! the Free Software Foundation; either version 2, or (at your option)
 ! any later version.
 !
 ! WHIZARD is distributed in the hope that it will be useful, but
 ! WITHOUT ANY WARRANTY; without even the implied warranty of
-! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ! GNU General Public License for more details.
 !
 ! You should have received a copy of the GNU General Public License
@@ -34,7 +34,7 @@
 ! to the source 'whizard.nw'
 
 module process_configurations_uti
-  
+
   use iso_varying_string, string_t => varying_string
   use particle_specifiers, only: new_prt_spec
   use prclib_stacks
@@ -42,7 +42,7 @@ module process_configurations_uti
   use rt_data
 
   use process_configurations
-    
+
   implicit none
   private
 
@@ -89,11 +89,11 @@ contains
 
        call prc_config%init (prc_name, size (prt_in), n_components, global)
        call prc_config%setup_component (1, &
-            new_prt_spec (prt_in), new_prt_spec (prt_out), global)
+            new_prt_spec (prt_in), new_prt_spec (prt_out), global%model, global%var_list)
        call prc_config%record (global)
 
        deallocate (prt_in, prt_out)
-       
+
     end if
 
     if (btest (mode, 1)) then
@@ -115,13 +115,13 @@ contains
 
        call prc_config%init (prc_name, size (prt_in), n_components, global)
        call prc_config%setup_component (1, &
-            new_prt_spec (prt_in), new_prt_spec (prt_out), global)
+            new_prt_spec (prt_in), new_prt_spec (prt_out), global%model, global%var_list)
        call prc_config%record (global)
 
        deallocate (prt_in, prt_out)
-       
+
     end if
-    
+
     if (btest (mode, 2)) then
 
        call global%select_model (var_str ("Test"))
@@ -141,20 +141,20 @@ contains
 
        call prc_config%init (prc_name, size (prt_in), n_components, global)
        call prc_config%setup_component (1, &
-            new_prt_spec (prt_in), new_prt_spec (prt_out), global)
+            new_prt_spec (prt_in), new_prt_spec (prt_out), global%model, global%var_list)
        call prc_config%record (global)
 
        deallocate (prt_in, prt_out)
-       
+
     end if
 
   end subroutine prepare_test_library
-    
+
 
   subroutine process_configurations_1 (u)
     integer, intent(in) :: u
     type(rt_data_t), target :: global
-    
+
     write (u, "(A)")  "* Test output: process_configurations_1"
     write (u, "(A)")  "*   Purpose: configure test processes"
     write (u, "(A)")
@@ -180,16 +180,16 @@ contains
 
     call global%final ()
     call syntax_model_file_final ()
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: process_configurations_1"
-    
+
   end subroutine process_configurations_1
-  
+
   subroutine process_configurations_2 (u)
     integer, intent(in) :: u
     type(rt_data_t), target :: global
-    
+
     type(string_t) :: libname
     type(prclib_entry_t), pointer :: lib
     type(string_t) :: prc_name
@@ -204,12 +204,12 @@ contains
     call syntax_model_file_init ()
 
     call global%global_init ()
-    
+
     write (u, "(A)")  "* Configure processes as omega, model QED"
     write (u, *)
 
     libname = "prc_config_lib_2"
-    
+
     allocate (lib)
     call lib%init (libname)
     call global%add_prclib (lib)
@@ -232,7 +232,7 @@ contains
     call global%set_log (var_str ("?report_progress"), &
          .true., is_known = .true.)
     call prc_config%setup_component (1, &
-         new_prt_spec (prt_in), new_prt_spec (prt_out), global)
+         new_prt_spec (prt_in), new_prt_spec (prt_out), global%model, global%var_list)
 
     call global%set_log (var_str ("?report_progress"), &
          .false., is_known = .true.)
@@ -244,12 +244,12 @@ contains
          var_str ("-fusion:progress_file omega_prc_config.log"), &
          is_known = .true.)
     call prc_config%setup_component (2, &
-         new_prt_spec (prt_in), new_prt_spec (prt_out), global)
-    
+         new_prt_spec (prt_in), new_prt_spec (prt_out), global%model, global%var_list)
+
     call prc_config%record (global)
 
     deallocate (prt_in, prt_out)
-    
+
     global%os_data%fc = "Fortran-compiler"
     global%os_data%fcflags = "Fortran-flags"
 
@@ -264,12 +264,12 @@ contains
 
     call global%final ()
     call syntax_model_file_final ()
-    
+
     write (u, "(A)")
     write (u, "(A)")  "* Test output end: process_configurations_2"
-    
+
   end subroutine process_configurations_2
-  
+
 
 end module process_configurations_uti
-  
+
