@@ -1,9 +1,9 @@
 #!/bin/sh
-### Check WHIZARD/O'Mega Circe/ISR
+### Check WHIZARD/O'Mega Circe2 setup
 echo "Running script $0"
 if test -f OCAML_FLAG; then
     script=`basename @script@`
-    ./run_whizard.sh @script@ --no-logging --model SM
+    ./run_whizard.sh @script@ --no-logging
     diff ref-output/$script.ref $script.log
 else
     echo "|=============================================================================|"
